	SUBROUTINE BESSYR_JULCENT(BY,T)
C
C       This subroutine computes the number of Julian centuries since the
C  fundamental epoch, given the Besselian year.  BC years should be 
C  input as  negative numbers.
C  Reference:  The Explanatory Supplement (1961 ed.), p. 22, p. 98.
C			Donald H. Gudehus, February 16, 1994
C			Version 2, May 16, 2005, D. Gudehus
C			Corrected for a misinterpretation of Table 14.3
C			in the Explanatory Supplement (1961 ed.).
C			Revised code based on Lieske, J. H. 1979,
C			Astron & Astrophysics, 73, 282.
C
C	BY	(C)	The Besselian year
C	T	(R)	The number of Julian centuries of 36525 days each 
C			measured from the fundamental epoch of 1900
C			January 0 at 12 hours E.T. = JD 2415020.0
C
	REAL*8 BY,T,TEMP
C
C  The fundamental epoch of the Besselian year is 1900.0 or 1900
C  January 0.813 E.T., where Jan. 0.0 denotes Greenwich midnight of the
C  preceding day.
	TEMP=BY-1900.0D0 !  Interval in Besselian years
	TEMP=TEMP*365.242198781D0+0.31352D0  ! Interval in ephemeris days from
			!  fundamental epoch of January 0.5 E.T.
C  Convert to Julian centuries of 36525 ephemeris days.
	T=TEMP/36525.0D0
	RETURN
	END
