	SUBROUTINE BINARY_INDEXES(CHOICE,NUMPARAM)
C
C  Note:  This is the Unix version
C
C       This subroutine helps in the nonlinear least squares fit for the 
C  orbital parameters of a binary star system by calculating the indexes
C  of the PARAMETRS array of all the variables to be solved for, certain
C  logical values, and the data dimension.
C			Donald H. Gudehus, June, 1995
C			Version 2, Donald H. Gudehus, October 3, 1997
C			Changed COMMON blocks for OpenVMS Alpha and UNIX
C			Version 3, Donald H. Gudehus, November 25, 1999
C			Fixed bug by increasing upper index from 17 to 18
C			for CHOICE = 'T'
C			Version 4, Donald H. Gudehus, February 26, 2000 
C			Added code for apsidal motion and OFFSET_SYS_VEL
C			Version 5, Donald H. Gudehus, September 22, 2001
C			Added code for apsidal motion of subcomponents
C			Version 6, Donald H. Gudehus, September 28, 2001
C			Removed INDX from argument list and put it in the
C			COMMON block.
C			Version 7, Donald H. Gudehus, December 22, 2002
C			Changed logic for determining when the parallax
C			could be solved for.  This had caused an error in the
C			case of a combined solution with data from a visual
C			secondary with relative coordinates, and a
C			spectroscopic secondary.  Added writing of
C			NUMPARAM to log file.  Added support for 1-D
C			interferometric data.
C
C	CHOICE		(C)	The choice which determines the type of
C				solution to use for a given multiple star
C				system configuration.  The possibilities
C				are:  
C				CHOICE(1:1)='P'   Primary of visual binary
C				CHOICE(1:1)='S'   Secondary of visual binary
C				CHOICE(1:1)='B'	  Both members of visual
C						  binary
C				CHOICE(1:1)='N'   No visual binary data
C				CHOICE(2:2)='R'   Parallax and proper motion
C						  removed from primary visual
C						  binary data
C				CHOICE(2:2)='I'   Parallax and proper motion
C						  included in primary data
C				CHOICE(3:3)='A'   Parallax and proper motion
C						  removed from secondary visual
C						  binary data;  orbit type is
C						  absolute
C				CHOICE(3:3)='R'   Parallax and proper motion
C						  removed from secondary visual
C						  binary data;  orbit type is
C						  relative
C				CHOICE(3:3)='I'   Parallax and proper motion
C						  included in secondary data;
C						  implies absolute
C				CHOICE(4:4)='P'	  Velocity data available for
C						  primary
C				CHOICE(4:4)='S'	  Velocity data available for
C						  secondary
C				CHOICE(4:4)='B'	  Velocity data available for
C						  primary and secondary
C				CHOICE(4:4)='N'   No spectroscopic binary
C						  data available
C				CHOICE(5:5)='n'	  1) The primary includes 
C						  velocity subcomponents equal
C						  in number to n, where said
C						  primary is taken to be the
C						  primary of the subsystem. 
C						  n should be in the range
C						  1 to 5.  If this
C						  position is blank, no
C						  subcomponents exist.  The
C						  action to be carried out is
C						  controlled by  CHOICE(6:6).
C						  2) If CHOICE(6:6)='o', then
C						  n is the actual subcompo-
C						  nent.
C				CHOICE(6:6)=' '	  Adds all velocity sub-
C						  components, up to n, to the
C						  primary.
C					   ='o'	  Evaluates only primary
C						  subcomponent n.
C					   ='a'	  Evaluates the sum of all
C						  subcomponents.
C					   ='m'	  Evaluates only the main
C						  component
C				CHOICE(7:7)='n'	  1) The secondary includes
C						  velocity subcomponents equal
C						  in number to n, where said
C						  secondary is taken to be the
C						  primary of the subsystem.
C						  n should be in the range
C						  1 to 5.  If this
C						  position is blank, no
C						  subcomponents exist.  The
C						  action to be carried out is
C						  controlled by  CHOICE(8:8).
C						  2) If CHOICE(8:8)='o', then
C						  n is the actual subcompo-
C						  nent.
C				CHOICE(8:8)=' '	  Adds all velocity sub-
C						  components, up to n, to the
C						  secondary.
C					   ='o'	  Evaluates only secondary
C						  subcomponent n.
C					   ='a'	  Evaluates the sum of all
C						  subcomponents.
C					   ='m'   Evaluates only the main
C						  component
C				CHOICE(9:13)='T'  Evaluates one or more
C						  velocities of secondaries 
C						  of primary subcomponents 1:5
C				CHOICE(14:18)='T' Evaluates one or more
C						  velocities of secondaries 
C						  of secondary subcomponents 
C						  1:5
C				CHOICE(19:19)='A' Apsidal motion is evaluated
C						  for main components
C				CHOICE(20:24)='A' Apsidal motion is evaluated
C						  for one or more primary
C						  subcomponents 1:5
C				CHOICE(25:29)='A' Apsidal motion is evaluated
C						  for one or more secondary
C						  subcomponents 1:5
C				CHOICE(30:30)='P' Primary 1D interferometric
C						  data
C				CHOICE(30:30)='S' Secondary 1D interferometric
C						  data
C				CHOICE(30:30)='B' 1D interferometric data for
C						  both primary and secondary
C				CHOICE(30:30)='N' No 1D interferometric data
C				CHOICE(31:31)='R' Parallax and proper motion
C						  removed from primary 1D
C						  interferometric data
C				CHOICE(31:31)='I' Parallax and proper motion
C						  included in primary data
C				CHOICE(32:32)='A' Parallax and proper motion
C						  removed from secondary 1D
C						  data;  orbit type is
C						  absolute
C				CHOICE(32:32)='R' Parallax and proper motion
C						  removed from secondary 1D
C						  data;  orbit type is
C						  relative
C				CHOICE(32:32)='I' Parallax and proper motion
C						  included in secondary 1D 
C						  data;  implies absolute
C				
C	NUMPARAM	(R)	The number of parameters to be solved for
C				for a given value of CHOICE.  The sign is
C				made negative to allow the user to be
C				prompted for which parameters to hold
C				constant in NONLINFIT.
C
C			Explanation of offsets
C
C	OFFSET_S	Skips over node angle and inclination when not needed
C	OFFSET_S2	Skips over a(")
C	OFFSET_RATIO	Skips over ratio of semimajor axes (total to primary)
C	OFFSET_CMX	Skips over X center of mass
C	OFFSET_PMQX	Skips over X proper motion terms
C	OFFSET_CMY	Skips over Y center of mass
C	OFFSET_PMQY	Skips over Y proper motion terms
C	OFFSET_SYS_VEL	Skips over system velocity
C	OFFSET_S3	Skips over a(AU)sin(i)
C	OFFSET_PRI_SUB	Skips over parameters associated with spectroscopic
C			subcomponents of primary
C	OFFSET_SEC_SUB	Skips over parameters associated with spectroscopic
C			subcomponents of secondary
C	OFFSET_SUB_SEC	Skips over parameters associated with primary and
C			secondary subcomponent secondaries (CHOICE(9:18)='T')
C	OFFSET_AP_MAIN	Skips over parameter associated with apsidal motion
C	OFFSET_AP_SUB	Skips over parameters associated with apsidal motions
C			of subcomponents
C	OFFSET_CM_INTERFERO
C			Skips over X and Y center of mass for interferometric
C			data
C
C			Explanation of semimajor axis assignments
C
C	SEMIMAJOR_AXIS_ASSIGNMENT(1)	0	a(") not assigned
C	SEMIMAJOR_AXIS_ASSIGNMENT(1)	1	a(") assigned to primary
C	SEMIMAJOR_AXIS_ASSIGNMENT(1)	2	a(") assigned to secondary
C	SEMIMAJOR_AXIS_ASSIGNMENT(1)	3	a(") assigned to relative orbit
C	SEMIMAJOR_AXIS_ASSIGNMENT(2)	0	a(AU) not assigned
C	SEMIMAJOR_AXIS_ASSIGNMENT(2)	1	a(AU) assigned to primary
C	SEMIMAJOR_AXIS_ASSIGNMENT(2)	2	a(AU) assigned to secondary
C
C			Explanation of INDX
C
C	INDX		An array containing the indexes which
C			reference the associated variable in the
C			array PARAMETRS.  Each location in INDX
C			corresponds to one of many possible variables
C			that can be solved for in a multiple star
C			system.
C
C	INDX(1)		Period = P
C	INDX(2)		Epoch of periastron passage = T
C	INDX(3)		Eccentricity = e
C	INDX(4)		Node angle = big omega
C	INDX(5)		Inclination = i
C	INDX(6)		Angle from node to periastron = w
C	INDX(7)		Semimajor axis = a
C	INDX(8)		Ratio of semimajor axes (Total to primary)
C	INDX(9)		Parallax = pi
C	INDX(10)	X center of mass at epoch of periastron passage
C	INDX(11)	X proper motion
C	INDX(12)	X acceleration
C	INDX(13)	Y center of mass
C	INDX(14)	Y proper motion
C	INDX(15)	Y acceleration
C	INDX(16)	System velocity
C	INDX(17)	Projected semimajor axis in AU
C	INDX(18)	Period of primary subcomponent #1
C	INDX(19)	Epoch "
C	INDX(20)	Eccentricity "
C	INDX(21)	Angle from node to periastron "
C	INDX(22)	Projected semimajor axis in AU "
C	INDX(23)	Period of primary subcompoent #2
C	INDX(24)	Epoch "
C	INDX(25)	Eccentricity "
C	INDX(26)	Angle from node to periastron "
C	INDX(27)	Projected semimajor axis in AU "
C	INDX(28)	Period of primary subcompoent #3
C	INDX(29)	Epoch "
C	INDX(30)	Eccentricity "
C	INDX(31)	Angle from node to periastron "
C	INDX(32)	Projected semimajor axis in AU "
C	INDX(33)	Period of primary subcompoent #4
C	INDX(34)	Epoch "
C	INDX(35)	Eccentricity "
C	INDX(36)	Angle from node to periastron "
C	INDX(37)	Projected semimajor axis in AU "
C	INDX(38)	Period of primary subcompoent #5
C	INDX(39)	Epoch "
C	INDX(40)	Eccentricity "
C	INDX(41)	Angle from node to periastron "
C	INDX(42)	Projected semimajor axis in AU "
C	INDX(43)	Period of secondary subcomponent #1
C	INDX(44)	Epoch "
C	INDX(45)	Eccentricity "
C	INDX(46)	Angle from node to periastron "
C	INDX(47)	Projected semimajor axis in AU "
C	INDX(48)	Period of secondary subcomponent #2
C	INDX(49)	Epoch "
C	INDX(50)	Eccentricity "
C	INDX(51)	Angle from node to periastron "
C	INDX(52)	Projected semimajor axis in AU "
C	INDX(53)	Period of secondary subcomponent #3
C	INDX(54)	Epoch "
C	INDX(55)	Eccentricity "
C	INDX(56)	Angle from node to periastron "
C	INDX(57)	Projected semimajor axis in AU "
C	INDX(58)	Period of secondary subcomponent #4
C	INDX(59)	Epoch "
C	INDX(60)	Eccentricity "
C	INDX(61)	Angle from node to periastron "
C	INDX(62)	Projected semimajor axis in AU "
C	INDX(63)	Period of secondary subcomponent #5
C	INDX(64)	Epoch "
C	INDX(65)	Eccentricity "
C	INDX(66)	Angle from node to periastron "
C	INDX(67)	Projected semimajor axis in AU "
C	INDX(68)	Ratio of semimajor axes for primary subcomponent #1
C	INDX(69)	" #2
C	INDX(70)	" #3
C	INDX(71)	" #4
C	INDX(72)	" #5
C	INDX(73)	Ratio of semimajor axes for secondary subcomponent #1
C	INDX(74)	" #2
C	INDX(75)	" #3
C	INDX(76)	" #4
C	INDX(77)	" #5
C	INDX(78)	Apsidal motion in degrees per year
C	INDX(79)	Apsidal motion for primary subcomponent #1
C	INDX(80)	" #2
C	INDX(81)	" #3
C	INDX(82)	" #4
C	INDX(83)	" #5
C	INDX(84)	Apsidal motion for secondary subcomponent #1
C	INDX(85)	" #2
C	INDX(86)	" #3
C	INDX(87)	" #4
C	INDX(88)	" #5
C	INDX(89)	X center of mass from 1-D interferometric data
C	INDX(90)	Y "

	INCLUDE 'binary_parameters.for'
C
	INTEGER OFFSET_S,OFFSET_S2,OFFSET_RATIO,OFFSET_CMX,
	1OFFSET_PMQX,OFFSET_CMY,OFFSET_PMQY,OFFSET_PPM,
	2OFFSET_SYS_VEL,OFFSET_S3,OFFSET_CON,OFFSET_CON2,
	3OFFSET_PRI_SUB,OFFSET_SEC_SUB,OFFSET_SUB_SEC,OFFSET_AP_MAIN,
	4OFFSET_AP_SUB,OFFSET_CM_INTERFERO
C
	CHARACTER*(*) CHOICE
C
	INTEGER SEMIMAJOR_AXIS_ASSIGNMENT(2),DIMENSN_DATA,DIMENSN_DATA_USED,
	1INDX(MAXNUMPARAM)
	COMMON /BINARY_INTEGER4/ SEMIMAJOR_AXIS_ASSIGNMENT,
	1DIMENSN_DATA,DIMENSN_DATA_USED,INDX
C
	LOGICAL VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
	COMMON /BINARY_LOGICALS/ VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
C
	VISUAL=.FALSE.
	PAR_PM=.FALSE.
	PAR_PM_INTERFERO=.FALSE.
	SPECTROSCOPIC=.FALSE.
	SPECTROSCOPIC_PRI_SUB=.FALSE.
	SPECTROSCOPIC_SEC_SUB=.FALSE.
	INTERFEROMETRIC=.FALSE.
C
C  Establish some logical values
C
	IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'S'.OR.
	1CHOICE(1:1).EQ.'B') VISUAL=.TRUE.
	IF (CHOICE(2:2).EQ.'I'.OR.CHOICE(3:3).EQ.'I') PAR_PM=.TRUE.
	IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'S'.OR.
	1CHOICE(4:4).EQ.'B') SPECTROSCOPIC=.TRUE.
	IF (CHOICE(30:30).EQ.'P'.OR.CHOICE(30:30).EQ.'S'.OR.
	1CHOICE(30:30).EQ.'B') INTERFEROMETRIC=.TRUE.
	IF (CHOICE(31:31).EQ.'I'.OR.CHOICE(32:32).EQ.'I')
	1PAR_PM_INTERFERO=.TRUE.
C
C  Determine the assignment of the semimajor axis.
C
c	IF (.NOT.VISUAL) THEN
c	  SEMIMAJOR_AXIS_ASSIGNMENT(1)=0
c	ELSE
c	  IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B'.OR.
c	1 CHOICE(4:4).EQ.'B') SEMIMAJOR_AXIS_ASSIGNMENT(1)=1
c	  IF ((CHOICE(1:1).EQ.'S'.AND.(CHOICE(3:3).EQ.'A'.OR.CHOICE(3:3).EQ.
c	1 'I')).AND.CHOICE(4:4).NE.'B') SEMIMAJOR_AXIS_ASSIGNMENT(1)=2
c	  IF (CHOICE(1:1).EQ.'S'.AND.CHOICE(3:3).EQ.'R'.AND.CHOICE(4:4).NE.
c	1 'B') SEMIMAJOR_AXIS_ASSIGNMENT(1)=3
c	END IF
c	SEMIMAJOR_AXIS_ASSIGNMENT(2)=0  ! Initializes it
c	IF (((CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B').AND..NOT.VISUAL).OR.
c	1(CHOICE(4:4).EQ.'P'.AND.CHOICE(1:1).EQ.'S'))
c	2SEMIMAJOR_AXIS_ASSIGNMENT(2)=1
c	IF (CHOICE(4:4).EQ.'S'.AND.(CHOICE(1:1).EQ.'P'.OR.
c	1(CHOICE(1:1).EQ.'S'.AND.CHOICE(3:3).EQ.'R').OR.CHOICE(1:1).EQ.'N'))
c	2SEMIMAJOR_AXIS_ASSIGNMENT(2)=2
C
C  Determine the assignment of the semimajor axis.
C
	IF (.NOT.VISUAL.AND..NOT.INTERFEROMETRIC) THEN
	  SEMIMAJOR_AXIS_ASSIGNMENT(1)=0
	ELSE
	  IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B'.OR.
	1 CHOICE(4:4).EQ.'B'.OR.CHOICE(30:30).EQ.'P'.OR.CHOICE(30:30).EQ.'B')
	2 SEMIMAJOR_AXIS_ASSIGNMENT(1)=1 !  a(") to primary
	  IF (( (CHOICE(1:1).EQ.'S'.OR.CHOICE(30:30).EQ.'S').AND.
	1 (CHOICE(3:3).EQ.'A'.OR.CHOICE(3:3).EQ.'I'.OR.
	2 CHOICE(32:32).EQ.'A'.OR.CHOICE(32:32).EQ.'I') )
	3 .AND.CHOICE(4:4).NE.'B') SEMIMAJOR_AXIS_ASSIGNMENT(1)=2
				!  a(") to secondary
	  IF ((CHOICE(1:1).EQ.'S'.OR.CHOICE(30:30).EQ.'S').AND.
	1 (CHOICE(3:3).EQ.'R'.OR.CHOICE(32:32).EQ.'R').AND.CHOICE(4:4).NE.
	2 'B') SEMIMAJOR_AXIS_ASSIGNMENT(1)=3 !  a(") to relative orbit
	END IF
C
	SEMIMAJOR_AXIS_ASSIGNMENT(2)=0  ! Initializes it
	IF ( ((CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B').AND..NOT.
	1VISUAL.AND..NOT.INTERFEROMETRIC).OR.(CHOICE(4:4).EQ.'P'.AND.
	2(CHOICE(1:1).EQ.'S'.OR.CHOICE(30:30).EQ.'S')) )
	3SEMIMAJOR_AXIS_ASSIGNMENT(2)=1 !  a(AU) to primary
	IF ( (CHOICE(4:4).EQ.'S'.AND.(CHOICE(1:1).EQ.'P'.OR.
	1(CHOICE(1:1).EQ.'S'.AND.CHOICE(3:3).EQ.'R').OR.
	2CHOICE(1:1).EQ.'N')) .OR.
	3(CHOICE(4:4).EQ.'S'.AND.(CHOICE(30:30).EQ.'P'.OR.
	4(CHOICE(30:30).EQ.'S'.AND.CHOICE(32:32).EQ.'R').OR.
	5CHOICE(30:30).EQ.'N')) )
	6SEMIMAJOR_AXIS_ASSIGNMENT(2)=2 !  a(AU) to secondary
C  Determine in what cases parallax can be solved for by using a visual,
C  spectroscopic, or interferometric binary.  This excludes the case
C  of trigonometric parallax.  Note that when parallax can't be
C  determined by any means "a" for the spectroscopic binary will have
C  the units of AU.
	SOLVE_PAR=.FALSE.
	IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(30:30).EQ.'P') THEN
	  IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B') SOLVE_PAR=.TRUE.
	ELSE IF ((CHOICE(1:1).EQ.'S'.OR.CHOICE(30:30).EQ.'S').AND.
	1(SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1.OR.SEMIMAJOR_AXIS_ASSIGNMENT(1)
	2.EQ.2)) THEN
	  IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') SOLVE_PAR=.TRUE.
	ELSE IF ((CHOICE(1:1).EQ.'S'.OR.CHOICE(30:30).EQ.'S').AND.
	1SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.3) THEN
	  IF (CHOICE(4:4).EQ.'B') SOLVE_PAR=.TRUE.
	ELSE IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(30:30).EQ.'B') THEN
	  IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4)
	1 .EQ.'B') SOLVE_PAR=.TRUE.
	END IF
c to here
C
C  Handle the case of the primary having a subcomponent
	IF (CHOICE(5:5).NE.' ') THEN
	  SPECTROSCOPIC_PRI_SUB=.TRUE.
	  DECODE(1,1001,CHOICE(5:5)) IPRISUB
1001	  FORMAT(I1)
	END IF
C  Handle the case of the secondary having a subcomponent
	IF (CHOICE(7:7).NE.' ') THEN
	  SPECTROSCOPIC_SEC_SUB=.TRUE.
	  DECODE(1,1001,CHOICE(7:7)) ISECSUB
	END IF
C
C  Obtain the data dimension
C
	IF (CHOICE(1:1).EQ.'P') THEN  ! Primary
	  DIMENSN_DATA=2  ! Positions 1 and 2 are for visual primary data
	ELSE IF (CHOICE(1:1).EQ.'S'.OR.CHOICE(1:1).EQ.'B') THEN
	  DIMENSN_DATA=4  ! Secondary visual data is always in positions
                          ! 3 and 4
	END IF
	IF (CHOICE(4:4).EQ.'P') THEN
	  DIMENSN_DATA=5  ! Primary spectroscopic data is always in
			  ! position 5.  Positions 1 to 4 may or may not
			  ! be needed
	ELSE IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') THEN
	  DIMENSN_DATA=6  ! Secondary spectroscopic data is always in position
                          ! 6.
	END IF
C
	DIMENSN_DATA_USED=0
	IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'S')
	1DIMENSN_DATA_USED=DIMENSN_DATA_USED+2  ! X and Y
	IF (CHOICE(1:1).EQ.'B')
	1DIMENSN_DATA_USED=DIMENSN_DATA_USED+4  ! X and Y
	IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'S')
	1DIMENSN_DATA_USED=DIMENSN_DATA_USED+1  ! Velocity
	IF (CHOICE(4:4).EQ.'B')
	1DIMENSN_DATA_USED=DIMENSN_DATA_USED+2  ! Velocity
C
	DO J=9,18
	  IF (CHOICE(J:J).EQ.'T') THEN
	    DIMENSN_DATA=J-2 !  7 is the first available one after 6
	    DIMENSN_DATA_USED=DIMENSN_DATA_USED+1
	  END IF
	END DO
C
	IF (CHOICE(30:30).EQ.'P') DIMENSN_DATA=17 !  Primary interferometric
	IF (CHOICE(30:30).EQ.'S'.OR.CHOICE(30:30).EQ.'B')
	1DIMENSN_DATA=18 !  Secondary interferometric
	IF (CHOICE(30:30).EQ.'P'.OR.CHOICE(1:1).EQ.'S')
	1DIMENSN_DATA_USED=DIMENSN_DATA_USED+1
	IF (CHOICE(30:30).EQ.'B')
	1DIMENSN_DATA_USED=DIMENSN_DATA_USED+2
C
C  Establish the offsets and assign the indexes
C
C  Initialize
	DO I=1,MAXNUMPARAM
	  INDX(I)=I
	END DO
C  For the first five, P, T, e, Omega, theta, INDX(I)=I
	IF (SPECTROSCOPIC.AND..NOT.VISUAL.AND..NOT.INTERFEROMETRIC) THEN
C  If only spectroscopic data is available, we can't solve for the
C  node or inclination angle.  In this case, make an offset.  Note that
C  a is in units of AU.
	  OFFSET_S=2  ! For big omega and theta which can't be solved for;
		      ! Indexes 4 and 5
	  INCLINATION_AVAILABLE=.FALSE.
	ELSE IF (VISUAL.OR.INTERFEROMETRIC) THEN !  Includes SPECTROSCOPIC 
						 !  and not SPECTROSCOPIC
C  a is in units of arc seconds
	  OFFSET_S=0
	  INCLINATION_AVAILABLE=.TRUE.
	END IF
C
	INDX(6)=6-OFFSET_S  ! w
C
	INDX(7)=7-OFFSET_S  ! a(")
C  Now compute the offset associated with INDX(7).  This affects higher
C  numbered indexes.
	IF (VISUAL.OR.INTERFEROMETRIC) THEN
	  OFFSET_S2=0 !  a(")
	ELSE 
C  If not visual then "a" cannot be in units of ".  For a spectroscopic 
C  binary, without visual information for the same component, we must use 
C  "a" in units of AU.  This is introduced as parameter 17.
	  OFFSET_S2=1
	END IF
C
C  Because parameter number 8 is taken to be the ratio of semimajors axes
C  (total to primary) in the case of a pair of stars, and because this is
C  not used for a single star, and because parameters numbers greater than
C  8 may be used (to solve for proper motion and parallax), an offset is
C  used.
	INDX(8)=8-OFFSET_S-OFFSET_S2  ! Ratio of semimajor axes (total to pri.)
	IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(4:4).EQ.'B'
	1.OR.CHOICE(30:30).EQ.'B') THEN
		!  A visual, spectroscopic, or interferometric binary
	  OFFSET_RATIO=0
	  RATIO_AVAILABLE=.TRUE.
	ELSE !  Single member of binary
	  OFFSET_RATIO=1 !  Ignores ratio of semimajor axes
	  RATIO_AVAILABLE=.FALSE.
	END IF
C
	INDX(9)=9-OFFSET_S-OFFSET_S2-OFFSET_RATIO !  Parallax
	IF (SOLVE_PAR.OR.PAR_PM.OR.PAR_PM_INTERFERO) THEN
	  OFFSET_PAR=0
	ELSE
	  OFFSET_PAR=1
	END IF
C
	INDX(10)=10-OFFSET_S-OFFSET_S2-OFFSET_RATIO-OFFSET_PAR !  CM_X
	IF (PAR_PM) THEN
	  OFFSET_CMX=0
	ELSE
	  OFFSET_CMX=1
	END IF
C
C  X proper motion and quadratic term (PM_X, Q_X)
	INDX(11)=11-OFFSET_S-OFFSET_S2-OFFSET_RATIO-OFFSET_PAR-OFFSET_CMX
	INDX(12)=12-OFFSET_S-OFFSET_S2-OFFSET_RATIO-OFFSET_PAR-OFFSET_CMX
	IF (PAR_PM.OR.PAR_PM_INTERFERO) THEN
	  OFFSET_PMQX=0
	ELSE
	  OFFSET_PMQX=2
	END IF
C
	INDX(13)=13-OFFSET_S-OFFSET_S2-OFFSET_RATIO-OFFSET_PAR-OFFSET_CMX
	1-OFFSET_PMQX !  CM_Y
	IF (PAR_PM) THEN
	  OFFSET_CMY=0
	ELSE
	  OFFSET_CMY=1
	END IF
C
C  Y proper motion and quadratic term (PM_Y, Q_Y)
	INDX(14)=14-OFFSET_S-OFFSET_S2-OFFSET_RATIO-OFFSET_PAR-OFFSET_CMX
	1-OFFSET_PMQX-OFFSET_CMY
	INDX(15)=15-OFFSET_S-OFFSET_S2-OFFSET_RATIO-OFFSET_PAR-OFFSET_CMX
	1-OFFSET_PMQX-OFFSET_CMY
	IF (PAR_PM.OR.PAR_PM_INTERFERO) THEN
	  OFFSET_PMQY=0
	ELSE
	  OFFSET_PMQY=2
	END IF
C
C  Consolidate some offsets
	OFFSET_PPM=OFFSET_PAR+OFFSET_CMX+OFFSET_PMQX+OFFSET_CMY+OFFSET_PMQY
	OFFSET_CON=OFFSET_S+OFFSET_S2+OFFSET_RATIO+OFFSET_PPM
C
	INDX(16)=16-OFFSET_CON !  System velocity
C  Offset past the system velocity if not spectroscopic (such as when
C  solving for apsidal motion)
	IF (.NOT.SPECTROSCOPIC) THEN
	  OFFSET_SYS_VEL=1
	ELSE
	  OFFSET_SYS_VEL=0
	END IF
C
	INDX(17)=17-OFFSET_CON-OFFSET_SYS_VEL !  aAU
			!  (OFFSET_SYS_VEL should always be zero here)
C  In some cases a(AU) needs to be solved for
	IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).NE.0) THEN
	  OFFSET_S3=0
	ELSE
	  OFFSET_S3=1 !  Excludes a(AU)
		      !  Index 17
	END IF
C
	DO I=18,42 !  Spectroscopic subcomponents of primary
	  INDX(I)=I-OFFSET_CON-OFFSET_SYS_VEL-OFFSET_S3
	    !  P, T, e, w, aAU for primary subcomponent
	END DO
C  Handle the case of the primary having a subcomponent
	IF (SPECTROSCOPIC_PRI_SUB) THEN
	  OFFSET_PRI_SUB=5*(5-IPRISUB)
	ELSE
	  OFFSET_PRI_SUB=25 !  Indexes 18 to 42
	END IF
C
	DO I=43,67 !  Spectroscopic subcomponents of secondary
	  INDX(I)=I-OFFSET_CON-OFFSET_SYS_VEL-OFFSET_S3-OFFSET_PRI_SUB
	    !  P, T, e, w, aAU for secondary subcomponent
	END DO
C  Handle the case of the secondary having a subcomponent
	IF (SPECTROSCOPIC_SEC_SUB) THEN
	  OFFSET_SEC_SUB=5*(5-ISECSUB)
	ELSE
	  OFFSET_SEC_SUB=25 !  Indexes 43 to 67
	END IF
C
C  Consolidate some offsets
	OFFSET_CON2=OFFSET_CON+OFFSET_SYS_VEL+OFFSET_S3+OFFSET_PRI_SUB+
	1OFFSET_SEC_SUB
C  Continue assigning the index
	DO I=68,77 !   Secondaries of spectroscopic subcomponents
	  INDX(I)=I-OFFSET_CON2 !  First approximation (assumes all are used)
	END DO
	OFFSET_SUB_SEC=0
	DO J=9,18 !  We need 10, starting at position 9
	  IF (CHOICE(J:J).NE.'T') THEN !  No velocities of secondaries of
				       !  subcomponents
	    OFFSET_SUB_SEC=OFFSET_SUB_SEC+1 !  Indexes 68 to 77
	    DO I=60+J,77 !  The next higher one is moved into the absent 
			 !  one's place
	      INDX(I)=INDX(I)-1 !  Final calculation
	    END DO
	  END IF
	END DO
C
	INDX(78)=78-OFFSET_CON2-OFFSET_SUB_SEC !  Apsidal motion for main
					       !  components
	IF (CHOICE(19:19).EQ.'A') THEN
	  OFFSET_AP_MAIN=0
	ELSE
	  OFFSET_AP_MAIN=1
	END IF
C
	DO I=79,88 !  Apsidal motion for subcomponents
	  INDX(I)=I-OFFSET_CON2-OFFSET_SUB_SEC-OFFSET_AP_MAIN
		!  First approximation (assumes all are used)
	END DO
	OFFSET_AP_SUB=0
	DO J=20,29 !  We need 10, starting at position 20
	  IF (CHOICE(J:J).NE.'A') THEN !  No apsidal motion requested
	    OFFSET_AP_SUB=OFFSET_AP_SUB+1
	    DO I=60+J,88 !  The next higher one is moved into the absent
			 !  one's place
	      INDX(I)=INDX(I)-1 !  Final calculation
	    END DO
	  END IF
	END DO
C
	INDX(89)=89-OFFSET_CON2-OFFSET_SUB_SEC-OFFSET_AP_MAIN-OFFSET_AP_SUB
	INDX(90)=90-OFFSET_CON2-OFFSET_SUB_SEC-OFFSET_AP_MAIN-OFFSET_AP_SUB
	IF (PAR_PM_INTERFERO) THEN
	  OFFSET_CM_INTERFERO=0
	ELSE
	  OFFSET_CM_INTERFERO=2
	END IF
C
C  Write the offsets to the log file
	WRITE (7,1900)
1900	FORMAT (' ','OFFSETS')
	WRITE (7,2000) OFFSET_S,OFFSET_S2,OFFSET_RATIO,OFFSET_CMX
2000	FORMAT (' ','S = ',I3,' S2 = ',I3,' RATIO = ',I3,' CMX = ',I3)
	WRITE (7,2010) OFFSET_PMQX,OFFSET_CMY,OFFSET_PMQY,OFFSET_PPM
2010	FORMAT (' ','PMQX = ',I3,' CMY = ',I3,' PMQY = ',I3,' PPM = ',I3)
	WRITE (7,2020) OFFSET_SYS_VEL,OFFSET_S3,OFFSET_PRI_SUB,OFFSET_SEC_SUB
2020	FORMAT (' ','SYS_VEL = ',I3,' S3 = ',I3,' PRI_SUB = ',I3,
	1' SEC_SUB = ',I3)
	WRITE (7,2030) OFFSET_CON,OFFSET_CON2,OFFSET_SUB_SEC,OFFSET_AP_MAIN
2030	FORMAT (' ','CON = ',I3,' CON2 = ',I3,' SUB_SEC = ',I3,
	1' AP_MAIN = ',I3)
	WRITE (7,2040) OFFSET_AP_SUB,OFFSET_CM_INTERFERO
2040	FORMAT (' ','AP_SUB = ',I3,' CM_INTERFERO = ',I3)
C  Write the indexes to the log file
	WRITE (7,2033)
2033	FORMAT(' ','I   INDEX')
	DO I=1,MAXNUMPARAM
	  WRITE (7,2034) I,INDX(I)
2034	  FORMAT(' ',I3,3X,I3)
	END DO
C
C  Obtain the number of parameters to solve for.
	NUMPARAM=-(MAXNUMPARAM-OFFSET_CON2-OFFSET_SUB_SEC-OFFSET_AP_MAIN
	1-OFFSET_AP_SUB-OFFSET_CM_INTERFERO)
C
C  Write the number of parameters to the log file
	WRITE (7,3016) NUMPARAM
3016    FORMAT(' ','NUMPARAM = ',I3)
C
	RETURN
	END
