	SUBROUTINE BINARY_LTT_COR(TIME,MAXNPTS,DIM_IND,
	1PARAMETRS,CHOICE)
C
C  Note:  This is the Unix version
C
C       This subroutine computes the light travel time correction for a
C  binary star system.  The time is corrected to the center of mass of the
C  system.  Only the spectroscopic data are corrected.
C
C			Donald H. Gudehus, July, 1995
C			Version 2, Donald Gudehus, September, 1997
C			Changed COMMON blocks for OpenVMS Alpha and UNIX
C			Version 3, Donald H. Gudehus, November 25, 1999
C			Increased number of points to 5000
C			Version 4, Donald H. Gudehus, February 25, 2000
C			Added the array TIME to the argument list to
C			enable the calculation of apsidal motion for the
C			main components.
C			Version 5, Donald H. Gudehus, September 25, 2001
C			Added code for apsidal motion of subcomponents.
C			Removed call to BINARY_INDEXES so that INDX for
C			the solution can be used.  It is now available
C			in the COMMON block.
C			Version 6, D. Gudehus, December, 2002
C			Modified COMMON block for interferometric data
C			
C	TIME		(C)	The array containing the times of observation
C	MAXNPTS		(C)	The first dimension of the array TIME
C	DIM_IND		(C)	The second dimension of the array TIME
C	PARAMETRS	(C)	The array containing the parameters used to
C				evaluate the function
C	CHOICE		(C)	The choice determining which function
C				to use within a category.  The possibilities
C				are:  
C				CHOICE(1:1)='P'   Primary of visual binary
C					   ='S'   Secondary of visual binary
C					   ='B'	  Both members of visual
C						  binary
C					   ='N'   No visual binary data
C				CHOICE(2:2)='R'   Parallax and proper motion
C						  removed from primary visual
C						  binary data
C					   ='I'   Parallax and proper motion
C						  included in primary data
C				CHOICE(3:3)='A'   Parallax and proper motion
C						  removed from secondary visual
C						  binary data;  orbit type is
C						  absolute
C					   ='R'   Parallax and proper motion
C						  removed from secondary visual
C						  binary data;  orbit type is
C						  relative
C					   ='I'   Parallax and proper motion
C						  included in secondary data;
C						  implies absolute
C				CHOICE(4:4)='P'	  Velocity data available for
C						  primary
C					   ='S'	  Velocity data available for
C						  secondary
C					   ='B'	  Velocity data available for
C						  primary and secondary
C					   ='N'   No spectroscopic binary
C						  data available
C				CHOICE(5:5)='n'	  1) The primary includes 
C						  velocity subcomponents equal
C						  in number to n, where said
C						  primary is taken to be the
C						  primary of the subsystem.
C						  n should be in the range
C						  1 to 5.  If this
C						  position is blank, no
C						  subcomponents exist.  The
C						  action to be carried out is
C						  controlled by  CHOICE(6:6).
C						  2) If CHOICE(6:6)='o', then
C						  n is the actual subcompo-
C						  nent.
C				CHOICE(6:6)=' '	  Adds all velocity sub-
C						  components, up to n, to the
C						  primary.
C					   ='o'	  Evaluates only primary
C						  subcomponent n.
C					   ='a'	  Evaluates the sum of all
C						  subcomponents.
C				CHOICE(7:7)='n'	  1) The secondary includes
C						  velocity subcomponents equal
C						  in number to n, where said
C						  secondary is taken to be the
C						  primary of the subsystem.
C						  n should be in the range
C						  1 to 5.  If this
C						  position is blank, no
C						  subcomponents exist.  The
C						  action to be carried out is
C						  controlled by  CHOICE(8:8).
C						  2) If CHOICE(8:8)='o', then
C						  n is the actual subcompo-
C						  nent.
C				CHOICE(8:8)=' '	  Adds all velocity sub-
C						  components, up to n, to the
C						  secondary.
C					   ='o'	  Evaluates only secondary
C						  subcomponent n.
C					   ='a'	  Evaluates the sum of all
C						  subcomponents.
C				CHOICE(9:13)='T'  Evaluates one or more
C						  velocities of secondaries 
C						  of primary subcomponents 1:5
C				CHOICE(14:18)='T' Evaluates one or more
C						  velocities of secondaries 
C						  of secondary subcomponents 
C						  1:5
C				CHOICE(19:19)='A' Apsidal motion is evaluated
C						  for main components
C				CHOICE(20:24)='A' Apsidal motion is evaluated
C						  for one or more primary
C						  subcomponents 1:5
C				CHOICE(25:29)='A' Apsidal motion is evaluated
C						  for one or more secondary
C						  subcomponents 1:5
C
	INCLUDE 'binary_parameters.for'
C
	DIMENSION PARAMETRS(1)
	CHARACTER*(*) CHOICE
	REAL*8 TIME(MAXNPTS,DIM_IND),PARAMETRS,PI,
	1E,P,P_PRI_SUB(NSUBS_MAX),P_SEC_SUB(NSUBS_MAX),
	2T,T_PRI_SUB(NSUBS_MAX),T_SEC_SUB(NSUBS_MAX),
	3ECCENTR,ECCENTR_PRI_SUB(NSUBS_MAX),ECCENTR_SEC_SUB(NSUBS_MAX),
	4OMEGA,INCLINA,w,w_save,w_PRI_SUB(NSUBS_MAX),
	5w_save_PRI_SUB(NSUBS_MAX),
	4w_SEC_SUB(NSUBS_MAX),w_save_SEC_SUB(NSUBS_MAX),
	4a,aAU,aAU_PRI_SUB(NSUBS_MAX),
	5aAU_SEC_SUB(NSUBS_MAX),PARALLAX,v,FRAC,
	1RATIO_a,RATIO_a_PRI_SUB_SEC(NSUBS_MAX),
	2RATIO_a_SEC_SUB_SEC(NSUBS_MAX)
C
	CHARACTER*1 TYPE_P,TYPE_S,TYPE_VEL_P,TYPE_VEL_S,TYPE_INTER_P,
	1TYPE_INTER_S
	COMMON /CHAR_SCRATCH_BLOCK/ TYPE_P,TYPE_S,TYPE_VEL_P,TYPE_VEL_S,
	1TYPE_INTER_P,TYPE_INTER_S
C
	INTEGER NPOINTS_DATA_VEL_P_SUB_S(NSUBS_MAX),
	1NPOINTS_DATA_VEL_S_SUB_S(NSUBS_MAX)
	COMMON /INTEGER4_SCRATCH_BLOCK/NPOINTSPXPY,NPOINTS_DATA_P,
	1NPOINTS_DATA_S,NPOINTS_VIS,NPOINTS_DATA_VEL_P,
	2NPOINTS_DATA_VEL_S,NPOINTS_SP,NPOINTS_DATA_VEL_P_SUB_S,
	3NPOINTS_DATA_VEL_S_SUB_S,NPOINTS_VEL_P_S,NPOINTS_VEL_S_S,
	4NPOINTS_DATA_INTER_P,NPOINTS_DATA_INTER_S,NPOINTS_INTER
C
	REAL*8 PX(MAXNPOINTS_COM),PY(MAXNPOINTS_COM),
	1FR_ANGLE(MAXNPOINTS_COM),EPOCH,
	1VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB(NSUBS_MAX),
	2VEL_AMP_SEC_SUB(NSUBS_MAX),VEL_AMP_PRI_SUB_SEC(NSUBS_MAX),
	3VEL_AMP_SEC_SUB_SEC(NSUBS_MAX),INCLINA_WORK,
	4INCLINA_WORK_PRI_SUB(NSUBS_MAX),INCLINA_WORK_SEC_SUB(NSUBS_MAX),
	5ECC_ANOM(MAXNPOINTS_COM),ECC_ANOM_PRI_SUB(MAXNPOINTS_COM,NSUBS_MAX),
	6ECC_ANOM_SEC_SUB(MAXNPOINTS_COM,NSUBS_MAX),LTT_COR(MAXNPOINTS_COM)
	  !  PX and PY are the parallax factors.  VEL_AMP_XXX are returned 
	  !  values which are used in BINARY and NLDER3.
	COMMON /REAL8_SCRATCH_BLOCK/ PX,PY,FR_ANGLE,
	4EPOCH,VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB,VEL_AMP_SEC_SUB,
	5VEL_AMP_PRI_SUB_SEC,VEL_AMP_SEC_SUB_SEC,INCLINA_WORK,
	6INCLINA_WORK_PRI_SUB,INCLINA_WORK_SEC_SUB,
	7ECC_ANOM,ECC_ANOM_PRI_SUB,ECC_ANOM_SEC_SUB,LTT_COR
C
	INTEGER SEMIMAJOR_AXIS_ASSIGNMENT(2),DIMENSN_DATA,
	1DIMENSN_DATA_USED,INDX(MAXNUMPARAM)
	COMMON /BINARY_INTEGER4/ SEMIMAJOR_AXIS_ASSIGNMENT,
	1DIMENSN_DATA,DIMENSN_DATA_USED,INDX
C
	LOGICAL VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
	COMMON /BINARY_LOGICALS/ VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
C
	PI=3.14159265358979D0
	TIME_FACT=1.4959791D8/2.99792458D5/3600.0D0/24.0D0/365.2421988D0
			!  Units of years
C
	P=ABS(PARAMETRS(1))
	PARAMETRS(1)=P
	T=PARAMETRS(2)
	ECCENTR=ABS(PARAMETRS(3))
	IF (SPECTROSCOPIC.AND..NOT.VISUAL) THEN
C  If only spectroscopic data is available, we can't solve for the
C  node or inclination angle.  In this case, we use an offset.  Note that
C  a is in units of AU and is projected.
	  OMEGA=0.0
	  INCLINA=90.0
	ELSE IF (VISUAL) THEN !  Either SPECTROSCOPIC or not SPECTROSCOPIC
C  a is in units of arc seconds
	  OMEGA=MOD(PARAMETRS(4),360.0D0)
	  INCLINA=MOD(PARAMETRS(5),360.0D0)
	  PARAMETRS(5)=INCLINA
	END IF
	w_save=PARAMETRS(INDX(6)) !  For apsidal motion
	IF (VISUAL) THEN
C  The meaning of "a" comes from the type of file.  For a primary with an
C  absolute orbit, "a" is the primary semimajor axis.  For a secondary 
C  with a relative orbit, "a" is the relative semimajor axis of
C  that orbit.  These will differ by the factor, RATIO_a.  When both kinds
C  of file are opened, the "a" of the secondary is ignored.
	  a=PARAMETRS(INDX(7))
	ELSE 
C  For a spectroscopic binary, without visual information for the same
C  component, we must use projected "a" in units of AU.  This is introduced as
C  parameter 17.  The interpretation of this is the same as explained above.
	END IF
C  Because parameter number 8 is taken to be the ratio of semimajors axes
C  (total to primary) in the case of a pair of stars, and because this is
C  not used for a single star, and because parameters numbers greater than
C  8 may be used (to solve for proper motion and parallax), an offset is
C  used.
	IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(4:4).EQ.'B') THEN
		!  Visual binary pair or a spectroscopic binary pair
	  RATIO_a=PARAMETRS(INDX(8))
	ELSE !  Single member
C  Ignore ratio of semimajor axes
	END IF
C  Note that when parallax can't be determined by any means,
C  for the spectroscopic binary, a will have the units of AU.
	IF (PAR_PM) THEN
	  PARALLAX=PARAMETRS(INDX(9))
	ELSE 
	  IF (SOLVE_PAR) THEN
	    PARALLAX=PARAMETRS(INDX(9))
	  END IF
	END IF
	IF (SPECTROSCOPIC) THEN
	  IF (.NOT.PAR_PM.AND..NOT.SOLVE_PAR) aAU=PARAMETRS(INDX(17))
		!  aAU is a projected value and may apply to the primary
		!  or secondary, depending on whether data is available for
		!  both or the secondary components, respectively.
C  Handle the case of the primary having a subcomponent
	  IF (SPECTROSCOPIC_PRI_SUB) THEN
	    DECODE(1,1001,CHOICE(5:5)) IPRISUB
1001	    FORMAT(I1)
	    P_PRI_SUB(1)=PARAMETRS(INDX(18))
	    T_PRI_SUB(1)=PARAMETRS(INDX(19))
	    ECCENTR_PRI_SUB(1)=PARAMETRS(INDX(20))
	    w_PRI_SUB(1)=PARAMETRS(INDX(21))
	    w_save_PRI_SUB(1)=w_PRI_SUB(1)
	    aAU_PRI_SUB(1)=PARAMETRS(INDX(22))
	    P_PRI_SUB(2)=PARAMETRS(INDX(23))
	    T_PRI_SUB(2)=PARAMETRS(INDX(24))
	    ECCENTR_PRI_SUB(2)=PARAMETRS(INDX(25))
	    w_PRI_SUB(2)=PARAMETRS(INDX(26))
	    w_save_PRI_SUB(2)=w_PRI_SUB(2)
	    aAU_PRI_SUB(2)=PARAMETRS(INDX(27))
	    P_PRI_SUB(3)=PARAMETRS(INDX(28))
	    T_PRI_SUB(3)=PARAMETRS(INDX(29))
	    ECCENTR_PRI_SUB(3)=PARAMETRS(INDX(30))
	    w_PRI_SUB(3)=PARAMETRS(INDX(31))
	    w_save_PRI_SUB(3)=w_PRI_SUB(3)
	    aAU_PRI_SUB(3)=PARAMETRS(INDX(32))
	    P_PRI_SUB(4)=PARAMETRS(INDX(33))
	    T_PRI_SUB(4)=PARAMETRS(INDX(34))
	    ECCENTR_PRI_SUB(4)=PARAMETRS(INDX(35))
	    w_PRI_SUB(4)=PARAMETRS(INDX(36))
	    w_save_PRI_SUB(4)=w_PRI_SUB(4)
	    aAU_PRI_SUB(4)=PARAMETRS(INDX(37))
	    P_PRI_SUB(5)=PARAMETRS(INDX(38))
	    T_PRI_SUB(5)=PARAMETRS(INDX(39))
	    ECCENTR_PRI_SUB(5)=PARAMETRS(INDX(40))
	    w_PRI_SUB(5)=PARAMETRS(INDX(41))
	    w_save_PRI_SUB(5)=w_PRI_SUB(5)
	    aAU_PRI_SUB(5)=PARAMETRS(INDX(42))
	  END IF
C  Handle the case of the secondary having a subcomponent
	  IF (CHOICE(7:7).NE.' ') THEN
	    DECODE(1,1001,CHOICE(7:7)) ISECSUB
	    P_SEC_SUB(1)=PARAMETRS(INDX(43))
	    T_SEC_SUB(1)=PARAMETRS(INDX(44))
	    ECCENTR_SEC_SUB(1)=PARAMETRS(INDX(45))
	    w_SEC_SUB(1)=PARAMETRS(INDX(46))
	    w_save_SEC_SUB(1)=w_SEC_SUB(1)
	    aAU_SEC_SUB(1)=PARAMETRS(INDX(47))
	    P_SEC_SUB(2)=PARAMETRS(INDX(48))
	    T_SEC_SUB(2)=PARAMETRS(INDX(49))
	    ECCENTR_SEC_SUB(2)=PARAMETRS(INDX(50))
	    w_SEC_SUB(2)=PARAMETRS(INDX(51))
	    w_save_SEC_SUB(2)=w_SEC_SUB(2)
	    aAU_SEC_SUB(2)=PARAMETRS(INDX(52))
	    P_SEC_SUB(3)=PARAMETRS(INDX(53))
	    T_SEC_SUB(3)=PARAMETRS(INDX(54))
	    ECCENTR_SEC_SUB(3)=PARAMETRS(INDX(55))
	    w_SEC_SUB(3)=PARAMETRS(INDX(56))
	    w_save_SEC_SUB(3)=w_SEC_SUB(3)
	    aAU_SEC_SUB(3)=PARAMETRS(INDX(57))
	    P_SEC_SUB(4)=PARAMETRS(INDX(58))
	    T_SEC_SUB(4)=PARAMETRS(INDX(59))
	    ECCENTR_SEC_SUB(4)=PARAMETRS(INDX(60))
	    w_SEC_SUB(4)=PARAMETRS(INDX(61))
	    w_save_SEC_SUB(4)=w_SEC_SUB(4)
	    aAU_SEC_SUB(4)=PARAMETRS(INDX(62))
	    P_SEC_SUB(5)=PARAMETRS(INDX(63))
	    T_SEC_SUB(5)=PARAMETRS(INDX(64))
	    ECCENTR_SEC_SUB(5)=PARAMETRS(INDX(65))
	    w_SEC_SUB(5)=PARAMETRS(INDX(66))
	    w_save_SEC_SUB(5)=w_SEC_SUB(5)
	    aAU_SEC_SUB(5)=PARAMETRS(INDX(67))
	  END IF
	  DO I=1,5
	    IF (CHOICE(8+I:8+I).EQ.'T'.OR.CHOICE(8+I:8+I).EQ.'o') THEN
	      RATIO_a_PRI_SUB_SEC(I)=PARAMETRS(INDX(67+I))
	    END IF
	  END DO
	  DO I=1,5
	    IF (CHOICE(13+I:13+I).EQ.'T'.OR.CHOICE(13+I:13+I).EQ.'o') THEN
	      RATIO_a_SEC_SUB_SEC(I)=PARAMETRS(INDX(72+I))
	    END IF
	  END DO
	END IF
C
C  Get the time correction amplitudes.  INCLINA_WORK was prompted for in
C  BINARY.FOR
C
	IF (SPECTROSCOPIC) THEN
	  IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B') THEN
	    IF (VISUAL) THEN
	      IF (CHOICE(1:1).EQ.'S'.AND.CHOICE(4:4).EQ.'P') THEN
C  2 cases;  a(AU) is of primary
	        LTT_AMP_PRI=TIME_FACT*aAU*(1.0D0-ECCENTR**2)*
	1       DSIND(INCLINA_WORK)
	      ELSE  ! (CHOICE(1:1).EQ.'S'.AND.CHOICE(4:4).EQ.'B').OR.
		    ! (CHOICE(1:1).EQ.'P'.AND.CHOICE(4:4).EQ.'P').OR.
		    ! (CHOICE(1:1).EQ.'P'.AND.CHOICE(4:4).EQ.'B').OR.
		    ! (CHOICE(1:1).EQ.'B'.AND.CHOICE(4:4).EQ.'P').OR.
		    ! (CHOICE(1:1).EQ.'B'.AND.CHOICE(4:4).EQ.'B')
C  8 cases;  a of primary, is in arc seconds.  Spectroscopic data 
C  is used to solve for parallax, but not semimajor axis.
	        LTT_AMP_PRI=TIME_FACT*a/PARALLAX*(1.0D0-ECCENTR**2)*
	1       DSIND(INCLINA)
	      END IF
	    ELSE
C  Case is of spectroscopic binary with no visual binary information.
C  Projected semimajor axis, a, of primary is in astronomical units, therefore 
C  PARALLAX is neglected.  INCLINA can be neglected also.  2 cases.
	      LTT_AMP_PRI=TIME_FACT*aAU*(1.0D0-ECCENTR**2)*
	1     DSIND(INCLINA_WORK)
	    END IF
	  END IF
C  Compute the LTT of the secondary if there is spectroscopic
C  binary data
	  IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') THEN
	    IF (VISUAL) THEN
	      IF ((CHOICE(1:1).EQ.'P'.AND.CHOICE(4:4).EQ.'S').OR.
	1     CHOICE(1:1).EQ.'S'.AND.CHOICE(3:3).EQ.'R'.AND.
	2     CHOICE(4:4).EQ.'S') THEN
C  2 cases;  projected a(AU) is of secondary
	        LTT_AMP_SEC=TIME_FACT*aAU*(1.0D0-ECCENTR**2)*
	1       DSIND(INCLINA_WORK)
	      ELSE IF ((CHOICE(1:1).EQ.'S'.AND.CHOICE(4:4).EQ.'S').AND.
	1     (CHOICE(3:3).EQ.'A'.OR.CHOICE(3:3).EQ.'I')) THEN
C  1 case;  a is of secondary
	        LTT_AMP_SEC=TIME_FACT*a/PARALLAX*(1.0D0-ECCENTR**2)*
	1       DSIND(INCLINA)
	      ELSE IF (CHOICE(1:1).EQ.'B'.AND.CHOICE(4:4).EQ.'S') THEN
C  2 cases;  a is of primary
	        LTT_AMP_SEC=(RATIO_a-1.0D0)*TIME_FACT*a/PARALLAX*
	1       (1.0D0-ECCENTR**2)*DSIND(INCLINA)
	      ELSE !  CHOICE(4:4).EQ.'B'
C  5 cases;  a is of primary
	        LTT_AMP_SEC=(RATIO_a-1.0D0)*TIME_FACT*a/PARALLAX*
	1       (1.0D0-ECCENTR**2)*DSIND(INCLINA)
	      END IF
	    ELSE !  Not visual
	      IF (CHOICE(4:4).EQ.'B') THEN
C  1 case;  projected a(AU) is of primary.
	        LTT_AMP_SEC=(RATIO_a-1.0D0)*TIME_FACT*aAU*(1.0D0-ECCENTR**2)*
	1       DSIND(INCLINA_WORK)
	      ELSE IF (CHOICE(4:4).EQ.'S') THEN
C  1 case;  projected a(AU) is of secondary
	        LTT_AMP_SEC=TIME_FACT*aAU*(1.0D0-ECCENTR**2)*
	1       DSIND(INCLINA_WORK)
	      END IF
	    END IF
	  END IF
	END IF
C  Obtain the light travel time corrections.  The calculation is done in the
C  order the points were read in by the main program.
C  Ignore the correction for the visual binary points since such periods
C  are usually long
	NLO=NPOINTS_VIS+1
	NHI=NLO+NPOINTS_DATA_VEL_P-1
	DO N=NLO,NHI !  Spectroscopic primary
C  The eccentric anomaly was previously calculated in NLFUN3
	  E=ECC_ANOM(N)
	  FRAC=SQRT(ABS((1.0D0+ECCENTR)/(1.0D0-ECCENTR)))
	  v=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	  IF (CHOICE(19:19).EQ.'A') THEN !  Apsidal motion
	    w=w_save+PARAMETRS(INDX(78))*(TIME(N,1)-T)
C  Note that this time is not corrected for light travel time since
C  apsidal motion is usually slow.
	  ELSE
	    w=w_save
	  END IF
	  LTT_COR(N)=LTT_AMP_PRI*DSIND(v+w)/(1.0D0+ECCENTR*DCOSD(v))
C  Add on any subcomponent displacements
	  DO I=1,IPRISUB
	    E=ECC_ANOM_PRI_SUB(N,I)
	    FRAC=SQRT(ABS((1.0D0+ECCENTR_PRI_SUB(I))/
	1   (1.0D0-ECCENTR_PRI_SUB(I))))
	    v=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	    IF (CHOICE(19+I:19+I).EQ.'A') THEN !  Apsidal motion
	      w_PRI_SUB(I)=w_save_PRI_SUB(I)+PARAMETRS(INDX(78+I))*
	1     (TIME(N,1)-T)
	    ELSE
	      w_PRI_SUB(I)=w_save_PRI_SUB(I)
	    END IF
	    LTT_COR(N)=LTT_COR(N)+TIME_FACT*aAU_PRI_SUB(I)*
	1   (1.0D0-ECCENTR_PRI_SUB(I)**2)*DSIND(INCLINA_WORK_PRI_SUB(I))*
	2   DSIND(v+w_PRI_SUB(I))/(1.0D0+ECCENTR_PRI_SUB(I)*DCOSD(v))
	  END DO
	END DO
C
	NLO=NPOINTS_VIS+NPOINTS_DATA_VEL_P+1
	NHI=NLO+NPOINTS_DATA_VEL_S-1
	DO N=NLO,NHI !  Spectroscopic secondary
	  E=ECC_ANOM(N)
	  FRAC=SQRT(ABS((1.0D0+ECCENTR)/(1.0D0-ECCENTR)))
	  v=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	  IF (CHOICE(19:19).EQ.'A') THEN !  Apsidal motion
	    w=w_save+PARAMETRS(INDX(78))*(TIME(N,1)-T)
C  Note that this time is not corrected for light travel time since
C  apsidal motion is usually slow.
	  ELSE
	    w=w_save
	  END IF
	  LTT_COR(N)=-LTT_AMP_SEC*DSIND(v+w)/(1.0D0+ECCENTR*DCOSD(v))
C  Add on any subcomponent displacements
	  DO I=1,ISECSUB
	    E=ECC_ANOM_SEC_SUB(N,I)
	    FRAC=SQRT(ABS((1.0D0+ECCENTR_SEC_SUB(I))/
	1   (1.0D0-ECCENTR_SEC_SUB(I))))
	    v=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	    IF (CHOICE(24+I:24+I).EQ.'A') THEN !  Apsidal motion
	      w_SEC_SUB(I)=w_save_SEC_SUB(I)+PARAMETRS(INDX(83+I))*
	1     (TIME(N,1)-T)
	    ELSE
	      w_SEC_SUB(I)=w_save_SEC_SUB(I)
	    END IF
	    LTT_COR(N)=LTT_COR(N)+TIME_FACT*aAU_SEC_SUB(I)*
	1   (1.0D0-ECCENTR_SEC_SUB(I)**2)*DSIND(INCLINA_WORK_SEC_SUB(I))*
	2   DSIND(v+w_SEC_SUB(I))/(1.0D0+ECCENTR_SEC_SUB(I)*DCOSD(v))
	  END DO
	END DO
C  If any of the primary spectroscopic subcomponents has complementary 
C  spectral data from the associated secondary, evaluate the corrections.
	DO I=1,NSUBS_MAX
	  IF (NPOINTS_DATA_VEL_P_SUB_S(I).NE.0) THEN
	    NLO=NHI+1
	    NHI=NLO+NPOINTS_DATA_VEL_P_SUB_S(I)-1
	    DO N=NLO,NHI
C  First get the displacement of the primary assuming no subcomponents
	      E=ECC_ANOM(N)
	      FRAC=SQRT(ABS((1.0D0+ECCENTR)/(1.0D0-ECCENTR)))
	      v=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	      IF (CHOICE(19:19).EQ.'A') THEN !  Apsidal motion
	        w=w_save+PARAMETRS(INDX(78))*(TIME(N,1)-T)
C  Note that this time is not corrected for light travel time since
C  apsidal motion is usually slow.
	      ELSE
	        w=w_save
	      END IF
	      LTT_COR(N)=LTT_AMP_PRI*DSIND(v+w)/(1.0D0+ECCENTR*DCOSD(v))
C  Now add on the displacement of the one secondary (of the primary)
C  under consideration
	      E=ECC_ANOM_PRI_SUB(N,I)
	      FRAC=SQRT(ABS((1.0D0+ECCENTR_PRI_SUB(I))/
	1     (1.0D0-ECCENTR_PRI_SUB(I))))
	      v=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	      IF (CHOICE(19+I:19+I).EQ.'A') THEN !  Apsidal motion
	        w_PRI_SUB(I)=w_save_PRI_SUB(I)+PARAMETRS(INDX(78+I))*
	1       (TIME(N,1)-T)
	      ELSE
	        w_PRI_SUB(I)=w_save_PRI_SUB(I)
	      END IF
	      LTT_COR(N)=LTT_COR(N)-(RATIO_a_PRI_SUB_SEC(I)-1.0D0)*
	1     TIME_FACT*aAU_PRI_SUB(I)*(1.0D0-ECCENTR_PRI_SUB(I)**2)*
	2     DSIND(INCLINA_WORK_PRI_SUB(I))*
	3     DSIND(v+w_PRI_SUB(I))/(1.0D0+ECCENTR_PRI_SUB(I)*DCOSD(v))
	    END DO
	  END IF
	END DO
C  If any of the secondary spectroscopic subcomponents has complementary 
C  spectral data from the associated secondary, evaluate the corrections.
	DO I=1,NSUBS_MAX
	  IF (NPOINTS_DATA_VEL_S_SUB_S(I).NE.0) THEN
	    NLO=NHI+1
	    NHI=NLO+NPOINTS_DATA_VEL_S_SUB_S(I)-1
	    DO N=NLO,NHI
C  First get the displacement of the secondary assuming no subcomponents
	      E=ECC_ANOM(N)
	      FRAC=SQRT(ABS((1.0D0+ECCENTR)/(1.0D0-ECCENTR)))
	      v=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	      IF (CHOICE(19:19).EQ.'A') THEN !  Apsidal motion
	        w=w_save+PARAMETRS(INDX(78))*(TIME(N,1)-T)
C  Note that this time is not corrected for light travel time since
C  apsidal motion is usually slow.
	      ELSE
	        w=w_save
	      END IF
	      LTT_COR(N)=-LTT_AMP_SEC*DSIND(v+w)/(1.0D0+ECCENTR*DCOSD(v))
C  Now add on the displacement of the one secondary (of the secondary)
C  under consideration
	      E=ECC_ANOM_SEC_SUB(N,I)
	      FRAC=SQRT(ABS((1.0D0+ECCENTR_SEC_SUB(I))/
	1     (1.0D0-ECCENTR_SEC_SUB(I))))
	      v=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	      IF (CHOICE(24+I:24+I).EQ.'A') THEN !  Apsidal motion
	        w_SEC_SUB(I)=w_save_SEC_SUB(I)+PARAMETRS(INDX(83+I))*
	1       (TIME(N,1)-T)
	      ELSE
	        w_SEC_SUB(I)=w_save_SEC_SUB(I)
	      END IF
	      LTT_COR(N)=LTT_COR(N)-(RATIO_a_SEC_SUB_SEC(I)-1.0D0)*
	1     TIME_FACT*aAU_SEC_SUB(I)*(1.0D0-ECCENTR_SEC_SUB(I)**2)*
	2     DSIND(INCLINA_WORK_SEC_SUB(I))*
	3     DSIND(v+w_SEC_SUB(I))/(1.0D0+ECCENTR_SEC_SUB(I)*DCOSD(v))
	    END DO
	  END IF
	END DO
	RETURN
	END
