	SUBROUTINE DATE_JD(DA,MO,YR,JD,DOW)
C
C       This subroutine computes the Julian Day number and day of week.
C  BC years should be input as negative numbers.
C				Donald H. Gudehus, 1983
C				Version 2, D. Gudehus, January 21, 1990
C				Fixed bug with day of week.
C				Version 3, D. Gudehus, March 7, 2005
C				Changed argument list to all integer and
C				made constants double precision.
C
C	DA	(C)	The day with time assumed to be noon.
C	MO	(C)	The month expressed as a number
C	YR	(C)	The year in the Gregorian calendar.  Note that
C			in the Gregorian proleptic calendar (dates
C			before its inception on October 15, 1582) the
C			year 0 is regarded as a leap year.
C	JD	(R)	The Julian Day number.  The Julian Day number
C			gives the number of days elapsed since noon
C			on January 1, 4713 BC;  note however that this
C			date is based on the Julian calendar (leap year
C			every 4 years and the civil system (no year 0).
C	DOW	(R)	The day of the week given by numbers 0 through
C			6, where 0 stands for Sunday.
C
	INTEGER DA,MO,YR,JD,DOW
C
	INTEGER MOP,KP,K,YRP,YRPP
	REAL*8 XNUM
C
	IF (MO.LE.2) THEN
	  MOP=MO+13
	  YRP=YR-1
	ELSE
	  MOP=MO+1
	  YRP=YR
	END IF
	IF (YRP.GE.0) THEN
	  YRPP=YRP
	  KP=0
	ELSE
	  YRPP=YRP+1
	  KP=-366
	END IF
	K=-JIDINT(YRPP/100.0D0)+JIDINT(YRPP/400.0D0)
	JD=JIDINT(365.25D0*YRPP)+JIDINT(30.6001D0*MOP)+DA+1720997+K+KP
C  Now get the day of the week
	XNUM=(DBLE(JD)-1720997+1.0D9)/7.0D0
	DOW=NINT(7.0D0*(XNUM-JIDINT(XNUM)))
	RETURN
	END
