	SUBROUTINE EARTH(ET,DA,MO,YR,A,ECCENTR,ARGPERI,MEANANOM,
	1OBLIQ,ECCENTRANOM,TRUEANOM,RADIUS)
C
C       This subroutine computes certain parameters of the earth's orbit
C  about the sun and the obliquity of the earth's axis.  BC years should 
C  be input as negative numbers.
C				Donald H. Gudehus, April 2, 1990
C			Version 2, D. Gudehus, March 7, 2005
C			Made DA, MO, and YR INTEGER and made constants
C			doubel precision
C
C	ET		(C)	Ephemeris time
C	DA		(C)	The day
C	MO		(C)	The month expressed as a number
C	YR		(C)	The year in the Gregorian calendar.  Note that
C				in the Gregorian proleptic calendar (dates
C				before its inception on October 15, 1582) the
C				year 0 is regarded as a leap year.
C	A		(R)	The semimajor axis in AU
C	ECCENTR		(R)	The eccentricity 
C	ARGPERI		(R)	The argument of perihelion, omega in degrees
C	MEANANOM	(R)	The mean anomaly in degrees
C	OBLIQ		(R)	The obliquity in degrees
C	ECCENTRANOM	(R)	The eccentric anomaly in degrees
C	TRUEANOM	(R)	The true anomaly in degrees
C	RADIUS		(R)	The radius in AU
C
	INTEGER  DA,MO,YR
	REAL*8 ET,T,A,ECCENTR,ARGPERI,MEANANOM,OBLIQ,ECCENTRANOM,
	1TRUEANOM,RADIUS,D,D1,D2,D3,T2,T3,TEMP,BY
C
	CALL ETDATE_JC_BY(ET,DA,MO,YR,T,BY)
C  Most of the following was taken from The Explanatory Supplement,
C  in particular, page 98.
	D=36525.0D0*T
	D1=3.6525D0*T
	D2=D1**2
	D3=D1**3
	T2=T**2
	T3=T**3
	A=1.00000023D0
	ECCENTR=0.01675104D0-0.00004180D0*T-0.000000126D0*T**2
	ARGPERI=101.220833D0+0.0000470684D0*D+0.0000339D0*D2+0.00000007D0*D3
	TEMP=99.0D0*T !  The argument of perhelion of the sun was reduced
		      !  by 180 degrees in order to get that of the earth.
	MEANANOM=358.475845D0+(TEMP-INT(TEMP))*360.0D0+
	1(1292579.1D0*T-0.54D0*T2-0.012D0*T3)/3600.0D0
	DO WHILE (MEANANOM.GT.360.0D0)
	  MEANANOM=MEANANOM-360.0D0
	END DO
	OBLIQ=23.452294D0-0.0130125D0*T-0.00000164D0*T2+0.000000503D0*T3
	CALL KEPLER(MEANANOM,ECCENTR,ECCENTRANOM)
	TRUEANOM=2.0D0*DATAND(DSQRT((1.0D0+ECCENTR)/(1.0D0-ECCENTR))*DTAND(
	1ECCENTRANOM/2.0D0))
	RADIUS=A*(1.0D0-ECCENTR*DCOSD(ECCENTRANOM))
	RETURN
	END
