	PROGRAM EDITBINARY
C
C	Note:  This is the UNIX version.
C
C	This program allows one to edit the standard binary star data 
C  files.  One can perform arithmetic operations on various columns,
C  precess coordinates from one equinox to another, bin the data,
C  enter data to a new or existing file, and create artificial
C  data.
C  For new files the following are noted:
C    1. The user is prompted for whether the equator should be
C       equal to the epoch only when the star is a secondary of
C       non-spectroscopic type.  Otherwise, the equator is constant
C       and equal to that listed in the header.
C    2. For non-spectroscopic systems, it is not permitted to have
C       a relative orbit with proper motion and parallax included.
C    3. Although it is possible to create a non-spectroscopic type
C       file where the primary has Type=Relative orbit, the actual
C       data is the same as for Type=Absolute.
C
C				Donald H. Gudehus, February, 1994
C			Version 2, Donald H. Gudehus, July, 1995
C			Added command to convert weights to sigmas
C			Version 2U, Donald H. Gudehus, June, 1997
C			Modified program to work under UNIX.
C			Version 3, Donald H. Gudehus, November 25, 1999
C			Increased number of points to 5000
C			Version 4, Donald H. Gudehus, February 26, 2000
C			Added apsidal motion for main components
C			Version 5, Donald H. Gudehus, September 25, 2001
C			Added apsidal motion for subcomponents.  Added
C			INDX to COMMON block to be consistent with new
C			version of BINARY_INDEXES.
C			Version 6, Donald H. Gudehus, October 8, 2001
C			Fixed bug where the ratio of photocentric to
C			true semimajor axis wasn't getting passed to
C			nlfun3, and where the parallax factors were
C			not being calculated.
C			Version 7, D. Gudehus, January 3, 2003
C			Added code for TYPE='r', constant equator.
C			Version 7U, D. Gudehus, January 7, 2003
C			Replaced call to NEL with INEL which hadn't
C			yet been done.
C			Version 8, D. Gudehus, January 14, 2003
C			Added command CI, for 1-D interferometric
C			data
C			Version 9, D. Gudehus, February 5, 2004
C			Increased the significance of C_ELEMENTS and
C			decoded more characters for the elements.
C			Version 10, D. Gudehus, February 26, 2005
C			Added command EI, for 1-D interferometric
C			data
C
	INCLUDE 'binary_parameters.for'
	PARAMETER (NCHOICESMAX=4,NCHOICESMAX_VEL=2,NCHOICESMAX_INTER=3,
	1NGROUP_MAX=200)
	CHARACTER*1 TYPE,PARALLAX_PROPER_MOTION,PRI_SEC,C_NSUBS
	CHARACTER*2 COMMAND
	CHARACTER*32 NLFUN_CHOICE
	CHARACTER*18 C_ELEMENTS(MAXNUMPARAM),C_ELEMENTS_S(MAXNUMPARAM)
	CHARACTER*11 FILETYPE
	CHARACTER*30 CHOICE
	CHARACTER*79 STRING
	CHARACTER*80 NAME
	INTEGER FUNCAT,ICHOICE(NCHOICESMAX),ITIME(2),
	1INDEX_SUB_SEC(NSUBS_MAX)
	LOGICAL GO,PPM,ADD_SUBCOMPONENT,OPERATE,ADDNOISE,EQUATOR_IS_EPOCH,
	1WRITE_SUB_SEC
	REAL RAPARAM(3),DECPARAM(3),ALTIPARAM(3),AZIPARAM(3),LONGIPARAM(3),
	1LATIPARAM(3),NUMBER(NCHOICESMAX)
	REAL*8 ARRAY(NCHOICESMAX),SCALE,POS_SEP(2),TEMP,TEMP2,TEMP3,T_LOW,
	1TJD_LOW,TEMPARRAY(2),ELEMENTS(MAXNUMPARAM),
	1ELEMENTS_S(MAXNUMPARAM),RIGHTASC,DECLINA,ALT,AZIMUTH,HA,PA,
	1ALT_INTER,AZIMUTH_INTER,LONGITUDE,LATITUDE,RIGHTASC_SUN,
	1DECLINA_SUN,ALT_SUN,AZIMUTH_SUN,HA_SUN,EQUATOR,EPOCH,
	2EQUINOX1,EQUINOX2,RIGHTASC_PRIMARY1,DECLINA_PRIMARY1,
	3RIGHTASC_PRIMARY2,DECLINA_PRIMARY2,RIGHTASC_SECONDARY1,
	4DECLINA_SECONDARY1,RIGHTASC_SECONDARY2,DECLINA_SECONDARY2,
	5DELTA_NORTH,DELTA_EAST,POSANG,SEP,B_DATA(5),VAR1(NGROUP_MAX),
	6VAR2(NGROUP_MAX),VAR3(NGROUP_MAX),VAR4(NGROUP_MAX),
	7VAR5(NGROUP_MAX),WEIGHT(NGROUP_MAX),AVG,SIGMA,
	8TIME(MAXNPOINTS,DIMENSN_IND_MAX),TIME_JD(MAXNPOINTS),
	9DATA(MAXNPOINTS,DIMENSN_DATA_MAX),PARAMETRS(MAXNUMPARAM),
	1NOISE_AS,NOISE_DEG,NOISE_KMS,SUM,SIGMAS(2),SIGMA_VEL,SIGMA_INTER
C
	REAL*8 PX(MAXNPOINTS_COM),PY(MAXNPOINTS_COM),
	1FR_ANGLE(MAXNPOINTS_COM),EPOCH_WORK,
	1VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB(NSUBS_MAX),
	2VEL_AMP_SEC_SUB(NSUBS_MAX),VEL_AMP_PRI_SUB_SEC(NSUBS_MAX),
	3VEL_AMP_SEC_SUB_SEC(NSUBS_MAX),INCLINA_WORK,
	4INCLINA_WORK_PRI_SUB(NSUBS_MAX),
	5INCLINA_WORK_SEC_SUB(NSUBS_MAX),ECC_ANOM(MAXNPOINTS_COM),
	6ECC_ANOM_PRI_SUB(MAXNPOINTS_COM,NSUBS_MAX),
	7ECC_ANOM_SEC_SUB(MAXNPOINTS_COM,NSUBS_MAX),LTT_COR(MAXNPOINTS_COM),
	8PHOTOCENTRIC_TRUE_SEMIMAJAX,FUNCTN_TEMP(MAXNPOINTS_COM,4)
	!  PX and PY are the parallax factors.  EPOCH_WORK is used for
	!  the proper motion calculation.  VEL_AMP_XXX are returned
	!  values which are used in BINARY and NLDER3.
	COMMON /REAL8_SCRATCH_BLOCK/ PX,PY,FR_ANGLE,EPOCH_WORK,
	1VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB,VEL_AMP_SEC_SUB,
	2VEL_AMP_PRI_SUB_SEC,VEL_AMP_SEC_SUB_SEC,INCLINA_WORK,
	3INCLINA_WORK_PRI_SUB,INCLINA_WORK_SEC_SUB,
	4ECC_ANOM,ECC_ANOM_PRI_SUB,ECC_ANOM_SEC_SUB,LTT_COR,
	5PHOTOCENTRIC_TRUE_SEMIMAJAX,FUNCTN_TEMP
C
	INTEGER SEMIMAJOR_AXIS_ASSIGNMENT_BIN(2),DIMENSN_DATA_BIN,
	1DIMENSN_DATA_USED_BIN,INDX(MAXNUMPARAM)
	COMMON /BINARY_INTEGER4/ SEMIMAJOR_AXIS_ASSIGNMENT_BIN,
	1DIMENSN_DATA_BIN,DIMENSN_DATA_USED_BIN,INDX
C
	LOGICAL OFIL
	COMMON /OLDFILE/ OFIL
C
C  Make carriage control as in OpenVMS
	OPEN(UNIT=6,STATUS='NEW',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=7,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=49,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
C
   1	COMMAND=' '
	DO WHILE (COMMAND.NE.'A'.AND.COMMAND.NE.'CI'.AND.COMMAND.NE.
	1'CS'.AND.COMMAND.NE.'CV'.AND.COMMAND.NE.'EI'.AND.COMMAND.NE.
	2'ES'.AND.COMMAND.NE.'EV'.AND.COMMAND.NE.'M'.AND.COMMAND.NE.
	2'P'.AND.COMMAND.NE.'PM'.AND.COMMAND.NE.'Q'.AND.COMMAND.NE.
	3'S'.AND.COMMAND.NE.'WS')
	  COMMAND='A'
	  CALL MESSAGE('Enter a command:')
	  CALL MESSAGE('     A  = Adds one or more numbers to the data'//
	1 ' (default)')
	  CALL MESSAGE('     CI = Creates a file of one-dimensional'//
	1 ' interferometric data from user')
	  CALL MESSAGE('          supplied elements')
	  CALL MESSAGE('     CS = Creates a file of artificial'//
	1 ' spectroscopic binary data from user')
	  CALL MESSAGE('          supplied elements')
	  CALL MESSAGE('     CV = Creates a file of artificial visual'//
	1 ' binary data from user')
	  CALL MESSAGE('          supplied elements')
	  CALL MESSAGE('     EI = Allows manual entry of one-dimensional'//
	1 ' interferometric data to')
	  CALL MESSAGE('          a new or existing file')
	  CALL MESSAGE('     ES = Allows manual entry of spectroscopic'//
	1 ' binary data to a new or')
	  CALL MESSAGE('          existing file')
	  CALL MESSAGE('     EV = Allows manual entry of visual binary'//
	1 ' data to a new or existing file')
	  CALL MESSAGE('     M  = Multiplies the data by one or more'//
	1 ' numbers')
	  CALL MESSAGE('     P  = Precesses the coordinates of the data.')
	  CALL MESSAGE('          Should only be used where all data points'//
	1 ' have the same equator.')
	  CALL MESSAGE('     PM = Precesses the position angle and'//
	1 ' separation of manually input data')
	  CALL MESSAGE('     Q  = Quits')
	  CALL MESSAGE('     S  = Smooths the data by averaging (blank line'//
	1 ' delimiters')
	  CALL MESSAGE('          must be in file)')
	  CALL MESSAGE('     WS = Converts weights to sigmas')
	  CALL USRINP(%REF(COMMAND),1,'A',2,1,'COMMAND=',' ')
	  CALL ALPHA(%REF(COMMAND),1,2,NC,NF,NR,2)
	END DO
C  Initialize ELEMENTS for proper functioning of WRITE_BINARY_HEADER
	DO I=1,MAXNUMPARAM
	  ELEMENTS(I)=-1.0D37
	END DO
C  Take care of initial opening of files	
	IF (COMMAND.EQ.'A'.OR.COMMAND.EQ.'M'.OR.COMMAND.EQ.'P'
	1.OR.COMMAND.EQ.'S'.OR.COMMAND.EQ.'WS') THEN
C  Open the input file
	  CALL MESSAGE('Open the data file to be edited')
	  IF (COMMAND.EQ.'P') THEN
	    CALL MESSAGE('Be sure the file does not contain, e.g., relative'//
	1  ' coordinates where the')
	    CALL MESSAGE('equator is equal to the epoch')
	  END IF
	  CALL OPENCF(21,1,'INPUT_FILE=',0)
	  REWIND (21)
	  CALL GET_BINARY_HEADER(21,NAME,RIGHTASC,DECLINA,
	1 EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	2 C_ELEMENTS,MAXNUMPARAM)
	  CALL PRINT_BINARY_HEADER(NAME,RIGHTASC,DECLINA,
	1 EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	2 C_ELEMENTS,MAXNUMPARAM)
C  Open the output file
	  CALL MESSAGE('Open the output file')
	  CALL OPENCF(22,2,'OUTPUT_FILE=',0)
	  IF (COMMAND.NE.'P') !  Information not yet available for 'P'
	1 CALL WRITE_BINARY_HEADER(22,NAME,RIGHTASC,DECLINA,
	2 EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	3 C_ELEMENTS,MAXNUMPARAM)
	ELSE IF (COMMAND.EQ.'CI'.OR.COMMAND.EQ.'CS'.OR.COMMAND.EQ.
	1'CV'.OR.COMMAND.EQ.'EI'.OR.COMMAND.EQ.'ES'.OR.COMMAND.EQ.'EV') THEN
C  Open the input/output file
	  IF (COMMAND.EQ.'CI'.OR.COMMAND.EQ.'CS'.OR.COMMAND.EQ.'CV') THEN
	    CALL MESSAGE('Open the new data file to be created')
	    CALL OPENCF(22,2,'EDIT_FILE=',0) !  Only new file
	  ELSE IF (COMMAND.EQ.'EI'.OR.COMMAND.EQ.'ES'.OR.COMMAND.EQ.'EV') THEN
	    CALL MESSAGE('Open the data file to be edited')
	    CALL OPENCF(22,0,'EDIT_FILE=',0) !  Old or new file
	  END IF
	  REWIND (22)
	END IF
C  Get auxilliary information
	IF (COMMAND.EQ.'A'.OR.COMMAND.EQ.'M') THEN  ! Adds or multiplies
	  CHOICE=' '
	  CALL MESSAGE('Enter one or more choices:')
	  IF (FILETYPE.EQ.'X_Y') THEN
	    CALL MESSAGE('     X  = Edits the X position')
	    CALL MESSAGE('     XE = Edits the X mean error')
	    CALL MESSAGE('     Y  = Edits the Y position')
	    CALL MESSAGE('     YE = Edits the Y mean error')
	  ELSE IF (FILETYPE.EQ.'THETA_RHO') THEN
	    CALL MESSAGE('     P  = Edits the position angle')
	    CALL MESSAGE('     PE = Edits the position angle error')
	    CALL MESSAGE('     S  = Edits the separation')
	    CALL MESSAGE('     SE = Edits the separation error')
	  ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	    CALL MESSAGE('     V  = Edits the radial velocity')
	    CALL MESSAGE('     VE = Edits the radial velocity error')
	  ELSE IF (FILETYPE(4:6).EQ.'RHO') THEN
	    CALL MESSAGE('     F  = Edits the fringe angle')
	    CALL MESSAGE('     R  = Edits the projected separation')
	    CALL MESSAGE('     RE = Edits the projected separation error')
	  END IF
	  CALL USRINP(%REF(CHOICE),1,'A',30,0,'CHOICES=',' ')
	  CALL ALPHA(%REF(CHOICE),NCHOICESMAX,5,NC,NF,NR,4)
	  DO I=1,NCHOICESMAX
	    J=ISETEND(ICHOICE(I))
	  END DO
	  II=1
	  DO I=1,16,5
	    IF (CHOICE(I:I+1).EQ.'X'.OR.CHOICE(I:I+1).EQ.'P'
	1   .OR.CHOICE(I:I+1).EQ.'V'.OR.CHOICE(I:I+1).EQ.'F') THEN
	      ICHOICE(II)=1
	      II=II+1
	    ELSE IF (CHOICE(I:I+1).EQ.'XE'.OR.CHOICE(I:I+1).EQ.'PE'
	1   .OR.CHOICE(I:I+1).EQ.'VE'.OR.CHOICE(I:I+1).EQ.'R') THEN
	      ICHOICE(II)=2
	      II=II+1
	    ELSE IF (CHOICE(I:I+1).EQ.'Y'.OR.CHOICE(I:I+1).EQ.'S'
	1   .OR.CHOICE(I:I+1).EQ.'RE') THEN
	      ICHOICE(II)=3
	      II=II+1
	    ELSE IF (CHOICE(I:I+1).EQ.'YE'.OR.CHOICE(I:I+1).EQ.'SE') THEN
	      ICHOICE(II)=NCHOICESMAX
	      II=II+1
	    END IF
  	  END DO
	  NUMCHOICES=INEL(ICHOICE,NCHOICESMAX)
	END IF
C
C  Carry out the commands
C
	IF (COMMAND.EQ.'A') THEN  ! Adds
	  DO I=1,NUMCHOICES
	    NUMBER(I)=0.0
	  END DO
	  CALL USRINP(NUMBER,NUMCHOICES,'F',4,1,'NUMBERS=',
	1 'Enter the numbers to be added:  (defaults = 0.0)')
	  DO WHILE (.TRUE.)
	    IF (FILETYPE.EQ.'X_Y'.OR.FILETYPE.EQ.'THETA_RHO') THEN
	      READ(21,3000,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX)
3000	      FORMAT(' ',F12.6,4X,F14.5,2X,F14.5,2X,F14.5,2X,F14.5)
	    ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	      READ(21,3100,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_VEL)
3100	      FORMAT(' ',F15.6,4X,F14.5,2X,F14.5)
	    ELSE IF (FILETYPE(4:6).EQ.'RHO') THEN
	      READ(21,3300,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_INTER)
3300	      FORMAT(' ',F15.6,4X,F14.5,2X,F14.6,2X,F14.6)
	    END IF
	    DO II=1,NUMCHOICES
	      ARRAY(ICHOICE(II))=ARRAY(ICHOICE(II))+NUMBER(II)
	    END DO
	    IF (FILETYPE.EQ.'X_Y'.OR.FILETYPE.EQ.'THETA_RHO') THEN
	      WRITE(22,3000) EPOCH,(ARRAY(I),I=1,NCHOICESMAX)
	    ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	      WRITE(22,3100) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_VEL)
	    ELSE IF (FILETYPE(4:6).EQ.'RHO') THEN
	      WRITE(22,3300) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_INTER)
	    END IF
	  END DO
C
	ELSE IF (COMMAND.EQ.'CV'.OR. !  Artificial data for visual
				     !  binary
	1COMMAND.EQ.'CS'.OR. !  Artificial data for spectroscopic binary
	2COMMAND.EQ.'CI'.OR. !  Artificial one-dimenional 
			     !  interferometric data
	3COMMAND.EQ.'ES'.OR. !  Manual entry of spectroscopic binary data
			     !  to a new or existing file
	4COMMAND.EQ.'EV'.OR. !  Manual entry of visual binary
			     !  data to a new or existing file
	5COMMAND.EQ.'EI') THEN !  Manual entry of one-dimensional
			       !  interferometric data
	  IF (.NOT.OFIL) THEN
	    CALL USRINP(%REF(NAME),1,'A',79,0,'NAME=',
	1  'Enter the name of the star:')
	    CALL ALPHA(%REF(NAME),1,79,NC,NF,NR,2)
	    CALL MESSAGE('Enter the right ascension of the primary'//
	1   ' in hours, minutes,')
	    CALL USRINP(RAPARAM,3,'F',4,0,'RIGHT_ASCENSION=',
	1   'and seconds:')
	    CALL HMSDH(RAPARAM(1),RAPARAM(2),RAPARAM(3),RIGHTASC)
	    CALL MESSAGE('Enter the declination of the primary in'//
	1   ' degrees, minutes,')
	    CALL USRINP(DECPARAM,3,'F',4,0,'DECLINATION=',
	1   'and seconds:')
	    CALL HMSDH(DECPARAM(1),DECPARAM(2),DECPARAM(3),
	1   DECLINA)
	    CALL USRINP(EQUATOR,1,'F',8,0,'EQUATOR=',
	1   'Enter the Besselian year of the equinox:')
C  Assume that the equator is the epoch for proper motion
	    EPOCH_WORK=EQUATOR
	    IF (COMMAND.NE.'CS'.AND.COMMAND.NE.'ES') THEN
	      IF (COMMAND.EQ.'CI') THEN
	        ALTIPARAM(1)=0.0
	        ALTIPARAM(2)=0.0
	        ALTIPARAM(3)=0.0
	        CALL MESSAGE('For a line connecting two '//
	1       'telescopes of the interferometer,')
	        CALL MESSAGE('and projected onto the sky, enter '//
	1       'the altitude in degrees, minutes,')
	        CALL USRINP(ALTIPARAM,3,'F',4,1,'ALTITUDE=',
	1       'and seconds (default = 0 0 0):')
	        CALL HMSDH(ALTIPARAM(1),ALTIPARAM(2),ALTIPARAM(3),
	1       ALT_INTER)
	        CALL MESSAGE('Enter the azimuth of the telescope '//
	1       'pair in degrees, minutes,')
	        CALL USRINP(AZIPARAM,3,'F',4,0,'AZIMUTH=',
	1       'and seconds:')
	        CALL HMSDH(AZIPARAM(1),AZIPARAM(2),AZIPARAM(3),
	1       AZIMUTH_INTER)
	        CALL MESSAGE('Enter the longitude of the '//
	1       'interferometer in degrees, minutes,')
	        CALL USRINP(LONGIPARAM,3,'F',4,0,'LONGITUDE=',
	1       'and seconds:')
	        CALL HMSDH(LONGIPARAM(1),LONGIPARAM(2),LONGIPARAM(3),
	1       LONGITUDE)
	        CALL MESSAGE('Enter the latitude of the '//
	1       'interferometer in degrees, minutes,')
	        CALL USRINP(LATIPARAM,3,'F',4,0,'LATITUDE=',
	1       'and seconds:')
	        CALL HMSDH(LATIPARAM(1),LATIPARAM(2),LATIPARAM(3),
	1       LATITUDE)
	      END IF
	      TYPE='R'
	      CALL MESSAGE('Enter the type of orbit:')
	      CALL MESSAGE('         A = Absolute orbit')
	      CALL MESSAGE('         R = Relative orbit, Equator '//
	1     'equals epoch (default)')
	      CALL MESSAGE('         r = Relative orbit, Equator '//
	1     'is constant')
	      CALL USRINP(%REF(TYPE),1,'A',1,1,'ORBIT_TYPE=',' ')
	      IF (TYPE.EQ.'A') THEN
	        PPM=.TRUE.
	        CALL MESSAGE('Are parallax and proper motion to be'//
	1       ' considered as removed from the data?')
	        CALL USRINP(PPM,1,'L',4,1,'PARALLAX_PROPER_MOTION_REMOVED=',
	1       '(default = Y)')
	        IF (PPM) THEN
	          PARALLAX_PROPER_MOTION='R'
	        ELSE
	          PARALLAX_PROPER_MOTION='I'
	        END IF
	      ELSE IF (TYPE.EQ.'R'.OR.TYPE.EQ.'r') THEN
	        PARALLAX_PROPER_MOTION='R'  ! The only possibility
	      END IF
	      PHOTOCENTRIC_TRUE_SEMIMAJAX=1.0D0
	      IF (TYPE.EQ.'A') THEN
	        CALL USRINP(PHOTOCENTRIC_TRUE_SEMIMAJAX,1,'F',8,1,
	1       'RATIO_PHOTOCENTRIC_TO_TRUE_SEMIMAJOR_AXIS=','Enter'//
	2       ' the ratio of the photocentric to true semimajor axis:'//
	3       '  (default = 1)')
	      END IF
	      SCALE=1.0D0
	      CALL USRINP(SCALE,1,'F',8,1,'SCALE=',
	1     'Enter the scale in arc seconds per unit:  (default = 1.0)')
	      FILETYPE=' '
	      IF (COMMAND.EQ.'CV'.OR.COMMAND.EQ.'EV') THEN
	        DO WHILE (FILETYPE.NE.'A'.AND.FILETYPE.NE.'B') 
	          FILETYPE='B'
	          CALL MESSAGE('Enter the type of file:')
	          CALL MESSAGE('     A = X and Y positions')
	          CALL MESSAGE('     B = Position angle and '//
	1         'separation (default)')
	          CALL USRINP(%REF(FILETYPE),1,'A',9,1,'FILE_TYPE=',' ')
	          CALL ALPHA(%REF(FILETYPE),1,9,NC,NF,NR,2)
	        END DO
	        IF (FILETYPE.EQ.'A') THEN
	          FILETYPE='X_Y'
	        ELSE IF (FILETYPE.EQ.'B') THEN
	          FILETYPE='THETA_RHO'
	        END IF
	      ELSE IF (COMMAND.EQ.'CI'.OR.COMMAND.EQ.'EI') THEN
	        FILETYPE=' '
		DO WHILE (FILETYPE.NE.'A'.AND.FILETYPE.NE.'B')
	          FILETYPE='B'
	          CALL MESSAGE('Enter the type of file:')
	          CALL MESSAGE('     A = Epoch in Besselian years')
	          CALL MESSAGE('     B = Epoch in Julian Date (default)')
	          CALL USRINP(%REF(FILETYPE),1,'A',9,1,'FILE_TYPE=',' ')
	          CALL ALPHA(%REF(FILETYPE),1,9,NC,NF,NR,2)
	        END DO
	        IF (FILETYPE.EQ.'A') THEN
	          FILETYPE='BY_RHO'
	        ELSE IF (FILETYPE.EQ.'B') THEN
	          FILETYPE='JD_RHO'
	        END IF
	      END IF
	      IF (COMMAND.EQ.'EI') THEN
C  Get the period in case the file is used with one2two.exe
	        CALL USRINP(%REF(C_ELEMENTS(1)),1,'A',11,0,'PERIOD=',
	1       'Enter the period in years:')
	        CALL ALPHA(%REF(C_ELEMENTS(1)),1,11,NC,NF,NR,5)
	        DECODE(11,2000,C_ELEMENTS(1)) ELEMENTS(1)
	      END IF
	    ELSE  ! Spectroscopic binary
	      TYPE='A'
	      PARALLAX_PROPER_MOTION=' '
	      SCALE=1.0
	      CALL USRINP(SCALE,1,'F',8,1,'SCALE=',
	1     'Enter the scale in kilometers per second per unit:'//
	1     '  (default = 1)')
	      FILETYPE=' '
	      DO WHILE (FILETYPE.NE.'A'.AND.FILETYPE.NE.'B') 
	        FILETYPE='B'
	        CALL MESSAGE('Enter the type of file:')
	        CALL MESSAGE('     A = Epoch in Besselian years')
	        CALL MESSAGE('     B = Epoch in Julian Date (default)')
	        CALL USRINP(%REF(FILETYPE),1,'A',9,1,'FILE_TYPE=',' ')
	        CALL ALPHA(%REF(FILETYPE),1,9,NC,NF,NR,2)
	      END DO
	      IF (FILETYPE.EQ.'A') THEN
	        FILETYPE='BY_VELOCITY'
	      ELSE IF (FILETYPE.EQ.'B') THEN
	        FILETYPE='JD_VELOCITY'
	      END IF
	    END IF
	  ELSE IF (OFIL) THEN
	    CALL GET_BINARY_HEADER(22,NAME,RIGHTASC,DECLINA,
	1   EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	2   C_ELEMENTS,MAXNUMPARAM)
	    CALL PRINT_BINARY_HEADER(NAME,RIGHTASC,DECLINA,
	1   EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	1   C_ELEMENTS,MAXNUMPARAM)
	  END IF
C  Check for impossible or unusual combinations
	  IF ((TYPE.EQ.'R'.OR.TYPE.EQ.'r').AND.PARALLAX_PROPER_MOTION.EQ.'I')
	1 THEN
	    CALL ERROR(-3,'Impossible combination of attributes')
	  END IF
	  IF (FILETYPE.EQ.'THETA_RHO'.AND.PARALLAX_PROPER_MOTION.EQ.'I') THEN
	    CALL MESSAGE('Position angle and separation, with parallax and'//
	1   ' proper motion')
	    CALL MESSAGE('included is an unusual combination, but it will'//
	1   ' be accepted.')
	  END IF
C  Now start entering the data.  For a new file, start
C  at the current location.  For an old file, advance to the end of the
C  file.
	  IF (OFIL) THEN
	    DO WHILE (.TRUE.)
	      READ (22,3900,END=99) STRING
3900	      FORMAT(A)
	    END DO
  99	  END IF
C  Artificial data (only new file)
	  IF (COMMAND.EQ.'CV'.OR.COMMAND.EQ.'CS'.OR.COMMAND.EQ.'CI') THEN
	    NLFUN_CHOICE(30:30)='N' !  Default is no 1D interferometric
	    IF (COMMAND.EQ.'CV') THEN
	      PRI_SEC='S'
	      CALL MESSAGE('Enter a choice for the orbit')
	      CALL MESSAGE('                P = Primary')
	      CALL MESSAGE('                S = Secondary (default)')
	      CALL USRINP(%REF(PRI_SEC),1,'A',1,1,'PRIMARY_SECONDARY=',' ')
	      IF (PARALLAX_PROPER_MOTION.EQ.'I') THEN
	        IF (PRI_SEC.EQ.'P') THEN
	          NLFUN_CHOICE=PRI_SEC//'I N'
	        ELSE IF (PRI_SEC.EQ.'S') THEN
	          NLFUN_CHOICE=PRI_SEC//' IN'
	        END IF
	      ELSE IF (PARALLAX_PROPER_MOTION.EQ.'R') THEN
	        IF (PRI_SEC.EQ.'P') THEN
	          NLFUN_CHOICE=PRI_SEC//'R N'
	        ELSE IF (PRI_SEC.EQ.'S') THEN
	          NLFUN_CHOICE=PRI_SEC//' RN'
	        END IF
	      END IF
	    ELSE IF (COMMAND.EQ.'CS') THEN
	      PRI_SEC='P'
	      CALL MESSAGE('Enter a choice for the orbit')
	      CALL MESSAGE('                P = Primary (default)')
	      CALL MESSAGE('                S = Secondary')
	      CALL USRINP(%REF(PRI_SEC),1,'A',1,1,'PRIMARY_SECONDARY=',' ')
	      NLFUN_CHOICE='N  '//PRI_SEC
	    ELSE IF (COMMAND.EQ.'CI') THEN
	      PRI_SEC='S'
	      CALL MESSAGE('Enter a choice for the orbit')
	      CALL MESSAGE('                P = Primary')
	      CALL MESSAGE('                S = Secondary (default)')
	      CALL USRINP(%REF(PRI_SEC),1,'A',1,1,'PRIMARY_SECONDARY=',' ')
	      NLFUN_CHOICE='N  N'
	      NLFUN_CHOICE(30:30)=PRI_SEC
	      IF (PARALLAX_PROPER_MOTION.EQ.'I') THEN
	        IF (PRI_SEC.EQ.'P') THEN
	          NLFUN_CHOICE(31:31)='I'
	        ELSE IF (PRI_SEC.EQ.'S') THEN
	          NLFUN_CHOICE(32:32)='I'
	        END IF
	      ELSE IF (PARALLAX_PROPER_MOTION.EQ.'R') THEN
	        IF (PRI_SEC.EQ.'P') THEN
	          NLFUN_CHOICE(31:31)='R'
	        ELSE IF (PRI_SEC.EQ.'S') THEN
	         NLFUN_CHOICE(32:32)='R'
	        END IF
	      END IF
	    END IF
C  Obtain the indexes up to this point
C  Get the elements from the user and write them to the file
	    C_ELEMENTS(1)='22.09'     !  Period 
	    CALL USRINP(%REF(C_ELEMENTS(1)),1,'A',11,1,'PERIOD=',
	1   'Enter the period in years:  (default = 22.09 years)')
	    CALL ALPHA(%REF(C_ELEMENTS(1)),1,11,NC,NF,NR,5)
	    DECODE(11,2000,C_ELEMENTS(1)) ELEMENTS(1)
2000	    FORMAT(F)
	    C_ELEMENTS(2)='1975.78'   !  Time of periastron passage
	    CALL USRINP(%REF(C_ELEMENTS(2)),1,'A',11,1,'TIME=',
	1   'Enter the time of periastron passage:  (default = 1975.78)')
	    CALL ALPHA(%REF(C_ELEMENTS(2)),1,11,NC,NF,NR,5)
	    DECODE(11,2000,C_ELEMENTS(2)) ELEMENTS(2)
	    C_ELEMENTS(3)='0.58'      !  Eccentricity
	    CALL USRINP(%REF(C_ELEMENTS(3)),1,'A',11,1,'ECCENTRICITY=',
	1   'Enter the eccentricity:  (default = 0.58)')
	    CALL ALPHA(%REF(C_ELEMENTS(3)),1,11,NC,NF,NR,5)
	    DECODE(11,2000,C_ELEMENTS(3)) ELEMENTS(3)
	    IF (COMMAND.NE.'CS') THEN
	      C_ELEMENTS(4)='47.4'      !  Angle of line of nodes (big omega)
	      CALL USRINP(%REF(C_ELEMENTS(4)),1,'A',11,1,'ANGLE_OF_NODES=',
	1     'Enter the angle of the line of nodes in degrees:'//
	2     '  (default = 47.4)')
	      CALL ALPHA(%REF(C_ELEMENTS(4)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(4)) ELEMENTS(4)
	      C_ELEMENTS(5)='108.2'     !  Inclination
	      CALL USRINP(%REF(C_ELEMENTS(5)),1,'A',11,1,'INCLINATION=',
	1     'Enter the inclination in degrees: (default = 108.2)')
	      CALL ALPHA(%REF(C_ELEMENTS(5)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(5)) ELEMENTS(5)
	    END IF
	    C_ELEMENTS(6)='324.8'  
		!  Angle in the plane of the true orbit between the 
		!  line of nodes and the periastron points (little omega).
	    CALL MESSAGE('Enter the angle between the line of nodes and'//
	1   ' the periastron point')
	    CALL USRINP(%REF(C_ELEMENTS(6)),1,'A',11,1,
	1   'LITTLE_OMEGA=','in degrees:  (default = 324.8)')
	    CALL ALPHA(%REF(C_ELEMENTS(6)),1,11,NC,NF,NR,5)
	    DECODE(11,2000,C_ELEMENTS(6)) ELEMENTS(6)
	    C_ELEMENTS(78)='0.0'
	    CALL MESSAGE('Enter the rate of change of the above angle '//
	1   'in degrees per year:')
	    CALL USRINP(%REF(C_ELEMENTS(78)),1,'A',11,1,
	1   'APSIDAL_MOTION=','(default = 0.0)')
	    CALL ALPHA(%REF(C_ELEMENTS(78)),1,11,NC,NF,NR,5)
	    DECODE(11,2000,C_ELEMENTS(78)) ELEMENTS(78)
C  If the apsidal motion is zero, prevent the header line from being written
	    IF (ELEMENTS(78).EQ.0.0) THEN
	      ELEMENTS(78)=-1.0D37
	      NLFUN_CHOICE(19:19)=' '
	    ELSE
	      NLFUN_CHOICE(19:19)='A'
	    END IF
	    IF (COMMAND.NE.'CS') THEN !  Not a spectroscopic binary;  assumed 
				      !  to be a Visual binary or 
				      !  interferometric
	      C_ELEMENTS(7)='0.190' !  Semimajor axis
	      CALL USRINP(%REF(C_ELEMENTS(7)),1,'A',11,1,'SEMIMAJOR_AXIS=',
	2     'Enter the semimajor axis in arc seconds:  (default = 0.19)')
	      CALL ALPHA(%REF(C_ELEMENTS(7)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(7)) ELEMENTS(7)
	    ELSE IF (COMMAND.EQ.'CS') THEN !  Spectroscopic binary
	      CALL MESSAGE('Enter the semimajor axis times the sine of'//
	1     ' the inclination,')
	      C_ELEMENTS(17)='1.35'
	      CALL USRINP(%REF(C_ELEMENTS(17)),1,
	2     'A',11,1,'a_sin(i)=','in astronomical units:  (default = 1.35)')
	      CALL ALPHA(%REF(C_ELEMENTS(17)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(17)) ELEMENTS(17)
	    END IF
	    IF (PRI_SEC.EQ.'S') THEN
	      C_ELEMENTS(8)='2.0' !  Ratio of semimajor axes
	      CALL MESSAGE('Enter the ratio of total to primary semimajor'//
	1     ' axes:  (default = 2.0)')
	      CALL USRINP(%REF(C_ELEMENTS(8)),1,'A',11,1,
	1     'RATIO=','If the secondary alone is solved for, this is'//
	2     ' ignored.')
	      CALL ALPHA(%REF(C_ELEMENTS(8)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(8)) ELEMENTS(8)
	    END IF
	    IF (TYPE.EQ.'A'.AND.PARALLAX_PROPER_MOTION.EQ.'I') THEN
	      C_ELEMENTS(9)='0.1368'    !  Parallax
	      CALL USRINP(%REF(C_ELEMENTS(9)),1,'A',11,1,'PARALLAX=',
	2     'Enter the parallax in arc seconds:  (default = 0.1368)')
	      CALL ALPHA(%REF(C_ELEMENTS(9)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(9)) ELEMENTS(9)
	      IF (COMMAND.NE.'CI') THEN
	        C_ELEMENTS(10)='-58.5' !  X center
	        CALL USRINP(%REF(C_ELEMENTS(10)),
	1       1,'A',11,1,'X_CENTER=',
	2       'Enter the X center in arc seconds:  (default = -58.5)')
	        CALL ALPHA(%REF(C_ELEMENTS(10)),1,11,NC,NF,NR,5)
	        DECODE(11,2000,C_ELEMENTS(10)) ELEMENTS(10)
	      ELSE
	        C_ELEMENTS(89)='-58.5' !  X center
	        CALL USRINP(%REF(C_ELEMENTS(89)),
	1       1,'A',11,1,'X_CENTER=',
	2       'Enter the X center in arc seconds:  (default = -58.5)')
	        CALL ALPHA(%REF(C_ELEMENTS(89)),1,11,NC,NF,NR,5)
	        DECODE(11,2000,C_ELEMENTS(89)) ELEMENTS(89)
	      END IF
	      C_ELEMENTS(11)='3.4364' 
				!  X proper motion
	      CALL USRINP(%REF(C_ELEMENTS(11)),
	1     1,'A',11,1,'X_PROPER_MOTION=',
	2     'Enter the X proper motion in arc seconds/year:'//
	3     '  (default = 3.4364)')
	      CALL ALPHA(%REF(C_ELEMENTS(11)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(11)) ELEMENTS(11)
	      C_ELEMENTS(12)='0.0'
					!  X proper motion acceleration
	      CALL USRINP(%REF(C_ELEMENTS(12)),
	1     1,'A',11,1,'X_PROPER_MOTION_ACCELERATION=',
	2     'Enter the X proper motion acceleration:  (default = 0.0)')
	      CALL ALPHA(%REF(C_ELEMENTS(12)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(12)) ELEMENTS(12)
	      IF (COMMAND.NE.'CI') THEN
	        C_ELEMENTS(13)='23.0'    !  Y center
	        CALL USRINP(%REF(C_ELEMENTS(13)),1,'A',11,1,'Y_CENTER=',
	2       'Enter the Y center in arc seconds:  (default = 23.0)')
	        CALL ALPHA(%REF(C_ELEMENTS(13)),1,11,NC,NF,NR,5)
	        DECODE(11,2000,C_ELEMENTS(13)) ELEMENTS(13)
	        C_ELEMENTS(14)='-1.5787'
	      ELSE
	        C_ELEMENTS(90)='23.0'    !  Y center
	        CALL USRINP(%REF(C_ELEMENTS(90)),1,'A',11,1,'Y_CENTER=',
	2       'Enter the Y center in arc seconds:  (default = 23.0)')
	        CALL ALPHA(%REF(C_ELEMENTS(90)),1,11,NC,NF,NR,5)
	        DECODE(11,2000,C_ELEMENTS(90)) ELEMENTS(90)
	      END IF
	      C_ELEMENTS(14)='-1.5787'  
				!  Y proper motion
	      CALL USRINP(%REF(C_ELEMENTS(14)),1,'A',11,1,'Y_PROPER_MOTION=',
	2     'Enter the Y proper motion in arc seconds/year:'//
	3     '  (default = -1.5787)')
	      CALL ALPHA(%REF(C_ELEMENTS(14)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(14)) ELEMENTS(14)
	      C_ELEMENTS(15)='0.0'
				!  Y proper motion acceleration
	      CALL USRINP(%REF(C_ELEMENTS(15)),
	1     1,'A',11,1,'Y_PROPER_MOTION_ACCELERATION=',
	2     'Enter the Y proper motion acceleration:  (default = 0.0)')
	      CALL ALPHA(%REF(C_ELEMENTS(15)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(15)) ELEMENTS(15)
	    END IF
	    IF (COMMAND.EQ.'CS') THEN
	      C_ELEMENTS(16)='0.0'
	      CALL USRINP(%REF
	1     (C_ELEMENTS(16)),1,'A',11,1,'SYSTEM_RADIAL_VELOCITY=',
	3     'Enter the system radial velocity in km/sec:  (default = 0)')
	      CALL ALPHA(%REF(C_ELEMENTS(16)),1,11,NC,NF,NR,5)
	      DECODE(11,2000,C_ELEMENTS(16)) ELEMENTS(16)
C  ELEMENTS(17) was established above
	      ADD_SUBCOMPONENT=.FALSE.
	      CALL MESSAGE('Do you wish to add any reflex velocities from'//
	1     ' subcomponents?')
	      CALL USRINP(ADD_SUBCOMPONENT,1,'L',4,1,'ADD_SUBCOMPONENT=',
	1     '(default = N)')
	      IF (ADD_SUBCOMPONENT) THEN
C  We assume that the reflex velocity of the subcomponent belongs to the
C  primary or secondary referred to above.
	        NSUBS=1
	        CALL MESSAGE('Enter the desired number of spectroscopic'//
	1       ' subcomponents:')
	        CALL USRINP(NSUBS,1,'I',4,1,'NUMBER_OF_SUBCOMPONENTS=',
	1       '(default = 1, maximum = 5)')
C No checking yet
	        ENCODE(1,1001,C_NSUBS) NSUBS
1001	        FORMAT(I1)
C  Modify NLFUN_CHOICE for the number of subcomponents, assumed to belong
C  to the original primary or secondary.
	        IF (PRI_SEC.EQ.'P') THEN
	          NLFUN_CHOICE(5:5)=C_NSUBS
	          IPRISEC=0
	        ELSE IF (PRI_SEC.EQ.'S') THEN
	          NLFUN_CHOICE(7:7)=C_NSUBS
	          IPRISEC=25
	        END IF
	        I_SUB_SEC=0
	        DO I=1,NSUBS
	          WRITE(6,1400) I
1400	          FORMAT(' ','For subcomponent #',I1)
	          WRITE(7,1400) I
	          IOFF=5*(I-1)+IPRISEC
	          C_ELEMENTS(18+IOFF)='5.0'     !  Period 
	          CALL USRINP(%REF(C_ELEMENTS(18+IOFF)),1,'A',11,
	1         1,'PERIOD_SUB=',
	2         'Enter the period in years:  (default = 5.0 years)')
	          CALL ALPHA(%REF(C_ELEMENTS(18+IOFF)),1,11,NC,NF,NR,5)
	          DECODE(11,2000,C_ELEMENTS(18+IOFF)) ELEMENTS(18+IOFF)
	          C_ELEMENTS(19+IOFF)='1975.78'   
					!  Time of periastron passage
	          CALL USRINP(%REF(C_ELEMENTS(19+IOFF)),1,'A',11,
	1         1,'TIME_SUB=',
	2         'Enter the time of periastron passage:  (default = 1975.78)')
	          CALL ALPHA(%REF(C_ELEMENTS(19+IOFF)),1,11,NC,NF,NR,5)
	          DECODE(11,2000,C_ELEMENTS(19+IOFF)) ELEMENTS(19+IOFF)
	          C_ELEMENTS(20+IOFF)='0.58'      !  Eccentricity
	          CALL USRINP(%REF(C_ELEMENTS(20+IOFF)),1,'A',11,1,
	1         'ECCENTRICITY_SUB=',
	2         'Enter the eccentricity:  (default = 0.58)')
	          CALL ALPHA(%REF(C_ELEMENTS(20+IOFF)),1,11,NC,NF,NR,5)
	          DECODE(11,2000,C_ELEMENTS(20+IOFF)) ELEMENTS(20+IOFF)
	          C_ELEMENTS(21+IOFF)='324.8'  !  Angle in the plane of the 
			!  true orbit between the line of nodes and the
			!  periastron points (little omega).
	          CALL MESSAGE('Enter the angle between the line of'//
	1         ' nodes and the periastron point')
	          CALL USRINP(%REF(C_ELEMENTS(21+IOFF)),1,'A',11,1,
	1         'LITTLE_OMEGA_SUB=','in degrees:  (default = 324.8)')
	          CALL ALPHA(%REF(C_ELEMENTS(21+IOFF)),1,11,NC,NF,NR,5)
	          DECODE(11,2000,C_ELEMENTS(21+IOFF)) ELEMENTS(21+IOFF)
	          IF (PRI_SEC.EQ.'P') THEN
	            ISTART=78
	            JSTART=19
	          ELSE IF (PRI_SEC.EQ.'S') THEN
	            ISTART=83
	            JSTART=24
	          END IF
	          C_ELEMENTS(ISTART+I)='0.0'
	          CALL MESSAGE('Enter the rate of change of the above angle '//
	1         'in degrees per year:')
	          CALL USRINP(%REF(C_ELEMENTS(ISTART+I)),1,'A',11,1,
	1         'APSIDAL_MOTION=','(default = 0.0)')
	          CALL ALPHA(%REF(C_ELEMENTS(ISTART+I)),1,11,NC,NF,NR,5)
	          DECODE(11,2000,C_ELEMENTS(ISTART+I)) ELEMENTS(ISTART+I)
C  If the apsidal motion is zero, prevent the header line from being written
	          IF (ELEMENTS(ISTART+I).EQ.0.0) THEN
	            ELEMENTS(ISTART+I)=-1.0D37
	            NLFUN_CHOICE(JSTART+I:JSTART+I)=' '
	          ELSE
	            NLFUN_CHOICE(JSTART+I:JSTART+I)='A'
	          END IF
	          CALL MESSAGE('Enter the semimajor axis times the sine of'//
	1         ' the inclination,')
	          C_ELEMENTS(22+IOFF)='1.35'
	          CALL USRINP(%REF(C_ELEMENTS(22+IOFF)),1,'A',11,1,
	1         'a_sin(i)_SUB=','in astronomical units:  (default = 1.35)')
	          CALL ALPHA(%REF(C_ELEMENTS(22+IOFF)),1,11,NC,NF,NR,5)
	          DECODE(11,2000,C_ELEMENTS(22+IOFF)) ELEMENTS(22+IOFF)
C  Ask the user if a file for this subcomponent velocity should also
C  be written.  If so, ask for the ratio of semimajor axes (total to that
C  of the primary or secondary parent).
	          CALL MESSAGE('Do you wish to write a file of velocities'//
	1         ' for this subcomponent')
	          WRITE_SUB_SEC=.FALSE.
	          CALL USRINP(WRITE_SUB_SEC,1,'L',4,1,
	1         'SUBCOMPONENT_SECONDARY=','as well?  (default = N)')
	          IF (WRITE_SUB_SEC) THEN
	            I_SUB_SEC=I_SUB_SEC+1
	            CALL OPENCF(22+I_SUB_SEC,2,
	1           'SUBCOMPONENT_SECONDARY_FILE=',0)
	            IF (PRI_SEC.EQ.'P') THEN
C  INDEX_SUB_SEC is used to reference the location in DATA and also ELEMENTS.
	              INDEX_SUB_SEC(I_SUB_SEC)=I+6
	            ELSE IF (PRI_SEC.EQ.'S') THEN
	              INDEX_SUB_SEC(I_SUB_SEC)=I+11
	            END IF
	            IOFF_SUB_SEC=5*(I_SUB_SEC-1)
C  Initialize ELEMENTS_S for proper functioning of WRITE_BINARY_HEADER
	            DO IE=1,MAXNUMPARAM
	              C_ELEMENTS_S(IE)=' '
	              ELEMENTS_S(IE)=-1.0D37
	            END DO
C  Set up the file header.  Include the parameters for the main component 
C  velocity and the particular subcomponent velocity
	            DO IE=1,17
	              C_ELEMENTS_S(IE)=C_ELEMENTS(IE)
	              ELEMENTS_S(IE)=ELEMENTS(IE)
	            END DO
	            DO IE=18,22
C  Each time this loop is carried out for successive subcomponents, IOFF 
C  increases by 5 
	              C_ELEMENTS_S(IE+IOFF)=C_ELEMENTS(IE+IOFF)
	              ELEMENTS_S(IE+IOFF)=ELEMENTS(IE+IOFF)
	            END DO
	            IF (PRI_SEC.EQ.'P') THEN
C  I_SUB_SEC is incremented by 1 for each secondary file written
	              ITEMP=67+I
	            ELSE IF (PRI_SEC.EQ.'S') THEN
	              ITEMP=72+I
	            END IF
	            CALL MESSAGE('Enter the ratio of semimajor axes'//
	1           ' (total to that of parent) for')
	            C_ELEMENTS_S(ITEMP)='2.0'
	            CALL USRINP(%REF(C_ELEMENTS_S(ITEMP)),1,'A',11,
	1           1,'RATIO_a_SUBCOMPONENT=','this subcomponent secondary'//
	2           ' (default = 2.0)')
	            CALL ALPHA(%REF(C_ELEMENTS_S(ITEMP)),1,11,
	1           NC,NF,NR,5)
	            DECODE(11,2000,C_ELEMENTS_S(ITEMP)) ELEMENTS_S(ITEMP)

C  Temporarily store this value in ELEMENTS so that it can be copied into
C  PARAMETRS, a few lines below.
	            ELEMENTS(ITEMP)=ELEMENTS_S(ITEMP)
C  Write the file header for the subcomponent secondary.  Include the
C  ratio of semimajor axes since it is a secondary.
	            CALL WRITE_BINARY_HEADER(22+I_SUB_SEC,NAME,
	1           RIGHTASC,DECLINA,EQUATOR,TYPE,PARALLAX_PROPER_MOTION,
	2           SCALE,FILETYPE,ELEMENTS_S,C_ELEMENTS_S,MAXNUMPARAM)
	            IF (PRI_SEC.EQ.'P') THEN
	              NLFUN_CHOICE(I+8:I+8)='T'
	            ELSE IF (PRI_SEC.EQ.'S') THEN
	              NLFUN_CHOICE(I+13:I+13)='T'
	            END IF
	          END IF
	        END DO !  Ends loop for NSUBS;  index is I
	      END IF
	    END IF
C
	    CALL BINARY_INDEXES(NLFUN_CHOICE,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      PARAMETRS(INDX(I))=ELEMENTS(I)
	    END DO
C
C  Write the file header for the main star, but do not include the 
C  ratios of semimajor axes.  If the apsidal motion is zero, do not
C  write it to the header.
	    DO I=1,I_SUB_SEC
	      ELEMENTS(61+INDEX_SUB_SEC(I))=-1.0D37 !  Voids the ratios
	    END DO
	    CALL WRITE_BINARY_HEADER(22,NAME,RIGHTASC,DECLINA,
	1   EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	2   C_ELEMENTS,MAXNUMPARAM)
C
	  END IF !  Ends section for COMMAND='CI','CS','CV', new file
C
C
	  IF (COMMAND.EQ.'CS'.OR.COMMAND.EQ.'CV'.OR.COMMAND.EQ.'CI') THEN
C  Print the header information on the screen and in the log file
	    CALL PRINT_BINARY_HEADER(NAME,RIGHTASC,DECLINA,
	1   EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	2   C_ELEMENTS,MAXNUMPARAM)
C
	    CALL USRINP(NPOINTS_DATA,1,'I',4,0,'NPOINTS_DATA=',
	1   'Enter the number of data points desired (maximum = 10000):')
	    IF (FILETYPE.EQ.'BY_VELOCITY'.OR.FILETYPE.EQ.'THETA_RHO'
	1   .OR.FILETYPE.EQ.'X_Y') THEN
	      CALL USRINP(TEMPARRAY,2,'F',8,0,'TIMES=',
	1     'Enter the beginning and ending times in Besselian years:')
	      TEMP=(TEMPARRAY(2)-TEMPARRAY(1))/(NPOINTS_DATA-1)
	      DO N=1,NPOINTS_DATA
	        TIME(N,1)=TEMPARRAY(1)+(N-1)*TEMP
	      END DO
	    ELSE IF (FILETYPE.EQ.'JD_VELOCITY') THEN
	      CALL USRINP(TEMPARRAY,2,'F',8,0,'TIMES=',
	1     'Enter the beginning and ending times in Julian date:')
	      TEMP=(TEMPARRAY(2)-TEMPARRAY(1))/(NPOINTS_DATA-1)
	      DO N=1,NPOINTS_DATA
	        TIME_JD(N)=TEMPARRAY(1)+(N-1)*TEMP
	      END DO
	      DO N=1,NPOINTS_DATA
	        CALL JD_BY(TIME_JD(N),TIME(N,1))
	      END DO
	    ELSE IF (FILETYPE.EQ.'BY_RHO') THEN
 500	      CALL USRINP(TEMPARRAY,2,'F',8,0,'TIMES=',
	1     'Enter the beginning and ending times in Besselian years:')
	      TEMP=TEMPARRAY(2)-TEMPARRAY(1)
C  Use random times when sun is below horizon, and object is above horizon
	      CALL MESSAGE('Random times when the sun is below the '//
	1     'horizon and the object is above')
	      CALL MESSAGE('the horizon will be used.')
	      ISEED=12345
	      CALL USRINP(ISEED,1,'I',4,1,'TIME_SEED=',
	1     'Enter the seed for the random number generator:'//
	2     '  (default = 12345)')
	      N_BAD_HOR=0
	      N=0
	      DO WHILE (N.LT.NPOINTS_DATA)
	        TEMP2=TEMPARRAY(1)+TEMP*RAN(ISEED)
	        CALL BY_JD(TEMP2,TEMP3)
	        CALL JD_SUN(TEMP3,LONGITUDE,LATITUDE,RIGHTASC_SUN,
	1       DECLINA_SUN,ALT_SUN,AZIMUTH_SUN,HA_SUN)
	        IF (ALT_SUN.LT.0.0D0) THEN
	          CALL FRINGE_ANGLE(TEMP3,LONGITUDE,LATITUDE,RIGHTASC,
	1         DECLINA,ALT_INTER,AZIMUTH_INTER,1,HA,ALT,PA)
	          IF (ALT.GT.0.0D0) THEN
	            N=N+1
	            TIME(N,1)=TEMP2
	            FR_ANGLE(N)=PA
	          ELSE
	            N_BAD_HOR=N_BAD_HOR+1
	          END IF
	        ELSE
	          N_BAD_HOR=N_BAD_HOR+1
	        END IF
	        IF (N_BAD_HOR.GE.100*NPOINTS_DATA) THEN
	          CALL MESSAGE('For the time interval selected, the object '//
	1         'is either below the horizon')
	          CALL MESSAGE('or the sun is above the horizon.')
	          GO TO 500
	        END IF
	      END DO
	      IF (N_BAD_HOR.GT.0) THEN
	        TEMP=1.0*N_BAD_HOR/(N_BAD_HOR+NPOINTS_DATA)
	        CALL MESSAGE('For the given time interval, the fraction of '//
	1       'the time that the star was')
	        CALL MESSAGE('below the horizon, and/or the sun was above '//
	1       'the horizon was')
	        WRITE (6,3990) TEMP
3990	        FORMAT(' ',G13.7)
	        WRITE (7,3990) TEMP
	      END IF
C  Now sort the times so that they increase monotonically
	      I_START=1
	      DO WHILE(I_START.LT.NPOINTS_DATA)
	        T_LOW=1.0D37
	        DO I=I_START,NPOINTS_DATA
	          IF (TIME(I,1).LE.T_LOW) THEN
	            T_LOW=TIME(I,1)
	            I_LOW=I
	            PA=FR_ANGLE(I)
	          END IF
	        END DO
C  Shuffle the points higher.  Start at the upper end
	        DO I=I_LOW,I_START+1,-1
	          TIME(I,1)=TIME(I-1,1)
	          FR_ANGLE(I)=FR_ANGLE(I-1)
	        END DO
	        TIME(I_START,1)=T_LOW
	        FR_ANGLE(I_START)=PA
	        I_START=I_START+1
	      END DO
	    ELSE IF (FILETYPE.EQ.'JD_RHO') THEN
 510	      CALL USRINP(TEMPARRAY,2,'F',8,0,'TIMES=',
	1     'Enter the beginning and ending times in Julian date:')
	      TEMP=TEMPARRAY(2)-TEMPARRAY(1)
C  Use random times when sun is below horizon, and object is above horizon
	      CALL MESSAGE('Random times when the sun is below the '//
	1     'horizon and the object is above')
	      CALL MESSAGE('the horizon will be used.')
	      ISEED=12345
	      CALL USRINP(ISEED,1,'I',4,1,'TIME_SEED=',
	1     'Enter the seed for the random number generator:'//
	2     '  (default = 12345)')
	      N_BAD_HOR=0
	      N=0
	      DO WHILE (N.LT.NPOINTS_DATA)
	        TEMP3=TEMPARRAY(1)+TEMP*RAN(ISEED)
	        CALL JD_SUN(TEMP3,LONGITUDE,LATITUDE,RIGHTASC_SUN,
	1       DECLINA_SUN,ALT_SUN,AZIMUTH_SUN,HA_SUN)
	        IF (ALT_SUN.LT.0.0D0) THEN
	          CALL FRINGE_ANGLE(TEMP3,LONGITUDE,LATITUDE,RIGHTASC,
	1         DECLINA,ALT_INTER,AZIMUTH_INTER,1,HA,ALT,PA)
	          IF (ALT.GT.0.0D0) THEN
	            N=N+1
	            CALL JD_BY(TEMP3,TEMP2)
	            TIME(N,1)=TEMP2
C  Save the JD for writing to the file
	            TIME_JD(N)=TEMP3
	            FR_ANGLE(N)=PA
	          ELSE
	            N_BAD_HOR=N_BAD_HOR+1
	          END IF
	        ELSE
	          N_BAD_HOR=N_BAD_HOR+1
	        END IF
	        IF (N_BAD_HOR.GE.100*NPOINTS_DATA) THEN
	          CALL MESSAGE('For the time interval selected, the object '//
	1         'is either below the horizon')
	          CALL MESSAGE('or the sun is above the horizon.')
	          GO TO 510
	        END IF
	      END DO
	      IF (N_BAD_HOR.GT.0) THEN
	        TEMP=1.0*N_BAD_HOR/(N_BAD_HOR+NPOINTS_DATA)
	        CALL MESSAGE('For the given time interval, the fraction of '//
	1       'the time that the star was')
	        CALL MESSAGE('below the horizon, and/or the sun was above '//
	1       'the horizon was')
	        WRITE (6,3990) TEMP
	        WRITE (7,3990) TEMP
	      END IF
C  Now sort the times so that they increase monotonically
	      I_START=1
	      DO WHILE(I_START.LT.NPOINTS_DATA)
	        T_LOW=1.0D37
	        DO I=I_START,NPOINTS_DATA
	          IF (TIME(I,1).LE.T_LOW) THEN
	            T_LOW=TIME(I,1)
	            TJD_LOW=TIME_JD(I)
	            PA=FR_ANGLE(I)
	            I_LOW=I
	          END IF
	        END DO
C  Shuffle the points higher.  Start at the upper end
	        DO I=I_LOW,I_START+1,-1
	          TIME(I,1)=TIME(I-1,1)
	          TIME_JD(I)=TIME_JD(I-1)
	          FR_ANGLE(I)=FR_ANGLE(I-1)
	        END DO
	        TIME(I_START,1)=T_LOW
	        TIME_JD(I_START)=TJD_LOW
	        FR_ANGLE(I_START)=PA
	        I_START=I_START+1
	      END DO
	    END IF
C  Compute the parallax factors
	    IF (PARALLAX_PROPER_MOTION.EQ.'I') THEN
	      DO I=1,NPOINTS_DATA
	        CALL PARALLAXFACTORS(TIME(I,1),RIGHTASC,DECLINA,
	1       EQUATOR,0,PX(I),PY(I))
	      END DO
	    END IF
	    DIMENSN_IND=1
c the following may not be needed
c	    IF (COMMAND.EQ.'CI') THEN
c	      DIMENSN_DATA=22
c	    ELSE
c	      IF (WRITE_SUB_SEC) THEN
c	        DIMENSN_DATA=16
c	      ELSE
c	        DIMENSN_DATA=6
c	      END IF
c	    END IF
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_DATA
	    FUNCAT=3
	    OPERATE=.FALSE.
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   NLFUN_CHOICE,OPERATE,ITIME,DATA,DIMENSN_DATA)
C  Convert TIME to JD if necessary.  This can now be done since NLFUN has been
C  called.
	    IF (FILETYPE.EQ.'JD_VELOCITY') THEN
	      DO N=1,NPOINTS_DATA
	        TIME(N,1)=TIME_JD(N)
	      END DO
	    END IF
	    ADDNOISE=.TRUE.
	    CALL USRINP(ADDNOISE,1,'L',4,1,'ADD_NOISE=',
	1   'Do you want to add noise to the data?  (default = Y)')
	    NOISE_AS=0.0D0
	    NOISE_DEG=0.0D0
	    NOISE_KMS=0.0D0
	    IF (ADDNOISE) THEN
	      IF (FILETYPE.EQ.'X_Y') THEN
	        NOISE_AS=0.01D0
	        CALL USRINP(NOISE_AS,1,'F',8,1,'NOISE_ARC_SECONDS=',
	1       'Enter the noise in arc seconds:  (default = 0.01)')
	      ELSE IF (FILETYPE.EQ.'THETA_RHO') THEN
	        NOISE_DEG=1.0D0
	        CALL USRINP(NOISE_DEG,1,'F',8,1,'NOISE_DEGREES=',
	1       'Enter the noise for the position angle in degrees:'//
	2       '  (default = 1.0)')
	        NOISE_AS=0.01D0
	        CALL USRINP(NOISE_AS,1,'F',8,1,'NOISE_ARC_SECONDS=',
	1       'Enter the noise for the separation in arc seconds:'//
	2       '  (default = 0.01)')
	      ELSE IF (FILETYPE.EQ.'BY_RHO'.OR.FILETYPE.EQ.'JD_RHO') THEN
	        NOISE_AS=0.001D0
	        CALL USRINP(NOISE_AS,1,'F',8,1,'NOISE_ARC_SECONDS=',
	1       'Enter the noise for the projected separation in arc '//
	2       'seconds:  (default = 0.001)')
	      ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	        NOISE_KMS=2.0D0
	        CALL USRINP(NOISE_KMS,1,'F',8,1,'NOISE_KMS=',
	1       'Enter the noise for the radial velocity in km/sec:'//
	2       '  (default = 2.0)')
	      END IF
	      ISEED=12345
	      CALL USRINP(ISEED,1,'I',4,1,'NOISE_SEED=',
	1     'Enter the seed for the random number generator:'//
	2     '  (default = 12345)')
	    END IF
	    IF (COMMAND.EQ.'CV') THEN
	      IF (PRI_SEC.EQ.'P') THEN
	        K1=1
	        K2=2
	      ELSE IF (PRI_SEC.EQ.'S') THEN
	        K1=3
	        K2=4
	      END IF
	    ELSE IF (COMMAND.EQ.'CS') THEN
	      IF (PRI_SEC.EQ.'P') THEN
	        KS=5
	      ELSE IF (PRI_SEC.EQ.'S') THEN
	        KS=6
	      END IF
	    ELSE IF (COMMAND.EQ.'CI') THEN
	      IF (PRI_SEC.EQ.'P') THEN
	        KI=17
	      ELSE IF (PRI_SEC.EQ.'S') THEN
	        KI=18
	      END IF
	    END IF
	    IF (COMMAND.NE.'CS') THEN
	      IF (TYPE.EQ.'R') EQUATOR_IS_EPOCH=.TRUE.
	      IF (TYPE.EQ.'A'.OR.TYPE.EQ.'r') EQUATOR_IS_EPOCH=.FALSE.
	    END IF
C  Write the data
	    DO N=1,NPOINTS_DATA  ! Steps through the number of data points
	      IF (FILETYPE.EQ.'X_Y') THEN
	        IF (EQUATOR_IS_EPOCH) THEN
	          POS_SEP(2)=DSQRT(DATA(N,K1)**2+DATA(N,K2)**2)
	          POS_SEP(1)=DATAN2D(DATA(N,K1),DATA(N,K2))
C  Precess the given RA and DEC of the star to the epoch of the
C  observation.
	          CALL PRECESSIO(0,EQUATOR,TIME(N,1),RIGHTASC,
	1         DECLINA,RIGHTASC_PRIMARY2,DECLINA_PRIMARY2)
C  Do the same for the position of the secondary
C  First get the RA and DEC of the secondary
	          DECLINA_SECONDARY1=DECLINA+DATA(N,K2)/3600.0
	          RIGHTASC_SECONDARY1=RIGHTASC+DATA(N,K1)/15.0/3600.0/
	1         DCOSD((DECLINA+DECLINA_SECONDARY1)/2.0D0)  ! In hours
C  Precess the secondary's position to the equator of the epoch
	          CALL PRECESSIO(0,EQUATOR,TIME(N,1),RIGHTASC_SECONDARY1,
	1         DECLINA_SECONDARY1,RIGHTASC_SECONDARY2,DECLINA_SECONDARY2)
C  Obtain a new position angle and separation valid at the epoch of observation
C  One could use the cosine law to get the separation, but it is
C  more accurate to simply use the original separation, POS_SEP(2)
	          DATA(N,K2)=(DECLINA_SECONDARY2-DECLINA_PRIMARY2)*3600  ! "
	          DATA(N,K1)=(RIGHTASC_SECONDARY2-RIGHTASC_PRIMARY2)*
	1         DCOSD((DECLINA_PRIMARY2+DECLINA_SECONDARY2)/2.0)*15.0*3600.0
	        END IF
	        IF (ADDNOISE) THEN
	          DO K=K1,K2
	            SUM=0.0
	            DO KK=1,12
	              SUM=SUM+RAN(ISEED)
	            END DO
	            DATA(N,K)=DATA(N,K)+NOISE_AS*(SUM-6.0)
	          END DO
	        END IF
	        WRITE (22,3000) TIME(N,1),DATA(N,K1)/SCALE,NOISE_AS/SCALE,
	1       DATA(N,K2)/SCALE,NOISE_AS/SCALE
	      ELSE IF (FILETYPE.EQ.'THETA_RHO') THEN
	        POS_SEP(2)=DSQRT(DATA(N,K1)**2+DATA(N,K2)**2)
	        IF (DATA(N,K1).NE.0.0) THEN
	          POS_SEP(1)=DATAN2D(DATA(N,K1),DATA(N,K2))
	        ELSE
	          POS_SEP(1)=0.0
	        END IF
	        IF (EQUATOR_IS_EPOCH) THEN
C  Precess the given RA and DEC of the star to the epoch of the
C  observation.
	          CALL PRECESSIO(0,EQUATOR,TIME(N,1),RIGHTASC,
	1         DECLINA,RIGHTASC_PRIMARY2,DECLINA_PRIMARY2)
C  Do the same for the position of the secondary
C  First get the RA and DEC of the secondary
	          DECLINA_SECONDARY1=DECLINA+DATA(N,K2)/3600.0
	          RIGHTASC_SECONDARY1=RIGHTASC+DATA(N,K1)/15.0/3600.0/
	1         DCOSD((DECLINA+DECLINA_SECONDARY1)/2.0D0)  ! In hours
C  Precess the secondary's position to the equator of the epoch
	          CALL PRECESSIO(0,EQUATOR,TIME(N,1),RIGHTASC_SECONDARY1,
	1         DECLINA_SECONDARY1,RIGHTASC_SECONDARY2,DECLINA_SECONDARY2)
C  Obtain a new position angle and separation valid at the epoch of observation
C  One could use the cosine law to get the separation, but it is
C  more accurate to simply use the original separation, POS_SEP(2)
	          DELTA_NORTH=(DECLINA_SECONDARY2-DECLINA_PRIMARY2)  ! Degrees
	          DELTA_EAST=(RIGHTASC_SECONDARY2-RIGHTASC_PRIMARY2)*
	1         DCOSD((DECLINA_PRIMARY2+DECLINA_SECONDARY2)/2.0)*15.0
	          IF (DELTA_EAST.NE.0.0) THEN
	            POS_SEP(1)=DATAN2D(DELTA_EAST,DELTA_NORTH)
	          ELSE
	            POS_SEP(1)=0.0
	          END IF
	        END IF
	        IF (ADDNOISE) THEN
	          SUM=0.0
	          DO KK=1,12
	            SUM=SUM+RAN(ISEED)
	          END DO
	          POS_SEP(2)=POS_SEP(2)+NOISE_AS*(SUM-6.0)
	          SUM=0.0
	          DO KK=1,12
	            SUM=SUM+RAN(ISEED)
	          END DO
	          POS_SEP(1)=POS_SEP(1)+NOISE_DEG*(SUM-6.0)
	        END IF
	        IF (POS_SEP(1).LT.0.0) POS_SEP(1)=POS_SEP(1)+360.0
	        WRITE (22,3000) TIME(N,1),POS_SEP(1),NOISE_DEG,
	1       POS_SEP(2)/SCALE,NOISE_AS/SCALE
	      ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	        IF (ADDNOISE) THEN
	          SUM=0.0
	          DO KK=1,12
	            SUM=SUM+RAN(ISEED)
	          END DO
	          DATA(N,KS)=DATA(N,KS)+NOISE_KMS*(SUM-6.0)
	        END IF
C  Write the main component.  Note that TIME can be either BY or JD
	        WRITE (22,3100) TIME(N,1),DATA(N,KS)/SCALE,NOISE_KMS/SCALE
	        DO I=1,I_SUB_SEC
	          IF (ADDNOISE) THEN
	            SUM=0.0
	            DO KK=1,12
	              SUM=SUM+RAN(ISEED)
	            END DO
	            DATA(N,INDEX_SUB_SEC(I))=DATA(N,INDEX_SUB_SEC(I))+
	1           NOISE_KMS*(SUM-6.0)
	          END IF
C  Write the subcomponent secondary
	          WRITE (22+I,3100) TIME(N,1),DATA(N,INDEX_SUB_SEC(I))/SCALE,
	1         NOISE_KMS/SCALE
	        END DO
	      ELSE IF (FILETYPE(4:6).EQ.'RHO') THEN
                IF (EQUATOR_IS_EPOCH) THEN
C  For an interferometer observation, we don't measure where the second star
C  is, only its projected distance, therefore the methods used above
C  will not work.  Instead, just consider that the direction to the north
C  celestial pole is different for each epoch.  Then use a call to
C  POSITION_ANGLE to recompute that angle.  The projected separation doesn't
C  change.
C
C  Precess the given RA and DEC of the star to the epoch of the
C  observation.  Time is in BY
	          CALL PRECESSIO(0,EQUATOR,TIME(N,1),RIGHTASC,
	1         DECLINA,RIGHTASC_PRIMARY2,DECLINA_PRIMARY2)
C  Recompute the fringe direction angle
	          CALL FRINGE_ANGLE(TIME(N,1),LONGITUDE,LATITUDE,
	1         RIGHTASC_PRIMARY2,DECLINA_PRIMARY2,ALT_INTER,
	2         AZIMUTH_INTER,0,HA,ALT,FR_ANGLE(N))
C  The projected separation doesn't change
	        END IF
	        IF (ADDNOISE) THEN
	          SUM=0.0
	          DO KK=1,12
	            SUM=SUM+RAN(ISEED)
	          END DO
	          DATA(N,KI)=DATA(N,KI)+NOISE_AS*(SUM-6.0)
	        END IF
c                 POS_SEP(2)=DSQRT(DATA(N,K1)**2+DATA(N,K2)**2)
c                 POS_SEP(1)=DATAN2D(DATA(N,K1),DATA(N,K2))

	        IF (FILETYPE.EQ.'BY_RHO') THEN
	          WRITE (22,3300) TIME(N,1),FR_ANGLE(N),
	1         DATA(N,KI)/SCALE,NOISE_AS/SCALE
	        ELSE IF (FILETYPE.EQ.'JD_RHO') THEN
	          WRITE (22,3300) TIME_JD(N),FR_ANGLE(N),
	1         DATA(N,KI)/SCALE,NOISE_AS/SCALE
	        END IF
	      END IF
	    END DO
C  User supplied data points
	  ELSE IF (COMMAND.EQ.'EI'.OR.COMMAND.EQ.'ES'.OR.COMMAND.EQ.'EV') THEN
	    IF (.NOT.OFIL) THEN
	      CALL WRITE_BINARY_HEADER(22,NAME,RIGHTASC,DECLINA,
	1     EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	2     C_ELEMENTS,MAXNUMPARAM)
	    END IF
	    GO=.TRUE.
	    DO WHILE(GO)
	      IF (FILETYPE.EQ.'BY_RHO') THEN
	        CALL MESSAGE('Enter the epoch in Besselian years, angle'//
	1       ' on the sky, projected separation,')
	        CALL USRINP(B_DATA,4,'F',8,0,'BINARY_DATA=',
	1       'and separation mean error.  Enter 0 0 0 0 when done.')
	      ELSE IF (FILETYPE.EQ.'JD_RHO') THEN
	        CALL MESSAGE('Enter the epoch in Julian date, angle'//
	1       ' on the sky, projected separation,')
	        CALL USRINP(B_DATA,4,'F',8,0,'BINARY_DATA=',
	1       'and separation mean error.  Enter 0 0 0 0 when done.')
	      ELSE IF (FILETYPE.EQ.'BY_VELOCITY') THEN
	        CALL MESSAGE('Enter the epoch in Besselian years, radial'//
	1       ' velocity, and radial')
	        CALL USRINP(B_DATA,3,'F',8,0,'BINARY_DATA=',
	1       'velocity mean error.  Enter 0 0 0 when done.')
	      ELSE IF (FILETYPE.EQ.'JD_VELOCITY') THEN
	        CALL MESSAGE('Enter the epoch in Julian date, radial'//
	1       ' velocity, and radial')
	        CALL USRINP(B_DATA,3,'F',8,0,'BINARY_DATA=',
	1       'velocity mean error.  Enter 0 0 0 when done.')
	      ELSE IF (FILETYPE.EQ.'X_Y') THEN
	        CALL MESSAGE('Enter the epoch, x position, x mean error,'//
	1       ' y position, and y mean error')
	        CALL USRINP(B_DATA,5,'F',8,0,'BINARY_DATA=',
	1       'Enter 0 0 0 0 0 when done.')
	      ELSE IF (FILETYPE.EQ.'THETA_RHO') THEN
	        CALL MESSAGE('Enter the epoch, position angle in degrees,'//
	1       ' p.a. mean error in degrees,')
	        CALL USRINP(B_DATA,5,'F',8,0,'BINARY_DATA=',
	1       'separation and separation mean error.  Enter 0 0 0 0 0'//
	2       ' when done.')
	      END IF
	      IF (FILETYPE(4:6).EQ.'RHO') THEN
	        IF (B_DATA(1).LT.1.0E-36.AND.B_DATA(2).LT.1.0E-36.AND.
	1       B_DATA(3).LT.1.0E-36.AND.B_DATA(4).LT.1.0E-36)
	2       GO=.FALSE.
	      ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	        IF (B_DATA(1).LT.1.0E-36.AND.B_DATA(2).LT.1.0E-36.AND.
	1       B_DATA(3).LT.1.0E-36) GO=.FALSE.
	      ELSE
	        IF (B_DATA(1).LT.1.0E-36.AND.B_DATA(2).LT.1.0E-36.AND.
	1       B_DATA(3).LT.1.0E-36.AND.B_DATA(4).LT.1.0E-36.AND.
	2       B_DATA(5).LT.1.0E-36) GO=.FALSE.
	      END IF
	      IF (FILETYPE(4:6).EQ.'RHO') THEN
	        IF (GO) WRITE (22,3300) (B_DATA(I),I=1,4)
	      ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	        IF (GO) WRITE (22,3100) (B_DATA(I),I=1,3)
	      ELSE
	        IF (GO) WRITE (22,3000) (B_DATA(I),I=1,5)
	      END IF
	    END DO
	  END IF
C
	ELSE IF (COMMAND.EQ.'M') THEN  ! Multiplies the data
C
	  DO I=1,NUMCHOICES
	    NUMBER(I)=1.0
	  END DO
	  CALL USRINP(NUMBER,NUMCHOICES,'F',4,1,'NUMBERS=',
	1 'Enter the numbers to be multiplied:  (defaults = 1.0)')
	  DO WHILE (.TRUE.)
	    IF (FILETYPE.EQ.'X_Y'.OR.FILETYPE.EQ.'THETA_RHO') THEN
	      READ(21,3000,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX)
	    ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	      READ(21,3100,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_VEL)
	    ELSE IF (FILETYPE(4:6).EQ.'RHO') THEN
	      READ(21,3300,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_INTER)
	    END IF
	    DO II=1,NUMCHOICES
	      IF (ARRAY(ICHOICE(II)).NE.9999.99999D0) !  Key for zero weight
	1     ARRAY(ICHOICE(II))=ARRAY(ICHOICE(II))*NUMBER(II)
	    END DO
	    IF (FILETYPE.EQ.'X_Y'.OR.FILETYPE.EQ.'THETA_RHO') THEN
	      WRITE(22,3000) EPOCH,(ARRAY(I),I=1,NCHOICESMAX)
	    ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	      WRITE(22,3100) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_VEL)
	    ELSE IF (FILETYPE(4:6).EQ.'RHO') THEN
	      WRITE(22,3300) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_INTER)
	    END IF
	  END DO
C
	ELSE IF (COMMAND.EQ.'P') THEN !  Precesses the coordinates
	  TEMP=1.0/15.0/3600.0
	  CALL MESSAGE('The initial equinox is assumed to be the one in'//
	1 ' the file')
	  EQUINOX1=EQUATOR
	  WRITE (6,4000) EQUINOX1
4000	  FORMAT(' ','The initial equinox is ',F11.5)
	  WRITE (7,4000) EQUINOX1
	  CALL USRINP(EQUINOX2,1,'F',8,1,'FINAL_EQUINOX=',
	1 'Enter the new equinox:')
	  RIGHTASC_PRIMARY1=RIGHTASC
	  DECLINA_PRIMARY1=DECLINA
C  Get the precessed coordinates of the primary
	  CALL PRECESSIO(0,EQUINOX1,EQUINOX2,RIGHTASC_PRIMARY1,
	1 DECLINA_PRIMARY1,RIGHTASC_PRIMARY2,DECLINA_PRIMARY2)
C  Write the new header
	  CALL WRITE_BINARY_HEADER(22,NAME,RIGHTASC_PRIMARY2,
	1 DECLINA_PRIMARY2,EQUINOX2,TYPE,PARALLAX_PROPER_MOTION,
	2 SCALE,FILETYPE,ELEMENTS,
	3 C_ELEMENTS,MAXNUMPARAM)
	  DO WHILE (.TRUE.)
	    READ(21,3000,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX)
	    IF (FILETYPE.EQ.'X_Y') THEN
C  +X is to the east;  +Y is to the north
	      POS_SEP(2)=DSQRT(ARRAY(1)**2+ARRAY(3)**2)
	      POS_SEP(1)=DATAN2D(ARRAY(1),ARRAY(3))
	      IF (POS_SEP(1).LT.0.0) POS_SEP(1)=POS_SEP(1)+360.0
	    ELSE IF (FILETYPE.EQ.'THETA_RHO') THEN  !  Unlikely scenario
	      POS_SEP(1)=ARRAY(1)
	      POS_SEP(2)=ARRAY(3)
	    END IF
C  Get the declination of the secondary
	    DECLINA_SECONDARY1=DECLINA_PRIMARY1+POS_SEP(2)*
	1   DCOSD(1.0D0*POS_SEP(1))/3600.0
C  Get the right ascension of the secondary
	    RIGHTASC_SECONDARY1=RIGHTASC_PRIMARY1+POS_SEP(2)*
	1   DSIND(1.0D0*POS_SEP(1))*TEMP/
	2   DCOSD((DECLINA_PRIMARY1+DECLINA_SECONDARY1)/2.0) !  In hours
C  Get the precessed coordinates of the secondary
	    CALL PRECESSIO(0,EQUINOX1,EQUINOX2,RIGHTASC_SECONDARY1,
	1   DECLINA_SECONDARY1,RIGHTASC_SECONDARY2,DECLINA_SECONDARY2)
C  If proper motion is the same for both primary and secondary,
C  the separation will not change.
	    DELTA_NORTH=(DECLINA_SECONDARY2-DECLINA_PRIMARY2)*3600.0
	    DELTA_EAST=(RIGHTASC_SECONDARY2-RIGHTASC_PRIMARY2)*
	1   DCOSD((DECLINA_PRIMARY2+DECLINA_SECONDARY2)/2.0)/TEMP
	    SEP=DSQRT(DELTA_NORTH**2+DELTA_EAST**2)
	    POSANG=DATAN2D(DELTA_EAST,DELTA_NORTH)
	    IF (POSANG.LT.0.0) POSANG=POSANG+360.0
	    IF (FILETYPE.EQ.'X_Y') THEN
C	      ARRAY(1)=POS_SEP(2)*DSIND(POSANG) !  X
C	      ARRAY(3)=POS_SEP(2)*DCOSD(POSANG) !  Y
	      ARRAY(1)=DELTA_EAST !  X
	      ARRAY(3)=DELTA_NORTH !  Y
	    ELSE IF (FILETYPE.EQ.'THETA_RHO') THEN
	      ARRAY(1)=POSANG
	      ARRAY(3)=POS_SEP(2) !  Original value is more accurate
C	      ARRAY(3)=SEP
	    END IF
	    WRITE(22,3000) EPOCH,(ARRAY(I),I=1,NCHOICESMAX)
	  END DO
	ELSE IF (COMMAND.EQ.'PM') THEN !  Precesses manually input data
	  TEMP=1.0D0/15.0D0/3600.0D0
	  CALL USRINP(EQUINOX1,1,'F',8,1,'INITIAL_EQUINOX=',
	1 'Enter the initial equinox:')
	  CALL MESSAGE('Enter the right ascension of the primary'//
	1 ' in hours, minutes,')
	  CALL USRINP(RAPARAM,3,'F',4,1,'RIGHT_ASCENSION=',
	1 'and seconds:  (default = last entry)')
	  CALL HMSDH(RAPARAM(1),RAPARAM(2),RAPARAM(3),RIGHTASC_PRIMARY1)
	  CALL MESSAGE('Enter the declination of the primary in'//
	1 ' degrees, minutes,')
	  CALL USRINP(DECPARAM,3,'F',4,1,'DECLINATION=',
	1 'and seconds:  (default = last entry)')
	  CALL HMSDH(DECPARAM(1),DECPARAM(2),DECPARAM(3),
	1 DECLINA_PRIMARY1)
	  CALL MESSAGE('Enter the position angle and separation in'//
	1 ' degrees and')
	  CALL USRINP(POS_SEP,2,'F',8,0,'POSITION_SEPARATION=',
	1 'arc seconds, respectively')
	  CALL USRINP(EQUINOX2,1,'F',8,1,'FINAL_EQUINOX=',
	1 'Enter the final equinox:')
C  Get the declination of the secondary
	  DECLINA_SECONDARY1=DECLINA_PRIMARY1+POS_SEP(2)*
	1 DCOSD(1.0D0*POS_SEP(1))/3600.0
C  Get the right ascension of the secondary
	  RIGHTASC_SECONDARY1=RIGHTASC_PRIMARY1+POS_SEP(2)*
	1 DSIND(1.0D0*POS_SEP(1))*TEMP/
	2 DCOSD((DECLINA_PRIMARY1+DECLINA_SECONDARY1)/2.0)  ! In hours
C  Get the precessed coordinates of the primary
	  CALL PRECESSIO(0,EQUINOX1,EQUINOX2,RIGHTASC_PRIMARY1,
	1 DECLINA_PRIMARY1,RIGHTASC_PRIMARY2,DECLINA_PRIMARY2)
C  Get the precessed coordinates of the secondary
	  CALL PRECESSIO(0,EQUINOX1,EQUINOX2,RIGHTASC_SECONDARY1,
	1 DECLINA_SECONDARY1,RIGHTASC_SECONDARY2,DECLINA_SECONDARY2)
	  DELTA_NORTH=(DECLINA_SECONDARY2-DECLINA_PRIMARY2)*3600.0
	  DELTA_EAST=(RIGHTASC_SECONDARY2-RIGHTASC_PRIMARY2)*
	1 DCOSD((DECLINA_PRIMARY2+DECLINA_SECONDARY2)/2.0)/TEMP
	  SEP=DSQRT(DELTA_NORTH**2+DELTA_EAST**2)
	  POSANG=DATAN2D(DELTA_EAST,DELTA_NORTH)
	  WRITE (6,4200) POSANG
4200	  FORMAT (' ','The position angle is ',F14.5' degrees')
	  WRITE (7,4200) POSANG
	  WRITE (6,4250) SEP
4250	  FORMAT (' ','The separation is',F14.5,' arc seconds')
	  WRITE (7,4250) SEP
	  GO TO 1
C
	ELSE IF (COMMAND.EQ.'Q') THEN !  Quits
C
	  STOP
C
	ELSE IF (COMMAND.EQ.'S') THEN !  Smooths the data
C
	  DO I=1,NGROUP_MAX
	    WEIGHT(I)=1.0
	  END DO
	  I=1
	  DO WHILE (.TRUE.)
	    READ(21,3000,END=999) VAR1(I),VAR2(I),VAR3(I),VAR4(I),VAR5(I)
C  Convert the mean errors to weights
	    IF (VAR3(I).NE.0.0) VAR3(I)=1.0/VAR3(I)**2
	    IF (VAR5(I).NE.0.0) VAR5(I)=1.0/VAR5(I)**2
	    IF (VAR1(I).EQ.0.0.AND.VAR2(I).EQ.0.0.AND.VAR3(I).EQ.0.0
	1   .AND.VAR4(I).EQ.0.0) THEN !  Blank line delimiter detected;
				      !  time to start the calculation.
	      I=I-1
C  Average the epochs using uniform weights	      
	      CALL DPAVERSIGWT(VAR1,WEIGHT,I,1,1,1,I,1,AVG,SIGMA)
	      B_DATA(1)=AVG
C  Average the x positions (or position angles) using the supplied weights
	      CALL DPAVERSIGWT(VAR2,VAR3,I,1,1,1,I,1,AVG,SIGMA)
	      B_DATA(2)=AVG
C  Average the x position (or position angle) mean errors by averaging the 
C  weights.
	      CALL DPAVERSIGWT(VAR3,WEIGHT,I,1,1,1,I,1,AVG,SIGMA)
C  Also reduce the mean errors by the square root of the number of points
C  averaged.
	      B_DATA(3)=1.0/SQRT(I*AVG)
C  Average the y positions (or separations) using the supplied weights
	      CALL DPAVERSIGWT(VAR4,VAR5,I,1,1,1,I,1,AVG,SIGMA)
	      B_DATA(4)=AVG
C  Average the y position (or separation) mean errors by averaging the
C  weights
	      CALL DPAVERSIGWT(VAR5,WEIGHT,I,1,1,1,I,1,AVG,SIGMA)
C  Also reduce the mean errors by the square root of the number of points
C  averaged.
	      B_DATA(5)=1.0/SQRT(I*AVG)
	      I=1
	      WRITE(22,3000) (B_DATA(K),K=1,5)
	    ELSE
	      I=I+1
	    END IF
	  END DO
C
	ELSE IF (COMMAND.EQ.'WS')  THEN !  Converts weights to sigmas
C
	  IF (FILETYPE.EQ.'X_Y') THEN
	    SIGMAS(1)=0.002
	    SIGMAS(2)=0.002
	    CALL MESSAGE('Enter the sigmas in arc seconds for the X and Y'//
	1   ' directions which')
	    CALL USRINP(SIGMAS,2,'F',8,1,'SIGMAS=',
	1   'correspond to a weight of 1: (defaults = 0.002 0.002)')
	  ELSE IF (FILETYPE.EQ.'THETA_RHO') THEN
	    SIGMAS(1)=1.0D0
	    SIGMAS(2)=0.002D0
	    CALL MESSAGE('Enter the sigmas for theta and rho'//
	1   ' in degrees and arc seconds,')
	    CALL USRINP(SIGMAS,2,'F',8,1,'SIGMAS=',
	1   'respectively, which correspond to a weight of 1:'//
	2   '  (defaults = 1 0.002)')
	  ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	    SIGMA_VEL=1.0D0
	    CALL MESSAGE('Enter the sigma in kilometers per second,'//
	1   ' which corresponds to a')
	    CALL USRINP(SIGMA_VEL,1,'F',8,1,'SIGMA_VELOCITY=',
	1   'weight of 1:  (default = 1)')
	  ELSE IF (FILETYPE(4:6).EQ.'RHO') THEN
	    SIGMA_INTER=0.001D0
	    CALL MESSAGE('Enter the sigma in arc seconds which'//
	1   ' corresponds to a weight of 1:')
	    CALL USRINP(SIGMA_INTER,1,'F',8,1,'SIGMA_RHO=',
	1   '(default = 0.001)')
	  END IF
	  DO WHILE (.TRUE.)
	    IF (FILETYPE.EQ.'X_Y'.OR.FILETYPE.EQ.'THETA_RHO') THEN
	      READ(21,3000,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX)
	      IF (ARRAY(2).NE.0.0) THEN
	        ARRAY(2)=SIGMAS(1)/SQRT(ARRAY(2))
	      ELSE
	        ARRAY(2)=9999.99999D0
	      END IF
	      IF (ARRAY(4).NE.0.0) THEN
	        ARRAY(4)=SIGMAS(2)/SQRT(ARRAY(4))
	      ELSE
	        ARRAY(4)=9999.99999D0
	      END IF
	      WRITE(22,3000) EPOCH,(ARRAY(I),I=1,NCHOICESMAX)
	    ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	      READ(21,3100,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_VEL)
	      IF (ARRAY(2).NE.0.0) THEN
	        ARRAY(2)=SIGMA_VEL/SQRT(ARRAY(2))
	      ELSE
	        ARRAY(2)=9999.99999D0
	      END IF
	      WRITE(22,3100) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_VEL)
	    ELSE IF (FILETYPE(4:6).EQ.'RHO') THEN
	      READ(21,3300,END=999) EPOCH,(ARRAY(I),I=1,NCHOICESMAX_INTER)
	      IF (ARRAY(3).NE.0.0) THEN
	        ARRAY(3)=SIGMA_INTER/SQRT(ARRAY(3))
	      ELSE
	        ARRAY(3)=9999.99999D0
	      END IF
	    END IF
	  END DO
	END IF
C
 999	IF (COMMAND.EQ.'A'.OR.COMMAND.EQ.'M'.OR.COMMAND.EQ.'P'
	1.OR.COMMAND.EQ.'S'.OR.COMMAND.EQ.'WS') THEN
	  CLOSE (UNIT=21)
	  CLOSE (UNIT=22)
	ELSE IF (COMMAND.EQ.'CI'.OR.COMMAND.EQ.'CS'.OR.COMMAND.EQ.'CV'
	1.OR.COMMAND.EQ.'EI'.OR.COMMAND.EQ.'ES'.OR.COMMAND.EQ.'EV') THEN
	  CLOSE (UNIT=22)
	END IF
	IF (WRITE_SUB_SEC) THEN
	  DO I=23,22+I_SUB_SEC
	    CLOSE (UNIT=I)
	  END DO
	END IF
	END
