	SUBROUTINE ETDATE_JC_BY(ET,DA,MO,YR,T,BY)
C
C       This subroutine computes the number of Julian centuries since the
C  fundamental epoch, and the epoch in Besselian years, given the ephemeris 
C  time and date.  BC years should be input as  negative numbers.
C			Donald H. Gudehus, April 2, 1990
C			Version 2, D. Gudehus, March 7, 2005
C			Changed DA,MO, and YR to INTEGER
C			Version 3, D. Gudehus, May 16, 2005
C			Revised code based on Lieske, J. H. 1979,
C			Astron & Astrophysics, 73, 282.
C
C	ET	(C)	Ephemeris time
C	DA	(C)	The day
C	MO	(C)	The month expressed as a number
C	YR	(C)	The year in the Gregorian calendar.  Note that
C			in the Gregorian proleptic calendar (dates
C			before its inception on October 15, 1582) the
C			year 0 is regarded as a leap year.
C	T	(R)	The number of Julian centuries of 36525 days each 
C			measured from the fundamental epoch of 1900
C			January 0 at 12 hours E.T.
C	BY	(R)	The epoch in Besselian years
C
	INTEGER*4 DA,MO,YR,JD,DOW
	REAL*8 ET,T,TEMP,BY
C
	CALL DATE_JD(DA,MO,YR,JD,DOW)
	T=(JD+(ET-12.0D0)/24.0D0-2415020.0D0)/36525.0D0
C  The fundamental epoch of the Besselian year is 1900.0 or 1900
C  January 0.81352 E.T.  First compute the number of days from the
C  fundamental epoch.
	TEMP=(JD+(ET-12.0D0)/24.0D0-2415020.0D0-0.31352D0)
C  Convert to years
	BY=TEMP/365.242198781D0+1900.0D0
	RETURN
	END
