	SUBROUTINE FRINGE_ANGLE(EPOCH,LONGI,LATI,RIGHTASC,DECLINA,
	1ALT_TEL,AZIMUTH_TEL,KEY,HOURANGL,ALT,POSANGLE)
C
C	This subroutine computes the position angle on the sky of the fringe
C  direction of a pair of telescopes acting as an interferometer.
C				Donald H. Gudehus, November 14, 2001
C
C	EPOCH	    (C)	The epoch of observation in Besselian years (KEY=1)
C			or Julian Date (KEY=2)
C	LONGI	    (C)	The longitude of the telescopes in decimal degrees
C	LATI	    (C)	The latitude of the telescopes in decimal degrees
C	RIGHTASC    (C)	The right ascension of the object in decimal hours
C	DECLINA	    (C)	The declination of the object in decimal degrees
C	ALT_TEL	    (C)	The altitude of the line connecting the two telescopes
C			in decimal degrees
C	AZIMUTH_TEL (C)	The azimuth of the line connecting the two telescopes
C			in decimal degrees
C	KEY	    (C)	A key such that if equal to 0, the EPOCH is in units
C			of Besselian years, and if equal to 1 it is in units
C			of Julian Days
C	HOURANGL    (R)	The hour angle of the object in decimal hours
C	ALT	    (R)	The altitude of the object in decimal degrees
C	POSANGLE    (R)	The position angle of the fringe direction in decimal
C			degrees.  The position angle of the fringe
C			direction is measured counterclockwise from north,
C			and lies between 0 and 180 degrees.
C
	REAL*8 EPOCH,LONGI,LATI,RIGHTASC,DECLINA,ALT_TEL,AZIMUTH_TEL,
	1HOURANGL,ALT,AZIMUTH,ZEN,POSANGLE
C
	INTEGER*4 JDN
	REAL*8 GST,JDATE,LST,UT,HOURANGL_TEL,RIGHTASC_TEL,DECLINA_TEL,
	1TANPOS1,TANPOS2
C
C  Check the value of KEY
	IF (KEY.EQ.0) THEN
	  CALL BY_JD(EPOCH,JDATE)
	ELSE IF (KEY.EQ.1) THEN
	  JDATE=EPOCH
	ELSE
	  CALL ERROR(-4,'Wrong value of KEY in FRINGE_ANGLE.FOR')
	END IF
C  Get the hour angle of the object first
	JDN=INT(JDATE) !  Julian Day Number
	UT=(JDATE-JDN+0.5D0)*24.0D0
	CALL UTGST(JDN,UT,GST)
	LST=GST-LONGI/15.0D0
	IF (LST.GE.24.0D0) LST=LST-24.0D0
	IF (LST.LT.0.0D0) LST=LST+24.0D0
	HOURANGL=LST-RIGHTASC
	IF (HOURANGL.GT.12.0D0) HOURANGL=HOURANGL+24.0D0
	IF (HOURANGL.LT.-12.0D0) HOURANGL=HOURANGL-24.0D0
C  Get the altitude of the object
	CALL EQHOR(LATI,HOURANGL,DECLINA,AZIMUTH,ALT,ZEN)
C  Get the right ascension and declination of the line connecting the
C  telescopes
	CALL HOREQ(ALT_TEL,AZIMUTH_TEL,LATI,LST,DECLINA_TEL,HOURANGL_TEL,
	1RIGHTASC_TEL)
C
	IF (DECLINA_TEL.NE.90.0D0.AND.DECLINA_TEL.NE.-90.0D0) THEN
	  TANPOS1=DSIND(15.0D0*(RIGHTASC-RIGHTASC_TEL))
	  TANPOS2=DSIND(DECLINA)*DCOSD(15.0D0*(RIGHTASC-RIGHTASC_TEL))-
	1 DCOSD(DECLINA)*DTAND(DECLINA_TEL)
	  IF (TANPOS1.NE.0.0D0.OR.TANPOS2.NE.0.0D0) THEN
	    POSANGLE=DATAN2D(TANPOS1,TANPOS2)
	  ELSE
	    POSANGLE=0.0D0
	  END IF
	ELSE
	  POSANGLE=0.0D0
	END IF
	IF (POSANGLE.LT.0.0D0) POSANGLE=POSANGLE+180.0D0
C
	RETURN
	END
