	SUBROUTINE GET_BINARY_HEADER(UNIT,NAME,RIGHTASC,DECLINA,
	1EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	2C_ELEMENTS,N_ELEMENTS)
C
C	Note:  This is the UNIX version.
C
C	This subroutine reads the header of a binary star file and
C  returns the vital information.  We assume that the file has
C  been rewound if necessary.
C
C			Donald H. Gudehus, January, 1994
C			Version 2, D. Gudehus, April, 1994
C			Incorporated code for spectroscopic binary
C			Version 3, D. Gudehus, May, 1995
C			Put C_ELEMENTS into argument list and added
C			DECODE statements.
C			Version 3U, D. Gudehus, June, 1997 
C			Modified code to work under UNIX
C			Version 4, D. Gudehus, February 23, 2000
C			Added code for ALT_HEAD
C			Version 5, D. Gudehus, September 24, 2001
C			Added code for apsidal motion of subcomponents
C			Version 6, D. Gudehus, January 3, 2003
C			Added new header line:
C			"Relative orbit, Equator is constant"
C			Version 7, D. Gudehus, December, 2002
C			Added code for 1-D interferometric data
C			Version 8, D. Gudehus, February 5, 2004
C			Increased the significance of C_ELEMENTS and 
C			decoded more characters for the elements.
C			Version 9, D. Gudehus, April 6, 2005
C			Added a validity check to FILETYPE
C			Version 10, D. Gudehus, March21, 2008
C			Eliminated control characters, e.g., <CR>, from the
C			header strings.  This caused a problem in the decode
C			statements on Linux when the lines with numerical data
C			didn't have enough significant figures, or didn't have
C			sufficient trailing blanks.  Carriage returns can be 
C			added to a file if for example if the file is sent via
C			the Macintosh Mail utility to a Unix system which 
C			receives the file with pine.  In that case each line
C			not with the usual <LF>, but <CR><LF>.
C			
C
C	UNIT			(C)	The logical unit to be used for reading
C	NAME			(R)	The name of the binary star
C	RIGHTASC		(R)	The right ascension of the binary
C	DECLINA			(R)	The declination of the binary
C	EQUATOR			(R)	The epoch of the equator
C	TYPE			(R)	The type of data, absolute (A) or 
C					relative (R).
C	PARALLAX_PROPER_MOTION	(R)	A flag for whether parallax and
C					proper motion are included (I),
C					or removed (R).
C	SCALE			(R)	The scale, "/unit or km/sec/unit.
C	FILETYPE		(R)	The type of file, X_Y, THETA_RHO,
C					BY_VELOCITY, JD_VELOCITY, BY_RHO,
C					or JD_RHO.
C	ELEMENTS		(R)	An array containing the elements of
C					the orbit or orbits.  Before reading
C					the header, each location is set
C					equal to a test value of -1.0D37.
C					The calling program can then
C					determine if a particular element
C					was available in the header.
C	C_ELEMENTS		(R)	A character array containing the 
C					elements of the orbit or orbits.
C	N_ELEMENTS		(C)	The dimension of C_ELEMENTS and
C					ELEMENTS
C
c	INCLUDE 'binary_parameters.for'
c	INCLUDE 'binary_header.for'
C
	BYTE BSTRING(80)
C
	CHARACTER*1 CSIGN_D,TYPE,PARALLAX_PROPER_MOTION
	CHARACTER*18 C_ELEMENTS(N_ELEMENTS)
	CHARACTER*11 FILETYPE
	CHARACTER*80 NAME,STRING
	EQUIVALENCE (STRING,BSTRING)
C
	INTEGER UNIT,RA_H,RA_M,DEC_D,DEC_M
C
	LOGICAL GO
C
	REAL*8 SCALE,RIGHTASC,DECLINA,EQUATOR,ELEMENTS(N_ELEMENTS)
C
	INCLUDE 'binary_parameters.for'
	INCLUDE 'binary_header.for'
C
C  Initialize the array ELEMENTS
	DO N=1,N_ELEMENTS
	  ELEMENTS(N)=-1.0D37
	END DO
C  Initialize the file type
	FILETYPE='Unknown'
	GO=.TRUE.
	DO WHILE (GO)
	  READ (UNIT,1000,END=999) STRING
C  Remove any control characters to avoid errors in printing on the screen
C  and decoding
	  DO I=1,80
	    IF (BSTRING(I).LT.32) BSTRING(I)=32
	  END DO
1000	  FORMAT(' ',A) 
	  IF (STRING(1:4).EQ.'Name') NAME=STRING(6:80)
	  IF (STRING(1:15).EQ.'Right Ascension') THEN
	    DECODE (2,1002,STRING(17:18)) RA_H
1002	    FORMAT (I2)
	    DECODE (2,1002,STRING(20:21)) RA_M
	    DECODE (8,2000,STRING(23:30)) RA_S
2000	    FORMAT (F)
	    CALL HMSDH(RA_H*1.0,RA_M*1.0,RA_S,RIGHTASC)
	  END IF
	  IF (STRING(1:11).EQ.'Declination') THEN
	    CSIGN_D=STRING(13:13)
	    DECODE (2,1002,STRING(14:15)) DEC_D
	    DECODE (2,1002,STRING(17:18)) DEC_M
	    DECODE (7,2000,STRING(20:26)) DEC_S
	    IF (CSIGN_D.EQ.'-') DEC_D=-1*DEC_D
	    CALL HMSDH(DEC_D*1.0,DEC_M*1.0,DEC_S,DECLINA)
	  END IF
	  IF (STRING(1:7).EQ.'Equator') 
	1 DECODE (11,2000,STRING(9:19)) EQUATOR
	  IF (STRING(1:4).EQ.'Type') THEN
	    IF (STRING(6:19).EQ.'Absolute orbit') TYPE='A'
	    IF (STRING(6:19).EQ.'Relative orbit') THEN
	      TYPE='R'
	      IF (STRING(22:40).EQ.'Equator is constant') TYPE='r'
	    END IF
	  END IF
	  IF (STRING(1:26).EQ.'Parallax and proper motion') THEN
	    IF (STRING(28:35).EQ.'Included') PARALLAX_PROPER_MOTION='I'
	    IF (STRING(28:34).EQ.'Removed') PARALLAX_PROPER_MOTION='R'
	  END IF
	  IF (STRING(1:5).EQ.'Scale') THEN
	    N=INDEX(STRING,'"') !  Visual binary
	    IF (N.EQ.0) N=INDEX(STRING,'k') !  Spectroscopic binary
	    N=N-8
	    DECODE (N,2000,STRING(7:6+N)) SCALE
	  END IF
	  IF (STRING(1:7).EQ.ALT_HEAD(1)) THEN !  Old period without units
	    DECODE (11,2000,STRING(8:18)) ELEMENTS(1)
	    C_ELEMENTS(1)=STRING(8:18)
	  END IF
	  IF (STRING(1:15).EQ.HEAD(1)) THEN
	    DECODE (11,2000,STRING(16:26)) ELEMENTS(1)
	    C_ELEMENTS(1)=STRING(16:26)
	  END IF
	  IF (STRING(1:27).EQ.HEAD(2)) THEN
	    DECODE (11,2000,STRING(28:38)) ELEMENTS(2)
	    C_ELEMENTS(2)=STRING(28:38)
	  END IF
	  IF (STRING(1:13).EQ.HEAD(3)) THEN
	    DECODE (11,2000,STRING(14:24)) ELEMENTS(3)
	    C_ELEMENTS(3)=STRING(14:24)
	  END IF
	  IF (STRING(1:23).EQ.HEAD(4)) THEN
	    DECODE (11,2000,STRING(24:34)) ELEMENTS(4)
	    C_ELEMENTS(4)=STRING(24:34)
	  END IF
	  IF (STRING(1:12).EQ.HEAD(5)) THEN
	    DECODE (11,2000,STRING(13:23)) ELEMENTS(5)
	    C_ELEMENTS(5)=STRING(13:23)
	  END IF
	  IF (STRING(1:30).EQ.HEAD(6)) THEN
	    DECODE (11,2000,STRING(31:41)) ELEMENTS(6)
	    C_ELEMENTS(6)=STRING(31:41)
	  END IF
	  IF (STRING(1:20).EQ.HEAD(7)) THEN
	    DECODE (11,2000,STRING(21:31)) ELEMENTS(7)
	    C_ELEMENTS(7)=STRING(21:31)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(8)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(8)
	    C_ELEMENTS(8)=STRING(44:54)
	  END IF
	  IF (STRING(1:9).EQ.HEAD(9)) THEN
	    DECODE (11,2000,STRING(10:20)) ELEMENTS(9)
	    C_ELEMENTS(9)=STRING(10:20)
	  END IF
	  IF (STRING(1:9).EQ.HEAD(10)) THEN
	    DECODE (11,2000,STRING(10:20)) ELEMENTS(10)
	    C_ELEMENTS(10)=STRING(10:20)
	  END IF
	  IF (STRING(1:16).EQ.HEAD(11)) THEN
	    DECODE (11,2000,STRING(17:27)) ELEMENTS(11)
	    C_ELEMENTS(11)=STRING(17:27)
	  END IF
	  IF (STRING(1:29).EQ.HEAD(12)) THEN
	    DECODE (11,2000,STRING(30:40)) ELEMENTS(12)
	    C_ELEMENTS(12)=STRING(30:40)
	  END IF
	  IF (STRING(1:9).EQ.HEAD(13)) THEN
	    DECODE (11,2000,STRING(10:20)) ELEMENTS(13)
	    C_ELEMENTS(13)=STRING(10:20)
	  END IF
	  IF (STRING(1:16).EQ.HEAD(14)) THEN
	    DECODE (11,2000,STRING(17:27)) ELEMENTS(14)
	    C_ELEMENTS(14)=STRING(17:27)
	  END IF
	  IF (STRING(1:29).EQ.HEAD(15)) THEN
	    DECODE (11,2000,STRING(30:40)) ELEMENTS(15)
	    C_ELEMENTS(15)=STRING(30:40)
	  END IF
	  IF (STRING(1:23).EQ.HEAD(16)) THEN
	    DECODE (11,2000,STRING(24:34)) ELEMENTS(16)
	    C_ELEMENTS(16)=STRING(24:34)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(17)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(17)
	    C_ELEMENTS(17)=STRING(48:58)
	  END IF
	  IF (STRING(1:34).EQ.HEAD(18)) THEN
	    DECODE (11,2000,STRING(35:45)) ELEMENTS(18)
	    C_ELEMENTS(18)=STRING(35:45)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(19)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(19)
	    C_ELEMENTS(19)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(20)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(20)
	    C_ELEMENTS(20)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(21)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(21)
	    C_ELEMENTS(21)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(22)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(22)
	    C_ELEMENTS(22)=STRING(45:55)
	  END IF
	  IF (STRING(1:34).EQ.HEAD(23)) THEN
	    DECODE (11,2000,STRING(35:45)) ELEMENTS(23)
	    C_ELEMENTS(23)=STRING(35:45)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(24)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(24)
	    C_ELEMENTS(24)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(25)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(25)
	    C_ELEMENTS(25)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(26)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(26)
	    C_ELEMENTS(26)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(27)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(27)
	    C_ELEMENTS(27)=STRING(45:55)
	  END IF
	  IF (STRING(1:34).EQ.HEAD(28)) THEN
	    DECODE (11,2000,STRING(35:45)) ELEMENTS(28)
	    C_ELEMENTS(28)=STRING(35:45)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(29)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(29)
	    C_ELEMENTS(29)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(30)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(30)
	    C_ELEMENTS(30)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(31)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(31)
	    C_ELEMENTS(31)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(32)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(32)
	    C_ELEMENTS(32)=STRING(45:55)
	  END IF
	  IF (STRING(1:34).EQ.HEAD(33)) THEN
	    DECODE (11,2000,STRING(35:45)) ELEMENTS(33)
	    C_ELEMENTS(33)=STRING(35:45)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(34)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(34)
	    C_ELEMENTS(34)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(35)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(35)
	    C_ELEMENTS(35)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(36)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(36)
	    C_ELEMENTS(36)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(37)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(37)
	    C_ELEMENTS(37)=STRING(45:55)
	  END IF
	  IF (STRING(1:34).EQ.HEAD(38)) THEN
	    DECODE (11,2000,STRING(35:45)) ELEMENTS(38)
	    C_ELEMENTS(38)=STRING(35:45)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(39)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(39)
	    C_ELEMENTS(39)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(40)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(40)
	    C_ELEMENTS(40)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(41)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(41)
	    C_ELEMENTS(41)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(42)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(42)
	    C_ELEMENTS(42)=STRING(45:55)
	  END IF
	  IF (STRING(1:36).EQ.HEAD(43)) THEN
	    DECODE (11,2000,STRING(37:47)) ELEMENTS(43)
	    C_ELEMENTS(43)=STRING(37:47)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(44)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(44)
	    C_ELEMENTS(44)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(45)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(45)
	    C_ELEMENTS(45)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(46)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(46)
	    C_ELEMENTS(46)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(47)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(47)
	    C_ELEMENTS(47)=STRING(45:55)
	  END IF
	  IF (STRING(1:36).EQ.HEAD(48)) THEN
	    DECODE (11,2000,STRING(37:47)) ELEMENTS(48)
	    C_ELEMENTS(48)=STRING(37:47)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(49)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(49)
	    C_ELEMENTS(49)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(50)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(50)
	    C_ELEMENTS(50)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(51)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(51)
	    C_ELEMENTS(51)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(52)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(52)
	    C_ELEMENTS(52)=STRING(45:55)
	  END IF
	  IF (STRING(1:36).EQ.HEAD(53)) THEN
	    DECODE (11,2000,STRING(37:47)) ELEMENTS(53)
	    C_ELEMENTS(53)=STRING(37:47)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(54)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(54)
	    C_ELEMENTS(54)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(55)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(55)
	    C_ELEMENTS(55)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(56)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(56)
	    C_ELEMENTS(56)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(57)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(57)
	    C_ELEMENTS(57)=STRING(45:55)
	  END IF
	  IF (STRING(1:36).EQ.HEAD(58)) THEN
	    DECODE (11,2000,STRING(37:47)) ELEMENTS(58)
	    C_ELEMENTS(58)=STRING(37:47)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(59)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(59)
	    C_ELEMENTS(59)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(60)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(60)
	    C_ELEMENTS(60)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(61)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(61)
	    C_ELEMENTS(61)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(62)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(62)
	    C_ELEMENTS(62)=STRING(45:55)
	  END IF
	  IF (STRING(1:36).EQ.HEAD(63)) THEN
	    DECODE (11,2000,STRING(37:47)) ELEMENTS(63)
	    C_ELEMENTS(63)=STRING(37:47)
	  END IF
	  IF (STRING(1:47).EQ.HEAD(64)) THEN
	    DECODE (11,2000,STRING(48:58)) ELEMENTS(64)
	    C_ELEMENTS(64)=STRING(48:58)
	  END IF
	  IF (STRING(1:33).EQ.HEAD(65)) THEN
	    DECODE (11,2000,STRING(34:44)) ELEMENTS(65)
	    C_ELEMENTS(65)=STRING(34:44)
	  END IF
	  IF (STRING(1:37).EQ.HEAD(66)) THEN
	    DECODE (11,2000,STRING(38:48)) ELEMENTS(66)
	    C_ELEMENTS(66)=STRING(38:48)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(67)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(67)
	    C_ELEMENTS(67)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(68)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(68)
	    C_ELEMENTS(68)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(69)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(69)
	    C_ELEMENTS(69)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(70)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(70)
	    C_ELEMENTS(70)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(71)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(71)
	    C_ELEMENTS(71)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(72)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(72)
	    C_ELEMENTS(72)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(73)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(73)
	    C_ELEMENTS(73)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(74)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(74)
	    C_ELEMENTS(74)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(75)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(75)
	    C_ELEMENTS(75)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(76)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(76)
	    C_ELEMENTS(76)=STRING(45:55)
	  END IF
	  IF (STRING(1:44).EQ.HEAD(77)) THEN
	    DECODE (11,2000,STRING(45:55)) ELEMENTS(77)
	    C_ELEMENTS(77)=STRING(45:55)
	  END IF
	  IF (STRING(1:42).EQ.HEAD(78)) THEN
	    DECODE (11,2000,STRING(43:53)) ELEMENTS(78)
	    C_ELEMENTS(78)=STRING(43:53)
	  END IF
	  IF (STRING(1:43).EQ.ALT_HEAD(78)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(78)
	    C_ELEMENTS(78)=STRING(44:54)
	  END IF
C  Apsidal motion of subcomponents
	  IF (STRING(1:43).EQ.HEAD(79)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(79)
	    C_ELEMENTS(79)=STRING(44:54)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(80)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(80)
	    C_ELEMENTS(80)=STRING(44:54)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(81)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(81)
	    C_ELEMENTS(81)=STRING(44:54)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(82)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(82)
	    C_ELEMENTS(82)=STRING(44:54)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(83)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(83)
	    C_ELEMENTS(83)=STRING(44:54)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(84)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(84)
	    C_ELEMENTS(84)=STRING(44:54)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(85)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(85)
	    C_ELEMENTS(85)=STRING(44:54)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(86)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(86)
	    C_ELEMENTS(86)=STRING(44:54)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(87)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(87)
	    C_ELEMENTS(87)=STRING(44:54)
	  END IF
	  IF (STRING(1:43).EQ.HEAD(88)) THEN
	    DECODE (11,2000,STRING(44:54)) ELEMENTS(88)
	    C_ELEMENTS(88)=STRING(44:54)
	  END IF
	  IF (STRING(1:11).EQ.HEAD(89)) THEN
	    DECODE (11,2000,STRING(10:20)) ELEMENTS(89)
	    C_ELEMENTS(89)=STRING(10:20)
	  END IF
	  IF (STRING(1:11).EQ.HEAD(90)) THEN
	    DECODE (11,2000,STRING(10:20)) ELEMENTS(90)
	    C_ELEMENTS(90)=STRING(10:20)
	  END IF
C
	  IF (STRING.EQ.     '   Epoch                 X'//
	1'             Sigma_X            Y            Sigma_Y') THEN
	    FILETYPE='X_Y'
	    GO=.FALSE.
	  ELSE IF (STRING.EQ.'   Epoch               Theta'//
	1'         Sigma_Theta         Rho          Sigma_Rho') THEN
	    FILETYPE='THETA_RHO'
	    GO=.FALSE.
	  ELSE IF (STRING.EQ.'    Epoch(BY)           v(radial)      '//
	1'   Sigma_v') THEN
	    FILETYPE='BY_VELOCITY'
	    GO=.FALSE.
	  ELSE IF (STRING.EQ.'    Epoch(JD)           v(radial)      '//
	1'   Sigma_v') THEN
	    FILETYPE='JD_VELOCITY'
	    GO=.FALSE.
	  ELSE IF (STRING.EQ.'    Epoch(BY)       Theta(interfer.)'//
	1'  Rho(projected)    Sigma_Rho') THEN
	    FILETYPE='BY_RHO'
	    GO=.FALSE.
	  ELSE IF (STRING.EQ.'    Epoch(JD)       Theta(interfer.)'//
	1'  Rho(projected)    Sigma_Rho') THEN
	    FILETYPE='JD_RHO'
	    GO=.FALSE.
	  END IF
	END DO
 999	IF (FILETYPE.EQ.'Unknown')
	1CALL ERROR(-4,'Unrecognized Data Header')
	RETURN
	END
