	SUBROUTINE GET_ECLIPSE_EPOCHS(PARAMETRS,ECLIPSE_EPOCHS)
C
C  Note:  This is the Unix version
C
C       This subroutine calculates the epochs when there is a possibility
C  of eclipse in a spectroscopic binary.  We assume i=+-90 degrees exactly,
C  do not include a light travel time correction, and consider two cases:
C  1) primary on the near side of the orbit;  w+v=270 for i=90 or w+v=90
C  for i=-90, and 2) primary on the far side of the orbit;  w+v=90 for i=90
C  or w+v=270 for i=-90.
C
C			Version 1, Donald H. Gudehus, May 18, 2005
C			following a suggestion by Keivan Stasuun
C			Version 2, D. H. Gudehus, October 24, 2005
C			Allowed for negative a*sin(i)
C	
	INCLUDE 'binary_parameters.for'
C
	REAL*8 PARAMETRS(MAXNUMPARAM),ECLIPSE_EPOCHS(2)
	REAL*8 PI,P,T,ECCENTR,v,w,MU,FRAC,E,M,asini
C
	INTEGER SEMIMAJOR_AXIS_ASSIGNMENT(2),DIMENSN_DATA_DUM,
	1DIMENSN_DATA_USED,INDX(MAXNUMPARAM)
	COMMON /BINARY_INTEGER4/ SEMIMAJOR_AXIS_ASSIGNMENT,
	1DIMENSN_DATA_DUM,DIMENSN_DATA_USED,INDX
C
C  P       Period
C  T       Time of periastron passage
C  ECCENTR Eccentricity
C  w       Angle from node to periastron
C  v       True anomaly
C  E       Eccentric anomaly
C  M       Mean anomaly
C  asini   Semimajor axis times sine of inclination

	PI=3.14159265358979D0
C
	P=PARAMETRS(INDX(1))
	T=PARAMETRS(INDX(2))
	ECCENTR=PARAMETRS(INDX(3)) !  No check here for e < 0
	w=PARAMETRS(INDX(6))
	MU=2.0D0*PI/P !  Rate of orbital revolution in radians per year
	asini=PARAMETRS(INDX(17))
C
	DO I=1,2
	  IF (I.EQ.1) THEN
C  First we consider the primary on the near side of the orbit
	    IF (asini.GT.0.0D0) THEN
	      v=270.0D0-w
	    ELSE
	      v=90.0D0-w
	    END IF
	  ELSE IF (I.EQ.2) THEN
C  Now we do the primary on the far side of the orbit
	    IF (asini.GT.0.0D0) THEN
	      v=90.0D0-w
	    ELSE
	      v=270.0D0-w
	    END IF
	  END IF
	  FRAC=SQRT(ABS((1.0D0+ECCENTR)/(1.0D0-ECCENTR)))
	  E=2.0D0*DATAN(DTAND(v/2.0D0)/FRAC)
	  M=E-ECCENTR*DSIN(E)
	  ECLIPSE_EPOCHS(I)=M/MU+T
	END DO
C
	END
