	SUBROUTINE JD_SUN(JDATE,LONGI,LATI,RIGHTASC,DECLINA,ALT,
	1AZIMUTH,HA)
C
C       This subroutine calculates the right ascension, declination,
C  altitude, azimuth, and hour angle of the sun, given the Julian Date,
C  the latitude, and longitude.
C				Donald H. Gudehus, November 15, 2001
C			Version 2, D. Gudehus, March 7, 2005
C			Made JDN,DA,MO, and YR INTEGER
C
C	JDATE	 (C)	The Julian Date
C	LONGI	 (C)	The longitude in decimal degrees
C	LATI     (C)    The latitude in decimal degrees
C	RIGHTASC (R)	The right ascension of the sun in decimal hours
C	DECLINA	 (R)	The declination of the sun in decimal hours
C	ALT	 (R)	The altitude of the sun in decimal degrees
C	AZIMUTH	 (R)	The azimuth of the sun in decimal degrees
C	HA	 (R)	The hour angle of the sun in decimal hours
C
	REAL*8 JDATE,LATI,LONGI,RIGHTASC,DECLINA,ALT,AZIMUTH,HA
C
	INTEGER JDN,DA,MO,YR,DOW
	REAL*8 UT,GST,LST,ET,A_E,ECCENTR_E,ARGPERI_E,MEANANOM_E,OBLIQ,
	1ECCENTRANOM_E,TRUEANOM_E,RADIUS_E,INCLINA_E,NODEANGLE_E,
	2x_e,y_e,z_e,x_ec,y_ec,z_ec,Xi,Eta,Zeta,Delta,ZEN
C
	JDN=INT(JDATE)
	UT=(JDATE-JDN+0.5D0)*24.0D0
	ET=UT !  Approximate
	CALL JD_DATE(JDN,DA,MO,YR,DOW)
C
	CALL EARTH(ET,DA,MO,YR,A_E,ECCENTR_E,ARGPERI_E,MEANANOM_E,
	1OBLIQ,ECCENTRANOM_E,TRUEANOM_E,RADIUS_E)
C
	INCLINA_E=0.0D0
	NODEANGLE_E=0.0D0
	CALL XYZECEQ(RADIUS_E,TRUEANOM_E,ARGPERI_E,NODEANGLE_E,INCLINA_E,
	1OBLIQ,x_ec,y_ec,z_ec,x_e,y_e,z_e)
C
	Xi=-x_e
	Eta=-y_e
	Zeta=-z_e
	Delta=DSQRT(Xi**2+Eta**2+Zeta**2)
	Xi=Xi/Delta
	Eta=Eta/Delta
	Zeta=Zeta/Delta
	RIGHTASC=DATAN2D(Eta,Xi)/15.0D0
	IF (RIGHTASC.LT.0.0D0) RIGHTASC=RIGHTASC+24.0D0
	DECLINA=DASIND(Zeta)
C
	CALL UTGST(JDN,UT,GST)
	LST=GST-LONGI/15.0D0
	IF (LST.GT.24.0D0) LST=LST-24.0D0
	IF (LST.LT.0.0D0) LST=LST+24.0D0
	HA=LST-RIGHTASC
	IF (HA.GT.12.0D0) HA=HA-24.0D0
	IF (HA.LT.-12.0D0) HA=HA+24.0D0
C
	CALL EQHOR(LATI,HA,DECLINA,AZIMUTH,ALT,ZEN)
C
	RETURN
	END
