	PROGRAM NL_DEMO_1
C
C	Note:  This is the UNIX version.
C
C       This is a demonstration of the usage of NONLINFIT to fit
C  either a constant, a straight line, or two intersecting straight lines
C  to a set of x, and y values.  The fit by a constant could of course 
C  be done by averaging;  the fit by a single straight line could be done 
C  by linear least squares;  but two intersecting straight lines requires 
C  nonlinear least squares fitting.  
C					Donald H. Gudehus, April, 1986
C			Version 2, D. Gudehus, September, 1992
C			Added new features.
C			Version 3, D. Gudehus, April, 1994
C			Incorporated new version of NONLINFIT
C			Version 3U, D. H. G., May, 1997
C			Modified call to USRINP for UNIX.
C			Version 3UM, D. H. G., Mar. 29, 2005
C			Modified code for Absoft compiler under Mac OS
C
	INTEGER DIMENSN_IND,DIMENSN_DATA
	PARAMETER (MAXNUMPARAM=4,MAXNPOINTS=104,DIMENSN_IND=1,
	1DIMENSN_DATA=1,MAXNPLOTS=1)
	INTEGER FUNCAT,IX(2)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,OPERATE
	REAL*8 X(MAXNPOINTS,DIMENSN_IND),DATA(MAXNPOINTS,DIMENSN_DATA),
	1WEIGHT(MAXNPOINTS,DIMENSN_DATA),
	1PARAMETRS(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	2MEANERROR(DIMENSN_DATA),MEANERRORMEANERROR(DIMENSN_DATA),
	3REDCHISQ(DIMENSN_DATA),AVREDCHISQ,RESIDUALS(MAXNPOINTS,DIMENSN_DATA),
	4MPRIME
C
	REAL XPLOT(MAXNPOINTS,MAXNPLOTS),DATPLOT(MAXNPOINTS,MAXNPLOTS)
C
	CHARACTER*1 ITERATION_SCHEME
	CHARACTER*6 CHOICE
C  Enter user declarations below this line
	LOGICAL ADD_NOISE
	REAL NOISE,SUM
	REAL*8 TEMPVAL,TEMPVAL2
C
C  Make carriage control as in OpenVMS
C	OPEN(UNIT=6,STATUS='NEW',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=7,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=49,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
C
C  Create some artificial data.  These data correspond to the run of cluster
C  galaxy Hubble log radius parameters with absolute V magnitude.  These
C  data are described by two intersecting lines of slopes -0.4 and -0.167
C  intersecting at X=-23.04 (V magnitude),Y=2.97 (log r).
	DO I=1,50  ! Bright range
	  X(I,1)=-25.0+0.04*(I-1)  ! Magnitude
	  DATA(I,1)=2.97-0.4*(X(I,1)+23.04)  ! log radius parameter
	  WEIGHT(I,1)=1.0D0
	END DO
	DO I=51,100  ! Faint range
	  X(I,1)=-23.04+0.04*(I-50-1)  ! Magnitude
	  DATA(I,1)=2.97-0.167*(X(I,1)+23.04)  ! log radius parameter
	  WEIGHT(I,1)=1.0D0
	END DO
	FUNCAT=-1  ! First demo
	IX(1)=1
	IX(2)=100
	CHOICE=' '
	DO WHILE (CHOICE.NE.'K'.AND.CHOICE.NE.'L'.AND.CHOICE.NE.'T')
	  CALL MESSAGE('Enter a choice:')
	  CALL MESSAGE('                 K = Constant')
	  CALL MESSAGE('                 L = Single straight line')
	  CALL MESSAGE('                 T = Two intersecting'//
	1 ' straight lines (default)')
	  CHOICE='T'
	  CALL USRINP_CHAR(1,'A',6,1,'CHOICE=',' ',%REF(CHOICE))
	  CALL ALPHA(%REF(CHOICE),1,6,NC,NF,NR,2)
	END DO
	ADD_NOISE=.TRUE.
	CALL USRINP(ADD_NOISE,1,'L',4,1,'ADD_NOISE=',
	1'Noise can be added to the data.  Would you like to add'//
	2' noise? (default = Y)')
	IF (ADD_NOISE) THEN
	  NOISE=0.15
	  CALL USRINP(NOISE,1,'F',4,1,'NOISE=',
	1 'Enter the noise (sigma) in the Y direction:  (default = 0.15)')
	  IF (NOISE.NE.0.0) THEN
	    ISEED=12345
	    DO I=1,100
	      SUM=0.0
	      DO K=1,12
	        SUM=SUM+RAN(ISEED)
	      END DO
	      DATA(I,1)=DATA(I,1)+NOISE*(SUM-6.0)
	    END DO
	  END IF
	END IF
C  Revise the weights if noise was added.  Now the reduced chi squared
C  values will make sense.
	IF (ADD_NOISE) THEN
	  DO I=1,100
	    WEIGHT(I,1)=1.0/NOISE**2
	  END DO
	END IF
	IF (CHOICE.EQ.'K') THEN
	  NUMPARAM=1
	  PARAMETRS(1)=3.0
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,1,'PARAMETERS=',
	1 'Enter the initial estimate of the average:  (default = 3)')
	ELSE IF (CHOICE.EQ.'L') THEN
	  NUMPARAM=2
	  PARAMETRS(1)=-3.0
	  PARAMETRS(2)=-0.3
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,3,'PARAMETERS=',
	1 'Enter the initial estimates of the constant and slope:'//
	2 '  (default = -3.0 -0.3')
	ELSE IF (CHOICE.EQ.'T') THEN
	  NUMPARAM=4
	  CALL MESSAGE('Enter the initial estimates of the intersection'//
	1 ' point and the slopes')
	  PARAMETRS(1)=-23.0D0  ! V
	  PARAMETRS(2)=3.0D0  ! log r
	  PARAMETRS(3)=-0.4D0  ! Bright slope
	  PARAMETRS(4)=-0.17D0  ! Faint slope
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,1,'INTERSECTION&SLOPES=',
	1 'for the bright and faint ends:  (default = -23.0 3.0 -0.4 -0.17')
	  TEMPVAL=PARAMETRS(1) 
	  TEMPVAL2=PARAMETRS(2)
	  PARAMETRS(1)=TEMPVAL2+TEMPVAL*(PARAMETRS(4)-PARAMETRS(3))/
	1 (1.0-PARAMETRS(4)/PARAMETRS(3))  ! Bright constant
	  PARAMETRS(2)=PARAMETRS(3)  ! Bright slope
	  PARAMETRS(3)=TEMPVAL2+TEMPVAL*(PARAMETRS(4)-PARAMETRS(3))/
	1 (PARAMETRS(3)/PARAMETRS(4)-1.0)  ! Faint constant
	  PARAMETRS(4)=PARAMETRS(4)  ! Faint slope
	  CALL MESSAGE('However, the parameters which we will use in the'//
	1 ' solution are the')
	  CALL MESSAGE('constants and slopes for the bright and faint ends')
	END IF
	NPLPOINTS=100
	ITERATION_SCHEME=' '  ! User is prompted
	TOLERANCE=0.0
	INTERMEDIATE=.TRUE.  ! Shows intermediate results
	OPERATE=.FALSE.  ! No user defined operation
	CALL NONLINFIT(X,MAXNPOINTS,DIMENSN_IND,DATA,DIMENSN_DATA,WEIGHT,IX,
	1FUNCAT,CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	2ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,MEANERROR,
	3MEANERRORMEANERROR,REDCHISQ,AVREDCHISQ,XPLOT,DATPLOT,
	4NPLPOINTS,RESIDUALS)
	WRITE (6,1000) MEANERROR(1)
1000	FORMAT(' Mean error = ',G15.6)
	WRITE (7,1000)
	IF (CHOICE.EQ.'K') THEN
	  WRITE (6,1100) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,1)
1100	  FORMAT(' Average = ',G15.6,'  +-',G15.6)
	  WRITE (7,1100) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,1)
	ELSE IF (CHOICE.EQ.'L') THEN
	  WRITE (6,1200) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,2)
1200	  FORMAT(' Constant = ',G15.6,'  +-',G15.6/
	1 ' Slope = ',G15.6,'  +-',G15.6)
	  WRITE (7,1200) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,2)
	ELSE IF (CHOICE.EQ.'T') THEN
	  WRITE (6,1200) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,4)
1300	  FORMAT(' Constant #1 = ',G15.6,'  +-',G15.6/
	1 ' Slope #1 = ',G15.6,'  +-',G15.6/
	2 ' Constant #2 = ',G15.6,'  +-',G15.6/
	1 ' Slope #2 = ',G15.6,'  +-',G15.6)
	  WRITE (7,1200) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,4)
	  MPRIME=(PARAMETRS(3)-PARAMETRS(1))/
	1 (PARAMETRS(2)-PARAMETRS(4))
	  WRITE (6,1350) MPRIME
1350	  FORMAT(' The intersection point = ',G15.6)
	  WRITE (7,1350)
	END IF
	DO J=1,NUMPARAM
	  DO I=1,J
	    K=K+1
	      WRITE (6,1400) I,J,COVARIANCE(I,J)
	      WRITE (7,1400) I,J,COVARIANCE(I,J)
1400	      FORMAT(' Covariance(',I1,',',I1,') = ',G20.8)
	  END DO
	END DO
C
	END
