	PROGRAM NL_DEMO_2
C
C  Note:  This is the Unix version
C
C       This program is an example of the useage of NONLINFIT to solve
C  for the equation of an ellipse.  The X and Y points on the ellipse 
C  are assumed to be parametrically related to an angle measured from the
C  center of the ellipse.  The angle constitutes the independent dimension.
C
C			Donald H. Gudehus, April, 1986
C			Version 2, D. Gudehus, April, 1994
C			Incorported new version of NONLINFIT
C			Version 3, D. Gudehus, September 17, 1999
C			Minor fixing up.
C			Version 3U, D. Gudehus, September, 1999
C			Modified code for Unix, and commented
C			out calls to XYPLOT.
C			Version 4, D. Gudehus, April 16, 2001
C			Minor fixing up.
C			Version 4UM, D. Gudehus, Mar. 29, 2005
C			Modified code for Abosft compiler under Mac OS
C
	INTEGER DIMENSN_IND,DIMENSN_DATA
	PARAMETER (MAXNUMPARAM=5,MAXNPOINTS=100,DIMENSN_IND=1,DIMENSN_DATA=2)
	PARAMETER (MAXNPLOTPOINTS=104,MAXNPLOTS=3) !  For XYPLOT.  Here, the
	!  array dimension should be 4 larger than the number of points
	INTEGER COLOR(2)
	INTEGER FUNCAT,IX(2),NPLPOINTS,NPOINTS_XYPLOT(MAXNPLOTS),
	1LINVECT(2),INC(2),LINTYPE(2),NSYMBOL(2)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,OPERATE,
	1INVERT(2),INTERACT,CROSSHAIRS,ADDNOISE,PLOTD
	REAL XPLOT(MAXNPOINTS,DIMENSN_IND),DATPLOT(MAXNPOINTS,DIMENSN_DATA),
	1XARRAY(MAXNPLOTPOINTS,MAXNPLOTS),YARRAY(MAXNPLOTPOINTS,MAXNPLOTS)
	REAL*8 X(MAXNPOINTS,DIMENSN_IND),DATA(MAXNPOINTS,DIMENSN_DATA),
	1WEIGHT(MAXNPOINTS,DIMENSN_DATA),
	1PARAMETRS(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	2MEANERROR(DIMENSN_DATA),MEANERRORMEANERROR(DIMENSN_DATA),
	3REDCHISQ(DIMENSN_DATA),AVREDCHISQ,RESIDUALS(MAXNPOINTS,DIMENSN_DATA),
	4PI,THETA,e,F,CS,SN,A,SUM,NOISE
C
	CHARACTER*1 ITERATION_SCHEME
	CHARACTER*6 CHOICE
	CHARACTER*50 XLABEL,YLABEL,TITLE
C
C  Make carriage control as in OpenVMS
C	OPEN(UNIT=6,STATUS='NEW',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=7,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=49,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
C
C  Determine what display and plot devices are available
	CALL GET_DP_AVAILABILITY
	FUNCAT=-2
	PI=3.1415926535
	CALL MESSAGE('This program is a demonstration of NONLINFIT in'//
	1' fitting an ellipse')
	CALL MESSAGE(' ')
C  Create some artificial data
	e=0.6D0 !  Eccentricity
	F=SQRT(1.0-e**2) !  Foreshortening
	THETA=45.0D0 !  Inclination
	CS=DCOSD(THETA)
	SN=DSIND(THETA)	
	A=2.0D0
C  The independent variable is angle and the dependent variables are x and y
C  Create some artificial data
	NPOINTS_DATA=30
	DO I=1,NPOINTS_DATA
	  X(I,1)=2.0D0*PI*(I-1)/NPOINTS_DATA !  Angle measured in radians
	  DATA(I,1)=A*DCOS(X(I,1))*CS-A*F*DSIN(X(I,1))*SN !  Position in X
	  DATA(I,2)=A*DCOS(X(I,1))*SN+A*F*DSIN(X(I,1))*CS !  Position in Y
	  WEIGHT(I,1)=1.0D0
	  WEIGHT(I,2)=1.0D0
	END DO
	ADDNOISE=.TRUE.
	CALL USRINP(ADDNOISE,1,'L',4,1,'ADD_NOISE=',
	1'Do you want to add noise to the data?  (default = Y)')
	IF (ADDNOISE) THEN
	  NOISE=0.05
	  CALL USRINP(NOISE,1,'F',8,1,'NOISE=',
	1'Enter the noise:  (default = 0.05)')
	  ISEED=12345
	  CALL USRINP(ISEED,1,'I',4,1,'SEED=',
	1'Enter the seed for the random number generator:  (default = 12345)')
	  DO N=1,NPOINTS_DATA  ! Steps through the number of data points
	    DO K=1,DIMENSN_DATA
	      SUM=0.0
	      DO KK=1,12
	        SUM=SUM+RAN(ISEED)
	      END DO
	      DATA(N,K)=DATA(N,K)+NOISE*(SUM-6.0)
	    END DO
	  END DO
	END IF
	IF (ADDNOISE) THEN
	  DO I=1,NPOINTS_DATA
	    WEIGHT(I,1)=1.0/NOISE**2
	    WEIGHT(I,2)=1.0/NOISE**2
	  END DO
	END IF
C  Fill the plotting array
	NPLPOINTS=100 !  The fitted function
	DO I=1,NPLPOINTS
	  XPLOT(I,1)=2.0*PI*(I-1)/NPLPOINTS
	END DO
	IX(1)=1
	IX(2)=NPOINTS_DATA
	CHOICE='E'
	NUMPARAM=-3 !  Negative value gives prompt for holding some parameters
		    !  constant
	ITERATION_SCHEME=' '
	TOLERANCE=0.0
	INTERMEDIATE=.TRUE.
	OPERATE=.FALSE.
	CALL MESSAGE('Data has parameters 45 0.6 2.0 for inclination'//
	1' (degrees), eccentricity,')
	CALL MESSAGE('and semimajor axis')
	CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',
	1'Enter the initial estimates of the parameters:  (no defaults here)')
	CALL NONLINFIT(X,MAXNPOINTS,DIMENSN_IND,DATA,DIMENSN_DATA,WEIGHT,IX,
	1FUNCAT,CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	2ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,MEANERROR,
	3MEANERRORMEANERROR,REDCHISQ,AVREDCHISQ,XPLOT,DATPLOT,
	4NPLPOINTS,RESIDUALS)
	WRITE (6,1321) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,3)
1321	FORMAT(' Inclination = ',G15.6,'  +-',G15.6/
	1' Eccentricity = ',G15.6,'  +-',G15.6/
	2' Semi-major axis = ',G15.6,'  +-',G15.6)
	WRITE (7,1321) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,3)
	WRITE (6,1350) MEANERROR(1),MEANERRORMEANERROR(1)
1350	FORMAT(' The mean error for the X dimension = ',G15.6,'  +-',G15.6)
	WRITE (7,1350) MEANERROR(1),MEANERRORMEANERROR(1)
	WRITE (6,1351) MEANERROR(2),MEANERRORMEANERROR(2)
1351	FORMAT(' The mean error for the Y dimension = ',G15.6,'  +-',G15.6)
	WRITE (7,1351) MEANERROR(2),MEANERRORMEANERROR(2)
	WRITE (6,1360) REDCHISQ(1)
1360	FORMAT(' The reduced chi squared for the X dimension = ',G15.6)
	WRITE (7,1360) REDCHISQ(1)
	WRITE (6,1361) REDCHISQ(2)
1361	FORMAT(' The reduced chi squared for the Y dimension = ',G15.6)
	WRITE (7,1361) REDCHISQ(2)
	DO J=1,NUMPARAM
	  DO I=1,J
	    K=K+1
	      WRITE (6,1400) I,J,COVARIANCE(I,J)
	      WRITE (7,1400) I,J,COVARIANCE(I,J)
1400	      FORMAT(' Covariance(',I1,',',I1,') = ',G20.8)
	  END DO
	END DO
	PLOTD=.TRUE.
	CALL USRINP(PLOTD,1,'L',4,1,'PLOT=',
	1'Do you wish to plot the function and the data?  (default = Y)')
	IF (PLOTD) THEN
C  Set up the plotting parameters
	  NPLOTS=2
	  LINVECT(1)=0
	  LINVECT(2)=0
	  INC(1)=1
	  INC(2)=1
	  LINTYPE(1)=0
	  LINTYPE(2)=-1
	  NSYMBOL(1)=0
	  NSYMBOL(2)=3 !  + signs
	  INVERT(1)=.FALSE.
	  INVERT(2)=.FALSE.
	  COLOR(1)=4251856 !  Turquoise
	  COLOR(2)=16753920 !  Orange
	  INTERACT=.FALSE.
	  CROSSHAIRS=.TRUE.
	  XLABEL='X AXIS'
	  YLABEL='Y AXIS'
	  TITLE='ELLIPSE'
C  Example of plotting the function and the data
	  NPOINTS_XYPLOT(1)=NPLPOINTS !  The fitted function
	  NPOINTS_XYPLOT(2)=NPOINTS_DATA !  The data
	  DO I=1,NPOINTS_XYPLOT(1)
	    XARRAY(I,1)=DATPLOT(I,1)
	    YARRAY(I,1)=DATPLOT(I,2)
	  END DO
	  DO I=1,NPOINTS_XYPLOT(2)
	    XARRAY(I,2)=DATA(I,1)
	    YARRAY(I,2)=DATA(I,2)
	  END DO
	  CALL XYPLOT(XARRAY,YARRAY,MAXNPLOTPOINTS,
	1 NPLOTS,NPOINTS_XYPLOT,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2 INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	  CALL RESETTEK
	END IF
C
	END
