	PROGRAM ONE2TWO
C
C  Note:  this is the UNIX version
C
C       This program computes the two-dimensional information, position
C  angle and separation (theta and rho), and their mean errors, of a binary
C  star from a set of one-dimensional measurements of projected separation
C  in different directions on the sky.  The time interval covering the
C  measurements should be much less than the orbital period of the star
C  system.  The angle of the direction on the sky increases counterclockwise
C  from north and should be between 0 and 180 degrees.  The derived position
C  angle is likewise measured in the usual sense, counterclockwise from
C  north, and will possess a 180 degree ambiguity.
C       The data should be contained in a file with the standard formats
C  supported by the combined solution binary star program.  Data can be
C  entered by using an existing sample file, e.g., sample_1d.dat as a
C  template, or by using the program editbinary with command EI.
C
C			Donald Gudehus, February 26, 2005
C			Version 2, D. Gudehus, Mar. 1, 2005
C			Added calculation of weighted mean epoch
C			Version 2UM, D. Gudehus, Mar. 28, 2005
C			Modified IOMIT for Absoft compiler on Mac OS
C
	INTEGER DIMENSN_IND_MAX,DIMENSN_DATA_MAX
	PARAMETER (MAXNUMPARAM=90,MAXNPOINTS=10004,
	1DIMENSN_IND_MAX=1,DIMENSN_DATA_MAX=1,MAXNPLOTS=10,
	2MAXHEAD=90)
C
	CHARACTER*1 TYPE,PARALLAX_PROPER_MOTION,ITERATION_SCHEME
	CHARACTER*11 FILETYPE
	CHARACTER*18 C_ELEMENTS(MAXNUMPARAM)
	CHARACTER*6 CHOICE
	CHARACTER*50 XLABEL,YLABEL,TITLE
	CHARACTER*80 NAME
	INTEGER DIMENSN_IND,DIMENSN_DATA,FUNCAT,IX(2),
	1NPLPOINTS(MAXNPLOTS),LINVECT(MAXNPLOTS),INC(MAXNPLOTS),
	2LINTYPE(MAXNPLOTS),NSYMBOL(MAXNPLOTS),COLOR(MAXNPLOTS)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,OPERATE,
	1INVERT(2),INTERACT,CROSSHAIRS,PLOTD,PLOTE,SHOW_RESID,
	2EXCLUDE,REDO_SOLUTION,INITIALIZED
	REAL XPLOT(MAXNPOINTS,MAXNPLOTS),DATPLOT(MAXNPOINTS,MAXNPLOTS)
	REAL*8 FR_ANGLE(MAXNPOINTS,DIMENSN_IND_MAX),TIME(MAXNPOINTS),
	1TIME_SAVE_BY(MAXNPOINTS,1),TIME_SAVE_JD(MAXNPOINTS,1),
	2DATA(MAXNPOINTS,DIMENSN_DATA_MAX),
	2DATA_SAVE(MAXNPOINTS,DIMENSN_DATA_MAX),
	2DATA_TEMP(MAXNPOINTS,2),WEIGHT(MAXNPOINTS,DIMENSN_DATA_MAX),
	3WEIGHT_SAVE(MAXNPOINTS,DIMENSN_DATA_MAX),
	4RESIDUALS(MAXNPOINTS,DIMENSN_DATA_MAX),
	5PARAMETRS(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	7INITIAL_ELEMENTS(MAXNUMPARAM),TEMP1(18),TEMP2(18),
	7MEANERROR(DIMENSN_DATA_MAX),MEANERRORMEANERROR(DIMENSN_DATA_MAX),
	8AVREDCHISQ,REDCHISQ(DIMENSN_DATA_MAX),
	9SIGMA(MAXNPOINTS,DIMENSN_DATA_MAX),SIGMA_SAVE(MAXNPOINTS,4),
	1SUM,RIGHTASC,DECLINA,EQUATOR,SCALE,
	1THRESHOLD_FACTOR,AVERAGE_TIME,SIGMA_TIME

	BYTE BCNTI
	CHARACTER*1 CNTI !  For toggling to Greek characters
	EQUIVALENCE (BCNTI,CNTI)
C
	INTEGER IOMIT
	EQUIVALENCE (IOMIT,OMIT)
	DATA IOMIT /'FF7FFFFF'X/
C
	BCNTI='09'X	
C  Make carriage control as in OpenVMS
C	OPEN(UNIT=6,STATUS='NEW',CARRIAGECONTROL='FORTRAN')
C  Open the log file
	OPEN(UNIT=7,NAME='one2two.log',STATUS='UNKNOWN',
	1CARRIAGECONTROL='FORTRAN')
C
C  Determine what display and plot devices are available
	CALL GET_DP_AVAILABILITY
C
	CALL MESSAGE('Open the file of one-dimensional interferometric '//
	1'data')
	CALL OPENCF(35,3,'FILE=',0)
	REWIND(35)
	CALL GET_BINARY_HEADER(35,NAME,RIGHTASC,DECLINA,
	1EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,
	2FILETYPE,INITIAL_ELEMENTS,C_ELEMENTS,MAXNUMPARAM)
	CALL PRINT_BINARY_HEADER(NAME,RIGHTASC,DECLINA,
	1EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,
	2INITIAL_ELEMENTS,C_ELEMENTS,MAXNUMPARAM)
C
	SUM=0.0D0
	NPOINTS_DATA=0
	N=NPOINTS_DATA+1
	DO WHILE (.TRUE.)
	  READ (35,1000,END=997) TIME(N),FR_ANGLE(N,1),
	1 DATA(N,1),SIGMA(N,1)
1000	  FORMAT(' ',F15.6,4X,F14.5,2X,F14.6,2X,F14.6)
	  IF (SIGMA(N,1).NE.9999.99999D0) THEN
	    IF (FILETYPE.EQ.'BY_RHO') THEN
	      CALL BY_JD(TIME(N),TIME_SAVE_JD(N,1))
C  To convert from Besselian years to Julian years of emphemeris time,
C  uncomment the following two lines.
C	      CALL BESSYR_JULCENT(TIME(N),TIME(N))
C	      TIME(N)=TIME(N)*100+1900.0
	    ELSE IF (FILETYPE.EQ.'JD_RHO') THEN
C  Convert from Julian Days to Besselian years
	      TIME_SAVE_JD(N,1)=TIME(N)
	      CALL JD_BY(TIME(N),TIME(N))
C  To convert from Besselian years to Julian years of ephemeris time,
C  uncomment the following line.
C	        TIME(N)=(TIME(N)-2415020.0)/365.25+1900.0
	    END IF
	    TIME_SAVE_BY(N,1)=TIME(N)
C  Take account of the scale in the file.
	    IF (FILETYPE(4:6).NE.'RHO')
	1     CALL ERROR(-4,'Non 1-D interferometric file opened as '//
	2     'interferometric')
	    DATA(N,1)=SCALE*DATA(N,1)
	    SIGMA(N,1)=SCALE*SIGMA(N,1)
C	    SIGMA_SAVE(N,1)=SIGMA(N,1)
C	    DATA_SAVE(N,1)=DATA(N,1)
	    IF (SIGMA(N,1).GT.0.0) THEN
	      WEIGHT(N,1)=1.0D0/SIGMA(N,1)**2
	    ELSE
	      WEIGHT(N,1)=1.0D10 !  Arbitrarily large
	    END IF
	    WEIGHT_SAVE(N,1)=WEIGHT(N,1)
	    SUM=SUM+WEIGHT(N,1)
	    N=N+1
	  END IF
	END DO
 997	N=N-1
	NPOINTS_DATA=N
C  Be sure that theta' is within the range 0 to 180 degrees
	II=0
	DO I=1,NPOINTS_DATA
	  DO WHILE (FR_ANGLE(I,1).GT.180.0D0)
	    FR_ANGLE(I,1)=FR_ANGLE(I,1)-180.0D0
	    II=II+1
	  END DO
	  DO WHILE (FR_ANGLE(I,1).LT.0.0D0)
	    FR_ANGLE(I,1)=FR_ANGLE(I,1)+180.0D0
	    II=II+1
	  END DO
	END DO
	IF (II.GT.0) CALL MESSAGE('One or more angles were outside the '//
	1'standard limits and were corrected')
C  Return point for redoing the calculation by excluding points deviating
C  beyond a given factor times the mean error.
	REDO_SOLUTION=.FALSE.
 100	NPOINTS_DATA_OMIT=0
	IF (REDO_SOLUTION) THEN
	  DO N=1,NPOINTS_DATA
	    WEIGHT(N,1)=WEIGHT_SAVE(N,1)
	    IF (ABS(RESIDUALS(N,1)).GT.THRESHOLD_FACTOR*MEANERROR(1)) THEN
	      WEIGHT(N,1)=0.0D0
	      NPOINTS_DATA_OMIT=NPOINTS_DATA_OMIT+1
	    END IF
	  END DO
	END IF
C  Compare the range in time to the period in the header if it is
C  available.
	IF (INITIAL_ELEMENTS(1).NE.-1.0D37) THEN
	  INITIAILIZED=.FALSE.
	  DO I=1,NPOINTS_DATA
	    IF (.NOT.INITIALIZED) THEN
	      IF (WEIGHT(I,1).GT.0.0D0) THEN
	        TIME_MAX=TIME_SAVE_BY(I,1)
	        TIME_MIN=TIME_MAX
	        INITIALIZED=.TRUE.
	      END IF
	    ELSE
	      IF (TIME_SAVE_BY(I,1).GT.TIME_MAX.AND.WEIGHT(I,1).GT.0.0D0)
	1     TIME_MAX=TIME_SAVE_BY(I,1)
	      IF (TIME_SAVE_BY(I,1).LT.TIME_MIN.AND.WEIGHT(I,1).GT.0.0D0)
	1     TIME_MIN=TIME_SAVE_BY(I,1)
	    END IF
	  END DO
	  RATIO=(TIME_MAX-TIME_MIN)/INITIAL_ELEMENTS(1)
	  CALL MESSAGE('The ratio of the time span of the observations '//
	1 'to the estimated')
	  WRITE(6,1050) RATIO
1050	  FORMAT(' ','period is ',G13.7)
	  WRITE(7,1050) RATIO
C  Calculate the weighted average of the epoch
	  CALL DPAVERSIGWT(TIME_SAVE_BY,WEIGHT,MAXNPOINTS,1,1,1,
	1 NPOINTS_DATA,1,AVERAGE_TIME,SIGMA_TIME)
	  WRITE (6,1060) AVERAGE_TIME,SIGMA_TIME
1060	  FORMAT(' ','The weighted average of the epoch is ',
	1 F15.8,'  +-',F14.8,/' in Besselian years')
	  WRITE (7,1060) AVERAGE_TIME,SIGMA_TIME
	  CALL DPAVERSIGWT(TIME_SAVE_JD,WEIGHT,MAXNPOINTS,1,1,1,
	1 NPOINTS_DATA,1,AVERAGE_TIME,SIGMA_TIME)
	  WRITE (6,1065) AVERAGE_TIME,SIGMA_TIME
1065	  FORMAT(' ','The weighted average of the epoch is ',
	1 F14.5,'  +-',F11.5,/' in Julian Date')
	  WRITE (7,1065) AVERAGE_TIME,SIGMA_TIME
	  CALL MESSAGE(' ')
	END IF
C
	DIMENSN_IND=1
	DIMENSN_DATA=1
C  Calculate some initial estimates
	SUM=0.0D0
	DO I=1,NPOINTS_DATA
	  SUM=SUM+DATA(I,1)
	END DO
	PARAMETRS(1)=SUM/NPOINTS_DATA/0.707D0
	DO I=1,18
	  TEMP1(I)=(I-1)*10.0D0
	  SUM=0.0D0
	  DO N=1,NPOINTS_DATA
	    SUM=SUM+
	1   (DATA(N,1)-PARAMETRS(1)*ABS(DCOSD(FR_ANGLE(N,1)-TEMP1(I))))**2
	  END DO
	  TEMP2(I)=SUM
	END DO
	PARAMETRS(2)=TEMP1(1)
	SUM=TEMP2(1)
	DO I=2,18
	  IF (TEMP2(I).LT.SUM) THEN
	    PARAMETRS(2)=TEMP1(I)
	    SUM=TEMP2(I)
	  END IF
	END DO
C  Done with initial estimates
	IX(1)=1
	IX(2)=NPOINTS_DATA
	FUNCAT=14
	CHOICE='?' !  Not used
	NUMPARAM=-2 !  User will be prompted for which of the two 
		    !  parameters to hold constant
	ITERATION_SCHEME=' '
	TOLERANCE=0.0
	INTERMEDIATE=.TRUE.
	OPERATE=.FALSE.
	DO I=1,181
	  XPLOT(I,1)=I-1
	END DO
	NPLPOINTS(1)=181
	CALL MESSAGE('The initial estimates are:')
	WRITE (6,1100) PARAMETRS(1)
1100	FORMAT(' ','     Rho = ',G13.7)
	WRITE (7,1100) PARAMETRS(1)
	WRITE (6,1110) PARAMETRS(2)
1110	FORMAT(' ','     Theta = ',G13.7)
	WRITE (7,1110) PARAMETRS(2)
	CALL USRINP(PARAMETRS,ABS(NUMPARAM),'F',8,1,'PARAMETERS=',
	1'Enter the initial estimates of the parameters:  (default = above)')
C
	CALL NONLINFIT(FR_ANGLE,MAXNPOINTS,DIMENSN_IND,DATA,DIMENSN_DATA,
	1WEIGHT,
	1IX,FUNCAT,CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	2ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,MEANERROR,
	3MEANERRORMEANERROR,REDCHISQ,AVREDCHISQ,
	4XPLOT,DATPLOT,NPLPOINTS(1),RESIDUALS)
C
	WRITE (6,1300) PARAMETRS(1),SQRT(COVARIANCE(1,1))
1300	FORMAT(' ','The separation in " is ',G16.10,' +-',G13.7)
	WRITE (7,1300) PARAMETRS(1),SQRT(COVARIANCE(1,1))
	WRITE (6,1310) PARAMETRS(2),SQRT(COVARIANCE(2,2))
1310	FORMAT(' ','The position angle in deg. is ',G16.10,' +-',G13.7)
	WRITE (7,1310) PARAMETRS(2),SQRT(COVARIANCE(2,2))
	CALL MESSAGE('     Note that the position angle contains a 180 deg. '//
	1'ambiguity')
	WRITE (6,1435) REDCHISQ(1)
1435	FORMAT(' ','The reduced chi squared is ',G18.8)
	WRITE (7,1435) REDCHISQ(1)
	CALL MESSAGE('The mean error for an observation of average weight is')
	WRITE (6,1438) MEANERROR(1),MEANERRORMEANERROR(1)
1438	FORMAT(' ',G18.8,' +-',G16.6)
	WRITE (7,1438) MEANERROR(1),MEANERRORMEANERROR(1)
	WRITE (6,1450) NPOINTS_DATA-NPOINTS_DATA_OMIT
1450	FORMAT(' ','The total number of points used in the solution = ',I5)
	WRITE (7,1450) NPOINTS_DATA-NPOINTS_DATA_OMIT
	WRITE (6,1500) COVARIANCE(1,1)
1500	FORMAT(' ','Covariance(1,1) = ',G20.8)
	WRITE (7,1500) COVARIANCE(1,1)
	WRITE (6,1501) COVARIANCE(1,2)
1501	FORMAT(' ','Covariance(1,2) = ',G20.8)
	WRITE (7,1501) COVARIANCE(1,2)
	WRITE (6,1502) COVARIANCE(2,2)
1502	FORMAT(' ','Covariance(2,2) = ',G20.8)
	WRITE (7,1502) COVARIANCE(2,2)
C
C  Ask if the user wants to redo the solution where points beyond
C  some multiple of the MEOAW are rejected.
	CALL MESSAGE('Do you wish to redo the solution, eliminating '//
	1'points which deviate')
	REDO_SOLUTION=.FALSE.
	CALL USRINP(REDO_SOLUTION,1,'L',4,1,'REDO_SOLUTION=',
	1'more than a given factor times the mean error?  (default = N)')
	IF (REDO_SOLUTION) THEN
	  THRESHOLD_FACTOR=3.0D0
	  CALL USRINP(THRESHOLD_FACTOR,1,'F',8,1,'THRESHOLD_FACTOR=',
	1 'Enter the threshold factor:  (default = 3.0)')
	END IF
	IF (REDO_SOLUTION) GO TO 100
C
C  Beginning of plotting section
C
	PLOTD=.TRUE.
	CALL USRINP(PLOTD,1,'L',4,1,'PLOT=',
	1'Do you wish to plot the function and the data?  (default = Y)')
	IF (PLOTD) THEN
	  PLOTE=.TRUE.
	  CALL USRINP(PLOTE,1,'L',4,1,'PLOT_ERRORBARS=',
	1 'Do you wish to plot errorbars for the'//
	2 ' data?  (default = Y)')
	  CALL MESSAGE('The first plot will be of rho'' against theta''')
C  The curve was already generated by NONLINFIT, so just put in the data
	  DO I=1,NPOINTS_DATA
	    IF (WEIGHT(I,1).NE.0.0D0) THEN
	      XPLOT(I,2)=FR_ANGLE(I,1)
	      DATPLOT(I,2)=DATA(I,1)
	    ELSE
	      XPLOT(I,2)=OMIT
	    END IF
	  END DO
	  IF (PLOTE) THEN
	    NPLOTS=4
	    DO I=1,NPOINTS_DATA
	      IF (WEIGHT(I,1).NE.0.0D0) THEN
	        XPLOT(I,3)=XPLOT(I,2)
	        DATPLOT(I,3)=DATPLOT(I,2)+SIGMA(I,1)
	        XPLOT(I,4)=XPLOT(I,2)
	        DATPLOT(I,4)=DATPLOT(I,2)-SIGMA(I,1)
	      ELSE
	        XPLOT(I,3)=OMIT
	        XPLOT(I,4)=OMIT
	      END IF
	    END DO
	    NSYMBOL(2)=1+3*256**2 !  1 is octagon symbol;  value in second
				  !  byte is symbol height in 0.01 in.
	  ELSE
	    NPLOTS=2
	    NSYMBOL(2)=1+6*256**2
	  END IF
	  NPLPOINTS(2)=NPOINTS_DATA
	  NPLPOINTS(3)=NPOINTS_DATA
	  NPLPOINTS(4)=NPOINTS_DATA
	  LINVECT(1)=0 !  Solid lines
	  LINVECT(2)=0
	  LINVECT(3)=0
	  LINVECT(4)=0
          INC(1)=1
	  INC(2)=1
	  INC(3)=1
	  INC(4)=1
	  LINTYPE(1)=0 !  Line
	  LINTYPE(2)=-1 !  Symbols with no connecting lines
	  LINTYPE(3)=997 !  Errorbars
	  LINTYPE(4)=997 !  Errorbars
	  NSYMBOL(1)=0
	  NSYMBOL(3)=0 
	  NSYMBOL(4)=0
          INVERT(1)=.FALSE.
          INVERT(2)=.FALSE.
	  COLOR(1)=16776960 !  Yellow
	  COLOR(2)=16711935 !  Magenta
	  COLOR(3)=15794175 !  Azure
	  COLOR(4)=15794175 !  Azure
          INTERACT=.FALSE.
          CROSSHAIRS=.TRUE.
	  XLABEL=CNTI//'j'''//CNTI//' (degrees)'
	  YLABEL=CNTI//'r'''//CNTI//' (")'
	  TITLE=NAME
	  CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1 NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2 INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	  CALL RESETTEK
C
	  CALL MESSAGE(
	1 'The second plot will be of rho''/rho against theta'' - theta')
C  First generate the curve
	  DO I=-90,90
	    II=I+91
	    XPLOT(II,1)=I
	    DATPLOT(II,1)=DCOSD(1.0D0*I)
	  END DO
C  Then load the data points
	  NPLPOINTS(1)=181
	  DO I=1,NPOINTS_DATA
	    IF (WEIGHT(I,1).NE.0.0D0) THEN
	      XPLOT(I,2)=FR_ANGLE(I,1)-PARAMETRS(2)
C  Make sure the values are within the range -90 to +90
	      IF (XPLOT(I,2).LT.-90.0) XPLOT(I,2)=XPLOT(I,2)+180.0
	      IF (XPLOT(I,2).GT.90.0) XPLOT(I,2)=XPLOT(I,2)-180.0
	      DATPLOT(I,2)=DATA(I,1)/PARAMETRS(1)
	    ELSE
	      XPLOT(I,2)=OMIT
	    END IF
	  END DO
	  IF (PLOTE) THEN
	    DO I=1,NPOINTS_DATA
	      IF (WEIGHT(I,1).NE.0.0D0) THEN
	        XPLOT(I,3)=XPLOT(I,2)
	        DATPLOT(I,3)=DATPLOT(I,2)+SIGMA(I,1)/PARAMETRS(1)
	        XPLOT(I,4)=XPLOT(I,2)
	        DATPLOT(I,4)=DATPLOT(I,2)-SIGMA(I,1)/PARAMETRS(1)
	      ELSE
	        XPLOT(I,3)=OMIT
	        XPLOT(I,4)=OMIT
	      END IF
	    END DO
	  END IF
	  XLABEL=CNTI//'j''-j'//CNTI//' (degrees)'
	  YLABEL=CNTI//'r''/r'
C  Predetermine the scaling of the axes for this plot
	  XPLOT(MAXNPOINTS-3,1)=6.0
	  XPLOT(MAXNPOINTS-2,1)=6.1025 !  Coded
	  XPLOT(MAXNPOINTS-1,1)=-90.0
	  XPLOT(MAXNPOINTS,1)=30.0
	  DATPLOT(MAXNPOINTS-3,1)=5.0
	  DATPLOT(MAXNPOINTS-2,1)=10.0095 !  Coded
	  DATPLOT(MAXNPOINTS-1,1)=0.0
	  DATPLOT(MAXNPOINTS,1)=0.1
	  CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1 NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2 INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	  CALL RESETTEK
	END IF
C
C  Beginning of section for displaying the residuals
C
	SHOW_RESID=.TRUE.
	CALL USRINP(SHOW_RESID,1,'L',4,1,'SHOW_RESIDUALS=',
	1'Do you wish to see the residuals?  (default = Y)')
	IF (SHOW_RESID) THEN
	  CALL MESSAGE(
	1'                                                       '//
	2'     |DELTA_RHO''|')
	  CALL MESSAGE(
	1'                                                DELTA_RHO'''//
	2'  ___________')
	  CALL MESSAGE(
	1'  EPOCH(BY)         EPOCH(JD)       THETA''         (")       '//
	2'MEAN ERROR')
	  DO I=1,NPOINTS_DATA
C  If the weight is zero, the residuals are not displayed
	    IF (WEIGHT(I,1).GT.0.0D0) THEN
	      WRITE (6,4000) TIME_SAVE_BY(I,1),TIME_SAVE_JD(I,1),FR_ANGLE(I,1),
	1     RESIDUALS(I,1),RESIDUALS(I,1)/MEANERROR(1)
4000	      FORMAT(' ',G16.10,1X,F15.6,2X,G12.6,1X,G12.6,2X,G12.6)
	      WRITE (7,4000) TIME_SAVE_BY(I,1),TIME_SAVE_JD(I,1),FR_ANGLE(I,1),
	1     RESIDUALS(I,1),RESIDUALS(I,1)/MEANERROR(1)
	    END IF
	  END DO
	END IF

	CLOSE(UNIT=35)
	END
