	SUBROUTINE PRINT_BINARY_HEADER(NAME,RIGHTASC,DECLINA,
	1EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	2C_ELEMENTS,N_ELEMENTS)
C
C	Note:  This is the Unix version.
C
C	This subroutine prints the binary file header on the screen
C  and in the log file.
C			Donald H. Gudehus, March 25, 1994
C			Version 2, D. Gudehus, April, 1994
C			Incorporated code for spectrocopic binaries
C			Version 3, D. Gudehus, May, 1995
C			Put C_ELEMENTS in argument list and added
C			ENCODE statements.
C			Version 4, D. Gudehus, September 16, 1999
C			Declared UNIT as INTEGER.  In its undeclared state
C			the VAX compiler didn't complain, but the Portland
C			Group compiler did.
C			Version 4U, D. Gudehus, September 16, 1999
C			Changed INCLUDE files to lower case for Unix
C			Version 5, D. Gudehus, January 3, 2003
C			Added code for TYPE='r' (constant equator)
C			Version 6, D. Gudehus, December, 2002
C			Added code for 1-D interferometric data
C			Version 7, D. Gudehus, February 5, 2004
C			Increased the significance of C_ELEMENTS
C			Version 7UM, D. Gudehus, March 29, 2005
C			Modified variable format statement for Absoft compiler
C			under Mac OS
C
C	NAME			(C)	The name of the binary star
C	RIGHTASC		(C)	The right ascension of the binary
C	DECLINA			(C)	The declination of the binary
C	EQUATOR			(C)	The epoch of the equator
C	TYPE			(C)	The type of data, absolute (A) or 
C					relative (R).
C	PARALLAX_PROPER_MOTION	(C)	A flag for whether parallax and
C					proper motion are included (I),
C					or removed (R).
C	SCALE			(C)	The scale, "/unit or km/sec/unit.
C	FILETYPE		(C)	The type of file, X_Y, THETA_RHO,
C					BY_VELOCITY, or JD_VELOCITY.
C	ELEMENTS		(C)	An array containing the elements of
C					the orbit or orbits.  Before reading
C					the header, each location is set
C					equal to a test value of -1.0D37.
C					The calling program can then
C					determine if a particular element
C					was available in the header.
C	C_ELEMENTS		(C)	A character array containing the 
C					elements of the orbit or orbits.
C	N_ELEMENTS		(C)	The dimension of the above 2 arrays
C
	CHARACTER*1 TYPE,PARALLAX_PROPER_MOTION,CSIGN
	CHARACTER*18 C_ELEMENTS(N_ELEMENTS)
	CHARACTER*11 FILETYPE
	CHARACTER*80 VFORMAT
	CHARACTER*80 NAME
	INTEGER UNIT
	REAL RAPARAM(3),DECPARAM(3),ERROR
	REAL*8 SCALE,RIGHTASC,DECLINA,EQUATOR,ELEMENTS(N_ELEMENTS)
	INCLUDE 'binary_parameters.for'
	INCLUDE 'binary_header.for'
C
	CALL MESSAGE(' ')
	CALL MESSAGE('The file has the following information:')
	WRITE (6,3600) NAME
3600	FORMAT(' ','The name of the star is ',A50)
	WRITE (7,3600) NAME
	ERROR=1.0E-6
	CALL DHHMS(RIGHTASC,ERROR,RAPARAM(1),RAPARAM(2),RAPARAM(3),
	1CSIGN)
	WRITE (6,3601) INT(RAPARAM(1)),INT(RAPARAM(2)),RAPARAM(3)
3601	FORMAT(' ','The right ascension is ',I2,' hour(s) ',I2,
	1' minute(s) ',F5.2,' second(s)')
	WRITE (7,3601) INT(RAPARAM(1)),INT(RAPARAM(2)),RAPARAM(3)
	CALL DHHMS(DECLINA,ERROR,DECPARAM(1),DECPARAM(2),DECPARAM(3),
	1 CSIGN)
	WRITE (6,3602) CSIGN,INT(DECPARAM(1)),INT(DECPARAM(2)),DECPARAM(3)
3602	FORMAT(' ','The declination is ',A1,I2,' degree(s) ',I2,
	1' minute(s) ',F6.3,' second(s)')
	WRITE (7,3602) CSIGN,INT(DECPARAM(1)),INT(DECPARAM(2)),DECPARAM(3)
	WRITE (6,3603) EQUATOR
3603	FORMAT(' ','The equator is ',F11.5)
	WRITE (7,3603) EQUATOR
	IF (TYPE.EQ.'A') THEN
	  WRITE (6,3604)
3604	  FORMAT(' ','The type of binary orbit is absolute')
	  WRITE (7,3604)
	ELSE IF (TYPE.EQ.'R') THEN
	  WRITE (6,3605)
3605	  FORMAT(' ','The type of binary orbit is relative')
	  WRITE (7,3605)
	ELSE IF (TYPE.EQ.'r') THEN
	  WRITE (6,3606)
3606	  FORMAT (' ','The type of binary orbit is relative with ',
	1 'constant equator')
	  WRITE (7,3606)
	END IF
	IF (PARALLAX_PROPER_MOTION.EQ.'I') THEN
	  WRITE (6,3616)
3616	  FORMAT(' ','Parallax and proper motion are included')
	  WRITE (7,3616)
	ELSE IF (PARALLAX_PROPER_MOTION.EQ.'R') THEN
	  WRITE (6,3617)
3617	  FORMAT(' ','Parallax and proper motion have been removed')
	  WRITE (7,3617)
	END IF
	IF (FILETYPE.EQ.'X_Y'.OR.FILETYPE.EQ.'THETA_RHO') THEN
	  WRITE (6,3618) SCALE
3618	  FORMAT(' ','The scale is ',F12.6,' arc seconds per unit')
	  WRITE (7,3618) SCALE
	ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	  WRITE (6,3619) SCALE
3619	  FORMAT(' ','The scale is ',F12.6,' kilometers per second',
	1 ' per unit')
	END IF
C  Take care of the elements
	DO J=1,2
	  IF (J.EQ.1) THEN
	    UNIT=6
	  ELSE IF (J.EQ.2) THEN
	    UNIT=7
	  END IF
	  DO I=1,10
	    IF (ELEMENTS(I).GT.-1.0D37) THEN
	      CALL ALPHA(%REF(HEAD(I)),1,47,IA,NR,NF,1)
	      WRITE(VFORMAT,5000) IA
5000	      FORMAT('('' '',A',I,',A)')
	      WRITE(UNIT,VFORMAT) HEAD(I),C_ELEMENTS(I)
C	      WRITE(UNIT,5000) HEAD(I),C_ELEMENTS(I)
C5000	      FORMAT(' ',A<IA>,A)
	    END IF
	  END DO
C  Put X center for 1-D interferometric data in the correct sequence
	  IF (ELEMENTS(89).GT.-1.0D37) THEN
	    CALL ALPHA(%REF(HEAD(89)),1,47,IA,NR,NF,1)
	    WRITE(VFORMAT,5000) IA
	    WRITE(UNIT,VFORMAT) HEAD(89),C_ELEMENTS(89)
C	    WRITE(UNIT,5000) HEAD(89),C_ELEMENTS(89)
	  END IF
	  DO I=11,13
	    IF (ELEMENTS(I).GT.-1.0D37) THEN
	      CALL ALPHA(%REF(HEAD(I)),1,47,IA,NR,NF,1)
	      WRITE(VFORMAT,5000) IA
	      WRITE(UNIT,VFORMAT) HEAD(I),C_ELEMENTS(I)
C	      WRITE(UNIT,5000) HEAD(I),C_ELEMENTS(I)
	    END IF
	  END DO
C  Put Y center for 1-D interferometric data in the correct sequence
	  IF (ELEMENTS(90).GT.-1.0D37) THEN
	    CALL ALPHA(%REF(HEAD(90)),1,47,IA,NR,NF,1)
	    WRITE(VFORMAT,5000) IA
	    WRITE(UNIT,VFORMAT) HEAD(90),C_ELEMENTS(90)
C	    WRITE(UNIT,5000) HEAD(90),C_ELEMENTS(90)
	  END IF
	  DO I=14,MAXHEAD
	    IF (ELEMENTS(I).GT.-1.0D37) THEN
	      CALL ALPHA(%REF(HEAD(I)),1,47,IA,NR,NF,1)
	      WRITE(VFORMAT,5000) IA
	      WRITE(UNIT,VFORMAT) HEAD(I),C_ELEMENTS(I)
C	      WRITE(UNIT,5000) HEAD(I),C_ELEMENTS(I)
	    END IF
	  END DO
	END DO
	IF (FILETYPE.EQ.'X_Y') THEN
	  WRITE (6,5640)
5640	  FORMAT(' ','The file contains X and Y positions')
	  WRITE (7,5640)
	ELSE IF (FILETYPE.EQ.'THETA_RHO') THEN
	  WRITE (6,5641)
5641	  FORMAT(' ','The file contains position angles and separations')
	  WRITE (7,5641)
	ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	  WRITE (6,5642)
5642	  FORMAT(' ','The file contains radial velocities')
	  WRITE (7,5642)
	ELSE IF (FILETYPE(4:6).EQ.'RHO') THEN
	  WRITE (6,5643)
5643	  FORMAT(' ','The file contains 1-D interferometric data')
	END IF
	CALL MESSAGE(' ')
	RETURN
	END
