	PROGRAM NL_DEMO_3
C
C  Note:  This is the Unix version
C
C       This program is an example of the useage of NONLINFIT to solve for
C  the equation of a circle.  This example is unusual in that only the X
C  Y data points are given.  If an angle were available as well, then the
C  angle would serve as the independent variable, and the setup of the
C  problem would be straightforward, i.e., the independent dimension would
C  be 1, and the dependent dimension would be 2, and the function and
C  derivative could be simply calculated.  One way to handle this situation
C  is to create a COMMON block and transfer the X and Y data points to the
C  function and derivative subroutines.  Then, the difference between the
C  distance from the solved for center and the solved for radius is 
C  minimized.  The dependent dimension only needs to be 1 for this part,
C  but for plotting the results, two dimensions are required.  This is the
C  approach used here.  Alternately, one could make the independent dimension
C  be 2, transfer the X and Y data points in the independent variable
C  array, and do without the extra COMMON block.
C			Donald H. Gudehus, Mar. 3, 2005
C			Version 2, D. Gudehus, August 31, 2006
C			Added some documentation
C			Version 3, D. Gudehus, November 11, 2013
C			Added support for 64-bit processors
C
	INTEGER DIMENSN_IND,DIMENSN_DATA
	PARAMETER (MAXNUMPARAM=5,MAXNPOINTS=300,DIMENSN_IND=1,DIMENSN_DATA=2)
	!  Note that MAXNPOINTS should match MAXNPOINTS_COM in nlfun_demo_3
	!  and nlder_demo_3
	PARAMETER (NPLPOINTS=300)
	PARAMETER (MAXNPLOTPOINTS=304,MAXNPLOTS=3) !  For XYPLOT.  Here, the
	!  array dimension should be 4 larger than the number of points
	INTEGER COLOR(2)
	INTEGER FUNCAT,IX(2),NPOINTS_XYPLOT(MAXNPLOTS),
	1LINVECT(2),INC(2),LINTYPE(2),NSYMBOL(2)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,OPERATE,
	1INVERT(2),INTERACT,CROSSHAIRS,PLOTD
	REAL XPLOT(MAXNPOINTS,DIMENSN_IND),DATPLOT(MAXNPOINTS,DIMENSN_DATA),
	1XARRAY(MAXNPLOTPOINTS,MAXNPLOTS),YARRAY(MAXNPLOTPOINTS,MAXNPLOTS)
	REAL*8 X(MAXNPOINTS,DIMENSN_IND),DATA(MAXNPOINTS,DIMENSN_DATA),
	1WEIGHT(MAXNPOINTS,DIMENSN_DATA),POINTS(MAXNPOINTS,DIMENSN_DATA),
	1PARAMETRS(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	2MEANERROR(DIMENSN_DATA),MEANERRORMEANERROR(DIMENSN_DATA),
	3REDCHISQ(DIMENSN_DATA),AVREDCHISQ,RESIDUALS(MAXNPOINTS,DIMENSN_DATA),
	4PI
C
	CHARACTER*1 ITERATION_SCHEME
	CHARACTER*6 CHOICE
	CHARACTER*50 XLABEL,YLABEL,TITLE
C  Set up a COOMON block to transfer the data to NLFUN_DEMO_5 and NLDER_DEMO_5
C  since we don't have the usual situation with a true independent variable.
	COMMON /SCRATCH_BLOCK/ POINTS
C
C  Get the number of processor bits
	CALL GET_OS_BYTES
C
C  Make carriage control as in OpenVMS
	OPEN(UNIT=6,STATUS='NEW',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=7,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=49,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
C
C  Determine what display and plot devices are available
	CALL GET_DP_AVAILABILITY
	FUNCAT=-3
	PI=3.1415926535D0
	CALL MESSAGE('This program is a demonstration of NONLINFIT in'//
	1' fitting a circle when')
	CALL MESSAGE('X and Y but no angles are provided')
	CALL MESSAGE(' ')
C  Enter the data
	POINTS(1,1)=1.9
	POINTS(1,2)=2.0
	POINTS(2,1)=4.2
	POINTS(2,2)=1.1
        POINTS(3,1)=6.7
        POINTS(3,2)=2.8
        POINTS(4,1)=6.9
        POINTS(4,2)=5.6
        POINTS(5,1)=5.5
        POINTS(5,2)=7.0
        POINTS(6,1)=3.5
        POINTS(6,2)=7.1
        POINTS(7,1)=1.6
        POINTS(7,2)=6.6
        POINTS(8,1)=1.0
        POINTS(8,2)=4.0
	NPOINTS_DATA=8
	DO I=1,NPOINTS_DATA
	  X(I,1)=I !  An index, just for convenience
C  Make the DATA array be zero to allow proper calculation of chi^2.
	  DATA(I,1)=0.0D0
	  DATA(I,2)=0.0D0
	  WEIGHT(I,1)=1.0D0
	  WEIGHT(I,2)=1.0D0
	END DO
C  Fill the plotting array
	DO I=1,NPLPOINTS !  The fitted function
	  XPLOT(I,1)=2.0*PI*(I-1)/NPLPOINTS
	END DO
	IX(1)=1
	IX(2)=NPOINTS_DATA
	CHOICE='C'
	NUMPARAM=-3 !  Negative value gives prompt for holding some parameters
		    !  constant
	ITERATION_SCHEME=' '
	TOLERANCE=0.0
	INTERMEDIATE=.TRUE.
	OPERATE=.FALSE.
	CALL MESSAGE('The initial estimates are:  3.5, 4.0, and 3.0 for '//
	1'X Center, Y Center,')
	CALL MESSAGE('and radius')
	PARAMETRS(1)=3.5
	PARAMETRS(2)=4.0
	PARAMETRS(3)=3.0
	N=ABS(NUMPARAM)
	CALL USRINP(PARAMETRS,N,'F',8,1,'PARAMETERS=',
	1'Enter the initial estimates of the parameters:  (defaults = above)')
	CALL NONLINFIT(X,MAXNPOINTS,DIMENSN_IND,DATA,DIMENSN_DATA,WEIGHT,IX,
	1FUNCAT,CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	2ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,MEANERROR,
	3MEANERRORMEANERROR,REDCHISQ,AVREDCHISQ,XPLOT,DATPLOT,
	4NPLPOINTS,RESIDUALS)
	WRITE (6,1321) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,3)
1321	FORMAT(' X Center = ',G15.6,'  +-',G15.6/
	1' Y Center = ',G15.6,'  +-',G15.6/
	2' Radius = ',G15.6,'  +-',G15.6)
	WRITE (7,1321) (PARAMETRS(I),SQRT(COVARIANCE(I,I)),I=1,3)
	WRITE (6,1350) MEANERROR(1),MEANERRORMEANERROR(1)
1350	FORMAT(' The mean error for the radius = ',G15.6,'  +-',G15.6)
	WRITE (7,1350) MEANERROR(1),MEANERRORMEANERROR(1)
	WRITE (6,1360) REDCHISQ(1)
1360	FORMAT(' The reduced chi squared for the radius = ',G15.6)
	WRITE (7,1360) REDCHISQ(1)
	DO J=1,NUMPARAM
	  DO I=1,J
	    K=K+1
	      WRITE (6,1400) I,J,COVARIANCE(I,J)
	      WRITE (7,1400) I,J,COVARIANCE(I,J)
1400	      FORMAT(' Covariance(',I1,',',I1,') = ',G20.8)
	  END DO
	END DO
	PLOTD=.TRUE.
	CALL USRINP(PLOTD,1,'L',4,1,'PLOT=',
	1'Do you wish to plot the function and the data?  (default = Y)')
	IF (PLOTD) THEN
C  Set up the plotting parameters
	  NPLOTS=2
	  LINVECT(1)=0
	  LINVECT(2)=0
	  INC(1)=1
	  INC(2)=1
	  LINTYPE(1)=0
	  LINTYPE(2)=-1
	  NSYMBOL(1)=0
	  NSYMBOL(2)=3 !  + signs
	  INVERT(1)=.FALSE.
	  INVERT(2)=.FALSE.
	  COLOR(1)=4251856 !  Turquoise
	  COLOR(2)=16416882 !  Salmon
	  INTERACT=.FALSE.
	  CROSSHAIRS=.TRUE.
	  XLABEL='X AXIS'
	  YLABEL='Y AXIS'
	  TITLE='CIRCLE'
C  Example of plotting the function and the data
	  NPOINTS_XYPLOT(1)=NPLPOINTS !  The fitted function
	  NPOINTS_XYPLOT(2)=NPOINTS_DATA !  The data
	  DO I=1,NPOINTS_XYPLOT(1)
	    XARRAY(I,1)=DATPLOT(I,1)
	    YARRAY(I,1)=DATPLOT(I,2)
	  END DO
	  DO I=1,NPOINTS_XYPLOT(2)
	    XARRAY(I,2)=POINTS(I,1)
	    YARRAY(I,2)=POINTS(I,2)
	  END DO
	  CALL XYPLOT(XARRAY,YARRAY,MAXNPLOTPOINTS,
	1 NPLOTS,NPOINTS_XYPLOT,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2 INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	  CALL RESETTEK
	END IF
C
	END
