	PROGRAM NL_DEMO_4
C
C	Note:  This is the UNIX version
C
C       This is a demonstration of the usage of NONLINFIT to fit
C  the de Vaucouleurs formula to a one-dimensional file of surface 
C  brightness data.  Although convolution could be included as an operation,
C  it is not included here.  
C			Donald H. Gudehus, April, 1986
C			Version 1U, D. H. G., June, 1997
C			Modified USRINP, OPENCF, and plotting for UNIX.
C			Versopm 2. D. H. G., November 11, 2013
C			Added support for 64-bit processors
C
	INTEGER DIMENSN_IND,DIMENSN_DATA
	PARAMETER (MAXNUMPARAM=8,MAXNPOINTS=5000,DIMENSN_IND=1,
	1DIMENSN_DATA=1)
	INTEGER COLOR
	INTEGER FUNCAT,IRADII(2),NPLOTPOINTS,
	1LINVECT,INC,LINTYPE,NSYMBOL
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,
	1INVERT(2),INTERACT,CROSSHAIRS,PLOTD,ADJUST,CONVOLVE
	REAL ECCENTRICITY(MAXNPOINTS),
	5RADII(2),READOUT(2),
	4XPLOT(MAXNPOINTS),DATPLOT(MAXNPOINTS),OBMINUSCALC(MAXNPOINTS,2),
	5INCLINA,LOGINTADU,SIGMAPARAM(MAXNUMPARAM)
	REAL*8 RADIUS(MAXNPOINTS,DIMENSN_IND),
	1SURFBRIGHTOB(MAXNPOINTS,DIMENSN_DATA),
	1WEIGHT(MAXNPOINTS,DIMENSN_DATA),
	1PARAMETRS(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	2MEANERROR(DIMENSN_DATA),MEANERRORMEANERROR(DIMENSN_DATA),
	2REDCHISQ(DIMENSN_DATA),AVREDCHISQ,RESIDUALS(MAXNPOINTS,DIMENSN_DATA),
	3SKY,SIGMASUM,SUMWEIGHT,
	4LOGWEIGHT(MAXNPOINTS),SUMLOGWEIGHT
C
	CHARACTER*1 WEIGHT_SCHEME,ITERATION_SCHEME
	CHARACTER*6 CHOICE,IDREAD
	CHARACTER*50 XLABEL,YLABEL,TITLE
C
	CHARACTER*6 ID
	COMMON /ID/ ID
C
	CHARACTER*40 FILENAME
	COMMON /CFILE/ FILENAME
C
	DATA TWOPI /6.2831853/
C
C  Get the number of processor bits
	CALL GET_OS_BYTES
C
C  Make carriage control as in OpenVMS
	OPEN(UNIT=6,STATUS='NEW',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=7,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
	OPEN(UNIT=49,STATUS='UNKNOWN',CARRIAGECONTROL='FORTRAN')
C
	FUNCAT=-4
	CONVOLVE=.FALSE.
	ITERATION_SCHEME=' '
	TOLERANCE=0.0
	INTERMEDIATE=.TRUE.
	CALL MESSAGE('This is a demonstration of the usage of NONLINFIT to'//
	1' fit the de Vaucouleurs')
	CALL MESSAGE('formula to a one-dimensional file of surface'//
	1' brightness data.')
C  Get the data to be fitted
C  Because we are supplying the default information, call OPENCF with
C  4 arguments in order to supress its prompt.
	FILENAME='galaxy_profiles.dat' ! Extension appended for
C					       ! UNIX
	CALL USRINP(%REF(FILENAME),1,'A',40,1,'FILE_21=',
	1'Open the data file:  (default = galaxy_profiles.dat)')
	CALL ALPHA(%REF(FILENAME),1,40,NC,NF,NR,2)
	CALL OPENCF(21,3,' ',1)
	REWIND 21
	ID='70'
	CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
	1'Enter the ID of the object:  (default = 70)')
	CALL ALPHA(%REF(ID),1,6,NC,NF,NR,2)
	READ(21,1100,ERR=14)
	GO TO 16
  14	CALL MESSAGE('Couldn''t read the header')
	STOP
  16	CONTINUE
1100	FORMAT(' ','ID',8X,'COLUMN',5X,'ROW',1X,'INCLINATION',
	11X,'ECCENTRICITY',1X,'RADIUS',5X,'ADU',3X,'LOG(INTEGRAL)')
	IDREAD=' '
	DO WHILE (ID.NE.IDREAD)
	  READ(21,1101,END=22) IDREAD,COLUMN,ROW,INCLINA,ECCENTRICITY(1),
	1 RADIUS(1,1),SURFBRIGHTOB(1,1),LOGINTADU
1101	  FORMAT(' ',A6,1X,F9.3,1X,F9.3,1X,F7.2,6X,F6.3,F8.3,1X,
	1 F13.6,1X,F11.8)
	END DO
	IF (RADIUS(1,1).NE.0.0) CALL MESSAGE('Warning!  The file does not'//
	1' begin at the center of the galaxy')
	GO TO 24
  22	CALL MESSAGE('Couldn''t find the object')
	STOP
  24	CONTINUE
	NPOINTS_FILE=1
	DO WHILE (ID.EQ.IDREAD)
	  NPOINTS_FILE=NPOINTS_FILE+1
	  READ(21,1101,END=42) IDREAD,COLUMN,ROW,INCLINA,
	1 ECCENTRICITY(NPOINTS_FILE),RADIUS(NPOINTS_FILE,1),
	1 SURFBRIGHTOB(NPOINTS_FILE,1),LOGINTADU
	END DO
  42	NPOINTS_FILE=NPOINTS_FILE-1  ! Ignores the last number read
C  Load the plotting array
	DO I=1,NPOINTS_FILE
	  XPLOT(I)=RADIUS(I,1)
	END DO
	SKY=SURFBRIGHTOB(1,1)
	DO I=1,NPOINTS_FILE
	  SKY=MIN(SKY,SURFBRIGHTOB(I,1))
	END DO
	CHOICE='V'  ! De Vaucouleurs formula
C  Now get the range of data to be fit
	CALL MESSAGE('Enter the starting and ending radii for the'//
	1' least squares fit:')
	RADII(1)=RADIUS(1,1)
	RADII(2)=RADIUS(NPOINTS_FILE,1)
	CALL USRINP(RADII,2,'F',4,1,'RADII=',
	1'(default = the entire range of the data file)')
	IF (RADII(1).EQ.0.0) THEN
	  CALL MESSAGE('Do you wish to adjust the first data point to'//
	1' a radius of')
	  ADJUST=.TRUE.
	  CALL USRINP(ADJUST,1,'L',4,1,'ADJUST=',
	1 '0.25 pixels?  (default = Y)')
	END IF
C  Get the number of points, allowing for RADIUS to be variably spaced.
	NPOINTS_FIT=0
	DO I=1,NPOINTS_FILE
	  IF (RADIUS(I,1).GE.RADII(1).AND.RADIUS(I,1).LE.RADII(2)) NPOINTS_FIT
	1 =NPOINTS_FIT+1
	END DO
C  Now convert the radii to indexes
	DO I=1,NPOINTS_FIT
	  IF (RADIUS(I,1).LE.RADII(1)) IRADII(1)=I
	  IF (RADIUS(I,1).LE.RADII(2)) IRADII(2)=I
	END DO
	IF (ADJUST) RADIUS(IRADII(1),1)=0.25
	IF (CHOICE.EQ.'V') THEN
	  NUMPARAM=3
	  CALL MESSAGE('Enter the estimates for the surface brightness'//
	1' at 1/2 the total light')
	  CALL MESSAGE('(ADU), the corresponding radius (pixels), and'//
	1' the sky level (ADU):')
	  DO I=1,MAXNUMPARAM
	    J=SETEND(PARAMETRS(I))
	  END DO
	  PARAMETRS(1)=5.0D0
	  PARAMETRS(2)=60.0D0
	  PARAMETRS(3)=490.0D0
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,1,'PARAMETERS=',
	1 '(default = 5 60 490')
	END IF
	NUMPARAM=NELD(PARAMETRS,MAXNUMPARAM)
C  Get the weighting scheme
  67	WEIGHT_SCHEME='R'
	CALL MESSAGE('Select the weighting scheme:')
	CALL MESSAGE('     R = Weight equal to circumference/(noise)**2'//
	1' (default)')
	CALL MESSAGE('     S = Weight equal to the square root of'//
	1' the circumference')
	CALL MESSAGE('     N = Weight equal to 1/(noise)**2')
	CALL USRINP(%REF(WEIGHT_SCHEME),1,'A',1,1,'WEIGHT_SCHEME=',' ')
	IF (WEIGHT_SCHEME.NE.'R'.AND.WEIGHT_SCHEME.NE.'S'.AND.
	1WEIGHT_SCHEME.NE.'N') GO TO 67
C  Get the readout noise in order to compute the weights
	READOUT(1)=65.0
	READOUT(2)=25.0
	IF (WEIGHT_SCHEME.NE.'S') THEN
	  CALL MESSAGE('Enter the readout noise in electrons and the'//
	1' gain in electrons per ADU:')
	  CALL USRINP(READOUT,2,'F',4,1,'NOISE_PARAMETERS=',
	1 '(defaults = 65 25)')
	END IF
	SUMWEIGHT=0.0
	SUMLOGWEIGHT=0.0
	DO N=IRADII(1),IRADII(2)
C  Compute the weights for NONLINFIT based on readout noise and shot noise
C  and the number of pixels in each contour.
	  IF (WEIGHT_SCHEME.EQ.'N'.OR.WEIGHT_SCHEME.EQ.'R') THEN
	    WEIGHT(N,1)=1.0/(READOUT(1)**2+
	1   SURFBRIGHTOB(N,1)*READOUT(2))
	    IF (WEIGHT_SCHEME.EQ.'R') THEN
	      WEIGHT(N,1)=WEIGHT(N,1)*MAX(1.0,TWOPI*SNGL(RADIUS(N,1)))
	    END IF
	  ELSE IF (WEIGHT_SCHEME.EQ.'S') THEN  ! Square root of radius
					       ! only
	    WEIGHT(N,1)=SQRT(RADIUS(N,1))
	  END IF
C  Convert to 1/ADU**2
	  WEIGHT(N,1)=WEIGHT(N,1)*READOUT(2)**2
	  SUMWEIGHT=WEIGHT(N,1)+SUMWEIGHT  ! For use below
	  LOGWEIGHT(N)=LOG10(WEIGHT(N,1))
	  SUMLOGWEIGHT=LOGWEIGHT(N)+SUMLOGWEIGHT
	END DO
	NPLOTPOINTS=NPOINTS_FIT
C
	CALL NONLINFIT(RADIUS,MAXNPOINTS,DIMENSN_IND,SURFBRIGHTOB,
	1DIMENSN_DATA,WEIGHT,IRADII,FUNCAT,CHOICE,CONSTANTTERMS,
	2PARAMETRS,MAXNUMPARAM,NUMPARAM,ITERATION_SCHEME,TOLERANCE,
	3INTERMEDIATE,CONVOLVE,COVARIANCE,MEANERROR,MEANERRORMEANERROR,
	4REDCHISQ,AVREDCHISQ,XPLOT,DATPLOT,NPLOTPOINTS,
	5RESIDUALS)
C
	CALL MESSAGE(' ')
	CALL MESSAGE('SOLUTION')
	WRITE(6,1300) MEANERROR(1),MEANERRORMEANERROR(1)
1300	FORMAT(' Mean error for an observation of average weight',
	1' = '/G18.6,  '   +-',G18.6)
	WRITE(7,1300) MEANERROR(1),MEANERRORMEANERROR(1)
C  Calculate the residuals in magnitudes and load the plot arrays
	DO N=IRADII(1),IRADII(2)
	  OBMINUSCALC(N,1)=RESIDUALS(N,1)
	  SKY=PARAMETRS(3)
	  IF (SURFBRIGHTOB(N,1)-SKY.GT.0.0.AND.DATPLOT(N)-SKY.GT.0.0) THEN
	    OBMINUSCALC(N,2)=-2.5*LOG10(ABS((SURFBRIGHTOB(N,1)-SKY)/
	1   (DATPLOT(N)-SKY)))
	  ELSE
	    OBMINUSCALC(N,2)=0.0
	  END IF
	  SIGMASUM=OBMINUSCALC(N,2)**2*LOGWEIGHT(N)
	1 +SIGMASUM
	END DO
	AVWEIGHT=SUMWEIGHT/NPOINTS_FIT
	AVLOGWEIGHT=SUMLOGWEIGHT/NPOINTS_FIT
	SIGMA=SQRT(SIGMASUM/((NPOINTS_FIT-NUMPARAM)*AVLOGWEIGHT))
	SIGMASIGMA=SIGMA/SQRT(2.0*(NPOINTS_FIT-NUMPARAM))
	IF (CHOICE.NE.'E3') THEN
	  WRITE(6,1305) SIGMA,SIGMASIGMA
1305	  FORMAT('                         and in magnitudes it is',
	1' = '/G18.6,  '   +-',G18.6)
	  WRITE(7,1305) SIGMA,SIGMASIGMA
	END IF
C  Compute the expected mean error for a perfect fit
	IF (WEIGHT_SCHEME.EQ.'R'.OR.WEIGHT_SCHEME.EQ.'N') THEN
	  EXPCT_SIGMA=SQRT(NPOINTS_FIT/SUMWEIGHT)
	  EXPCT_SIGMASIGMA=EXPCT_SIGMA/SQRT(2.0*(NPOINTS_FIT-NUMPARAM))
	  CALL MESSAGE('For a perfect fit:')
	  WRITE (6,1300) EXPCT_SIGMA,EXPCT_SIGMASIGMA
	  WRITE (7,1300) EXPCT_SIGMA,EXPCT_SIGMASIGMA
	END IF
C  Print the reduced chi squared
	IF (WEIGHT_SCHEME.NE.'S') THEN
	  WRITE (6,1415) REDCHISQ
	  WRITE (7,1415) REDCHISQ
1415	  FORMAT(' The reduced chi squared = ',F10.4)
	  CALL MESSAGE('The reduced chi squared may be large because seeing'//
	1 ' has not been taken into')
	  CALL MESSAGE('account and because the de Vaucouleurs formula does'//
	1 ' not produce a perfect fit.')
	END IF
C  Print the average weight
	WRITE (6,1410) AVWEIGHT
	WRITE (7,1410) AVWEIGHT
1410	FORMAT(' The average weight = ',F15.6)
	DO I=1,NUMPARAM
	  SIGMAPARAM(I)=SQRT(COVARIANCE(I,I))
	END DO
	IF (CHOICE.EQ.'V') THEN
1350	  FORMAT(' Effective surface brightness = ',F15.4,'   +-',
	1 F15.4/' Effective radius = ',F15.4,'   +-',F15.4)
	  WRITE (6,1350) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  WRITE (7,1350) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  SKY=PARAMETRS(3)  ! Used below in integration
	  WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
1310	  FORMAT(' Sky surface brightness = ',F15.4,'   +-',F15.4)
	  WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	END IF
	DO J=1,NUMPARAM
	  DO I=1,J
	    K=K+1
	      WRITE (6,1400) I,J,COVARIANCE(I,J)
	      WRITE (7,1400) I,J,COVARIANCE(I,J)
1400	      FORMAT(' Covariance(',I1,',',I1,') = ',G20.8)
	  END DO
	END DO
C
C  Set up the plotting parameters
	ISTART=IRADII(1)
	NPLOTS=1
	LINVECT=0
	INC=1
	LINTYPE=0
	NSYMBOL=0
	INVERT(1)=.FALSE.
	INVERT(2)=.FALSE.
	COLOR=16776960
	INTERACT=.FALSE.
	CROSSHAIRS=.TRUE.
	XLABEL='RADIUS (pixels)'
	YLABEL='OSBSERVED-CALCULATED (ADU)'
	TITLE='RESIDUALS'
	PLOTD=.TRUE.
	CALL USRINP(PLOTD,1,'L',4,1,'PLOT=',
	1'Do you wish to plot the residuals?  (default = Y)')
	IF (PLOTD) THEN
C  Determine what display and plot devices are available
	CALL GET_DP_AVAILABILITY
C  Reset the array
	  DO I=MAXNPOINTS-ISTART+1-3,MAXNPOINTS-ISTART+1
	    XPLOT(I)=0
	  END DO
	  CALL XYPLOT(XPLOT(ISTART),OBMINUSCALC(ISTART,1),
	1 MAXNPOINTS-ISTART+1,NPLOTS,NPLOTPOINTS,LINVECT,INC,LINTYPE,
	2 NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,
	3 ' ',.FALSE.)
	  CALL RESETTEK
	  YLABEL='OBSERVED-CALCULATED (MAGNITUDES)'
C  Reset the array
	  DO I=MAXNPOINTS-ISTART+1-3,MAXNPOINTS-ISTART+1
	    XPLOT(I)=0
	  END DO
	  CALL XYPLOT(XPLOT(ISTART),OBMINUSCALC(ISTART,2),
	1 MAXNPOINTS-ISTART+1,NPLOTS,NPOINTS_FIT,LINVECT,INC,LINTYPE,
	2 NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,
	3 ' ',.FALSE.)
	  CALL RESETTEK
	END IF
	CLOSE(UNIT=21)
	END
