	SUBROUTINE NLDER14(FR_ANGLE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,
	1MAXNUMPARAM,CHOICE,CONSTANTTERMS,OPERATE,IX,FUNCTN,
	2DIMENSN_DATA,DERIV)
C
C       This subroutine selects the appropriate function choice and
C  computes the derivatives for NONLINFIT when solving for the
C  separation and position angle of a binary star from one-
C  dimensional interferometric data.
C		Donald H. Gudehus, February 23, 2005
C
C	FR_ANGLE	(C)	The array containing the independent variable
C				values
C	MAXNPOINTS	(C)	The first dimension of the array FR_ANGLE
C	DIMENSN_IND	(C)	The second dimension of the array FR_ANGLE
C	PARAMETRS	(C)	The array containing the parameters used to
C				evaluate the function
C	MAXNUMPARAM	(C)	The maximum number of parameters to be used in
C				the calling program
C	CHOICE		(C)	The choice determining which function to use.
C				Not used here.
C	CONSTANTTERMS	(C)	The terms which are to be held constant in the
C				solution.  Time is saved if these derivatives
C				are not calculated.
C	OPERATE		(C)	A logical key which determines what to do.
C				If .FALSE., the function is evaluated, if 
C				.TRUE.,  a user defined operation is carried
C				out, e.g. convolution.  Not used here.
C	IX		(C)	An array containing the starting and ending
C				indexes for the evaluation of DERIV
C	FUNCTN		(C)	The output array for the function.
C	DIMENSN_DATA	(C)	The second dimension of the array FUNCTN
C	DERIV		(R)	The ouput array containing the derivative
C				
C
	CHARACTER*(*) CHOICE
	INTEGER DIMENSN_IND,DIMENSN_DATA,IX(2)
	DIMENSION PARAMETRS(MAXNUMPARAM),FR_ANGLE(MAXNPOINTS,DIMENSN_IND),
	1FUNCTN(MAXNPOINTS,DIMENSN_DATA),
	2DERIV(MAXNPOINTS,MAXNUMPARAM,DIMENSN_DATA)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),OPERATE
	REAL*8 FR_ANGLE,PARAMETRS,FUNCTN,DERIV
C
	REAL*8 DEG_RAD
C
	DEG_RAD=57.29577951308232D0
C
	DO I=IX(1),IX(2)
	  DERIV(I,1,1)=FUNCTN(I,1)/PARAMETRS(1)
	  DERIV(I,2,1)=PARAMETRS(1)*DSIND(FR_ANGLE(I,1)-PARAMETRS(2))/
	1 DEG_RAD !  -90 to +90
	  IF (((FR_ANGLE(I,1)-PARAMETRS(2).GT.90.0D0).AND.
	1   (FR_ANGLE(I,1)-PARAMETRS(2).LE.180.0D0)).OR.
	2   ((FR_ANGLE(I,1)-PARAMETRS(2).GE.-180.0D0).AND.
	3   (FR_ANGLE(I,1)-PARAMETRS(2).LT.-90.0D0))) THEN 
	    DERIV(I,2,1)=-1.0D0*DERIV(I,2,1)
	  END IF
	END DO
C
	RETURN
	END
