	SUBROUTINE NLDER3(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,
	1MAXNUMPARAM,CHOICE,CONSTANTTERMS,OPERATE,ITIME,FUNCTN,
	2DIMENSN_DATA,DERIV)
C
C       This subroutine selects the appropriate function choice and
C  computes the derivatives for NONLINFIT when solving a binary star, taking
C  into account a user defined operation if necessary.
C		Donald H. Gudehus, and James W. Haywood, March, 1986
C		Version 2, Donald H. Gudehus, May, 1994
C		Added capability to solve for primary and secondary
C		simultaneously.  Added capability to solve for spectroscopic
C		binary, and spectroscopic and visual binary simultaneously.
C		Version 3, Donald H. Gudehus, June, 1995
C		Added capability to solve for velocity of subcomponent
C		of primary and/or secondary.
C		Version 4, Donald H. Gudehus, September, 1997
C		Changed COMMON blocks for OpenVMS and UNIX
C		Version 5, Donald H. Gudehus, November 25, 1999
C		Increased number of points to 5000
C		Version 6, Donald H. Gudehus, February 25, 2000
C		Added code for apsidal motion
C		Version 7, D. H. G., May 2, 2001
C		Program now solves for the true proper motion acceleration
C		rather than the coefficient of the quadratic time factor.
C		Version 8, D. H. G., September 25, 2001
C		Fixed bug where apsidal motion failed for spectroscopic
C		primary.  Added apsidal motion for subcomponents.  Removed
C		call to BINARY_INDEXES.  The call is made in BINARY.FOR so 
C		that INDX for the solution can be used to properly address
C		PARAMETRS when plotting.
C
C	TIME		(C)	The array containing the independent variable
C				values
C	MAXNPOINTS	(C)	The first dimension of the array TIME
C	DIMENSN_IND	(C)	The second dimension of the array TIME
C	PARAMETRS	(C)	The array containing the parameters used to
C				evaluate the function
C	MAXNUMPARAM	(C)	The maximum number of parameters to be used in
C				the calling program
C	CHOICE		(C)	The choice determining which function
C				to use within a category.  The possibilities
C				are:  
C				CHOICE(1:1)='P'   Primary of visual binary
C					   ='S'   Secondary of visual binary
C					   ='B'	  Both members of visual
C						  binary
C				CHOICE(2:2)='R'   Parallax and proper motion
C						  removed from primary visual
C						  binary data
C					   ='I'   Parallax and proper motion
C						  included in primary data
C					   ='N'   No primary visual binary data
C				CHOICE(3:3)='A'   Parallax and proper motion
C						  removed from secondary visual
C						  binary data;  orbit type is
C						  absolute
C					   ='R'   Parallax and proper motion
C						  removed from secondary visual
C						  binary data;  orbit type is
C						  relative
C					   ='I'   Parallax and proper motion
C						  included in secondary data;
C						  implies absolute
C					   ='N'   No secondary visual binary
C						  data
C				CHOICE(4:4)='P'	  Velocity data available for
C						  primary
C					   ='S'	  Velocity data available for
C						  secondary
C					   ='B'	  Velocity data available for
C						  primary and secondary
C					   ='N'   No spectroscopic binary
C						  data available
C				CHOICE(5:5)='n'	  1) The primary includes 
C						  velocity subcomponents equal
C						  in number to n, where said
C						  primary is taken to be the
C						  primary of the subsystem.
C						  n should be in the range
C						  1 to 5.  If this
C						  position is blank, no
C						  subcomponents exist.  The
C						  action to be carried out is
C						  controlled by  CHOICE(6:6).
C						  2) If CHOICE(6:6)='o', then
C						  n is the actual subcompo-
C						  nent.
C				CHOICE(6:6)=' '	  Adds all velocity sub-
C						  components, up to n, to the
C						  primary.
C					   ='o'	  Evaluates only primary
C						  subcomponent n.
C					   ='a'	  Evaluates the sum of all
C						  subcomponents.
C				CHOICE(7:7)='n'	  1) The secondary includes
C						  velocity subcomponents equal
C						  in number to n, where said
C						  secondary is taken to be the
C						  primary of the subsystem.
C						  n should be in the range
C						  1 to 5.  If this
C						  position is blank, no
C						  subcomponents exist.  The
C						  action to be carried out is
C						  controlled by  CHOICE(8:8).
C						  2) If CHOICE(8:8)='o', then
C						  n is the actual subcompo-
C						  nent.
C				CHOICE(8:8)=' '	  Adds all velocity sub-
C						  components, up to n, to the
C						  secondary.
C					   ='o'	  Evaluates only secondary
C						  subcomponent n.
C					   ='a'	  Evaluates the sum of all
C						  subcomponents.
C				CHOICE(9:13)='T'  Evaluates one or more
C						  velocities of secondaries 
C						  of primary subcomponents 1:5
C				CHOICE(14:18)='T' Evaluates one or more
C						  velocities of secondaries 
C						  of secondary subcomponents 
C						  1:5
C				CHOICE(19:19)='A' Apsidal motion is evaluated
C						  for main components
C				CHOICE(20:24)='A' Apsidal motion is evaluated
C						  for one or more primary
C						  subcomponents 1:5
C				CHOICE(25:29)='A' Apsidal motion is evaluated
C						  for one or more secondary
C						  subcomponents 1:5
C				CHOICE(30:30)='P' Primary 1D interferometric
C						data
C				CHOICE(30:30)='S' Secondary 1D interferometric
C						  data
C				CHOICE(30:30)='B' 1D interferometric data for
C						  both primary and secondary
C				CHOICE(30:30)='N' No 1D interferometric data
C				CHOICE(31:31)='R' Parallax and proper motion
C						  removed from primary 1D
C						  interferometric data
C				CHOICE(31:31)='I' Parallax and proper motion
C						  included in primary data
C				CHOICE(32:32)='A' Parallax and proper motion
C						  removed from secondary 1D
C						  data;  orbit type is
C						  absolute
C				CHOICE(32:32)='R' Parallax and proper motion
C						  removed from secondary 1D
C						  data;  orbit type is
C						  relative
C				CHOICE(32:32)='I' Parallax and proper motion
C						  included in secondary 1D
C						  data;  implies absolute
C
C	CONSTANTTERMS	(C)	The terms which are to be held constant in the
C				solution.  Time is saved if these derivatives
C				are not calculated.
C	OPERATE		(C)	A logical key which determines what to do.
C				If .FALSE., the function is evaluated, if 
C				.TRUE.,  a user defined operation is carried
C				out, e.g. convolution.  In this case, when
C				.TRUE., the light travel time correction
C				is made for spectroscopic orbits.
C	ITIME		(C)	An array containing the starting and ending
C				indexes for the evaluation of DERIV
C	FUNCTN		(C)	The output array for the function.
C	DIMENSN_DATA	(C)	The second dimension of the array FUNCTN
C				The indexes are assigned as follows:
C				1	Primary X position
C				2	Primary Y position
C				3	Secondary X position
C				4	Secondary Y position
C				5	Primary radial velocity plus any
C					subcomponents
C				6	Secondary radial velocity plus any
C					subcomponents
C				7:11	Radial velocities of secondaries as 
C					subcomponents of primary
C				12:16	Radial velocities of secondaries as
C					subcomponents of secondary
C				17      One-dimensional interferometric
C					projected separation for primary
C				18      One-dimensional interferometric
C					projected separation for secondary
C	DERIV		(R)	The ouput array containing the derivative
C				
C
	PARAMETER (MAXNUMPARAM_DUM=90,NSUBS_MAX=5,MAXNPOINTS_COM=10000)
	CHARACTER*(*) CHOICE
	INTEGER DIMENSN_IND,DIMENSN_DATA,ITIME(2)
	DIMENSION PARAMETRS(MAXNUMPARAM),TIME(MAXNPOINTS,DIMENSN_IND),
	1FUNCTN(MAXNPOINTS,DIMENSN_DATA),
	2DERIV(MAXNPOINTS,MAXNUMPARAM,DIMENSN_DATA)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),OPERATE
	REAL*8 TIME,PARAMETRS,FUNCTN,DERIV
C
	REAL*8 PI,DEG_RAD,E,
	1P,P_PRI_SUB(NSUBS_MAX),P_SEC_SUB(NSUBS_MAX),T,T_PRI_SUB(NSUBS_MAX),
	2T_SEC_SUB(NSUBS_MAX),ECCENTR,ECCENTR_PRI_SUB(NSUBS_MAX),
	3ECCENTR_SEC_SUB(NSUBS_MAX),a,aAU,aAU_PRI_SUB(NSUBS_MAX),
	4aAU_SEC_SUB(NSUBS_MAX),FRAC,OMEGA,INCLINA,w,w_save,
	4APSIDAL_MOTION,APSIDAL_MOTION_PRI_SUB(NSUBS_MAX),
	5APSIDAL_MOTION_SEC_SUB(NSUBS_MAX),w_PRI_SUB(NSUBS_MAX),
	5w_save_PRI_SUB(NSUBS_MAX),w_SEC_SUB(NSUBS_MAX),
	5w_save_SEC_SUB(NSUBS_MAX),RATIO_a,RATIO_a_PRI_SUB_SEC(NSUBS_MAX),
	6RATIO_a_SEC_SUB_SEC(NSUBS_MAX),A1,PARALLAX,MU,MU_PRI_SUB(NSUBS_MAX),
	7MU_SEC_SUB(NSUBS_MAX),M,DELTATIME,DELTATIME2,r,V_SYS,VEL_P,VEL_S,
	8X1,Y1,X,Y,XI1,XI2,v,v_PRI_SUB(NSUBS_MAX),v_SEC_SUB(NSUBS_MAX),
	9SINTHETA_OM,COSTHETA_OM,
	1ATANTHETA_OM,THETA1,CS,SN,RHO,
	2ALPHA1,ALPHA2,ALPHA3,ALPHA4,ALPHA5,ALPHA6,
	3BETA1,BETA2,BETA3,BETA4,BETA5,BETA6,GAMMA1,GAMMA2,GAMMA3,
	4ALPHA6_PRI_SUB(NSUBS_MAX),ALPHA6_SEC_SUB(NSUBS_MAX),
	5BETA1_PRI_SUB(NSUBS_MAX),BETA5_PRI_SUB(NSUBS_MAX),
	6BETA6_PRI_SUB(NSUBS_MAX),BETA1_SEC_SUB(NSUBS_MAX),
	7BETA5_SEC_SUB(NSUBS_MAX),BETA6_SEC_SUB(NSUBS_MAX),
	8GAMMA1_PRI_SUB(NSUBS_MAX),GAMMA2_PRI_SUB(NSUBS_MAX),
	9GAMMA1_SEC_SUB(NSUBS_MAX),GAMMA2_SEC_SUB(NSUBS_MAX)
C
	CHARACTER*1 TYPE_P,TYPE_S,TYPE_VEL_P,TYPE_VEL_S,TYPE_INTER_P,
	1TYPE_INTER_S
	COMMON /CHAR_SCRATCH_BLOCK/TYPE_P,TYPE_S,TYPE_VEL_P,TYPE_VEL_S,
	1TYPE_INTER_P,TYPE_INTER_S

	INTEGER NPOINTS_DATA_VEL_P_SUB_S(NSUBS_MAX),
	1NPOINTS_DATA_VEL_S_SUB_S(NSUBS_MAX)
	COMMON /INTEGER4_SCRATCH_BLOCK/NPOINTSPXPY,NPOINTS_DATA_P,
	1NPOINTS_DATA_S,NPOINTS_VIS,NPOINTS_DATA_VEL_P,
	2NPOINTS_DATA_VEL_S,NPOINTS_SP,NPOINTS_DATA_VEL_P_SUB_S,
	3NPOINTS_DATA_VEL_S_SUB_S,NPOINTS_VEL_P_S,NPOINTS_VEL_S_S,
	4NPOINTS_DATA_INTER_P,NPOINTS_DATA_INTER_S,NPOINTS_INTER

	REAL*8 PX(MAXNPOINTS_COM),PY(MAXNPOINTS_COM),
	1FR_ANGLE(MAXNPOINTS_COM),EPOCH,
	1VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB(NSUBS_MAX),
	2VEL_AMP_SEC_SUB(NSUBS_MAX),VEL_AMP_PRI_SUB_SEC(NSUBS_MAX),
	3VEL_AMP_SEC_SUB_SEC(NSUBS_MAX),INCLINA_WORK,
	4INCLINA_WORK_PRI_SUB(NSUBS_MAX),INCLINA_WORK_SEC_SUB(NSUBS_MAX),
	5ECC_ANOM(MAXNPOINTS_COM),ECC_ANOM_PRI_SUB(MAXNPOINTS_COM,NSUBS_MAX),
	6ECC_ANOM_SEC_SUB(MAXNPOINTS_COM,NSUBS_MAX),LTT_COR(MAXNPOINTS_COM),
	7PHOTOCENTRIC_TRUE_SEMIMAJAX,FUNCTN_TEMP(MAXNPOINTS_COM,4)
	  !  PX and PY are the parallax factors.  VEL_AMP_XXX are returned
	  !  values which are used in BINARY and NLDER3.
	COMMON /REAL8_SCRATCH_BLOCK/ PX,PY,FR_ANGLE,
	4EPOCH,VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB,VEL_AMP_SEC_SUB,
	5VEL_AMP_PRI_SUB_SEC,VEL_AMP_SEC_SUB_SEC,INCLINA_WORK,
	6INCLINA_WORK_PRI_SUB,INCLINA_WORK_SEC_SUB,
	7ECC_ANOM,ECC_ANOM_PRI_SUB,ECC_ANOM_SEC_SUB,LTT_COR,
	8PHOTOCENTRIC_TRUE_SEMIMAJAX,FUNCTN_TEMP
C
	INTEGER SEMIMAJOR_AXIS_ASSIGNMENT(2),DIMENSN_DATA_DUM,
	1DIMENSN_DATA_USED,INDX(MAXNUMPARAM_DUM)
	COMMON /BINARY_INTEGER4/ SEMIMAJOR_AXIS_ASSIGNMENT,
	1DIMENSN_DATA_DUM,DIMENSN_DATA_USED,INDX

	LOGICAL VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
	COMMON /BINARY_LOGICALS/ VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
C
	PI=3.14159265358979D0
	DEG_RAD=180.0D0/PI
C
	P=PARAMETRS(INDX(1))
	T=PARAMETRS(INDX(2))
	ECCENTR=PARAMETRS(INDX(3))
	FRAC=DSQRT(ABS((1.0D0+ECCENTR)/(1.0D0-ECCENTR)))
	IF (SPECTROSCOPIC.AND..NOT.VISUAL.AND..NOT.INTERFEROMETRIC) THEN
C  If only spectroscopic data is available, we can't solve for the
C  node or inclination angle.  In this case, an offset is made.  Note that
C  a is in units of AU.
	  OMEGA=0.0D0
	  INCLINA=90.0D0
	ELSE IF (VISUAL.OR.INTERFEROMETRIC) THEN !  Either SPECTROSCOPIC
						 !  not SPECTROSCOPIC
C  a is in units of arc seconds
	  OMEGA=PARAMETRS(INDX(4))
	  INCLINA=PARAMETRS(INDX(5))
	END IF
C  If not VISUAL and not SPECTROSCOPIC then no useful data exists
	w=PARAMETRS(INDX(6)) !  Little omega
	w_save=w !  For apsidal motion
	IF (VISUAL.OR.INTERFEROMETRIC) THEN
	  a=PARAMETRS(INDX(7))
	ELSE 
C  If no visual information exists, the spectroscopic data can only solve
C  for projected "a" in units of AU.  This is parameter INDX(17).
	END IF
	IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(4:4).EQ.'B'.OR.
C		!  Visual binary pair or a spectroscopic binary pair
	1CHOICE(30:30).EQ.'B') THEN
C		!  Interferometric binary pair
	  RATIO_a=PARAMETRS(INDX(8))
	ELSE !  Single member
C  Ignore ratio of semimajor axes
	END IF
C  Note that when parallax can't be determined by any means,
C  for the spectroscopic binary, a will have the units of AU.
	IF (PAR_PM.OR.PAR_PM_INTERFERO) THEN
	  PARALLAX=PARAMETRS(INDX(9))
C  We don't need to calculate CM_X, etc.
	ELSE
	  IF (SOLVE_PAR) THEN
	    PARALLAX=PARAMETRS(INDX(9))
	  END IF
	END IF
c to here
	IF (SPECTROSCOPIC) THEN
	  V_SYS=PARAMETRS(INDX(16))
	  IF (.NOT.PAR_PM.AND..NOT.PAR_PM_INTERFERO.AND.
	1 .NOT.SOLVE_PAR) aAU=PARAMETRS(INDX(17))
C  Handle the case of the primary having a subcomponent
	  IF (SPECTROSCOPIC_PRI_SUB) THEN
	    DECODE(1,1001,CHOICE(5:5)) IPRISUB
1001	    FORMAT(I1)
	    P_PRI_SUB(1)=PARAMETRS(INDX(18))
	    T_PRI_SUB(1)=PARAMETRS(INDX(19))
	    ECCENTR_PRI_SUB(1)=PARAMETRS(INDX(20))
	    w_PRI_SUB(1)=PARAMETRS(INDX(21))
	    w_save_PRI_SUB(1)=w_PRI_SUB(1)
	    aAU_PRI_SUB(1)=PARAMETRS(INDX(22))
	    P_PRI_SUB(2)=PARAMETRS(INDX(23))
	    T_PRI_SUB(2)=PARAMETRS(INDX(24))
	    ECCENTR_PRI_SUB(2)=PARAMETRS(INDX(25))
	    w_PRI_SUB(2)=PARAMETRS(INDX(26))
	    w_save_PRI_SUB(2)=w_PRI_SUB(2)
	    aAU_PRI_SUB(2)=PARAMETRS(INDX(27))
	    P_PRI_SUB(3)=PARAMETRS(INDX(28))
	    T_PRI_SUB(3)=PARAMETRS(INDX(29))
	    ECCENTR_PRI_SUB(3)=PARAMETRS(INDX(30))
	    w_PRI_SUB(3)=PARAMETRS(INDX(31))
	    w_save_PRI_SUB(3)=w_PRI_SUB(3)
	    aAU_PRI_SUB(3)=PARAMETRS(INDX(32))
	    P_PRI_SUB(4)=PARAMETRS(INDX(33))
	    T_PRI_SUB(4)=PARAMETRS(INDX(34))
	    ECCENTR_PRI_SUB(4)=PARAMETRS(INDX(35))
	    w_PRI_SUB(4)=PARAMETRS(INDX(36))
	    w_save_PRI_SUB(4)=w_PRI_SUB(4)
	    aAU_PRI_SUB(4)=PARAMETRS(INDX(37))
	    P_PRI_SUB(5)=PARAMETRS(INDX(38))
	    T_PRI_SUB(5)=PARAMETRS(INDX(39))
	    ECCENTR_PRI_SUB(5)=PARAMETRS(INDX(40))
	    w_PRI_SUB(5)=PARAMETRS(INDX(41))
	    w_save_PRI_SUB(5)=w_PRI_SUB(5)
	    aAU_PRI_SUB(5)=PARAMETRS(INDX(42))
	  END IF
C  Handle the case of the secondary having a subcomponent
	  IF (SPECTROSCOPIC_SEC_SUB) THEN
	    DECODE(1,1001,CHOICE(7:7)) ISECSUB
	    P_SEC_SUB(1)=PARAMETRS(INDX(43))
	    T_SEC_SUB(1)=PARAMETRS(INDX(44))
	    ECCENTR_SEC_SUB(1)=PARAMETRS(INDX(45))
	    w_SEC_SUB(1)=PARAMETRS(INDX(46))
	    w_save_SEC_SUB(1)=w_SEC_SUB(1)
	    aAU_SEC_SUB(1)=PARAMETRS(INDX(47))
	    P_SEC_SUB(2)=PARAMETRS(INDX(48))
	    T_SEC_SUB(2)=PARAMETRS(INDX(49))
	    ECCENTR_SEC_SUB(2)=PARAMETRS(INDX(50))
	    w_SEC_SUB(2)=PARAMETRS(INDX(51))
	    w_save_SEC_SUB(2)=w_SEC_SUB(2)
	    aAU_SEC_SUB(2)=PARAMETRS(INDX(52))
	    P_SEC_SUB(3)=PARAMETRS(INDX(53))
	    T_SEC_SUB(3)=PARAMETRS(INDX(54))
	    ECCENTR_SEC_SUB(3)=PARAMETRS(INDX(55))
	    w_SEC_SUB(3)=PARAMETRS(INDX(56))
	    w_save_SEC_SUB(3)=w_SEC_SUB(3)
	    aAU_SEC_SUB(3)=PARAMETRS(INDX(57))
	    P_SEC_SUB(4)=PARAMETRS(INDX(58))
	    T_SEC_SUB(4)=PARAMETRS(INDX(59))
	    ECCENTR_SEC_SUB(4)=PARAMETRS(INDX(60))
	    w_SEC_SUB(4)=PARAMETRS(INDX(61))
	    w_save_SEC_SUB(4)=w_SEC_SUB(4)
	    aAU_SEC_SUB(4)=PARAMETRS(INDX(62))
	    P_SEC_SUB(5)=PARAMETRS(INDX(63))
	    T_SEC_SUB(5)=PARAMETRS(INDX(64))
	    ECCENTR_SEC_SUB(5)=PARAMETRS(INDX(65))
	    w_SEC_SUB(5)=PARAMETRS(INDX(66))
	    w_save_SEC_SUB(5)=w_SEC_SUB(5)
	    aAU_SEC_SUB(5)=PARAMETRS(INDX(67))
	  END IF
	  DO I=1,5
	    IF (CHOICE(8+I:8+I).EQ.'T') THEN
	      RATIO_a_PRI_SUB_SEC(I)=PARAMETRS(INDX(67+I))
	    END IF
	  END DO
	  DO I=1,5
	    IF (CHOICE(13+I:13+I).EQ.'T') THEN
	      RATIO_a_SEC_SUB_SEC(I)=PARAMETRS(INDX(72+I))
	    END IF
	  END DO
	END IF
	IF (CHOICE(19:19).EQ.'A') THEN
	  APSIDAL_MOTION=PARAMETRS(INDX(78))
	ELSE
	  APSIDAL_MOTION=0.0D0
	END IF
	IF (SPECTROSCOPIC_PRI_SUB) THEN
	  DO I=1,5
	    IF (CHOICE(19+I:19+I).EQ.'A') THEN
	      APSIDAL_MOTION_PRI_SUB(I)=PARAMETRS(INDX(78+I))
	    ELSE
	      APSIDAL_MOTION_PRI_SUB(I)=0.0D0
	    END IF
	  END DO
	END IF
	IF (SPECTROSCOPIC_SEC_SUB) THEN
	  DO I=1,5
	    IF (CHOICE(24+I:24+I).EQ.'A') THEN
	      APSIDAL_MOTION_SEC_SUB(I)=PARAMETRS(INDX(83+I))
	    ELSE
	      APSIDAL_MOTION_SEC_SUB(I)=0.0D0
	    END IF
	  END DO
	END IF
 	MU=2.0D0*PI/P !  Rate of revolution in radians per year
	IF (SPECTROSCOPIC_PRI_SUB) THEN
	  DO I=1,IPRISUB
	    MU_PRI_SUB(I)=2.0D0*PI/P_PRI_SUB(I)
	  END DO
	END IF
	IF (SPECTROSCOPIC_SEC_SUB) THEN
	  DO I=1,ISECSUB
	    MU_SEC_SUB(I)=2.0D0*PI/P_SEC_SUB(I)
	  END DO
	END IF
	DO N=ITIME(1),ITIME(2)
	  DELTATIME=TIME(N,1)-EPOCH !  Epoch for parallax and proper motion
	  DELTATIME2=DELTATIME*DELTATIME
	  M=MU*(TIME(N,1)-T)
	  E=ECC_ANOM(N) !  Same value as in NLFUN3
C d(E)/dP = GAMMA1; d(E)/dT = GAMMA2; d(E)/d(ECCENTR) = GAMMA3
	  A1=1.0D0-ECCENTR*DCOS(E)
	  GAMMA1=-M/(P*A1)
	  GAMMA2=-2.0D0*PI/(P*A1)
	  GAMMA3=DSIN(E)/A1
	  v=2.0D0*DATAND(FRAC*TAN(E/2.0D0))
C  Take care of any apsidal motion
	  IF (CHOICE(19:19).EQ.'A') THEN
	    IF (OPERATE) THEN
	      w=w_save+APSIDAL_MOTION*(TIME(N,1)-LTT_COR(N)-T)
	    ELSE
	      w=w_save+APSIDAL_MOTION*(TIME(N,1)-T)
	    END IF
	  END IF
	  SINTHETA_OM=DSIND(v+w)*DCOSD(INCLINA)
	  COSTHETA_OM=DCOSD(v+w)
	  ATANTHETA_OM=DATAN2D(SINTHETA_OM,COSTHETA_OM)
	  THETA1=OMEGA+ATANTHETA_OM
	  CS=DCOSD(THETA1)
	  SN=DSIND(THETA1)
	  r=a*(1.0D0-ECCENTR*DCOS(E))
	  RHO=DABS(r*DCOSD(v+w)/DCOSD(ATANTHETA_OM))
	  X=RHO*SN
	  Y=RHO*CS
	  ALPHA1=a*ECCENTR*DSIN(E)*DSQRT((DCOSD(v+w)/DCOSD(ATANTHETA_OM))**2)
	  ALPHA2=-r*DSQRT((DCOSD(ATANTHETA_OM)/DCOSD(v+w))**2)
	1 *DCOSD(v+w)*DSIND(v+w)/DCOSD(ATANTHETA_OM)**2
	  ALPHA3=r*DSQRT((DCOSD(ATANTHETA_OM)/DCOSD(v+w))**2)
	1 *DCOSD(v+w)**2*DSIND(ATANTHETA_OM)/DCOSD(ATANTHETA_OM)**3
	  ALPHA4=r*DSQRT((DCOSD(v+w)/DCOSD(ATANTHETA_OM))**2)
C dv/dP = BETA1 d(E)/dP; d(THETA1)/dP = BETA1*BETA2 d(E)/dP
C  To derive the following, multiply numerator and denominator of original
C  expression by 1-ECCENTR, and then use trigonometric identities
	  BETA1=DSQRT(1.0D0-ECCENTR**2)/(1.0D0-ECCENTR*DCOS(E))
	  BETA2=DCOSD(INCLINA)/(COSTHETA_OM**2+SINTHETA_OM**2)
	  XI1=ALPHA1+BETA1*(ALPHA2+BETA2*ALPHA3)
	  XI2=BETA1*BETA2*ALPHA4
	  X1=XI1*SN+XI2*CS
	  Y1=XI1*CS-XI2*SN
C  Calculate the visual binary derivatives.  The primary derivative
C  is calculated first, then the secondary derivative is calculated
C  in terms of it.  If no primary data exists, the primary derivative 
C  will be nonzero in this code, but since the calling program ensures
C  that the weight is zero, no problem will be encountered in NONLINFIT.
	  IF (VISUAL) THEN !  Avoids unecessary calculation if not visual
	    DERIV(N,INDX(1),1)=X1*GAMMA1
	    DERIV(N,INDX(1),2)=Y1*GAMMA1
	    DERIV(N,INDX(2),1)=X1*GAMMA2
	    DERIV(N,INDX(2),2)=Y1*GAMMA2
	    ALPHA5=-A*DCOS(E)*DSQRT((DCOSD(v+w)/DCOSD(ATANTHETA_OM))**2)
	    BETA3=2.0D0*DSQRT(1.0D0/(1.0D0-ECCENTR**2))*DCOS(E/2.0D0)*
	1   DSIN(E/2.0D0)/(1.0D0-ECCENTR*DCOS(E))
	    X1=ALPHA5+BETA3*(ALPHA2+BETA2*ALPHA3)+XI1*GAMMA3
	    Y1=BETA2*ALPHA4*(BETA3+BETA1*GAMMA3)
	    DERIV(N,INDX(3),1)=X1*SN+Y1*CS
	    DERIV(N,INDX(3),2)=X1*CS-Y1*SN
	    DERIV(N,INDX(4),1)=Y/DEG_RAD !  For Omega
	    DERIV(N,INDX(4),2)=-X/DEG_RAD
	    BETA4=-DSIND(INCLINA)*DTAND(v+w)
	1   /(1.0D0+(DTAND(v+w)*DCOSD(INCLINA))**2)/DEG_RAD
	    DERIV(N,INDX(5),1)=(ALPHA3*SN+ALPHA4*CS)*BETA4
	    DERIV(N,INDX(5),2)=(ALPHA3*CS-ALPHA4*SN)*BETA4
	    DERIV(N,INDX(6),1)=((ALPHA2+BETA2*ALPHA3)*SN+BETA2*ALPHA4*CS)/
	1   DEG_RAD
	    DERIV(N,INDX(6),2)=((ALPHA2+BETA2*ALPHA3)*CS-BETA2*ALPHA4*SN)/
	1   DEG_RAD
	    DERIV(N,INDX(7),1)=X/a
	    DERIV(N,INDX(7),2)=Y/a
	    DERIV(N,INDX(8),1)=0.0D0 !  Ratio of semimajor axes;  
				     !  total to primary
	    DERIV(N,INDX(8),2)=0.0D0
	    IF (CHOICE(2:2).EQ.'I') THEN
	      DERIV(N,INDX(9),1)=PX(N) !  Parallax
	      DERIV(N,INDX(9),2)=PY(N)
	      DERIV(N,INDX(10),1)=1.0D0 !  CM_X
	      DERIV(N,INDX(10),2)=0.0D0
	      DERIV(N,INDX(11),1)=DELTATIME !  PM_X
	      DERIV(N,INDX(11),2)=0.0D0
	      DERIV(N,INDX(12),1)=0.5D0*DELTATIME2 !  Q_X
	      DERIV(N,INDX(12),2)=0.0D0
	      DERIV(N,INDX(13),1)=0.0D0
	      DERIV(N,INDX(13),2)=1.0D0 !  CM_Y
	      DERIV(N,INDX(14),1)=0.0D0
	      DERIV(N,INDX(14),2)=DELTATIME !  PM_Y
	      DERIV(N,INDX(15),1)=0.0D0
	      DERIV(N,INDX(15),2)=0.5D0*DELTATIME2 !  Q_Y
	    END IF
C  Section for visual primary apsidal motion
	    IF (CHOICE(19:19).EQ.'A') THEN
	      DERIV(N,INDX(78),1)=DERIV(N,INDX(6),1)*(TIME(N,1)-
	1     PARAMETRS(INDX(2)))
	      DERIV(N,INDX(78),2)=DERIV(N,INDX(6),2)*(TIME(N,1)-
	1     PARAMETRS(INDX(2)))
	    END IF
C  Consider the visual secondary
	    IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(1:1).EQ.'S') THEN
	      IF (CHOICE(1:1).EQ.'B') THEN
	        IF (CHOICE(3:3).EQ.'A'.OR.CHOICE(3:3).EQ.'I') THEN !  Absolute
	          DO I=INDX(1),INDX(7)
	            DERIV(N,I,3)=(1.0D0-RATIO_a)*DERIV(N,I,1)
	            DERIV(N,I,4)=(1.0D0-RATIO_a)*DERIV(N,I,2)
	          END DO
	        ELSE IF (CHOICE(3:3).EQ.'R') THEN !  Relative
	          DO I=INDX(1),INDX(7)
	            DERIV(N,I,3)=-RATIO_a*DERIV(N,I,1)
	            DERIV(N,I,4)=-RATIO_a*DERIV(N,I,2)
	          END DO
	        END IF
	      ELSE IF (CHOICE(1:1).EQ.'S') THEN
	        IF (CHOICE(4:4).EQ.'B') THEN
C  In this case the ratio is obtained from the spectroscopic orbit
	          DO I=INDX(1),INDX(7)
	            DERIV(N,I,3)=-RATIO_a*DERIV(N,I,1)
	            DERIV(N,I,4)=-RATIO_a*DERIV(N,I,2)
	          END DO
	        ELSE
	          DO I=INDX(1),INDX(7)
	            DERIV(N,I,3)=-DERIV(N,I,1)
	            DERIV(N,I,4)=-DERIV(N,I,2)
	          END DO
	        END IF
	      END IF
	      DERIV(N,INDX(8),3)=-X  !  RATIO_a
	      DERIV(N,INDX(8),4)=-Y
	      IF (CHOICE(3:3).EQ.'I') THEN
	        DERIV(N,INDX(9),3)=PX(N) !  Parallax
	        DERIV(N,INDX(9),4)=PY(N)
	        DERIV(N,INDX(10),3)=1.0D0 !  CM_X
	        DERIV(N,INDX(10),4)=0.0D0
	        DERIV(N,INDX(11),3)=DELTATIME !  PM_X
	        DERIV(N,INDX(11),4)=0.0D0
	        DERIV(N,INDX(12),3)=0.5D0*DELTATIME2 !  Q_X
	        DERIV(N,INDX(12),4)=0.0D0
	        DERIV(N,INDX(13),3)=0.0D0
	        DERIV(N,INDX(13),4)=1.0D0 !  CM_Y
	        DERIV(N,INDX(14),3)=0.0D0
	        DERIV(N,INDX(14),4)=DELTATIME !  PM_Y
	        DERIV(N,INDX(15),3)=0.0D0
	        DERIV(N,INDX(15),4)=0.5D0*DELTATIME2 !  Q_Y
C  Absolute or relative for the secondary
	      ELSE IF (CHOICE(3:3).EQ.'A'.OR.CHOICE(3:3).EQ.'R') THEN
	        DO I=INDX(9),INDX(15)
	          DERIV(N,I,3)=0.0D0
	          DERIV(N,I,4)=0.0D0
	        END DO
	      END IF
C  Section for visual secondary apsidal motion
	      IF (CHOICE(19:19).EQ.'A') THEN
	        DERIV(N,INDX(78),3)=DERIV(N,INDX(6),3)*(TIME(N,1)-
	1       PARAMETRS(INDX(2)))
	        DERIV(N,INDX(78),4)=DERIV(N,INDX(6),4)*(TIME(N,1)-
	1       PARAMETRS(INDX(2)))
	      END IF
	    END IF
	  END IF !  For visual binary
C  Now take care of the spectroscopic cases
	  IF (SPECTROSCOPIC) THEN
C  The primary derivatives will be calculated even if they are not neeeded
C  for the primary.  Since WEIGHT will be zero for these cases, there will 
C  be no problem.  Later they will be used for computing the secondary 
C  derivatives.
C  First do the primary derivatives
	    ALPHA6=DCOSD(v+w)+ECCENTR*DCOSD(w)
	    BETA5=(1.0D0+ECCENTR)/(1.0D0-ECCENTR)
	    BETA6=1.0D0-ECCENTR*DCOS(E)
C  Compute the velocity of the primary.  We can't simply use 
C  FUNCTN(N,5)-V_SYS, because that is a sum of the primary velocity and
C  a possible subcomponent(s) velocity.  We can however use
C  VEL_AMP_PRI which is available in the COMMON block. 
	    IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).NE.2) THEN
	      VEL_P=VEL_AMP_PRI*(DCOSD(v+w)+ECCENTR*DCOSD(w))
	    ELSE
C  We pretend that the secondary is the primary for just this one case
	      VEL_P=VEL_AMP_SEC*(DCOSD(v+w)+ECCENTR*DCOSD(w)) !  For secondary
	    END IF
C  For the case of spectroscopic binary with no visual binary information,
C  projected semimajor axis, a, is in astronomical units, therefore 
C  PARALLAX is neglected.  INCLINA can be neglected also.
	    DERIV(N,INDX(1),5)=-VEL_P/P-VEL_P/ALPHA6*DSIND(v+w)*
	1   BETA1*GAMMA1 !  P
	    DERIV(N,INDX(2),5)=-VEL_P/ALPHA6*DSIND(v+w)*
	1   BETA1*GAMMA2 !  T
	    DERIV(N,INDX(3),5)=VEL_P*(ECCENTR/(1.0D0-ECCENTR**2)-
	1   DSIND(v+w)*DSIN(E)/ALPHA6/DSQRT(1.0D0-ECCENTR**2)/BETA6-
	2   DSIND(v+w)*DSIN(E)/ALPHA6*DSQRT(1.0D0-ECCENTR**2)/BETA6**2
	3   +DCOSD(w)/ALPHA6) !  e
C  We set the derivatives of big omega and inclination to zero.  These
C  can only be solved for from visual binary data.
	    DERIV(N,INDX(4),5)=0.0D0 !  Big omega
	    DERIV(N,INDX(5),5)=0.0D0 !  i
	    DERIV(N,INDX(6),5)=-VEL_P/ALPHA6*(DSIND(v+w)+
	1   ECCENTR*DSIND(w))/DEG_RAD !  w
C  We set the derivative of a(") to zero.  For a spectroscopic binary,
C  we solve for a(AU), a(AU)sin(i), or parallax.
	    DERIV(N,INDX(7),5)=0.0D0 !  a(")
C  For the primary, as in the case of the visual data, the derivative of
C  the ratio is set to zero.
	    DERIV(N,INDX(8),5)=0.0D0
			!  Ratio of semimajor axes, total to primary
C  Don't worry about the different cases, just compute the derivatives and
C  INDX will take care of things as long as we proceed in numerical order.
	    IF (PARALLAX.NE.0.0D0) THEN
	      DERIV(N,INDX(9),5)=-VEL_P/PARALLAX !  p
	    END IF
	    DO II=INDX(10),INDX(15)
	      DERIV(N,II,5)=0.0D0 !  Proper motion not relevant here
	    END DO
	    DERIV(N,INDX(16),5)=1.0D0
	    IF (aAU.NE.0.0D0) THEN
	      DERIV(N,INDX(17),5)=VEL_P/aAU
	    END IF
C  Section for spectroscopic primary apsidal motion
	    IF (CHOICE(19:19).EQ.'A') THEN
	      DERIV(N,INDX(78),5)=DERIV(N,INDX(6),5)*(TIME(N,1)-
	1     PARAMETRS(INDX(2)))
	    END IF
C  Now do the spectroscopic secondary.
	    IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') THEN
C  Compute the velocity of the secondary.  We can't simply use 
C  FUNCTN(N,6)-V_SYS, because that is a sum of the secondary velocity and
C  a possible subcomponent(s) velocity.  We can however use
C  VEL_AMP_SEC which is available in the COMMON block. 
	      VEL_S=-VEL_AMP_SEC*(DCOSD(v+w)+ECCENTR*DCOSD(w))
	      IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1) THEN !  a(") of pri.
				!  Parallax and ratio, 5 cases
	        DO II=INDX(1),INDX(7)
	          DERIV(N,II,6)=(1.0D0-RATIO_a)*DERIV(N,II,5)
	        END DO
	        DERIV(N,INDX(8),6)=-VEL_P !  RATIO_a
	        DERIV(N,INDX(9),6)=-VEL_S/PARALLAX !  p
	        DO II=INDX(10),INDX(15)
	          DERIV(N,II,6)=0.0D0
	        END DO
	        DERIV(N,INDX(16),6)=1.0D0
	        DERIV(N,INDX(17),6)=0.0D0
	      END IF
	      IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.2) THEN !  a(") of sec.
				!  Parallax, no ratio, 1 case
	        DO II=INDX(1),INDX(8)
	          DERIV(N,II,6)=-DERIV(N,II,5)
	        END DO
	        DERIV(N,INDX(9),6)=-VEL_S/PARALLAX !  p
	        DO II=INDX(10),INDX(15)
	          DERIV(N,II,6)=0.0D0
	        END DO
	        DERIV(N,INDX(16),6)=1.0D0
	        DERIV(N,INDX(17),6)=0.0D0
	      ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.1) THEN !  a(AU) of pri.
			!  No parallax, ratio, 1 case:  CHOICE(4:4)='B'
	        DO II=INDX(1),INDX(7)
	          DERIV(N,II,6)=(1.0-RATIO_a)*DERIV(N,II,5)
	        END DO
	        DERIV(N,INDX(8),6)=-VEL_P !  RATIO_a
	        DERIV(N,INDX(9),6)=0.0D0 !  p
	        DO II=INDX(10),INDX(15)
	          DERIV(N,II,6)=0.0D0
	        END DO
	        DERIV(N,INDX(16),6)=1.0D0
	        DERIV(N,INDX(17),6)=(1.0D0-RATIO_a)*DERIV(N,INDX(17),5) !  aAU
	      ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.2) THEN !  a(AU) of sec.
			!  No parallax or ratio, 3 cases
C  a(AU) of secondary is allowed to override a(") of primary in the case
C  CHOICE(1:1)='P', CHOICE(4:4)='S'
	        DO II=INDX(1),INDX(8)
	          DERIV(N,II,6)=-DERIV(N,II,5)
	        END DO
	        DERIV(N,INDX(9),6)=0.0D0 !  p
	        DO II=INDX(10),INDX(15)
	          DERIV(N,II,6)=0.0D0
	        END DO
	        DERIV(N,INDX(16),6)=1.0D0
	        DERIV(N,INDX(17),6)=VEL_S/aAU
	      END IF
C  Section for spectroscopic secondary apsidal motion
	      IF (CHOICE(19:19).EQ.'A') THEN !  This is for the spectroscopic
					     !  secondary
	        DERIV(N,INDX(78),6)=DERIV(N,INDX(6),6)*(TIME(N,1)-
	1       PARAMETRS(INDX(2)))
	      END IF
	    END IF !  Ends IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B')
C  If the primary has a spectroscopic subcomponent, compute the derivatives
	    IF (SPECTROSCOPIC_PRI_SUB) THEN
	      DO I=1,IPRISUB !  Loops over the subcomponents
	        IF (OPERATE) THEN
	          w_PRI_SUB(I)=w_save_PRI_SUB(I)+APSIDAL_MOTION_PRI_SUB(I)*
	1         (TIME(N,1)-LTT_COR(N)-T)
C  If apsidal motion is not to be calculated for a particular subcomponent,
C  then APSIDAL_MOTION_PRI_SUB(I) will be zero in the above equation.
	        ELSE
	          w_PRI_SUB(I)=w_save_PRI_SUB(I)+APSIDAL_MOTION_PRI_SUB(I)*
	1         (TIME(N,1)-T)
	        END IF
	        IOFF=5*(I-1)
	        M=MU_PRI_SUB(I)*(TIME(N,1)-T_PRI_SUB(I))
	        E=ECC_ANOM_PRI_SUB(N,I) !  Was calculated in NLFUN3
	        FRAC=SQRT(ABS((1.0D0+ECCENTR_PRI_SUB(I))/
	1       (1.0D0-ECCENTR_PRI_SUB(I))))
	        v_PRI_SUB(I)=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	        ALPHA6_PRI_SUB(I)=DCOSD(v_PRI_SUB(I)+w_PRI_SUB(I))+
	1       ECCENTR_PRI_SUB(I)*DCOSD(w_PRI_SUB(I))
	        BETA1_PRI_SUB(I)=DSQRT(1.0D0-ECCENTR_PRI_SUB(I)**2)/
	1       (1.0D0-ECCENTR_PRI_SUB(I)*DCOS(E))
	        BETA5_PRI_SUB(I)=(1.0D0+ECCENTR_PRI_SUB(I))/
	1       (1.0D0-ECCENTR_PRI_SUB(I))
	        BETA6_PRI_SUB(I)=1.0D0-ECCENTR_PRI_SUB(I)*DCOS(E)
C d(E)/dP = GAMMA1; d(E)/dT = GAMMA2; d(E)/d(ECCENTR) = GAMMA3
	        A1=1.0D0-ECCENTR_PRI_SUB(I)*DCOS(E)
	        GAMMA1_PRI_SUB(I)=-M/(P_PRI_SUB(I)*A1)
	        GAMMA2_PRI_SUB(I)=-2.0D0*PI/(P_PRI_SUB(I)*A1)
C  Compute the velocity of the primary subcomponent.
C  VEL_AMP_PRI_SUB(I) is available in the COMMON block
	        VEL_P=VEL_AMP_PRI_SUB(I)*(DCOSD(v_PRI_SUB(I)+w_PRI_SUB(I))+
	1       ECCENTR_PRI_SUB(I)*DCOSD(w_PRI_SUB(I)))
	        DERIV(N,INDX(18)+IOFF,5)=-VEL_P/P_PRI_SUB(I)-
	1       VEL_P/ALPHA6_PRI_SUB(I)*DSIND(v_PRI_SUB(I)+w_PRI_SUB(I))*
	2       BETA1_PRI_SUB(I)*GAMMA1_PRI_SUB(I) !  P_PRI_SUB(I)
	        DERIV(N,INDX(19)+IOFF,5)=-VEL_P/ALPHA6_PRI_SUB(I)*
	1       DSIND(v_PRI_SUB(I)+w_PRI_SUB(I))*BETA1_PRI_SUB(I)*
	2       GAMMA2_PRI_SUB(I) !  T_PRI_SUB(I)
	        DERIV(N,INDX(20)+IOFF,5)=VEL_P*(ECCENTR_PRI_SUB(I)/
	1       (1.0D0-ECCENTR_PRI_SUB(I)**2)-
	2       DSIND(v_PRI_SUB(I)+w_PRI_SUB(I))*DSIN(E)/ALPHA6_PRI_SUB(I)/
	3       DSQRT(1.0D0-ECCENTR_PRI_SUB(I)**2)/BETA6_PRI_SUB(I)-
	4       DSIND(v_PRI_SUB(I)+w_PRI_SUB(I))*DSIN(E)/ALPHA6_PRI_SUB(I)*
	5       DSQRT(1.0D0-ECCENTR_PRI_SUB(I)**2)/BETA6_PRI_SUB(I)**2
	6       +DCOSD(w_PRI_SUB(I))/ALPHA6_PRI_SUB(I)) !  e_PRI_SUB(I)
	        DERIV(N,INDX(21)+IOFF,5)=-VEL_P/ALPHA6_PRI_SUB(I)*
	1       (DSIND(v_PRI_SUB(I)+w_PRI_SUB(I))+
	2       ECCENTR_PRI_SUB(I)*DSIND(w_PRI_SUB(I)))/DEG_RAD !  w_PRI_SUB(I)
	        DERIV(N,INDX(22)+IOFF,5)=VEL_P/aAU_PRI_SUB(I)
C  If any of the primary spectroscopic subcomponents has complementary 
C  spectral data from the associated secondary, compute the derivatives.
C  We use the range CHOICE(9:13) to indicate the presence of primary 
C  subcomponent secondaries.
C  The indexes of the data dimension run from 7 to 11.
C  The ratios of semimajor axes are referenced by INDX(68) to INDX(72)
	        IF (CHOICE(8+I:8+I).EQ.'T') THEN
	          DO II=1,5
	            DERIV(N,INDX(18)+II-1+IOFF,6+I)=
	1           (1.0D0-RATIO_a_PRI_SUB_SEC(I))*
	2           DERIV(N,INDX(18)+II-1+IOFF,5)
	            DERIV(N,INDX(67+I),6+I)=-VEL_P
	          END DO
	        END IF
	      END DO
	    END IF !  Ends IF (SPECTROSCOPIC_PRI_SUB)
C  If the secondary has a spectroscopic subcomponent, compute the derivatives
	    IF (SPECTROSCOPIC_SEC_SUB) THEN
	      DO I=1,ISECSUB !  Loops over the subcomponents
	        IF (OPERATE) THEN
	          w_SEC_SUB(I)=w_save_SEC_SUB(I)+APSIDAL_MOTION_SEC_SUB(I)*
	1         (TIME(N,1)-LTT_COR(N)-T)
	        ELSE
	          w_SEC_SUB(I)=w_save_SEC_SUB(I)+APSIDAL_MOTION_SEC_SUB(I)*
	1         (TIME(N,1)-T)
	        END IF
	        IOFF=5*(I-1)
	        M=MU_SEC_SUB(I)*(TIME(N,1)-T_SEC_SUB(I))
	        E=ECC_ANOM_SEC_SUB(N,I) !  Was calculated in NLFUN3
	        FRAC=SQRT(ABS((1.0D0+ECCENTR_SEC_SUB(I))/
	1       (1.0D0-ECCENTR_SEC_SUB(I))))
	        v_SEC_SUB(I)=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	        ALPHA6_SEC_SUB(I)=DCOSD(v_SEC_SUB(I)+w_SEC_SUB(I))+
	1       ECCENTR_SEC_SUB(I)*DCOSD(w_SEC_SUB(I))
	        BETA1_SEC_SUB(I)=DSQRT(1.0D0-ECCENTR_SEC_SUB(I)**2)/
	1       (1.0D0-ECCENTR_SEC_SUB(I)*DCOS(E))
	        BETA5_SEC_SUB(I)=(1.0+ECCENTR_SEC_SUB(I))/
	1       (1.0D0-ECCENTR_SEC_SUB(I))
	        BETA6_SEC_SUB(I)=1.0D0-ECCENTR_SEC_SUB(I)*DCOS(E)
C d(E)/dP = GAMMA1; d(E)/dT = GAMMA2; d(E)/d(ECCENTR) = GAMMA3
	        A1=1.0D0-ECCENTR_SEC_SUB(I)*DCOS(E)
	        GAMMA1_SEC_SUB(I)=-M/(P_SEC_SUB(I)*A1)
	        GAMMA2_SEC_SUB(I)=-2.0D0*PI/(P_SEC_SUB(I)*A1)
C  Compute the velocity of the secondary subcomponent.
C  VEL_AMP_SEC_SUB(I) is available in the COMMON block
	        VEL_S=VEL_AMP_SEC_SUB(I)*(DCOSD(v_SEC_SUB(I)+w_SEC_SUB(I))+
	1       ECCENTR_SEC_SUB(I)*DCOSD(w_SEC_SUB(I)))
	        DERIV(N,INDX(43)+IOFF,6)=-VEL_S/P_SEC_SUB(I)-
	1       VEL_S/ALPHA6_SEC_SUB(I)*DSIND(v_SEC_SUB(I)+w_SEC_SUB(I))*
	2       BETA1_SEC_SUB(I)*GAMMA1_SEC_SUB(I) !  P_SEC_SUB(I)
	        DERIV(N,INDX(44)+IOFF,6)=-VEL_S/ALPHA6_SEC_SUB(I)*
	1       DSIND(v_SEC_SUB(I)+w_SEC_SUB(I))*BETA1_SEC_SUB(I)*
	2       GAMMA2_SEC_SUB(I) !  T_SEC_SUB(I)
	        DERIV(N,INDX(45)+IOFF,6)=VEL_S*(ECCENTR_SEC_SUB(I)/
	1       (1.0D0-ECCENTR_SEC_SUB(I)**2)-
	2       DSIND(v_SEC_SUB(I)+w_SEC_SUB(I))*DSIN(E)/ALPHA6_SEC_SUB(I)/
	3       DSQRT(1.0D0-ECCENTR_SEC_SUB(I)**2)/BETA6_SEC_SUB(I)-
	4       DSIND(v_SEC_SUB(I)+w_SEC_SUB(I))*DSIN(E)/ALPHA6_SEC_SUB(I)*
	5       DSQRT(1.0D0-ECCENTR_SEC_SUB(I)**2)/BETA6_SEC_SUB(I)**2
	6       +DCOSD(w_SEC_SUB(I))/ALPHA6_SEC_SUB(I)) !  e_SEC_SUB(I)
	        DERIV(N,INDX(46)+IOFF,6)=-VEL_S/ALPHA6_SEC_SUB(I)*
	1       (DSIND(v_SEC_SUB(I)+w_SEC_SUB(I))+
	2       ECCENTR_SEC_SUB(I)*DSIND(w_SEC_SUB(I)))/DEG_RAD !  w_SEC_SUB(I)
	        DERIV(N,INDX(47)+IOFF,6)=VEL_S/aAU_SEC_SUB(I)
C  If any of the secondary spectroscopic subcomponents has complementary 
C  spectral data from the associated secondary, compute the derivatives.
C  We use the range CHOICE(14:18) to indicate the presence of secondary
C  subcomponent secondaries.
C  The indexes of the data dimension run from 12 to 16.
C  The ratios of semimajor axes are referenced by INDX(73) to INDX(77)
	        IF (CHOICE(13+I:13+I).EQ.'T') THEN
	          DO II=1,5
	            DERIV(N,INDX(43)+II-1+IOFF,11+I)=
	1           (1.0D0-RATIO_a_SEC_SUB_SEC(I))*
	2           DERIV(N,INDX(18)+II-1+IOFF,6)
	            DERIV(N,INDX(72+I),11+I)=-VEL_S
	          END DO
	        END IF
	      END DO
	    END IF !  Ends IF (SPECTROSCOPIC_SEC_SUB)
C  Section for apsidal motion of subcomponents
	    JJ=16
	    KK=14
	    DO I=20,24 !  Primary subcomponent
	      IF (CHOICE(I:I).EQ.'A') THEN !  20 to 24
	        II=I+59 !  79 to 83, Apsidal motion
	        JJ=JJ+5 !  21 to 41, w 
	        KK=KK+5 !  19 to 30, T
	        DERIV(N,INDX(II),5)=DERIV(N,INDX(JJ),5)*(TIME(N,1)-
	1       PARAMETRS(INDX(KK)))
C  If any of the primary spectroscopic subcomponents have complementary
C  spectral data from the associated seconday, calculate those derivatives
	        IF (CHOICE(I-11:I-11).EQ.'T') THEN !  9 to 13
	          DERIV(N,INDX(II),I-13)=DERIV(N,INDX(JJ),I-13)*(TIME(N,1)-
	1         PARAMETRS(INDX(KK)))
	        END IF
	      END IF
	    END DO
	    JJ=41
	    KK=39
	    DO I=25,29 !  Secondary subcomponent
	      IF (CHOICE(I:I).EQ.'A') THEN !  25 to 29
	        II=I+59 !  84 to 88
	        JJ=JJ+5 !  46 to 66
	        KK=KK+5 !  44 to 64
	        DERIV(N,INDX(II),6)=DERIV(N,INDX(JJ),6)*(TIME(N,1)-
	1       PARAMETRS(INDX(KK)))
C  If any of the secondary spectroscopic subcomponents have complementary
C  spectral data from the associated seconday, calculate those derivatives
	        IF (CHOICE(I-11:I-11).EQ.'T') THEN !  14 to 18
	          DERIV(N,INDX(II),I-13)=DERIV(N,INDX(JJ),I-13)*(TIME(N,1)-
	1         PARAMETRS(INDX(KK)))
	        END IF
	      END IF
	    END DO
	  END IF !  Ends IF (SPECTROSCOPIC)
	END DO
C
	RETURN
	END
