	SUBROUTINE NLFUN3(TIME,MAXNPOINTS_DUM,DIMENSN_IND,PARAMETRS,
	1CHOICE,OPERATE,ITIME,FUNCTN,DIMENSN_DATA)
C
C  Note:  This is the Unix version
C
C       This subroutine helps NONLINFIT in a least squares fit for the orbital
C  parameters of binary star system.  The X direction is assumed to be
C  east-west (increasing to the east) and the Y direction is assumed to be
C  north-south (increasing to the north).  Astrometric, speckle, and
C  spectroscopic data can be simulataneously be used in the fit.
C			Donald H. Gudehus, March, 1986
C		Version 2, Donald H. Gudehus, May, 1994
C		Added capability to solve for primary and secondary
C		simultaneously.  Added capability to solve for spectroscopic
C		binary, and spectroscopic and visual binary simultaneously.
C		Version 3, Donald H. Gudehus, June, 1995
C		Added capability to calculate velocity of subcomponent
C		of primary and/or secondary.
C		Version 4, D. H. G., September, 1997
C		Changed COMMON blocks for OpenVMS Alpha and UNIX
C		Version 5, D. H. G., November 25, 1999
C		Increased number of points to 5000
C		Version 6, D. H. G., February 25, 2000
C		Added code for apsidal motion of main components
C		Version 7, D. H. G., May 2, 2001
C		Program now solves for the true proper motion acceleration
C		rather than the coefficient of the quadratic time factor.
C		Version 8, D. H. G., September 25, 2001
C		Added apsidal motion for subcomponents.  Removed call
C		to BINARY_INDEXES.  The call is made in BINARY.FOR so that 
C		INDX for the solution can be used to properly address
C		PARAMETRS when plotting.
C		Version 9, D. H. G., October 9, 2002
C		Added include statement for binary_parameters.for
C		Version 10, D. H. G., February 26, 2004
C		Added boundary checking for eccentricity and angle from
C		node to periastron, for the spectroscopic subcomponents.
C		Version 11, D. H. G., February, 2005
C		Added support for one-dimensional interferometric data
C
C	TIME		(C)	The array containing the independent variable
C				values
C	MAXNPOINTS_DUM	(C)	The first dimension of the array TIME
C	DIMENSN_IND	(C)	The second dimension of the array TIME
C	PARAMETRS	(C)	The array containing the parameters used to
C				evaluate the function
C	CHOICE		(C)	The choice determining which function
C				to use within a category.  The possibilities
C				are:  
C				CHOICE(1:1)='P'   Primary of visual binary
C					   ='S'   Secondary of visual binary
C					   ='B'	  Both members of visual
C						  binary
C					   ='N'   No visual binary data
C				CHOICE(2:2)='R'   Parallax and proper motion
C						  removed from primary visual
C						  binary data
C					   ='I'   Parallax and proper motion
C						  included in primary data
C				CHOICE(3:3)='A'   Parallax and proper motion
C						  removed from secondary visual
C						  binary data;  orbit type is
C						  absolute
C					   ='R'   Parallax and proper motion
C						  removed from secondary visual
C						  binary data;  orbit type is
C						  relative
C					   ='I'   Parallax and proper motion
C						  included in secondary data;
C						  implies absolute
C				CHOICE(4:4)='P'	  Velocity data available for
C						  primary
C					   ='S'	  Velocity data available for
C						  secondary
C					   ='B'	  Velocity data available for
C						  primary and secondary
C					   ='N'   No spectroscopic binary
C						  data available
C				CHOICE(5:5)='n'	  1) The primary includes 
C						  velocity subcomponents equal
C						  in number to n, where said
C						  primary is taken to be the
C						  primary of the subsystem.
C						  n should be in the range
C						  1 to 5.  If this
C						  position is blank, no
C						  subcomponents exist.  The
C						  action to be carried out is
C						  controlled by  CHOICE(6:6).
C						  2) If CHOICE(6:6)='o', then
C						  n is the actual subcompo-
C						  nent.
C				CHOICE(6:6)=' '	  Adds all velocity sub-
C						  components, up to n, to the
C						  primary.
C					   ='o'	  Evaluates only primary
C						  subcomponent n.
C					   ='a'	  Evaluates the sum of all
C						  subcomponents.
C					   ='m'   Evaluates only the main
C						  component
C				CHOICE(7:7)='n'	  1) The secondary includes
C						  velocity subcomponents equal
C						  in number to n, where said
C						  secondary is taken to be the
C						  primary of the subsystem.
C						  n should be in the range
C						  1 to 5.  If this
C						  position is blank, no
C						  subcomponents exist.  The
C						  action to be carried out is
C						  controlled by  CHOICE(8:8).
C						  2)  If CHOICE(8:8)='o', then
C						  n is the actual subcompo-
C						  nent.
C				CHOICE(8:8)=' '	  Adds all velocity sub-
C						  components, up to n, to the
C						  secondary.
C					   ='o'	  Evaluates only secondary
C						  subcomponent n.
C					   ='a'	  Evaluates the sum of all
C						  subcomponents.
C					   ='m'	  Evaluates only the main 
C						  component
C				CHOICE(9:13)='T'  Evaluates one or more
C						  velocities of secondaries 
C						  of primary subcomponents 1:5
C				CHOICE(14:18)='T' Evaluates one or more
C						  velocities of secondaries 
C						  of secondary subcomponents 
C						  1:5
C				CHOICE(19:19)='A' Apsidal motion is evaluated
C						  for main components
C				CHOICE(20:24)='A' Apsidal motion is evaluated
C						  for one or more primary
C						  subcomponents 1:5
C				CHOICE(25:29)='A' Apsidal motion is evaluated
C						  for one or more secondary
C						  subcomponents 1:5
C				CHOICE(30:30)='P' Primary 1D interferometric
C					     ='S' Secondary 1D interferometric
C					     ='B' Both members
C					     ='N' No 1D interferometric data
C				CHOICE(31:31)='R' Parallax and proper motion
C						  removed from primary 1D
C						  interferometric data
C					     ='I' Parallax and proper motion
C						  included in primary 1D 
C						  interferometric data
C				CHOICE(32:32)='A' Parallax and proper motion
C						  removed from secondary 
C						  1D interferometric data;
C						  orbit type is absolute
C					     ='R' Parallax and proper motion
C						  removed from secondary 
C						  1D interferometric data;
C						  orbit type is relative
C					     ='I' Parallax and proper motion
C						  included in secondary 1D
C						  interferometric data;
C						  implies absolute
C
C	OPERATE		(C)	A logical key which determines what to do.
C				If .FALSE., the function is evaluated, if 
C				.TRUE.,  a user defined operation is also 
C				carried	out, e.g. convolution.  In this 
C				subroutine, when .TRUE., the light travel 
C				time correction	is made for spectroscopic 
C				orbits.
C	ITIME		(C)	An array containing the starting and ending
C				indexes for the evaluation of FUNCTN
C	FUNCTN		(R)	The output array for the function.
C	DIMENSN_DATA	(C)	The second dimension of the array FUNCTN
C				The indexes are assigned as follows:
C				1	Primary X position
C				2	Primary Y position
C				3	Secondary X position
C				4	Secondary Y position
C				5	Primary radial velocity plus any
C					subcomponents
C				6	Secondary radial velocity plus any
C					subcomponents
C				7:11	Radial velocities of secondaries as 
C					subcomponents of primary
C				12:16	Radial velocities of secondaries as
C					subcomponents of secondary
C				17	One-dimensional interferometric
C					projected separation for primary
C				18	One-dimensional interferometric
C					projected separation for secondary
C
	INCLUDE 'binary_parameters.for'
C
	INTEGER DIMENSN_IND,DIMENSN_DATA
	DIMENSION ITIME(2),TIME(MAXNPOINTS_DUM,DIMENSN_IND),PARAMETRS(1),
	1FUNCTN(MAXNPOINTS_DUM,DIMENSN_DATA)
	CHARACTER*(*) CHOICE
	LOGICAL OPERATE,PLOT,FIRSTTIME
	REAL*8 TIME,PARAMETRS,FUNCTN,PI,DEG_RAD,
	1E,E1,E2,ED,P,P_PRI_SUB(NSUBS_MAX),P_SEC_SUB(NSUBS_MAX),
	2T,T_PRI_SUB(NSUBS_MAX),T_SEC_SUB(NSUBS_MAX),
	3ECCENTR,ECCENTR_PRI_SUB(NSUBS_MAX),ECCENTR_SEC_SUB(NSUBS_MAX),
	4OMEGA,INCLINA,w,w_save,APSIDAL_MOTION,
	5APSIDAL_MOTION_PRI_SUB(NSUBS_MAX),
	6APSIDAL_MOTION_SEC_SUB(NSUBS_MAX),w_PRI_SUB(NSUBS_MAX),
	7w_save_PRI_SUB(NSUBS_MAX),
	4w_SEC_SUB(NSUBS_MAX),w_save_SEC_SUB(NSUBS_MAX),
	4a,aAU,aAU_PRI_SUB(NSUBS_MAX),
	5aAU_SEC_SUB(NSUBS_MAX),r,MU,MU_PRI_SUB(NSUBS_MAX),
	6MU_SEC_SUB(NSUBS_MAX),M,DELTATIME,PARALLAX,
	7PM_X,PM_Y,CM_X,CM_Y,Q_X,Q_Y,DELTATIME2,DELTA0,
	8v,v_PRI_SUB(NSUBS_MAX),v_SEC_SUB(NSUBS_MAX),FRAC,
	9SINTHETA_OM,COSTHETA_OM,ATANTHETA_OM,
	1THETA,RHO,DELTA1,X,Y,RATIO_a,RATIO_a_PRI_SUB_SEC(NSUBS_MAX),
	2RATIO_a_SEC_SUB_SEC(NSUBS_MAX),SPECT_FACT,V_SYS,TEMP(NSUBS_MAX),
	3V_PRI_SAVE,V_SEC_SAVE,SUM,TEMP1,TEMP2
C
	CHARACTER*1 TYPE_P,TYPE_S,TYPE_VEL_P,TYPE_VEL_S,TYPE_INTER_P,
	1TYPE_INTER_S
	COMMON /CHAR_SCRATCH_BLOCK/TYPE_P,TYPE_S,TYPE_VEL_P,TYPE_VEL_S,
	1TYPE_INTER_P,TYPE_INTER_S

	INTEGER NPOINTS_DATA_VEL_P_SUB_S(NSUBS_MAX),
	1NPOINTS_DATA_VEL_S_SUB_S(NSUBS_MAX)
	COMMON /INTEGER4_SCRATCH_BLOCK/NPOINTSPXPY,NPOINTS_DATA_P,
	1NPOINTS_DATA_S,NPOINTS_VIS,NPOINTS_DATA_VEL_P,
	2NPOINTS_DATA_VEL_S,NPOINTS_SP,NPOINTS_DATA_VEL_P_SUB_S,
	3NPOINTS_DATA_VEL_S_SUB_S,NPOINTS_VEL_P_S,NPOINTS_VEL_S_S,
	4NPOINTS_DATA_INTER_P,NPOINTS_DATA_INTER_S,NPOINTS_INTER

	REAL*8 PX(MAXNPOINTS_COM),PY(MAXNPOINTS_COM),
	1FR_ANGLE(MAXNPOINTS_COM),EPOCH,
	1VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB(NSUBS_MAX),
	2VEL_AMP_SEC_SUB(NSUBS_MAX),VEL_AMP_PRI_SUB_SEC(NSUBS_MAX),
	3VEL_AMP_SEC_SUB_SEC(NSUBS_MAX),INCLINA_WORK,
	4INCLINA_WORK_PRI_SUB(NSUBS_MAX),INCLINA_WORK_SEC_SUB(NSUBS_MAX),
	5ECC_ANOM(MAXNPOINTS_COM),ECC_ANOM_PRI_SUB(MAXNPOINTS_COM,NSUBS_MAX),
	6ECC_ANOM_SEC_SUB(MAXNPOINTS_COM,NSUBS_MAX),LTT_COR(MAXNPOINTS_COM),
	7PHOTOCENTRIC_TRUE_SEMIMAJAX,FUNCTN_TEMP(MAXNPOINTS_COM,4)
	  !  PX and PY are the parallax factors.  VEL_AMP_XXX are returned
	  !  values which are used in BINARY and NLDER3.
	COMMON /REAL8_SCRATCH_BLOCK/ PX,PY,FR_ANGLE,
	4EPOCH,VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB,VEL_AMP_SEC_SUB,
	5VEL_AMP_PRI_SUB_SEC,VEL_AMP_SEC_SUB_SEC,INCLINA_WORK,
	6INCLINA_WORK_PRI_SUB,INCLINA_WORK_SEC_SUB,
	7ECC_ANOM,ECC_ANOM_PRI_SUB,ECC_ANOM_SEC_SUB,LTT_COR,
	8PHOTOCENTRIC_TRUE_SEMIMAJAX,FUNCTN_TEMP
C
	INTEGER SEMIMAJOR_AXIS_ASSIGNMENT(2),DIMENSN_DATA_DUM,
	1DIMENSN_DATA_USED,INDX(MAXNUMPARAM)
	COMMON /BINARY_INTEGER4/ SEMIMAJOR_AXIS_ASSIGNMENT,
	1DIMENSN_DATA_DUM,DIMENSN_DATA_USED,INDX

	LOGICAL VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
	COMMON /BINARY_LOGICALS/ VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
C
	DATA FIRSTTIME /.TRUE./
C
C  The assignments of the second indexes of the array FUNCTN_TEMP are:
C       1      Primary interferometric X position
C       2      Primary interferometric Y position
C       3      Secondary interferometric X position
C       4      Secondary interferometric Y positio

	PI=3.14159265358979D0
	DEG_RAD=180.0D0/PI
	SPECT_FACT=2.0D0*PI*1.4959791D8/3600.0D0/24.0D0/365.2421988D0
C
	CALL ALPHA(%REF(CHOICE),1,LEN(CHOICE),NCHAR,NFULL,NREST,1)
	PLOT=.FALSE.
	IF (CHOICE(NCHAR:NCHAR).EQ.'%') THEN
	  PLOT=.TRUE.
	  CHOICE(NCHAR:NCHAR)=' '
	END IF
C
	P=ABS(PARAMETRS(1))
	PARAMETRS(1)=P
	T=PARAMETRS(2)
C	ECCENTR=ABS(PARAMETRS(3))
	ECCENTR=PARAMETRS(3)
	IF (ECCENTR.GT.1.0D0) ECCENTR=2.0D0-ECCENTR !  Prevents impossible
					!  solutions for binaries
	PARAMETRS(3)=ECCENTR
	IF (SPECTROSCOPIC.AND..NOT.VISUAL.AND..NOT.INTERFEROMETRIC) THEN
C  If only spectroscopic data is available, we can't solve for the
C  node or inclination angle.  In this case, we use an offset.  Note that
C  a is in units of AU and is projected.
	  OMEGA=0.0D0
	  INCLINA=90.0D0
	ELSE IF (VISUAL.OR.INTERFEROMETRIC) THEN !  Either SPECTROSCOPIC or
						 !  not SPECTROSCOPIC
C  a is in units of arc seconds
	  OMEGA=MOD(PARAMETRS(4),360.0D0)
	  IF (OMEGA.LT.0.0D0) OMEGA=OMEGA+360.0D0
	  PARAMETRS(4)=OMEGA
	  INCLINA=MOD(PARAMETRS(5),360.0D0)
	  PARAMETRS(5)=INCLINA
	END IF
	w=MOD(PARAMETRS(INDX(6)),360.0D0)
	IF (w.LT.0.0D0) w=w+360.0D0 
	PARAMETRS(INDX(6))=w
	w_save=w !  For apsidal motion
	IF (VISUAL.OR.INTERFEROMETRIC) THEN
C  The meaning of "a" comes from the types of file available.  See
C  BINARY_INDEXS.FOR.  
	  a=ABS(PARAMETRS(INDX(7)))
	  PARAMETRS(INDX(7))=a
	ELSE 
C  For a spectroscopic binary, without visual information for the same
C  component, we must use projected "a" in units of AU.  This is introduced as
C  parameter 17.  The interpretation of this is the same as explained above.
	END IF
C  Because parameter number 8 is taken to be the ratio of semimajors axes
C  (total to primary) in the case of a pair of stars, and because this is
C  not used for a single star, and because parameters numbers greater than
C  8 may be used (to solve for proper motion and parallax), an offset is
C  used.
	IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(4:4).EQ.'B'.OR.
C		!  Visual binary pair or a spectroscopic binary pair
	1CHOICE(30:30).EQ.'B') THEN
C		!  Interferometric binary pair 
	  RATIO_a=PARAMETRS(INDX(8))
	ELSE !  Single member
C  Ignore ratio of semimajor axes
	END IF
C  Note that when parallax can't be determined by any means,
C  for the spectroscopic binary, a will have the units of AU.
	IF (PAR_PM.OR.PAR_PM_INTERFERO) THEN
	  PARALLAX=PARAMETRS(INDX(9))
	  IF (PAR_PM) THEN
	    CM_X=PARAMETRS(INDX(10))
	  ELSE
	    CM_X=PARAMETRS(INDX(89))
	  END IF
	  PM_X=PARAMETRS(INDX(11))
	  Q_X=PARAMETRS(INDX(12))
	  IF (PAR_PM) THEN
	    CM_Y=PARAMETRS(INDX(13))
	  ELSE
	    CM_Y=PARAMETRS(INDX(90))
	  END IF
	  PM_Y=PARAMETRS(INDX(14))
	  Q_Y=PARAMETRS(INDX(15))
	ELSE 
	  IF (SOLVE_PAR) THEN
	    PARALLAX=PARAMETRS(INDX(9))
	  END IF
	END IF
	IF (SPECTROSCOPIC) THEN
	  V_SYS=PARAMETRS(INDX(16))
	  IF (.NOT.PAR_PM.AND..NOT.PAR_PM_INTERFERO.AND.
	1 .NOT.SOLVE_PAR) aAU=PARAMETRS(INDX(17))
		!  aAU is a projected value and may apply to the primary
		!  or secondary, depending on whether data is available for
		!  both or the secondary components, respectively.
C  Handle the case of the primary having a subcomponent
	  IF (SPECTROSCOPIC_PRI_SUB) THEN
	    DECODE(1,1001,CHOICE(5:5)) IPRISUB
1001	    FORMAT(I1)
	    P_PRI_SUB(1)=PARAMETRS(INDX(18))
	    T_PRI_SUB(1)=PARAMETRS(INDX(19))
	    ECCENTR_PRI_SUB(1)=PARAMETRS(INDX(20))
	    w_PRI_SUB(1)=MOD(PARAMETRS(INDX(21)),360.0D0)
	    aAU_PRI_SUB(1)=PARAMETRS(INDX(22))
	    P_PRI_SUB(2)=PARAMETRS(INDX(23))
	    T_PRI_SUB(2)=PARAMETRS(INDX(24))
	    ECCENTR_PRI_SUB(2)=PARAMETRS(INDX(25))
	    w_PRI_SUB(2)=MOD(PARAMETRS(INDX(26)),360.0D0)
	    aAU_PRI_SUB(2)=PARAMETRS(INDX(27))
	    P_PRI_SUB(3)=PARAMETRS(INDX(28))
	    T_PRI_SUB(3)=PARAMETRS(INDX(29))
	    ECCENTR_PRI_SUB(3)=PARAMETRS(INDX(30))
	    w_PRI_SUB(3)=MOD(PARAMETRS(INDX(31)),360.0D0)
	    aAU_PRI_SUB(3)=PARAMETRS(INDX(32))
	    P_PRI_SUB(4)=PARAMETRS(INDX(33))
	    T_PRI_SUB(4)=PARAMETRS(INDX(34))
	    ECCENTR_PRI_SUB(4)=PARAMETRS(INDX(35))
	    w_PRI_SUB(4)=MOD(PARAMETRS(INDX(36)),360.0D0)
	    aAU_PRI_SUB(4)=PARAMETRS(INDX(37))
	    P_PRI_SUB(5)=PARAMETRS(INDX(38))
	    T_PRI_SUB(5)=PARAMETRS(INDX(39))
	    ECCENTR_PRI_SUB(5)=PARAMETRS(INDX(40))
	    w_PRI_SUB(5)=MOD(PARAMETRS(INDX(41)),360.0D0)
	    aAU_PRI_SUB(5)=PARAMETRS(INDX(42))
	    DO I=1,NSUBS_MAX
	      IF (ECCENTR_PRI_SUB(I).GT.1.0D0)
	1     ECCENTR_PRI_SUB(I)=2.0D0-ECCENTR_PRI_SUB(I)
	      PARAMETRS(INDX(15+5*I))=ECCENTR_PRI_SUB(I)
C
	      IF (w_PRI_SUB(I).LT.0.0D0)
	1     w_PRI_SUB(I)=w_PRI_SUB(I)+360.0D0
	      PARAMETRS(INDX(16+5*I))=w_PRI_SUB(I)
	      w_save_PRI_SUB(I)=w_PRI_SUB(I)
	    END DO
	  END IF
C  Handle the case of the secondary having a subcomponent
	  IF (CHOICE(7:7).NE.' ') THEN
	    DECODE(1,1001,CHOICE(7:7)) ISECSUB
	    P_SEC_SUB(1)=PARAMETRS(INDX(43))
	    T_SEC_SUB(1)=PARAMETRS(INDX(44))
	    ECCENTR_SEC_SUB(1)=PARAMETRS(INDX(45))
	    w_SEC_SUB(1)=MOD(PARAMETRS(INDX(46)),360.0D0)
	    aAU_SEC_SUB(1)=PARAMETRS(INDX(47))
	    P_SEC_SUB(2)=PARAMETRS(INDX(48))
	    T_SEC_SUB(2)=PARAMETRS(INDX(49))
	    ECCENTR_SEC_SUB(2)=PARAMETRS(INDX(50))
	    w_SEC_SUB(2)=MOD(PARAMETRS(INDX(51)),360.0D0)
	    aAU_SEC_SUB(2)=PARAMETRS(INDX(52))
	    P_SEC_SUB(3)=PARAMETRS(INDX(53))
	    T_SEC_SUB(3)=PARAMETRS(INDX(54))
	    ECCENTR_SEC_SUB(3)=PARAMETRS(INDX(55))
	    w_SEC_SUB(3)=MOD(PARAMETRS(INDX(56)),360.0D0)
	    aAU_SEC_SUB(3)=PARAMETRS(INDX(57))
	    P_SEC_SUB(4)=PARAMETRS(INDX(58))
	    T_SEC_SUB(4)=PARAMETRS(INDX(59))
	    ECCENTR_SEC_SUB(4)=PARAMETRS(INDX(60))
	    w_SEC_SUB(4)=MOD(PARAMETRS(INDX(61)),360.0D0)
	    aAU_SEC_SUB(4)=PARAMETRS(INDX(62))
	    P_SEC_SUB(5)=PARAMETRS(INDX(63))
	    T_SEC_SUB(5)=PARAMETRS(INDX(64))
	    ECCENTR_SEC_SUB(5)=PARAMETRS(INDX(65))
	    w_SEC_SUB(5)=MOD(PARAMETRS(INDX(66)),360.0D0)
	    aAU_SEC_SUB(5)=PARAMETRS(INDX(67))
	    DO I=1,NSUBS_MAX
	      IF (ECCENTR_SEC_SUB(I).GT.1.0D0)
	1     ECCENTR_SEC_SUB(I)=2.0D0-ECCENTR_SEC_SUB(I)
	      PARAMETRS(INDX(40+5*I))=ECCENTR_SEC_SUB(I)
C
	      IF (w_SEC_SUB(I).LT.0.0D0)
	1     w_SEC_SUB(I)=w_SEC_SUB(I)+360.0D0
	      PARAMETRS(INDX(41+5*I))=w_SEC_SUB(I)
	      w_save_SEC_SUB(I)=w_SEC_SUB(I)
	    END DO
	  END IF
	  DO I=1,5
	    IF (CHOICE(8+I:8+I).EQ.'T'.OR.CHOICE(8+I:8+I).EQ.'o') THEN
	      RATIO_a_PRI_SUB_SEC(I)=PARAMETRS(INDX(67+I))
	    END IF
	  END DO
	  DO I=1,5
	    IF (CHOICE(13+I:13+I).EQ.'T'.OR.CHOICE(13+I:13+I).EQ.'o') THEN
	      RATIO_a_SEC_SUB_SEC(I)=PARAMETRS(INDX(72+I))
	    END IF
	  END DO
	END IF
	IF (CHOICE(19:19).EQ.'A') THEN
	  APSIDAL_MOTION=PARAMETRS(INDX(78))
	ELSE
	  APSIDAL_MOTION=0.0D0
	END IF
	IF (SPECTROSCOPIC_PRI_SUB) THEN
	  DO I=1,5
	    IF (CHOICE(19+I:19+I).EQ.'A') THEN
	      APSIDAL_MOTION_PRI_SUB(I)=PARAMETRS(INDX(78+I))
	    ELSE
	      APSIDAL_MOTION_PRI_SUB(I)=0.0D0
	    END IF
	  END DO
	END IF
	IF (SPECTROSCOPIC_SEC_SUB) THEN
	  DO I=1,5
	    IF (CHOICE(24+I:24+I).EQ.'A') THEN
	      APSIDAL_MOTION_SEC_SUB(I)=PARAMETRS(INDX(83+I))
	    ELSE
	      APSIDAL_MOTION_SEC_SUB(I)=0.0D0
	    END IF
	  END DO
	END IF
	MU=2.0D0*PI/P !  Rate of orbital revolution in radians per year
C  Get the velocity amplitudes
	IF (SPECTROSCOPIC) THEN !  Results of calculation are used for 
				!  velocity of primary and in 
				!  NLDER3 for deriv. of secondary.
	  IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B') THEN !  For primary
	    IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1) THEN !  a(") of pri.
	      VEL_AMP_PRI=SPECT_FACT*a/PARALLAX*DSIND(INCLINA)/P/
	1     SQRT(1.0D0-ECCENTR**2)	
	    ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.1) THEN !  a(AU) of pri.
	      VEL_AMP_PRI=SPECT_FACT*aAU/P/SQRT(1.0D0-ECCENTR**2)
	    END IF
	  END IF
	  IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') THEN !  For secondary
	    IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1) THEN !  a(") of pri.
	      VEL_AMP_SEC=(RATIO_a-1.0D0)*SPECT_FACT*a/PARALLAX*
	1     DSIND(INCLINA)/P/SQRT(1.0D0-ECCENTR**2)
	    END IF
	    IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.2) THEN !  a(") of sec.
	      VEL_AMP_SEC=SPECT_FACT*a/PARALLAX*DSIND(INCLINA)/P/
	1     SQRT(1.0D0-ECCENTR**2)
	    ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.1) THEN !  a(AU) of pri.
	      VEL_AMP_SEC=(RATIO_a-1.0D0)*SPECT_FACT*aAU/P/
	1     SQRT(1.0D0-ECCENTR**2)
	    ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.2) THEN !  a(AU) of sec.
C  a(AU) of secondary is allowed to override a(") of primary in the case
C  CHOICE(1:1)='P', CHOICE(4:4)='S'
	      VEL_AMP_SEC=SPECT_FACT*aAU/P/SQRT(1.0D0-ECCENTR**2)
	    END IF
	  END IF
	END IF
C
	IF (SPECTROSCOPIC_PRI_SUB) THEN
	  DO I=1,IPRISUB
	    MU_PRI_SUB(I)=2.0D0*PI/P_PRI_SUB(I)
	  END DO
	END IF
	IF (SPECTROSCOPIC_SEC_SUB) THEN
	  DO I=1,ISECSUB
	    MU_SEC_SUB(I)=2.0D0*PI/P_SEC_SUB(I)
	  END DO
	END IF
C  Obtain the light travel time correction.  Don't call BINARY_LTT_COR
C  the first time because BINARY_LTT_COR uses the eccentric anomaly 
C  calculated in this subroutine and the eccentric anomaly arrays will 
C  not yet have been filled.  The eccentric anomaly is calculated with the 
C  light travel time correction only after BINARY_LTT_CORR has been called.
	IF (OPERATE.AND..NOT.PLOT.AND..NOT.FIRSTTIME)
	1CALL BINARY_LTT_COR(TIME,MAXNPOINTS_DUM,DIMENSN_IND,PARAMETRS,CHOICE)
	DO N=ITIME(1),ITIME(2) !  Does all the visual and spectroscopic
			       !  binary data points
	  DELTATIME=TIME(N,1)-EPOCH !  Epoch for proper motion
	  DELTATIME2=DELTATIME*DELTATIME
C  Compute parallax and proper motion for the primary if necessary,
C  otherwise initialize the function.
	  IF (CHOICE(2:2).EQ.'I') THEN
	    FUNCTN(N,1)=PARALLAX*PX(N)+CM_X+PM_X*DELTATIME+
	1   0.5D0*Q_X*DELTATIME2
	    FUNCTN(N,2)=PARALLAX*PY(N)+CM_Y+PM_Y*DELTATIME+
	1   0.5D0*Q_Y*DELTATIME2
	  ELSE
	    FUNCTN(N,1)=0.0D0
	    FUNCTN(N,2)=0.0D0
	  END IF
C  Compute parallax and proper motion for the secondary if necessary,
C  otherwise initialize the function.
	  IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(1:1).EQ.'S') THEN
	    IF (CHOICE(3:3).EQ.'I') THEN
	      FUNCTN(N,3)=PARALLAX*PX(N)+CM_X+PM_X*DELTATIME+
	1     0.5D0*Q_X*DELTATIME2
	      FUNCTN(N,4)=PARALLAX*PY(N)+CM_Y+PM_Y*DELTATIME+
	1     0.5D0*Q_Y*DELTATIME2
	    ELSE
	      FUNCTN(N,3)=0.0D0
	      FUNCTN(N,4)=0.0D0
	    END IF
	  END IF
C  Do the above calculation for 1-D interferometric data
	  IF (CHOICE(31:31).EQ.'I') THEN
	    FUNCTN_TEMP(N,1)=PARALLAX*PX(N)+CM_X+PM_X*DELTATIME+
	1   0.5D0*Q_X*DELTATIME2
	    FUNCTN_TEMP(N,2)=PARALLAX*PY(N)+CM_Y+PM_Y*DELTATIME+
	1   0.5D0*Q_Y*DELTATIME2
	  ELSE
	    FUNCTN_TEMP(N,1)=0.0D0
	    FUNCTN_TEMP(N,2)=0.0D0
	  END IF
	  IF (CHOICE(30:30).EQ.'B'.OR.CHOICE(30:30).EQ.'S') THEN
	    IF (CHOICE(32:32).EQ.'I') THEN
	      FUNCTN_TEMP(N,3)=PARALLAX*PX(N)+CM_X+PM_X*DELTATIME+
	1     0.5D0*Q_X*DELTATIME2
	      FUNCTN_TEMP(N,4)=PARALLAX*PY(N)+CM_Y+PM_Y*DELTATIME+
	1     0.5D0*Q_Y*DELTATIME2
	    ELSE
	      FUNCTN_TEMP(N,3)=0.0D0
	      FUNCTN_TEMP(N,4)=0.0D0
	    END IF
	  END IF
C  M   Mean anomaly
C  T   Time of periastron passage
C  E   Eccentric anomaly
C  v   True anomaly
	  IF (OPERATE) THEN
	    M=MU*(TIME(N,1)-LTT_COR(N)-T)
C  The time is corrected for the travel time of light from the projected
C  distance of the primary or secondary as affected by all the respective 
C  subcomponents, to the center of mass.
	  ELSE
	    M=MU*(TIME(N,1)-T)
	  END IF
	  E1=ECC_ANOM(N)
	  DELTA0=1.0D0
	  DELTA1=DELTA0
C  Solve Kepler's equation to find E, the eccentric anomaly
	  DO WHILE (ABS(DELTA0).GT.1.0D-12)
	    E=M+ECCENTR*DSIN(E1)
	    ED=E+E1
	    IF (ABS(ED).LT.1.0D-20) ED=SIGN(1.0D0,ED)*1.0D-20
	    DELTA0=2.0D0*(E-E1)/ED
	    IF (DELTA0.EQ.-DELTA1) E=(E+E1)/2.0D0
	    DELTA1=DELTA0
	    E2=E1
	    E1=E
	  END DO
	  ECC_ANOM(N)=E
C  Take care of any apsidal motion
	  IF (CHOICE(19:19).EQ.'A') THEN
	    IF (OPERATE) THEN
	      w=w_save+APSIDAL_MOTION*(TIME(N,1)-LTT_COR(N)-T)
	    ELSE
	      w=w_save+APSIDAL_MOTION*(TIME(N,1)-T)
	    END IF
	  END IF
C  The following can be found on p. 62 and p. 120 of P. van de Kamp
	  FRAC=SQRT(ABS((1.0D0+ECCENTR)/(1.0D0-ECCENTR)))
	  v=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
	  SINTHETA_OM=DSIND(v+w)*DCOSD(INCLINA)
	  COSTHETA_OM=DCOSD(v+w)
	  IF (COSTHETA_OM.EQ.0.0D0.AND.SINTHETA_OM.EQ.0.0D0) THEN
	    ATANTHETA_OM=0.0D0
	  ELSE
	    ATANTHETA_OM=DATAN2D(SINTHETA_OM,COSTHETA_OM)
	  END IF
C  We know OMEGA since it is an element
	  THETA=OMEGA+ATANTHETA_OM
	  r=a*(1.0D0-ECCENTR*DCOS(E))
	  IF (DCOSD(ATANTHETA_OM).EQ.0.0D0) THEN
	    RHO=r
	  ELSE
	    RHO=r*DCOSD(v+w)/DCOSD(ATANTHETA_OM)
	  END IF
	  X=RHO*DSIND(THETA) !  Zero degrees is to
	  Y=RHO*DCOSD(THETA) !  the north
C  Compute the position of the primary star or the first of a pair
	  IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B') THEN
C  Make an adjustment for the astrometric data which usually yields a
C  photocentric orbit
	    FUNCTN(N,1)=FUNCTN(N,1)+X*PHOTOCENTRIC_TRUE_SEMIMAJAX
	    FUNCTN(N,2)=FUNCTN(N,2)+Y*PHOTOCENTRIC_TRUE_SEMIMAJAX
	  END IF
C  Do the same for the 1-D interferometric case
	  IF (CHOICE(30:30).EQ.'P'.OR.CHOICE(30:30).EQ.'B') THEN
c fix this with independent p_t_s later
	    FUNCTN_TEMP(N,1)=FUNCTN_TEMP(N,1)+X*PHOTOCENTRIC_TRUE_SEMIMAJAX
	    FUNCTN_TEMP(N,2)=FUNCTN_TEMP(N,2)+Y*PHOTOCENTRIC_TRUE_SEMIMAJAX
	  END IF
C  Compute the position of the secondary
C  For the factor PHOTOCENTRIC_TRUE_SEMIMAJAX, the best approach is to
C  introduce another keyword, but for now we take the following approach.
	  IF (CHOICE(1:1).EQ.'B'.AND.
	1 (CHOICE(3:3).EQ.'A'.OR.CHOICE(3:3).EQ.'I')) THEN !  Absolute
c	    FUNCTN(N,3)=FUNCTN(N,3)-(RATIO_a-1.0D0)*X*
c	1   PHOTOCENTRIC_TRUE_SEMIMAJAX_SEC
	    FUNCTN(N,3)=FUNCTN(N,3)-(RATIO_a-1.0D0)*X
c	    FUNCTN(N,4)=FUNCTN(N,4)-(RATIO_a-1.0D0)*Y*
c	1   PHOTOCENTRIC_TRUE_SEMIMAJAX_SEC
	    FUNCTN(N,4)=FUNCTN(N,4)-(RATIO_a-1.0D0)*Y
	  ELSE IF (CHOICE(1:1).EQ.'B'
	1 .AND.CHOICE(3:3).EQ.'R') THEN !  Relative
	    FUNCTN(N,3)=FUNCTN(N,3)-RATIO_a*X
	    FUNCTN(N,4)=FUNCTN(N,4)-RATIO_a*Y
	  ELSE IF (CHOICE(1:1).EQ.'S') THEN !  Relative or absolute
	    IF (CHOICE(4:4).EQ.'B') THEN
C  In this case, the ratio is obtained from the spectroscopic orbit
	      FUNCTN(N,3)=FUNCTN(N,3)-RATIO_a*X
	      FUNCTN(N,4)=FUNCTN(N,4)-RATIO_a*Y
	    ELSE
C  In this case, "a" refers to the relative orbit (if CHOICE(3:3) = 'R')
C  or to the secondary's absolute orbit (if CHOICE(3:3) = 'A' or 'I').
	      FUNCTN(N,3)=FUNCTN(N,3)-X
	      FUNCTN(N,4)=FUNCTN(N,4)-Y
	    END IF
	  END IF
C  Do the same calculation for the 1-D interferometric case
	  IF (CHOICE(30:30).EQ.'B'.AND.
	1 (CHOICE(32:32).EQ.'A'.OR.CHOICE(32:32).EQ.'I')) THEN !  Absolute
	    FUNCTN_TEMP(N,3)=FUNCTN_TEMP(N,3)-(RATIO_a-1.0D0)*X*
	1   PHOTOCENTRIC_TRUE_SEMIMAJAX
	    FUNCTN_TEMP(N,4)=FUNCTN_TEMP(N,4)-(RATIO_a-1.0D0)*Y*
	1   PHOTOCENTRIC_TRUE_SEMIMAJAX
	  ELSE IF (CHOICE(30:30).EQ.'B'
	1 .AND.CHOICE(32:32).EQ.'R') THEN !  Relative
	    FUNCTN_TEMP(N,3)=FUNCTN_TEMP(N,3)-RATIO_a*X
	    FUNCTN_TEMP(N,4)=FUNCTN_TEMP(N,4)-RATIO_a*Y
	  ELSE IF (CHOICE(30:30).EQ.'S') THEN !  Relative or absolute
	    IF (CHOICE(4:4).EQ.'B') THEN
C  In this case, the ratio is obtained from the spectroscopic orbit
	      FUNCTN_TEMP(N,3)=FUNCTN_TEMP(N,3)-RATIO_a*X
	      FUNCTN_TEMP(N,4)=FUNCTN_TEMP(N,4)-RATIO_a*Y
	    ELSE
C  In this case, "a" refers to the relative orbit (if CHOICE(32:32) = 'R')
C  or to the secondary's absolute orbit (if CHOICE(32:32) = 'A' or 'I').
	      FUNCTN_TEMP(N,3)=FUNCTN_TEMP(N,3)-X
	      FUNCTN_TEMP(N,4)=FUNCTN_TEMP(N,4)-Y
	    END IF
	  END IF
	  IF (INTERFEROMETRIC) THEN
	    IF (CHOICE(30:30).EQ.'P'.OR.CHOICE(30:30).EQ.'B') THEN
	      TEMP2=DSQRT(FUNCTN_TEMP(N,1)**2+FUNCTN_TEMP(N,2)**2)
				!  Distance in "
	      TEMP1=DATAN2D(FUNCTN_TEMP(N,1),FUNCTN_TEMP(N,2))
				!  Angle in deg.
	      FUNCTN(N,17)=ABS(TEMP2*DCOSD(TEMP1-FR_ANGLE(N)))
	    END IF
	    IF (CHOICE(30:30).EQ.'S'.OR.CHOICE(30:30).EQ.'B') THEN
	      TEMP2=DSQRT(FUNCTN_TEMP(N,3)**2+FUNCTN_TEMP(N,4)**2)
				!  Distance in "
	      TEMP1=DATAN2D(FUNCTN_TEMP(N,3),FUNCTN_TEMP(N,4))
				!  Angle in deg.
	      FUNCTN(N,18)=ABS(TEMP2*DCOSD(TEMP1-FR_ANGLE(N)))
	    END IF
	  END IF
C  Compute the radial velocity of the primary if there is spectroscopic
C  binary data.  The primary's velocity is calculated whether or not
C  data exists for it, since it is used later to calculate the subcomponent
C  secondary velocities and derivatives.  As long as the weight is zero, 
C  there will be no problem if no data exists.
	  IF (SPECTROSCOPIC) THEN !  Results of calculation are used for 
				  !  velocity of primary and in 
				  !  NLDER3 for deriv. of secondary.
	    FUNCTN(N,5)=V_SYS+VEL_AMP_PRI*(DCOSD(v+w)+ECCENTR*DCOSD(w))
C  Save this velocity for use with a subcomponent secondary.  As in its
C  normal usage, this velocity is considered to be that of the primary 
C  alone.  However, the light travel time correction should instead be
C  computed for the displacement of the subcomponent secondary from the 
C  center of mass position.
	    V_PRI_SAVE=FUNCTN(N,5) !  Used for subcomponent secondary
C  Compute the radial velocity of the secondary if there is spectroscopic
C  binary data
	    IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') THEN
	      FUNCTN(N,6)=V_SYS-VEL_AMP_SEC*(DCOSD(v+w)+ECCENTR*DCOSD(w))
	      V_SEC_SAVE=FUNCTN(N,6) !  Used for subcomponent secondary
	    END IF
C  If the primary has a spectroscopic subcomponent(s), take action with this 
C  extra velocity or velocities.
	    IF (SPECTROSCOPIC_PRI_SUB) THEN
	      IF (CHOICE(6:6).EQ.' '.OR.CHOICE(6:6).EQ.'a') THEN
	        IPRISUB_START=1
	      ELSE IF (CHOICE(6:6).EQ.'o') THEN
	        IPRISUB_START=IPRISUB
	      ELSE IF (CHOICE(6:6).EQ.'m') THEN
	        IPRISUB_START=IPRISUB+1 !  So that loop does not execute
	      END IF
	      SUM=0.0D0
C  IPRISUB is the highest number of the spectroscopic subcomponents to
C  be considered.
	      DO I=IPRISUB_START,IPRISUB
	        IF (OPERATE) THEN
	          M=MU_PRI_SUB(I)*(TIME(N,1)-LTT_COR(N)-T_PRI_SUB(I))
C  The time is corrected for the light travel time from the position of
C  the primary as affected by all the subcomponents
	          w_PRI_SUB(I)=w_save_PRI_SUB(I)+APSIDAL_MOTION_PRI_SUB(I)*
	1         (TIME(N,1)-LTT_COR(N)-T)
C  If apsidal motion is not to be calculated for a particular subcomponent,
C  then APSIDAL_MOTION_PRI_SUB(I) will be zero in the above equation.
	        ELSE
	          M=MU_PRI_SUB(I)*(TIME(N,1)-T_PRI_SUB(I))
	          w_PRI_SUB(I)=w_save_PRI_SUB(I)+APSIDAL_MOTION_PRI_SUB(I)*
	1         (TIME(N,1)-T)
	        END IF
	        E1=ECC_ANOM_PRI_SUB(N,I)
	        DELTA0=1.0D0
	        DELTA1=DELTA0
C  Solve Kepler's equation to find E, the eccentric anomaly
	        DO WHILE (ABS(DELTA0).GT.1.0D-12)
	          E=M+ECCENTR_PRI_SUB(I)*DSIN(E1)
	          ED=E+E1
	          IF (ABS(ED).LT.1.0D-20) ED=SIGN(1.0D0,ED)*1.0D-20
	          DELTA0=2.0D0*(E-E1)/ED
	          IF (DELTA0.EQ.-DELTA1) E=(E+E1)/2.0D0
	          DELTA1=DELTA0
	          E2=E1
	          E1=E
	        END DO
	        ECC_ANOM_PRI_SUB(N,I)=E
	        FRAC=SQRT(ABS((1.0D0+ECCENTR_PRI_SUB(I))/
	1       (1.0D0-ECCENTR_PRI_SUB(I))))
	        v_PRI_SUB(I)=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
C
	        VEL_AMP_PRI_SUB(I)=SPECT_FACT*aAU_PRI_SUB(I)/P_PRI_SUB(I)/
	1       SQRT(1.0D0-ECCENTR_PRI_SUB(I)**2)
	        TEMP(I)=VEL_AMP_PRI_SUB(I)*(DCOSD(v_PRI_SUB(I)+w_PRI_SUB(I))+
	1       ECCENTR_PRI_SUB(I)*DCOSD(w_PRI_SUB(I)))
	        SUM=SUM+TEMP(I)
	      END DO
C  Add the subcomponent velocities to the primary velocity.  All velocities
C  are calculated according to a time corrected by the position of the primary
C  affected by all the subcomonents.
	      IF (CHOICE(6:6).EQ.' ') THEN !  Adds all subcomp. to primary
	        FUNCTN(N,5)=FUNCTN(N,5)+SUM
	      ELSE IF (CHOICE(6:6).EQ.'o'.OR.CHOICE(6:6).EQ.'a') THEN
	        FUNCTN(N,5)=SUM
	      END IF
	    END IF
C  If the secondary has a spectroscopic subcomponent(s), take action with 
C  this extra velocity or velocities. 
	    IF (SPECTROSCOPIC_SEC_SUB) THEN
	      IF (CHOICE(8:8).EQ.' '.OR.CHOICE(8:8).EQ.'a') THEN
	        ISECSUB_START=1
	      ELSE IF (CHOICE(8:8).EQ.'o') THEN
	        ISECSUB_START=ISECSUB
	      ELSE IF (CHOICE(8:8).EQ.'m') THEN
	        ISECSUB_START=ISECSUB+1 !  So that loop does not execute
	      END IF
	      SUM=0.0D0
C  ISECSUB is the highest number of the spectroscopic subcomponents to
C  be considered.
	      DO I=ISECSUB_START,ISECSUB
	        IF (OPERATE) THEN
	          M=MU_SEC_SUB(I)*(TIME(N,1)-LTT_COR(N)-T_SEC_SUB(I))
C  The time is corrected for the light travel time from the position of
C  the primary as affected by all the subcomponents
	          w_SEC_SUB(I)=w_save_SEC_SUB(I)+APSIDAL_MOTION_SEC_SUB(I)*
	1         (TIME(N,1)-LTT_COR(N)-T)
	        ELSE
	          M=MU_SEC_SUB(I)*(TIME(N,1)-T_SEC_SUB(I))
	          w_SEC_SUB(I)=w_save_SEC_SUB(I)+APSIDAL_MOTION_SEC_SUB(I)*
	1         (TIME(N,1)-T)
	        END IF
	        E1=ECC_ANOM_SEC_SUB(N,I)
	        DELTA0=1.0D0
	        DELTA1=DELTA0
C  Solve Kepler's equation to find E, the eccentric anomaly
	        DO WHILE (ABS(DELTA0).GT.1.0D-12)
	          E=M+ECCENTR_SEC_SUB(I)*DSIN(E1)
	          ED=E+E1
	          IF (ABS(ED).LT.1.0D-20) ED=SIGN(1.0D0,ED)*1.0D-20
	          DELTA0=2.0D0*(E-E1)/ED
	          IF (DELTA0.EQ.-DELTA1) E=(E+E1)/2.0D0
	          DELTA1=DELTA0
	          E2=E1
	          E1=E
	        END DO
	        ECC_ANOM_SEC_SUB(N,I)=E
	        FRAC=SQRT(ABS((1.0D0+ECCENTR_SEC_SUB(I))/
	1       (1.0D0-ECCENTR_SEC_SUB(I))))
	        v_SEC_SUB(I)=2.0D0*DATAND(FRAC*DTAN(E/2.0D0))
C
	        VEL_AMP_SEC_SUB(I)=SPECT_FACT*aAU_SEC_SUB(I)/P_SEC_SUB(I)/
	1       SQRT(1.0D0-ECCENTR_SEC_SUB(I)**2)
	        TEMP(I)=VEL_AMP_SEC_SUB(I)*(DCOSD(v_SEC_SUB(I)+w_SEC_SUB(I))+
	1       ECCENTR_SEC_SUB(I)*DCOSD(w_SEC_SUB(I)))
	        SUM=SUM+TEMP(I)
	      END DO
C  Add the subcomponent velocities to the secondary velocity.  All velocities
C  are calculated according to a time corrected by the position of the 
C  secondary affected by all the subcomonents.
	      IF (CHOICE(8:8).EQ.' ') THEN !  Adds all subcomp. to secondary
	        FUNCTN(N,6)=FUNCTN(N,6)+SUM
	      ELSE IF (CHOICE(8:8).EQ.'o'.OR.CHOICE(8:8).EQ.'a') THEN
	        FUNCTN(N,6)=SUM
	      END IF
	    END IF
C  If any of the primary spectroscopic subcomponents has complementary 
C  spectral data from the associated secondary, evaluate the velocities.
C  We use the range CHOICE(9:13) for the primary subcomponent secondaries
C  and write the results to FUNCTN(N,7) to FUNCTN(N,11).
	    DO IC=9,13
	      IS=IC-8 !  Runs from 1 to 5
	      IF=IC-2 !  Runs from 7 to 11
	      IF (CHOICE(IC:IC).EQ.'T'.OR.CHOICE(IC:IC).EQ.'o') THEN
C  If a light travel time correction is to be made, v for the
C  subcomponent secondary will be different than for the primary 
C  subcomponent since the displacement is different.  We don't need to 
C  recalculate v however since each N refers to a specific time of 
C  observation.  Thus although the subcomponent velocities of the primary
C  were calculated above, they are not used for the N which applies to
C  the primary subcomponent secondary.
	        VEL_AMP_PRI_SUB_SEC(IS)=(RATIO_a_PRI_SUB_SEC(IS)-1.0D0)*
	1       SPECT_FACT*aAU_PRI_SUB(IS)/P_PRI_SUB(IS)/
	2       SQRT(1.0D0-ECCENTR_PRI_SUB(IS)**2)
	        FUNCTN(N,IF)=-VEL_AMP_PRI_SUB_SEC(IS)*
	1       (DCOSD(v_PRI_SUB(IS)+w_PRI_SUB(IS))+ECCENTR_PRI_SUB(IS)*
	2       DCOSD(w_PRI_SUB(IS)))
	        IF (CHOICE(IC:IC).EQ.'T') FUNCTN(N,IF)=FUNCTN(N,IF)+V_PRI_SAVE
	      END IF
	    END DO
C  If any of the secondary spectroscopic subcomponents has complementary 
C  spectral data from the associated secondary, evaluate the velocities.
C  We use the range CHOICE(14:18) for the secondary subcomponent secondaries
C  and write the results to FUNCTN(N,12) to FUNCTN(N,16).
	    DO IC=14,18
	      IS=IC-13 !  Runs from 1 to 5
	      IF=IC-2 !  Runs from 12 to 16
	      IF (CHOICE(IC:IC).EQ.'T'.OR.CHOICE(IC:IC).EQ.'o') THEN
	        VEL_AMP_SEC_SUB_SEC(IS)=(RATIO_a_SEC_SUB_SEC(IS)-1.0D0)*
	1       SPECT_FACT*aAU_SEC_SUB(IS)/P_SEC_SUB(IS)/
	2       SQRT(1.0D0-ECCENTR_SEC_SUB(IS)**2)
	        FUNCTN(N,IF)=-VEL_AMP_SEC_SUB_SEC(IS)*
	1       (DCOSD(v_SEC_SUB(IS)+w_SEC_SUB(IS))+ECCENTR_SEC_SUB(IS)*
	2       DCOSD(w_SEC_SUB(IS)))
	        IF (CHOICE(IC:IC).EQ.'T') FUNCTN(N,IF)=FUNCTN(N,IF)+V_SEC_SAVE
	      END IF
	    END DO
	  END IF
	END DO
	RETURN
	END
