	SUBROUTINE NLFUN_DEMO_1(X,MAXNPOINTS,DIMENSN_IND,PARAMETRS,
	1CHOICE,OPERATE,IX,FUNCTN,DIMENSN_DATA)
C
C       This subroutine helps NONLINFIT by selecting the chosen
C  fitting function, evaluating it while taking into account a user
C  defined operation if necessary, and returning the answer.  In this
C  simple demonstration application, a set of x, and y values can be fit by
C  a constant, a straight line, or two intersecting straight lines.  The
C  fit by a constant can of course be done by averaging;  the fit by a
C  straight line can be done by linear least squares;  but two intersecting
C  straight lines requires nonlinear least squares fitting.
C			Donald H. Gudehus, March, 1986
C
C	X		(C)	The array containing the independent variable
C				values
C	MAXNPOINTS	(C)	The first dimension of the array X
C	DIMENSN_IND	(C)	The second dimension of the array X
C	PARAMETRS	(C)	The array containing the parameters used to
C				evaluate the function
C	CHOICE		(C)	The choice determining which function
C				to use within a category
C	OPERATE		(C)	A logical key which determines what to do.
C				If .FALSE., the function is evaluated, if 
C				.TRUE., a user defined operation is carried
C				out, e.g., convolution.
C	IX		(C)	An array containing the starting and ending
C				indexes for the evaluation of of FUNCTN
C	FUNCTN		(R)	The output array containing the function.
C	DIMENSN_DATA	(C)	The second dimension of the output array.
C
	INTEGER DIMENSN_IND,DIMENSN_DATA
	DIMENSION IX(2),X(MAXNPOINTS,DIMENSN_IND),PARAMETRS(1),
	1FUNCTN(MAXNPOINTS,DIMENSN_DATA)
	LOGICAL OPERATE,PLOT
	REAL*8 X,PARAMETRS,FUNCTN
	CHARACTER*(*) CHOICE
C
C  Place user declarations below here
	REAL*8 MPRIME
C
	CALL ALPHA(%REF(CHOICE),1,LEN(CHOICE),NCHAR,NFULL,NREST,1)
	PLOT=.FALSE.
	IF (CHOICE(NCHAR:NCHAR).EQ.'%') THEN
	  PLOT=.TRUE.
	  CHOICE(NCHAR:NCHAR)=' '
	END IF
	MCFACTOR=0.4*LOG(10.0)
C  Place user code below this point.
	IF (OPERATE) THEN
C No user defined operation other than evaluation, is carried out in this demo
	ELSE
	  IF (CHOICE.EQ.'K') THEN  ! Constant
	    DO N=IX(1),IX(2)
	      FUNCTN(N,1)=PARAMETRS(1)
	    END DO
	  ELSE IF (CHOICE.EQ.'L') THEN  ! Straight line
	    DO N=IX(1),IX(2)
	      FUNCTN(N,1)=PARAMETRS(1)+PARAMETRS(2)*X(N,1)
	    END DO
	  ELSE IF (CHOICE.EQ.'T') THEN  ! Two intersecting lines
	    MPRIME=(PARAMETRS(3)-PARAMETRS(1))/
	1   (PARAMETRS(2)-PARAMETRS(4))	
	    DO N=IX(1),IX(2)
	      IF (X(N,1).LE.MPRIME) THEN
	        FUNCTN(N,1)=PARAMETRS(1)+PARAMETRS(2)*X(N,1)
	      ELSE
	        FUNCTN(N,1)=PARAMETRS(3)+PARAMETRS(4)*X(N,1)
	      END IF
	    END DO
	  END IF
	END IF
C
	RETURN
	END
