	SUBROUTINE NONLINFIT(X,MAXNPOINTS,DIMENSN_IND,DATA,DIMENSN_DATA,WEIGHT,
	1IX,FUNCAT,CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	2ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,MEANERROR,
	3MEANERRORMEANERROR,REDCHISQ,AVREDCHISQ,
	4XPLOT,DATPLOT,NPLPOINTS,RESIDUALS)
C
C       This subroutine does a nonlinear least squares fit to weighted data
C  and returns the parameters of the fit and their errors.  If desired,
C  certain coefficients may be held constant in the solution.  The dimensions
C  of the independent variable and the data are unrestricted.  The number of
C  points to be fit and the number of parameters to be solved for are also
C  unrestricted.  For plotting purposes, an array is filled with calculated
C  values corresponding to the values in the array XPLOT.
C       The weights can be relative or equal to the inverse square of the
C  standard deviation.  For relative weights however, the computed reduced
C  chi squared will be incorrect.
C       For each new function category, new subroutines which calculate
C  the function and its derivative must be created.  These must be referenced
C  in subroutines NLFUN and NLDER.  The object codes for subroutines NLFUN,
C  NLDER, NLFUN*, AND NLDER* belong in the NONLINFIT library.  In the calling
C  program, the plotting arrays XPLOT and DATPLOT must be dimensioned at least
C  as large as the number of points and the dimension of the data.  See the 
C  programs [MIIPS.FOR]NL_DEMO_1, NL_DEMO_2, NL_DEMO_4, NLTEST5,
C  NLTEST8, BINARY, DISPGAL, REDSHFTMAG, and SADFIT for further details on 
C  dimensioning and the subroutines NLFUN* and NLDER* for how to loop through 
C  the calculations.
C					Donald H. Gudehus, January, 1986
C			Version 2, D. Gudehus, April, 1994
C			Made MEANERROR multidimensional, and added
C			the arrays REDCHISQ and RESIDUALS
C			Version 3, D. Gudehus, May, 1995
C			Made CHOICE an arbitrary length
C			Version 4, D. Gudehus, November 11, 2013
C			Modified code to also be compatible with 64-bit
C			systems.
C
C	X		  (C)   An array containing the values of the
C			        independent variable.
C	MAXNPOINTS	  (C)   The dimension of the array X.  This is greater
C			        than or equal to the number of points to be fit.
C	DIMENSION_IND	  (C)	The dimension of the independent variable
C	DATA		  (C)   An array containing the observations to
C			        be fitted.
C	DIMENSION_DATA	  (C)	The dimension of the data.
C	WEIGHT		  (C)   An array for the weights of each point.
C	IX		  (C)	An array containing the starting and ending
C				indexes of the data for the fit.
C	FUNCAT		  (C)   The number of the function category;  ie., this
C				selects the NLFUN# and NLDER# that has
C				been written for a particular category of
C				solution.  If it is zero in the call, it is 
C				prompted for.
C	CHOICE		  (C/R) The choice of fitting function within a 
C				function category.  The position after the
C				last one in use by the calling program is used
C				as a flag by the NLFUN# when filling the plot 
C				array.  CHOICE should therefore be declared
C				large enough.
C	CONSTANTTERMS	  (C/R) An array which is .TRUE. if that coefficient
C			        is to be held constant and not solved for.
C				If NUMPARAM is .LE. zero in the call, the user
C				is prompted for which parameters to hold
C				constant.
C	PARAMETRS	  (C/R) An array containing the coefficients of the
C			        fitting function.
C	MAXNUMPARAM	  (C)	The maximum number of parameters desired to be
C				fit by the calling program.
C	NUMPARAM	  (C/R)	In the call, the absolute value gives the total
C				number of parameters in the function.  If it
C				is zero in the call, the user is prompted for
C				the number.  If it is less than zero in the 
C				call, the user is prompted for which parameters
C				to hold constant.  The absolute value is always
C				returned.
C	ITERATION_SCHEME  (C)	The scheme for iterating:  A is automatic with
C				maximum searching;  B is automatic with minimum
C				searching;  C is automatic with no searching;
C				M is manual.  If ITERATION_SCHEME
C				is blank in the call or M, the user is promted
C				for the value and also prompted for the initial
C				value of the fractional increment.
C	TOLERANCE	  (C/R)	The fractional tolerance for continuing the
C				iteration.  If it is zero in the call and the
C				ITERATION_SCHEME = 'A', 'B', or 'C', the user is
C				prompted for the value.  If the change in any
C				parameter is greater than TOLERANCE times the
C				sum of the parameter and its mean error,
C				iteration is continued.
C	INTERMEDIATE	  (C)	A logical variable which determnines whether
C				the intermediate results are printed on the
C				terminal.
C	OPERATE		  (C)	A logical variable which determines whether
C				to perform a user defined operation in the
C				subroutines NLFUN# and NLDER#.
C	COVARIANCE	  (R)   An array containing the covariances
C	MEANERROR	  (R)   An array giving the mean error for an 
C				observation of average weight for each of
C				the dependent variables
C	MEANERRORMEANERROR 
C			  (R)	An array giving the mean error of the mean
C				error for an observation of average weight
C				for each of the dependent variables
C	REDCHISQ	  (R)   An array giving the reduced chi squared
C				for each of the dependent variables
C	AVREDCHISQ	  (R)	The reduced chi squared averaged over
C				the dependent variables
C	XPLOT		  (C)   An array containing points from each dimension
C			        of the independent variable to be evaluated
C				with the fitted function.
C	DATPLOT		  (R)   An array containing the evaluated points from
C			        XPLOT.
C	NPLPOINTS	  (C)   The number of points to be plotted.  If it
C				equals zero in the call, the array DATPLOT is
C				not filled.  It must not be greater than
C				MAXNPOINTS
C	RESIDUALS	  (R)	An array containing the residuals for each of
C				the dependent variables.
C
	INTEGER DIMENSN_IND,DIMENSN_DATA,FUNCAT,IX(2)
	INTEGER*4 ADDRESS1_4,ADDRESS2_4,ADDRESS3_4,ADDRESS4_4,
	1ADDRESS5_4,ADDRESS6_4,ADDRESS7_4,ADDRESS8_4,ADDRESS9_4,ADDRESS10_4,
	2ADDRESS11_4,ADDRESS12_4,ADDRESS13_4,ADDRESS14_4,ADDRESS15_4,
	3ADDRESS16_4,ADDRESS17_4,ADDRESS18_4,ADDRESS19_4,ADDRESS20_4,
	4ADDRESS21_4
	INTEGER*8 ADDRESS1_8,ADDRESS2_8,ADDRESS3_8,ADDRESS4_8,
	1ADDRESS5_8,ADDRESS6_8,ADDRESS7_8,ADDRESS8_8,ADDRESS9_8,ADDRESS10_8,
	2ADDRESS11_8,ADDRESS12_8,ADDRESS13_8,ADDRESS14_8,ADDRESS15_8,
	3ADDRESS16_8,ADDRESS17_8,ADDRESS18_8,ADDRESS19_8,ADDRESS20_8,
	4ADDRESS21_8
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,OPERATE
	REAL XPLOT(MAXNPOINTS,DIMENSN_IND),DATPLOT(MAXNPOINTS,DIMENSN_DATA)
	REAL*8 X(MAXNPOINTS,DIMENSN_IND),DATA(MAXNPOINTS,DIMENSN_DATA),
	1WEIGHT(MAXNPOINTS,DIMENSN_DATA),
	2COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),PARAMETRS(MAXNUMPARAM),
	3MEANERROR(DIMENSN_DATA),MEANERRORMEANERROR(DIMENSN_DATA),
	4REDCHISQ(DIMENSN_DATA),AVREDCHISQ,
	5RESIDUALS(MAXNPOINTS,DIMENSN_DATA)
C
	CHARACTER*1 ITERATION_SCHEME
	CHARACTER*(*) CHOICE
C
	INTEGER SYSTEM_NUMBER_BYTES
	COMMON /SYS_NUM_BYTES/ SYSTEM_NUMBER_BYTES
C
C  Create some dynamic memory
	LENGTH1=MAXNPOINTS*DIMENSN_IND  ! XSAVE
	LENGTH2=MAXNUMPARAM  ! NCONSTANT
	LENGTH3=MAXNUMPARAM  ! ISHIFT
	LENGTH4=MAXNUMPARAM  ! KPVT
	LENGTH5=MAXNUMPARAM  ! PARAMETRS_SOLVE
	LENGTH6=MAXNUMPARAM  ! PARAMETRS_INIT
	LENGTH7=MAXNUMPARAM  ! PARAMETRS_PRIOR
	LENGTH8=MAXNUMPARAM  ! PARAMETRS_BEST
	LENGTH9=MAXNPOINTS*DIMENSN_DATA  ! FUNCT
	LENGTH10=MAXNPOINTS*MAXNUMPARAM*DIMENSN_DATA  ! DERIV
	LENGTH11=MAXNUMPARAM  ! SUMRIGHT
	LENGTH12=MAXNUMPARAM*MAXNUMPARAM  ! LEFTSIDE
	LENGTH13=MAXNUMPARAM*(MAXNUMPARAM+1)/2  ! UPTRIPART
	LENGTH14=MAXNUMPARAM-1  ! AUX
	LENGTH15=MAXNUMPARAM*MAXNUMPARAM  ! MINORARRAY
	LENGTH16=DIMENSN_DATA  ! NUM_DATA
	LENGTH17=DIMENSN_DATA  ! WEIGHT_DATA
	LENGTH18=DIMENSN_DATA  ! SUMSIGMA
	LENGTH19=MAXNUMPARAM  ! SIGMAPARAM
	LENGTH20=MAXNUMPARAM  ! SIGP_TEMP
	LENGTH21=MAXNUMPARAM*MAXNUMPARAM  ! COVARIANCE_TEMP
	IF (SYSTEM_NUMBER_BYTES.EQ.4) THEN
	  CALL DYNAMIC(ADDRESS1_4,LENGTH1,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS2_4,LENGTH2,'  I4',.TRUE.)
	  CALL DYNAMIC(ADDRESS3_4,LENGTH3,'  I4',.TRUE.)
	  CALL DYNAMIC(ADDRESS4_4,LENGTH4,'  I4',.TRUE.)
	  CALL DYNAMIC(ADDRESS5_4,LENGTH5,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS6_4,LENGTH6,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS7_4,LENGTH7,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS8_4,LENGTH8,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS9_4,LENGTH9,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS10_4,LENGTH10,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS11_4,LENGTH11,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS12_4,LENGTH12,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS13_4,LENGTH13,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS14_4,LENGTH14,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS15_4,LENGTH15,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS16_4,LENGTH16,'  I4',.TRUE.)
	  CALL DYNAMIC(ADDRESS17_4,LENGTH17,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS18_4,LENGTH18,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS19_4,LENGTH19,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS20_4,LENGTH20,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS21_4,LENGTH21,'  R8',.TRUE.)
	  CALL NONLINFIT1(X,%VAL(ADDRESS1_4),MAXNPOINTS,
	1 DIMENSN_IND,DATA,DIMENSN_DATA,WEIGHT,IX,FUNCAT,
	2 CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	3 ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,
	4 MEANERROR,MEANERRORMEANERROR,XPLOT,DATPLOT,NPLPOINTS,
	5 %VAL(ADDRESS2_4),%VAL(ADDRESS3_4),%VAL(ADDRESS4_4),
	6 %VAL(ADDRESS5_4),%VAL(ADDRESS6_4),%VAL(ADDRESS7_4),
	7 %VAL(ADDRESS8_4),%VAL(ADDRESS9_4),%VAL(ADDRESS10_4),
	8 %VAL(ADDRESS11_4),%VAL(ADDRESS12_4),%VAL(ADDRESS13_4),
	9 %VAL(ADDRESS14_4),%VAL(ADDRESS15_4),%VAL(ADDRESS16_4),
	1 %VAL(ADDRESS17_4),%VAL(ADDRESS18_4),
	2 REDCHISQ,AVREDCHISQ,%VAL(ADDRESS19_4),RESIDUALS,
	3%VAL(ADDRESS20_4),%VAL(ADDRESS21_4))
	  CALL DYNAMIC(ADDRESS1_4,LENGTH1,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS2_4,LENGTH2,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS3_4,LENGTH3,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS4_4,LENGTH4,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS5_4,LENGTH5,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS6_4,LENGTH6,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS7_4,LENGTH7,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS8_4,LENGTH8,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS9_4,LENGTH9,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS10_4,LENGTH10,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS11_4,LENGTH11,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS12_4,LENGTH12,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS13_4,LENGTH13,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS14_4,LENGTH14,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS15_4,LENGTH15,'  R8',.FALSE.)
 	  CALL DYNAMIC(ADDRESS16_4,LENGTH16,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS17_4,LENGTH17,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS18_4,LENGTH18,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS19_4,LENGTH19,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS20_4,LENGTH20,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS21_4,LENGTH21,'  R8',.FALSE.)
	ELSE IF (SYSTEM_NUMBER_BYTES.EQ.8) THEN
	  CALL DYNAMIC(ADDRESS1_8,LENGTH1,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS2_8,LENGTH2,'  I4',.TRUE.)
	  CALL DYNAMIC(ADDRESS3_8,LENGTH3,'  I4',.TRUE.)
	  CALL DYNAMIC(ADDRESS4_8,LENGTH4,'  I4',.TRUE.)
	  CALL DYNAMIC(ADDRESS5_8,LENGTH5,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS6_8,LENGTH6,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS7_8,LENGTH7,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS8_8,LENGTH8,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS9_8,LENGTH9,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS10_8,LENGTH10,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS11_8,LENGTH11,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS12_8,LENGTH12,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS13_8,LENGTH13,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS14_8,LENGTH14,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS15_8,LENGTH15,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS16_8,LENGTH16,'  I4',.TRUE.)
	  CALL DYNAMIC(ADDRESS17_8,LENGTH17,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS18_8,LENGTH18,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS19_8,LENGTH19,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS20_8,LENGTH20,'  R8',.TRUE.)
	  CALL DYNAMIC(ADDRESS21_8,LENGTH21,'  R8',.TRUE.)
	  CALL NONLINFIT1(X,%VAL(ADDRESS1_8),MAXNPOINTS,
	1 DIMENSN_IND,DATA,DIMENSN_DATA,WEIGHT,IX,FUNCAT,
	2 CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	3 ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,
	4 MEANERROR,MEANERRORMEANERROR,XPLOT,DATPLOT,NPLPOINTS,
	5 %VAL(ADDRESS2_8),%VAL(ADDRESS3_8),%VAL(ADDRESS4_8),
	6 %VAL(ADDRESS5_8),%VAL(ADDRESS6_8),%VAL(ADDRESS7_8),
	7 %VAL(ADDRESS8_8),%VAL(ADDRESS9_8),%VAL(ADDRESS10_8),
	8 %VAL(ADDRESS11_8),%VAL(ADDRESS12_8),%VAL(ADDRESS13_8),
	9 %VAL(ADDRESS14_8),%VAL(ADDRESS15_8),%VAL(ADDRESS16_8),
	1 %VAL(ADDRESS17_8),%VAL(ADDRESS18_8),
	2 REDCHISQ,AVREDCHISQ,%VAL(ADDRESS19_8),RESIDUALS,
	3 %VAL(ADDRESS20_8),%VAL(ADDRESS21_8))
	  CALL DYNAMIC(ADDRESS1_8,LENGTH1,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS2_8,LENGTH2,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS3_8,LENGTH3,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS4_8,LENGTH4,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS5_8,LENGTH5,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS6_8,LENGTH6,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS7_8,LENGTH7,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS8_8,LENGTH8,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS9_8,LENGTH9,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS10_8,LENGTH10,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS11_8,LENGTH11,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS12_8,LENGTH12,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS13_8,LENGTH13,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS14_8,LENGTH14,'  R8',.FALSE.)
  	  CALL DYNAMIC(ADDRESS15_8,LENGTH15,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS16_8,LENGTH16,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS17_8,LENGTH17,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS18_8,LENGTH18,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS19_8,LENGTH19,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS20_8,LENGTH20,'  R8',.FALSE.)
	  CALL DYNAMIC(ADDRESS21_8,LENGTH21,'  R8',.FALSE.)
	END IF
C
	RETURN
	END
