	SUBROUTINE NONLINFIT1(X,XSAVE,MAXNPOINTS,DIMENSN_IND,DATA,DIMENSN_DATA,
	1WEIGHT,IX,FUNCAT,CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,
	2NUMPARAM,ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,
	3MEANERROR,MEANERRORMEANERROR,XPLOT,DATPLOT,NPLPOINTS,NCONSTANT,
	4ISHIFT,KPVT,
	4PARAMETRS_SOLV,PARAMETRS_INIT,PARAMETRS_PRIOR,PARAMETRS_BEST,FUNCTN,
	5DERIV,SUMRIGHT,LEFTSIDE,UPTRIPART,AUX,MINORARRAY,
	6NUM_DATA,WEIGHT_DATA,SUMSIGMA,REDCHISQ,AVREDCHISQ,
	7SIGMAPARAM,RESIDUALS,SIGP_TEMP,COVARIANCE_TEMP)
C
C	Note:  This is the UNIX version.
C
C       This subroutine helps NONLINFIT to solve a nonlinear least squares
C  problem after aquisition of the dynamic memory.
C					Donald H. Gudehus, April, 1986
C			Version 2, Donald H. Gudehus, March, 1994
C			Enabled calculation of MEANERROR for each dependent
C			variable.
C			Version 3, Donald H. Gudehus, November, 1994
C			Introduced logical ZERO.
C			Version 4, Donald H. Gudehus, May, 1995
C			Made CHOICE and arbitrary length.
C			Version 5, Donald H. Gudehus, June, 1995
C			Changed format to G17.8 for display of intermediate
C			values of parameters, and corrected bug in format
C			statement.
C			Version 5U, D. Gudehus, May, 1997
C			Modified call to USRINP for UNIX
C                       Version 6, D. Gudehus, October, 1998
C                       Corrected logic of prompt for display of intermediate
C                       results when ITERATION_SCHEME='M'.
C			Version 6U, D. Gudehus, October, 1999
C			Incorporated calls to INEL
C			Version 7, Donald H. Gudehus, May 17, 2001
C			Fixed bug in setting elements of covariance
C			matrix to zero when some parameters are held constant.
C			Version 8, Donald H. Gudehus, August 23, 2001
C			Changed calculation of covariance matrix to allow for
C			nonzero values even when some dimensions have too few
C			points to give a nonzero meanerror.
C			Version 9, Donald H. Gudehus, October 15, 2001
C			Changed the default value of NUMCONSTANT to zero
C			rather than the END value so that an actual value
C			can be written to the save file in that case.
C			Version 10, D. Gudehus, January 14, 2003
C			Improved code for calculating reduced chi squared
C			and mean error when more than one dependent dimension
C			is used.
C
C	X		   (C)  An array containing the values of the
C			        independent variable.
C	XSAVE		   (C)	A working array so that X is not destroyed
C	MAXNPOINTS	   (C)  The dimension of the array X.  This is greater
C			        than or equal to the number of points to be fit.
C	DIMENSION_IND	   (C)	The dimension of the independent variable
C	DATA		   (C)	An array containing the observations to
C			        be fitted.
C	DIMENSION_DATA	   (C)	The dimension of the data.
C	WEIGHT		   (C)	An array for the weights of each point.
C	IX		   (C)	An array containing the starting and ending
C				indexes of the data for the fit.
C	FUNCAT		   (C)	The number of the function category;  ie., this
C				selects the NLFUN# and NLDER# that has
C				been written for a particular category of
C				solution.  If it is zero in the call, it is 
C				prompted for.
C	CHOICE		   (C)	The choice of fitting function within a 
C				function category.  The position after the
C				last one in use by the calling program is used
C				as a flag by the FCN# when filling the plot 
C				array.  CHOICE should therefore be declared
C				large enough.
C	CONSTANTTERMS	  (C/R) An array which is .TRUE. if that coefficient
C			        is to be held constant and not solved for.
C				If NUMPARAM is .LE. zero in the call, the user
C				is prompted for which parameters to hold
C				constant.
C	PARAMETRS	  (C/R) An array containing the coefficients of the
C			        fitting function.
C	MAXNUMPARAM	   (C)	The maximum number of parameters desired to be
C				fit by the calling program.
C	NUMPARAM	  (C/R)	In the call, the absolute value gives the total
C				number of parameters in the function.  If it
C				is zero in the call, the user is prompted for
C				the number.  The absolute value is always
C				returned.
C	ITERATION_SCHEME   (C)	The scheme for iterating:  A is automatic with
C				maximum searching;  B is automatic with minimum
C				searching;  C is automatic with no searching;
C				M is manual.  If ITERATION_SCHEME
C				is blank in the call or M, the user is promted
C				for the value, prompted for the initial value
C				of the fractional increment, and prompted for
C				whether intermediate values are to be
C				displayed.
C	TOLERANCE	  (C/R)	The fractional tolerance for continuing the
C				iteration.  If it is zero in the call and the
C				ITERATION_SCHEME = 'A', 'B', or 'C', the user is
C				prompted for the value.  If the change in any
C				parameter is greater than TOLERANCE times the
C				sum of the parameter and its mean error,
C				iteration is continued.
C	INTERMEDIATE	   (C)	A logical variable which determnines whether
C				the intermediate results are printed on the
C				terminal.
C	OPERATE		   (C)	A logical variable which determines whether
C				to perform a user defined operation in the
C				subroutines NLFUN# and NLDER#.
C	COVARIANCE	   (R)  An array containing the covariances
C	MEANERROR	   (R)  An array gving the mean error for an 
C				observation of average weight for each
C				dependent variable
C	MEANERRORMEANERROR (R)  An array gving the mean error of the mean
C				error for an observation of average weight 
C				for each dependent variable
C	XPLOT		   (C)  An array containing points from each dimension
C			        of the independent variable to be evaluated
C				with the fitted function.
C	DATPLOT		   (R)  An array containing the evaluated points from
C			        XPLOT.
C	NPLPOINTS	   (C)  The number of points to be plotted.  If it 
C				equals zero in the call, the array DATPLOT is
C				not filled.
C	NUM_DATA	   (R)	An array giving the number of data points 
C				with nonzero weight for each dependent 
C				dimension.
C	WEIGHT_DATA	   (R)	An array giving the sum of the weights of 
C				all the data points for each dependent
C				dimension.
C	SUMSIGMA	   (R)	An array containing an intermediate result
C	REDCHISQ	   (R)	An array containing the reduced chi squared
C				for each dependent variable
C	AVREDCHISQ	   (R)	The reduced chi squared averaged over the
C				dependent variables
C	RESIDUALS	   (R)	An array giving the residuals for each
C				data point for each dependent dimension,
C				in the sense, observed minus calculated.
C
	INTEGER DIMENSN_IND,DIMENSN_DATA,DIMENSN_DATA_USED,
	1FUNCAT,IX(2),NCONSTANT(MAXNUMPARAM),
	2ISHIFT(MAXNUMPARAM),IXPLOT(2),KPVT(MAXNUMPARAM),
	3NUM_DATA(DIMENSN_DATA),SUMNUM_DATA
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,INTERMEDIATEDUMMY,
	1OPERATE,PROMPTCONSTANT,FRACINCRPROMPT,INTERMEDIATEPROMPT,
	2ITERATE,FIRSTIME,ZERO
	REAL XPLOT(MAXNPOINTS,DIMENSN_IND),DATPLOT(MAXNPOINTS,DIMENSN_DATA)
	REAL*8 X(MAXNPOINTS,DIMENSN_IND),XSAVE(MAXNPOINTS,DIMENSN_IND),
	1DATA(MAXNPOINTS,DIMENSN_DATA),WEIGHT(MAXNPOINTS,DIMENSN_DATA),
	1COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	2PARAMETRS(MAXNUMPARAM),PARAMETRS_SOLV(MAXNUMPARAM),
	3PARAMETRS_INIT(MAXNUMPARAM),PARAMETRS_PRIOR(MAXNUMPARAM),
	4PARAMETRS_BEST(MAXNUMPARAM),FUNCTN(MAXNPOINTS,DIMENSN_DATA),
	2DERIV(MAXNPOINTS,MAXNUMPARAM,DIMENSN_DATA),SUMRIGHT(MAXNUMPARAM),
	5LEFTSIDE(MAXNUMPARAM,MAXNUMPARAM),
	1UPTRIPART(MAXNUMPARAM*(MAXNUMPARAM+1)/2),AUX(MAXNUMPARAM-1),DET(2),
	2WEIGHT_DATA(DIMENSN_DATA),SUMSUMSIGMA,
	5SUMSIGMA(DIMENSN_DATA),
	6MINOR(2),MINORARRAY(MAXNUMPARAM,MAXNUMPARAM),
	2SUMWEIGHT,NUMER,DENOM,SIGN,SIGMAPARAM(MAXNUMPARAM),
	6MEANERROR(DIMENSN_DATA),MEANERRORMEANERROR(DIMENSN_DATA),
	7REDCHISQ(DIMENSN_DATA),AVREDCHISQ,AVREDCHISQ_INIT,AVWEIGHT,
	8RESIDUALS(MAXNPOINTS,DIMENSN_DATA),SIGP_TEMP(MAXNUMPARAM),
	9COVARIANCE_TEMP(MAXNUMPARAM,MAXNUMPARAM)
C
	CHARACTER*1 ITERATION_SCHEME
	CHARACTER*(*) CHOICE
C
	IF (FUNCAT.EQ.0) THEN
	  CALL USRINP(FUNCAT,1,'I',4,0,'FUNCTION_CATEGORY=',
	1'Enter the function category number:')
	END IF
	LEN_CHOICE=LEN(CHOICE)
	IF (CHOICE.EQ.' ') THEN
	  CALL USRINP(%REF(CHOICE),1,'A',LEN_CHOICE,0,'FUNCTION_CHOICE=',
	1'Enter the function choice:')
	END IF
	PROMPTCONSTANT=.FALSE.
	IF (NUMPARAM.LE.0) THEN
	  PROMPTCONSTANT=.TRUE.
	  NUMPARAM=ABS(NUMPARAM)
	  DO I=1,MAXNUMPARAM
	    J=ISETEND(NCONSTANT(I))
	  END DO
	END IF
	IF (NUMPARAM.EQ.0) THEN
	  CALL USRINP(NUMPARAM,1,'I',4,0,'NUMBER_PARAMETERS=',
	1 'Enter the total number of parameters in the function:')
	END IF
	IF (PROMPTCONSTANT) THEN
C  Set the first index to 0 to be used as a key to determine whether
C  a default was taken or not.  If a default is taken, then the 0
C  will appear in the save file.  Because under the current version
C  of USRINP the END value does not appear in the save file, without 
C  setting the first index to 0, the save mechanism will not work. 
	  NCONSTANT(1)=0
	  CALL MESSAGE('Enter the indexes of the parameters which are to'//
	1 ' be held constant')
	  CALL USRINP(NCONSTANT,MAXNUMPARAM,'I',4,1,'NUMBERS_CONSTANT=',
	1 '(starting index = 1):  (default = none)')
	  IF (NCONSTANT(1).EQ.0) THEN
	    NUMCONSTANT=0
	  ELSE
	    NUMCONSTANT=INEL(NCONSTANT,MAXNUMPARAM)
	  END IF
	  DO I=1,NUMPARAM
	    CONSTANTTERMS(I)=.FALSE.
	  END DO
	  DO I=1,NUMCONSTANT
	    CONSTANTTERMS(NCONSTANT(I))=.TRUE.
	  END DO
	END IF
	NUMPARAM_SOLV=0
	DO I=1,NUMPARAM
	  IF (.NOT.CONSTANTTERMS(I)) NUMPARAM_SOLV=NUMPARAM_SOLV+1
	  ISHIFT(I)=0
	END DO
	I=1
	IS=1
	DO WHILE (I.LE.NUMPARAM)
	  IF (CONSTANTTERMS(I)) THEN
	    ISHIFT(IS)=ISHIFT(IS)+1
	    I=I+1
	  ELSE
	    IS=IS+1
	    ISHIFT(IS)=ISHIFT(IS-1)
	    I=I+1
	  END IF
	END DO
	DO I=1,NUMPARAM_SOLV
	  PARAMETRS_SOLV(I)=PARAMETRS(I+ISHIFT(I))
	END DO
	DO I=1,NUMPARAM_SOLV
	  PARAMETRS_INIT(I)=PARAMETRS_SOLV(I)
	END DO
C  Initialize the COVARIANCE matrix
	DO J=1,NUMPARAM
	  DO I=1,NUMPARAM
	    COVARIANCE(J,I)=0.0
	  END DO
	END DO
C  Get the iteration scheme
	FRACINCRPROMPT=.FALSE.
	IF (ITERATION_SCHEME.EQ.' ') THEN
	  FRACINCRPROMPT=.TRUE.
	  INTERMEDIATEPROMPT=.TRUE.
	  ITERATION_SCHEME=' '
	  DO WHILE (ITERATION_SCHEME.NE.'M'.AND.ITERATION_SCHEME.NE.'A'.AND.
	1 ITERATION_SCHEME.NE.'B'.AND.ITERATION_SCHEME.NE.'C')
	    ITERATION_SCHEME='A'
	    CALL MESSAGE('Select the iteration scheme:')
	    CALL MESSAGE('     M = Manual')
	    CALL MESSAGE('     A = Automatic selection of fractional'//
	1   ' increment with')
	    CALL MESSAGE('         maximum searching (default)')
	    CALL MESSAGE('     B = Automatic selection of fractional'//
	1   ' increment with')
	    CALL MESSAGE('         minimum searching')
	    CALL MESSAGE('     C = Automatic selection of fractional'//
	1   ' increment with')
	    CALL MESSAGE('         no searching')
	    CALL USRINP(%REF(ITERATION_SCHEME),1,'A',1,1,
	1   'ITERATION_SCHEME=',' ')
	  END DO
C  Get the tolerance for stopping the iteration for scheme A, B, or C.
	END IF
	IF (TOLERANCE.EQ.0.0.AND.(ITERATION_SCHEME.EQ.'A'.OR.ITERATION_SCHEME
	1.EQ.'B'.OR.ITERATION_SCHEME.EQ.'C')) THEN
	  TOLERANCE=.0001
	  CALL USRINP(TOLERANCE,1,'F',4,1,'TOLERANCE=',
	1 'Enter the tolerance for continuing the iteration:'//
	2 '  (default = 0.0001)')
	END IF
C  Determine whether or not to display intermediate results
	INTERMEDIATEDUMMY=INTERMEDIATE
        IF (INTERMEDIATEPROMPT.AND.ITERATION_SCHEME.NE.'M') THEN
	  INTERMEDIATEDUMMY=.FALSE.
	  CALL USRINP(INTERMEDIATEDUMMY,1,'L',4,1,'INTERMEDIATE=',
	1 'Do you want to display the intermediate results?  (default = N)')
	END IF
C  Get the fractional increment for the first iteration
	FRACINCR=1.0
	IF (ITERATION_SCHEME.EQ.'M'.OR.FRACINCRPROMPT) THEN
	  CALL USRINP(FRACINCR,1,'F',4,1,'FRACTIONAL_INCREMENT=',
	1 'Enter the initial fractional increment:  (default = 1.0)')
	END IF
C  Calculate the terms involving the weights
C  Initialize
	SUMWEIGHT=0.0
	SUMNUM_DATA=0
	DIMENSN_DATA_USED=0
	DO K=1,DIMENSN_DATA
	  WEIGHT_DATA(K)=0.0
	  NUM_DATA(K)=0.0
	  DO N=IX(1),IX(2)
	    WEIGHT_DATA(K)=WEIGHT(N,K)+WEIGHT_DATA(K)
	    IF (WEIGHT(N,K).GT.0.0) NUM_DATA(K)=NUM_DATA(K)+1
	  END DO
	  SUMWEIGHT=WEIGHT_DATA(K)+SUMWEIGHT
	  SUMNUM_DATA=NUM_DATA(K)+SUMNUM_DATA
C  In some applications, DIMENSN_DATA may be declared larger than the
C  number actually used.  For example, solving for only the secondary
C  of a visual binary star, has DIMENSN_DATA=4, but the first two 
C  dimensions, reserved for the primary, are not used.
	  IF (WEIGHT_DATA(K).GT.0.0) DIMENSN_DATA_USED=DIMENSN_DATA_USED+1
	END DO
	AVWEIGHT=SUMWEIGHT/SUMNUM_DATA
	FIRSTIME=.TRUE.
	NTIMES=0
C  Start of the loop for solving the normal equations which have been
C  linearized by expansion in a Taylor series.
  70	NTIMES=NTIMES+1
C  Initialize the sums for the normal equations
	  DO J=1,NUMPARAM_SOLV
	    SUMRIGHT(J)=0.0
	    DO I=1,NUMPARAM_SOLV
	      LEFTSIDE(J,I)=0.0
	    END DO
	  END DO
C  Initialize
	  DO K=1,DIMENSN_DATA
	    SUMSIGMA(K)=0.0
	  END DO
	  SUMSUMSIGMA=0.0
C  Compute the function
	  CALL NLFUN(X,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,CHOICE,
	1 OPERATE,IX,FUNCTN,DIMENSN_DATA)
C  Compute the derivatives for the normal equations.
	  CALL NLDER(X,MAXNPOINTS,DIMENSN_IND,PARAMETRS,MAXNUMPARAM,
	1 FUNCAT,CHOICE,CONSTANTTERMS,OPERATE,IX,FUNCTN,DIMENSN_DATA,
	2 DERIV)
C  Compute the sums for the normal equations
	  DO K=1,DIMENSN_DATA
	    DO N=IX(1),IX(2)
	      DO J=1,NUMPARAM_SOLV  ! Loop for rows in matrix
	        SUMRIGHT(J)=WEIGHT(N,K)*DERIV(N,J+ISHIFT(J),K)*
	1       (DATA(N,K)-FUNCTN(N,K))+SUMRIGHT(J)
	        DO I=1,NUMPARAM_SOLV  ! Loop for columns
	          LEFTSIDE(J,I)=WEIGHT(N,K)*DERIV(N,J+ISHIFT(J),K)*
	1         DERIV(N,I+ISHIFT(I),K)+LEFTSIDE(J,I)
	          SUMRIGHT(J)=WEIGHT(N,K)*DERIV(N,J+ISHIFT(J),K)*
	1         DERIV(N,I+ISHIFT(I),K)*PARAMETRS_SOLV(I)+SUMRIGHT(J)
	        END DO
	      END DO
	      IF (FIRSTIME) THEN
	        SUMSIGMA(K)=(FUNCTN(N,K)-DATA(N,K))**2*WEIGHT(N,K)
	1       +SUMSIGMA(K)
	      END IF
	    END DO
	    SUMSUMSIGMA=SUMSIGMA(K)+SUMSUMSIGMA
	  END DO
	  IF (FIRSTIME) THEN
C  The first time through, compute the reduced chi squared for each 
C  data dimension
	    DO K=1,DIMENSN_DATA
	      IF ((NUM_DATA(K)-NUMPARAM_SOLV/DIMENSN_DATA_USED).GT.0) THEN
	        REDCHISQ(K)=SUMSIGMA(K)/(NUM_DATA(K)-NUMPARAM_SOLV/
	1       DIMENSN_DATA_USED)
	      ELSE
	        REDCHISQ(K)=0.0
	      END IF
	    END DO
C  The first time through, compute the average reduced chi squared
	    IF ((SUMNUM_DATA-NUMPARAM_SOLV).GT.0.0) THEN
	      AVREDCHISQ=(SUMSUMSIGMA/(SUMNUM_DATA-NUMPARAM_SOLV))
	    ELSE
	      AVREDCHISQ=0.0
	    END IF
	    AVREDCHISQ_INIT=AVREDCHISQ
C  The first time through, calculate the mean error for the
C  initial guess
	    DO K=1,DIMENSN_DATA
	      IF ((NUM_DATA(K)-NUMPARAM_SOLV/DIMENSN_DATA_USED).GT.0.AND.
	1     WEIGHT_DATA(K).NE.0.0) THEN
C		NADJUST=NUMPARAM_SOLV/DIMENSN_DATA_USED
	        NADJUST=NUMPARAM_SOLV*1.0*NUM_DATA(K)/SUMNUM_DATA
	        MEANERROR(K)=SQRT(SUMSIGMA(K)/((NUM_DATA(K)-NADJUST)*
	1       WEIGHT_DATA(K)/NUM_DATA(K)))
	        MEANERRORMEANERROR(K)=MEANERROR(K)/
	1       SQRT(2.0D0*(NUM_DATA(K)-NADJUST))
	      ELSE
	        MEANERROR(K)=0.0
	        MEANERRORMEANERROR(K)=0.0
	      END IF
	    END DO
	  END IF
	  IF (INTERMEDIATEDUMMY.AND.FIRSTIME) THEN
	    CALL MESSAGE('For the initial guess:')
	    CALL MESSAGE('The reduced chi squared for each data index is:')
	    DO K=1,DIMENSN_DATA
	      I1300=INT(LOG10(FLOAT(K)))+1
	      WRITE(6,1300) K,REDCHISQ(K)
1300	      FORMAT(' Data dimension #',I<I1300>,
	1     G18.6)
	      WRITE(7,1300) K,REDCHISQ(K)
	    END DO
	    WRITE(6,1301) AVREDCHISQ
1301	    FORMAT(' The average reduced chi squared = ',G18.6)
	    WRITE(7,1301) AVREDCHISQ
	    CALL MESSAGE('The mean errors for an observation of average'//
	1   ' weight for each data index are:')
	    DO K=1,DIMENSN_DATA
	      I1311=INT(LOG10(FLOAT(K)))+1
	      WRITE(6,1311) K,MEANERROR(K),MEANERRORMEANERROR(K)
1311	      FORMAT(' Data dimension #',I<I1311>,
	1     G18.6,' +-',G16.4)
	      WRITE(7,1311) K,MEANERROR(K),MEANERRORMEANERROR(K)
	    END DO
	  END IF
C  Load the array UPTRIPART with the upper right part of LEFTSIDE in order
C  to solve the equations
	  K=0
	  DO J=1,NUMPARAM_SOLV
	    DO I=1,J
	      K=K+1
	      UPTRIPART(K)=LEFTSIDE(I,J)
	    END DO
	  END DO
	  IER=1.0E-14
	  CALL SYMINV(SUMRIGHT,UPTRIPART,NUMPARAM_SOLV,1,EPS,IER,AUX)
	  IF (IER.GT.0) WRITE(6,1200) IER+1
	  IF (IER.GT.0) WRITE(7,1200) IER+1
1200	  FORMAT(' Warning:  Possible loss of significance indicated at',
	1 ' elimination step',I4/' in SYMINV routine, where pivot element',
	1 ' was less than the internal tolerance'/ 'EPS=1.E-14 set up in'
	1 ' NONLINFIT1')
	  IF (IER.EQ.-1) CALL MESSAGE('Attention:  SYMINV failed to'//
	1 ' solve system of linear equations')
	  IF (IER.EQ.-1) RETURN
C  Adjust the solution by the fractional increment
	  DO I=1,NUMPARAM_SOLV
	    PARAMETRS_SOLV(I)=PARAMETRS_SOLV(I)+(SUMRIGHT(I)-
	1   PARAMETRS_SOLV(I))*FRACINCR
	  END DO
C  Update all the parameters
	  DO I=1,NUMPARAM_SOLV
	    PARAMETRS(I+ISHIFT(I))=PARAMETRS_SOLV(I)
	  END DO
	  CALL NLFUN(X,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,CHOICE,OPERATE,
	1 IX,FUNCTN,DIMENSN_DATA)
C  Update the solved parameter array to reflect any changes made by NLFUN, e.g.,
C  setting an eccentricity to a number <= 1.
	  DO I=1,NUMPARAM_SOLV
	    PARAMETRS_SOLV(I)=PARAMETRS(I+ISHIFT(I))
	  END DO
C  Compute the mean error for an observation of average weight
C  for the solution
	  SUMSUMSIGMA=0.0
	  DO K=1,DIMENSN_DATA
	    SUMSIGMA(K)=0.0
	    DO N=IX(1),IX(2)
	      SUMSIGMA(K)=(DATA(N,K)-FUNCTN(N,K))**2*WEIGHT(N,K)
	1     +SUMSIGMA(K)
	    END DO
	    SUMSUMSIGMA=SUMSIGMA(K)+SUMSUMSIGMA
	  END DO
C  Compute the reduced chi squared for each data dimension
	  DO K=1,DIMENSN_DATA
	    IF ((NUM_DATA(K)-NUMPARAM_SOLV/DIMENSN_DATA_USED).GT.0) THEN
C	      NADJUST=NUMPARAM_SOLV/DIMENSN_DATA_USED
	      NADJUST=NUMPARAM_SOLV*1.0*NUM_DATA(K)/SUMNUM_DATA
	      REDCHISQ(K)=SUMSIGMA(K)/(NUM_DATA(K)-NADJUST)
	    ELSE
	      REDCHISQ(K)=0.0
	    END IF
	  END DO
C  Compute the average reduced chi squared
	  IF ((SUMNUM_DATA-NUMPARAM_SOLV).GT.0.0) THEN
	    AVREDCHISQ=SUMSUMSIGMA/(SUMNUM_DATA-NUMPARAM_SOLV)
	  ELSE
	    AVREDCHISQ=0.0
	  END IF
C  Compute the mean error for each data dimension
	  DO K=1,DIMENSN_DATA
	    IF ((NUM_DATA(K)-NUMPARAM_SOLV/DIMENSN_DATA_USED).GT.0.AND.
	1   WEIGHT_DATA(K).NE.0.0) THEN
C	      NADJUST=NUMPARAM_SOLV/DIMENSN_DATA_USED
	      NADJUST=NUMPARAM_SOLV*1.0*NUM_DATA(K)/SUMNUM_DATA
	      MEANERROR(K)=SQRT(SUMSIGMA(K)/((NUM_DATA(K)-NADJUST)*
	1     WEIGHT_DATA(K)/NUM_DATA(K)))
	      MEANERRORMEANERROR(K)=MEANERROR(K)/
	1     SQRT(2.0D0*(NUM_DATA(K)-NADJUST))
	    ELSE
	      MEANERROR(K)=0.0
	      MEANERRORMEANERROR(K)=0.0
	    END IF
	  END DO
	  IF (INTERMEDIATEDUMMY) THEN
	    CALL MESSAGE('For the solution:')
	    IF (ITERATION_SCHEME.EQ.'A'.OR.ITERATION_SCHEME.EQ.
	1   'B'.OR.ITERATION_SCHEME.EQ.'C') THEN
	      WRITE (6,1350) FRACINCR
	      WRITE (7,1350) FRACINCR
1350	      FORMAT(' The fractional increment = ',G15.6)
	    END IF
	    CALL MESSAGE('The reduced chi squared for each data index is:')
	    DO K=1,DIMENSN_DATA
	      WRITE(6,1300) K,REDCHISQ(K)
	      WRITE(7,1300) K,REDCHISQ(K)
	    END DO
	    WRITE(6,1301) AVREDCHISQ
	    WRITE(7,1301) AVREDCHISQ
	    CALL MESSAGE('The mean errors for an observation of average'//
	1   ' weight for each data index are:')
	    DO K=1,DIMENSN_DATA
	      WRITE(6,1311) K,MEANERROR(K),MEANERRORMEANERROR(K)
	      WRITE(7,1311) K,MEANERROR(K),MEANERRORMEANERROR(K)
	    END DO
	  END IF
C  Compute the mean errors of each parameter
	  CALL DETERMNT(LEFTSIDE,MAXNUMPARAM,NUMPARAM_SOLV,DET,KPVT)
	  DENOM=DET(1)
	  IF (DENOM.EQ.0.0) THEN
	    CALL ERROR(-1,'Determinant was zero')
	    DENOM=DENOM+1.0
	  END IF
C  Initialize the error array
	  DO I=1,NUMPARAM
	    SIGMAPARAM(I)=0.0
	  END DO
	  DO I=1,NUMPARAM_SOLV
	    CALL COFACTOR(LEFTSIDE,MINORARRAY,MAXNUMPARAM,I,I,NUMPARAM_SOLV,
	1   MINOR,SIGN,KPVT)
	    NUMER=MINOR(1)*SIGN
	    ZERO=.FALSE.
	    IF (ABS(MINOR(2)-DET(2)).GT.74) THEN
	      CALL ERROR(-1,'Determinant and/or cofactor beyond'//
	1     ' allowable range')
	      SIGMAPARAM(I+ISHIFT(I))=0.0
	    ELSE
	      DO K=1,DIMENSN_DATA
	        IF (NUM_DATA(K).NE.0) THEN
	          SIGP_TEMP(K)=MEANERROR(K)*SQRT(SUMWEIGHT/NUM_DATA(K)*
	1         ABS(NUMER/DENOM))*10.0**(0.5*(MINOR(2)-DET(2)))
	          IF (SIGP_TEMP(K).LE.1.0D-18) ZERO=.TRUE.
	          IF (.NOT.ZERO) SIGMAPARAM(I+ISHIFT(I))=1.0/SIGP_TEMP(K)**2
	1         +SIGMAPARAM(I+ISHIFT(I))
	        END IF
	      END DO
	      IF (.NOT.ZERO) THEN
	        SIGMAPARAM(I+ISHIFT(I))=SQRT(1.0D0/SIGMAPARAM(I+ISHIFT(I)))
	      ELSE
	        SIGMAPARAM(I+ISHIFT(I))=0.0
	      END IF
	    END IF
	  END DO
	  IF (INTERMEDIATEDUMMY) THEN
	    DO II=1,NUMPARAM
	      I1321=INT(LOG10(FLOAT(II)))+1
	      WRITE (6,1321) II,PARAMETRS(II),SIGMAPARAM(II)
1321	      FORMAT(' Parameter #',I<I1321>,
	1     ' = ',G18.10,'  +-',G16.4)
	      WRITE (7,1321) II,PARAMETRS(II),SIGMAPARAM(II)
	    END DO
	  END IF
C  Get the fractional increment for the next pass and readjust the
C  parameters as necessary depending on the iteration scheme.
	CALL NLIT(FIRSTIME,PARAMETRS_SOLV,PARAMETRS_INIT,PARAMETRS_PRIOR,
	1PARAMETRS_BEST,NUMPARAM_SOLV,MAXNUMPARAM,AVREDCHISQ,SIGMAPARAM,
	2AVREDCHISQ_INIT,TOLERANCE,FRACINCR,ITERATION_SCHEME,CHOICE,
	3INTERMEDIATEDUMMY,ITERATE)
	DO I=1,NUMPARAM_SOLV
	  PARAMETRS(I+ISHIFT(I))=PARAMETRS_SOLV(I)
	END DO
	FIRSTIME=.FALSE.
	IF (ITERATE) GO TO 70
C  Calculate the covariance;  note that the matrix may be
C  smaller if some coefficients were held constant
	CALL MATRIXINV(LEFTSIDE,MAXNUMPARAM,NUMPARAM_SOLV,COVARIANCE,KPVT)
	K=0
C  Scale the covariance matrix properly and make the matrix its
C  original size if necessary.  Start the loops from the high end
C  so as not to overwrite.
	DO NCOL=NUMPARAM_SOLV,1,-1
	  DO NROW=NUMPARAM_SOLV,1,-1
	    COVARIANCE_TEMP(NROW,NCOL)=0.0
	    DO K=1,DIMENSN_DATA
	      ZERO=.FALSE.
	      IF (NUM_DATA(K).NE.0) THEN
	        SIGP_TEMP(K)=MEANERROR(K)**2*SUMWEIGHT/NUM_DATA(K)*
	1       COVARIANCE(NROW,NCOL)
	        IF (ABS(SIGP_TEMP(K)).LE.1.0D-37) ZERO=.TRUE.
	        IF (.NOT.ZERO) COVARIANCE_TEMP(NROW,NCOL)=1.0/SIGP_TEMP(K)+
	2       COVARIANCE_TEMP(NROW,NCOL)
	      END IF
	    END DO
	    IF (COVARIANCE_TEMP(NROW,NCOL).NE.0.0) THEN
	      COVARIANCE(NROW+ISHIFT(NROW),NCOL+ISHIFT(NCOL))=
	1     1.0D0/COVARIANCE_TEMP(NROW,NCOL)
	    ELSE
	      COVARIANCE(NROW+ISHIFT(NROW),NCOL+ISHIFT(NCOL))=0.0
	    END IF
C  Set the location from where this number came equal to zero if there was a
C  non-zero shift
	    IF (ISHIFT(NCOL).NE.0.OR.ISHIFT(NROW).NE.0) 
	1   COVARIANCE(NROW,NCOL)=0.0
	  END DO
	END DO
C  Compute the residuals and fill the array	
	DO N=IX(1),IX(2)
	  DO K=1,DIMENSN_DATA
	    RESIDUALS(N,K)=DATA(N,K)-FUNCTN(N,K)
	  END DO
	END DO
C  Fill the plot array if NPLPOINTS is not equal to zero
	IF (NPLPOINTS.NE.0) THEN
	  DO K=1,DIMENSN_IND
	    DO N=1,NPLPOINTS
	      XSAVE(N,K)=X(N,K)
	      XSAVE(N,K)=XPLOT(N,K)
	    END DO
	  END DO
	  IXPLOT(1)=1
	  IXPLOT(2)=NPLPOINTS
	  CALL ALPHA(%REF(CHOICE),1,LEN_CHOICE,NCHAR,NFULL,NREST,1)
	  CHOICE(NCHAR+1:NCHAR+1)='%'
	  CALL NLFUN(XSAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1 CHOICE,OPERATE,IXPLOT,FUNCTN,DIMENSN_DATA)
	  DO N=1,NPLPOINTS
	    DO K=1,DIMENSN_DATA
	      DATPLOT(N,K)=FUNCTN(N,K)
	    END DO
	  END DO
	END IF
C
	RETURN
	END
