	SUBROUTINE PARALLAXFACTORS(BESSYR,RIGHTASC,DECLINA,
	1EPOCH_EQ,KEY,FACT_RIGHTASC,FACT_DECLINA)
C
C       This subroutine computes the parallax factors, used for solving
C  visual and astrometric binary star orbits.
C  Reference:  Smart, p. 221
C				Donald H. Gudehus
C				February 16, 1994
C			Version 2, Donald H. Gudehus, April 27, 2001
C			Added argument KEY
C
C	BESSYR	 	(C)	The Besselian year.  This is the time of
C				observation, which is used to determine the
C				position of the earth in its orbit, and the
C				parallax factors
C	RIGHTASC	(C)	The right ascension of the object for the
C				epoch given by EPOCH_EQ, and measured in 
C				decimal hours
C	DECLINA	 	(C)	The declination of the object for the
C				epoch given by EPOCH_EQ, and measured in 
C				decimal degrees
C	EPOCH_EQ	(C)	The epoch of the equator system for the
C				right ascension and declination of the object
C	KEY		(C)	A key such that when equal to 0, the parallax
C				factors are computed for the equator system
C				corresponding to EPOCH_EQ, and when equal to
C				1, they are computed for the equator system
C				of the observation, i.e., BESSYR
C	FACT_RIGHTASC	(R)	The parallax factor in right ascension
C				(dimensionless) for the desired equinox.
C				If KEY=1 (equinox of date), it is left to 
C				the calling program which solves for
C				parallax, to convert the parallax factors
C				in RA and DEC to a standard epoch for the 
C				equator. 
C	FACT_DECLINA	(R)	The parallax factor in declination.
C				(dimensionless) for the equinox of date
C
	INTEGER FLAG,KEY
	INTEGER  JD,DA,MO,YR,DOW
	REAL*8 BESSYR,RIGHTASC,DECLINA,EPOCH_EQ,FACT_RIGHTASC,FACT_DECLINA,
	1T,DAYS,ET,ECCENTR,ARGPERI,MEANANOM,OBLIQ,ECCENTRANOM,TRUEANOM,
	2RADIUS,NODEANGLE,INCLINA,x_c,y_c,z_c,x,y,z,RIGHTASC_DATE,
	3DECLINA_DATE,RIGHTASC_DATE_DEG
C
	CALL BESSYR_JULCENT(BESSYR,T)
	DAYS=T*36525.0D0 !  Ephemeris days since fundamental epoch of 
			 !  January 0.5 E.T.
	JD=INT(DAYS)+2415020 !  Julian day number
	CALL JD_DATE(JD,DA,MO,YR,DOW) !  Gregorian calendar.  The time
				      !  of day is assumed to be noon
	ET=(DAYS-INT(DAYS)+0.5D0)*24.0D0 !  This is approximate because we
		!  assume ET=12h at noon.  Note that ET can be > 24 if the
		!  time is after midnight and before noon the following day.
	CALL EARTH(ET,DA,MO,YR,A,ECCENTR,ARGPERI,MEANANOM,
	1OBLIQ,ECCENTRANOM,TRUEANOM,RADIUS)
	NODEANGLE=0.0D0
	INCLINA=0.0D0
	CALL XYZECEQ(RADIUS,TRUEANOM,ARGPERI,NODEANGLE,INCLINA,
	1OBLIQ,x_c,y_c,z_c,x,y,z)
	IF (KEY.EQ.0) THEN
	  RIGHTASC_DATE_DEG=15.0D0*RIGHTASC
	  DECLINA_DATE=DECLINA
	ELSE IF (KEY.EQ.1) THEN
C  Precess the object's coordinates to the mean equinox of date.  We
C  approximate the epoch, required to be in tropical years, by the epoch
C  measured in Besselian years.
	  FLAG=0
	  CALL PRECESSIO(FLAG,EPOCH_EQ,BESSYR,RIGHTASC,DECLINA,
	1 RIGHTASC_DATE,DECLINA_DATE)
	  RIGHTASC_DATE_DEG=15.0D0*RIGHTASC_DATE
	END IF
	FACT_RIGHTASC=-y*DCOSD(RIGHTASC_DATE_DEG)+x*DSIND(RIGHTASC_DATE_DEG)
	FACT_DECLINA=-z*DCOSD(DECLINA_DATE)+x*DCOSD(RIGHTASC_DATE_DEG)*
	1DSIND(DECLINA_DATE)+y*DSIND(RIGHTASC_DATE_DEG)*DSIND(DECLINA_DATE)
	RETURN
	END
