	SUBROUTINE PRECESSIO(FLAG,EPOCH1,EPOCH2,RA1,DEC1,
	1RA2,DEC2)
C
C       This subroutine will precess right ascension and
C  declination from one epoch to another, either forward or
C  backward in time.  The transformation is based on the formulae
C  in 'The Explanatory Supplement to the Astronomical Epehmeris
C  and the American Ephemeris and Nautical Almanac', p. 31,
C  Her Majesty's Stationery Office, London, 1962.
C				Donald H. Gudehus, February, 1984
C
C			Version 2, D. H. Gudehus, April, 1991
C			Substituted trig functions in degrees
C			Version 3, D. H. Gudehus, September, 1991
C			Used new constants from J. H. Lieske, T. Lederle, 
C			W. Fricke, and B. Morando, Astronomy and Astrophysics,
C			58, 1 (1977)

C
C	FLAG	(C)	A flag which if equal to 0 accepts the 
C			epochs in tropical years elapsed since
C			2000.0 plus 2000, and if equal to 1 accepts
C			the epochs in Julian Day numbers.  In Version 1,
C			the above numbers were 1900.
C	EPOCH1	(C)	The initial epoch in the appropriate units
C	EPOCH2	(C)	The final epoch in the appropriate units
C	RA1	(C)	The initial right ascension in decimal hours
C	DEC1	(C)	The initial declination in decimal degrees
C	RA2	(R)	The final right ascension in decimal hours
C	DEC2	(R)	The final declination in decimal degrees
C
	INTEGER FLAG,QUAD
	REAL*8 EPOCH1,EPOCH2,RA1,DEC1,RA2,DEC2,TZERO,T,TPRIME,
	1TEMP,ZETA,ZED,THETA,SINDEC,SINCOS,COSCOS,COSDEC1,SINDEC1,
	2SINRAZ1,COSRAZ1,COSTHETA,SINTHETA
C
C  This is the code for version 1
C	IF (FLAG.EQ.0) THEN  ! Input units of tropical years elapsed since
C			     ! 1900.0 plus 1900.
C	  TZERO=(EPOCH1-1900.0)/100.0
C	  T=(EPOCH2-1900.0)/100.0-TZERO
C	ELSE IF (FLAG.EQ.1) THEN  ! Input units of Julian Day numbers
C	  TPRIME=(EPOCH1-2415020.0)/36525.0  ! Units of Julian centuries
C					     ! of 36525 ephemeris days
C					     ! elapsed since Jan. 0.5,
C					     ! 1900 (Julian Day
C					     ! 2415020.0).
C	  TZERO=(EPOCH1-2415020.313)/(36524.219879-0.000614*
C	1 (TPRIME/2.0))  ! Accounts for the changing length of the
C			 ! tropical century
C	  TPRIME=(EPOCH2-2415020.0)/36525.0
C	  T=(EPOCH2-2415020.313)/(36524.219879-0.000614*
C	1 (TPRIME/2.0))-TZERO
C	END IF
C	ZETA=(2304.25+1.396*TZERO)*T+0.302*T**2+0.018*T**3
C	ZED=ZETA+0.791*T**2
C	THETA=(2004.682-0.853*TZERO)*T-0.426*T**2-0.042*T**3
C  This is the code for version 2
	IF (FLAG.EQ.0) THEN  ! Input units of tropical years elapsed since
			     ! 2000.0 plus 2000.
	  TZERO=(EPOCH1-2000.0D0)/100.0D0
	  T=(EPOCH2-EPOCH1)/100.0D0
	ELSE IF (FLAG.EQ.1) THEN  ! Input units of Julian Day numbers
	  TPRIME=(EPOCH1-2415020.0D0)/36525.0D0  ! Units of Julian centuries
					 	 ! of 36525 ephemeris days
						 ! elapsed since Jan. 0.5,
					 	 ! 1900 (Julian Day
						 ! 2415020).
	  TZERO=(EPOCH1-2451544.0D0)/(36524.219879D0-0.000614D0*
	1 (TPRIME/2.0))  ! Accounts for the changing length of the
			 ! tropical century
C	  TPRIME=(EPOCH2-2415020.0D0)/36525.0D0
	  T=(EPOCH2-2451544.0D0)/(36524.219879D0-0.000614D0*
	1 (TPRIME/2.0D0))-TZERO
	END IF
C
	TEMP=(2306.2181D0+1.39656D0*TZERO-0.000139D0*TZERO**2)*T
	ZETA=TEMP+(0.30188D0-0.000345D0*TZERO)*T**2+0.017998D0*T**3
	ZED=TEMP+(1.09468D0+0.000066D0*TZERO)*T**2+0.018203D0*T**3
	THETA=(2004.3109D0-0.85330D0*TZERO-0.000217D0*TZERO**2)*T
	1+(-0.42665D0-0.000217D0*TZERO)*T**2-0.041833D0*T**3
C	WRITE(7,1000) ZETA,ZED,THETA
1000	FORMAT(' ','zeta = ',G14.6,3X,'z = ',G14.6,3X,'theta = ',G14.6)
	ZETA=ZETA/3600.0D0
	ZED=ZED/3600.0D0
	COSTHETA=DCOSD(THETA/3600.0D0)
	SINTHETA=DSIND(THETA/3600.0D0)
C
	SINRAZ1=DSIND(RA1*15.0D0+ZETA)
	COSRAZ1=DCOSD(RA1*15.0D0+ZETA)
	SINDEC1=DSIND(DEC1)
	COSDEC1=DCOSD(DEC1)
C
	SINDEC=COSTHETA*SINDEC1+SINTHETA*COSRAZ1*COSDEC1
	SINCOS=SINRAZ1*COSDEC1
	COSCOS=-SINTHETA*SINDEC1+COSTHETA*COSRAZ1*COSDEC1
C
	DEC2=DASIND(SINDEC)
C  Get the quadrant of the right ascension
	IF (SINCOS.GE.0.0) THEN
	  IF (COSCOS.GE.0.0) THEN
	    QUAD=1
	  ELSE
	    QUAD=2
	  END IF
	ELSE
	  IF (COSCOS.GE.0.0) THEN
	    QUAD=4
	  ELSE
	    QUAD=3
	  END IF
	END IF
	IF (DABS(COSCOS).LT.1.0D-17) THEN
	  IF (QUAD.EQ.3.OR.QUAD.EQ.4) THEN
	    RA2=18.0
	  ELSE
	    RA2=6.0
	  END IF
	ELSE
	  RA2=(DATAND(SINCOS/COSCOS)+ZED)/15.0
	  IF (QUAD.EQ.2.OR.QUAD.EQ.3) RA2=RA2+12.0
	END IF
	IF (RA2.LT.0.0) RA2=RA2+24.0
C
	RETURN
	END
