	SUBROUTINE UTGST(JD,UT,GST)
C
C       This subroutine computes the mean Greenwich sidereal time
C  given the Julian Day number and the Universal time.
C			Donald H. Gudehus, May, 1983
C
	REAL*8 UT,GST,JDATE,T
C
C  Convert the Julian Day number to the Julian Date at 0h Universal
C  time.
	JDATE=DBLE(JD)-0.5D0
C  Get the number of Julian centuries of 36525 days of universal time
C  elapsed since Greenwich mean noon (12h Universal Time) on Januaray 0, 
C  1900.
	T=(JDATE-2415020.0D0)/36525.0D0
C  Compute the sidereal time in revolutions for 0h UT (from Explanatory
C  Supplement, p. 73, converted to revolutions)
	GST=0.276919398D0+100.0021359D0*T+0.000001075D0*T**2
C  Get the fractional part and multiply by 24 to get hours
	GST=24.0D0*(GST-INT(GST))
C  Get the sidereal time for the given UT
	GST=GST+UT*1.002737909265D0+0.589D-10*T !  p. 76
C
C  To compute the apparent sidereal time one would correct for nutation in
C  right ascension.  That is,
C	CALL NUTATION(JD,UT,DELTAPSI,DELTAEPS)
C  Get the obliquity of the ecliptic
C	CALL OBLIQUITY(JD,UT,EPSILON)
C	GST=GST+DELTAPSI*DCOSD(EPSILON)/(15.0D0*3600.0D0)
C  However, since we are computing mean sidereal time, we will not do this.
C
C  Be sure that the time is not greater than or equal to 24h
	IF (GST.GE.24.0D0) GST=GST-24.0D0
	RETURN
	END
