	SUBROUTINE VEL_AMP_ERROR(CHOICE,INDX,PARAMETRS,COVARIANCE,
	1SIGMA_VEL_AMP,SIGMA_VEL_AMP_SUBS)
C
C  Note:  This is the Unix version
C
C       This subroutine calculates the error in the velocity amplitude K for
C  a spectroscopic binary.  Since K is not an element of the orbit the error
C  is not calculated by NONLINFIT, and so must be calculated manually.  This
C  calculation is to be carried out after the elements have been solved
C  for, and is called by BINARY.FOR.
C				Donald H. Gudehus, February, 2000
C		Version 2, D. H. G., May 6, 2001
C		Took absolute value of VEL_AMP*
C		Version 3, D. Gudehus, December, 2002
C		Modified COMMON block for interferometric data
C
C	CHOICE		   (C)	A character array containing the choices of
C				what type of solution to use for a given
C				multiple star.  See details in BINARY.FOR.
C	INDX		   (C)	An array containing the indexes which
C				reference the associated variable in the
C				array PARAMETRS.  Each location in INDX
C				corresponds to one of many possible variables
C				that can be solved for in a multiple star
C				system.
C	PARAMETRS	   (C)	The array containing the parameters that are
C				solved for.
C	COVARIANCE	   (C)	The covariance array for the parameters.
C	SIGMA_VEL_AMP	   (R)	An array containing the errors for the velocity 
C				amplitudes, K, for the primary and secondary.
C	SIGMA_VEL_AMP_SUBS (R)	An array containing the errors for the velocity
C				amplitudes, K, for the subcomponennts of the
C				primary and secondary.
C
	INCLUDE 'binary_parameters.for'
C
	CHARACTER*(*) CHOICE
	INTEGER INDX(MAXNUMPARAM)
	REAL*8 PARAMETRS(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	1SIGMA_VEL_AMP(2),SIGMA_VEL_AMP_SUBS(2*NSUBS_MAX)
C
	REAL*8 P,ECCENTR,INCLINA,a,aAU,RATIO_a,PARALLAX,DEG_RAD
C
	INTEGER SEMIMAJOR_AXIS_ASSIGNMENT(2)
	COMMON /BINARY_INTEGER4/ SEMIMAJOR_AXIS_ASSIGNMENT
C  Only part of this COMMON block is used here
C
	REAL*8 PX(MAXNPOINTS_COM),PY(MAXNPOINTS_COM),
	1FR_ANGLE(MAXNPOINTS_COM),EPOCH,
	1VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB(NSUBS_MAX),
	2VEL_AMP_SEC_SUB(NSUBS_MAX),VEL_AMP_PRI_SUB_SEC(NSUBS_MAX),
	3VEL_AMP_SEC_SUB_SEC(NSUBS_MAX)
	COMMON /REAL8_SCRATCH_BLOCK/ PX,PY,FR_ANGLE,
	1EPOCH,VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB,VEL_AMP_SEC_SUB,
	2VEL_AMP_PRI_SUB_SEC,VEL_AMP_SEC_SUB_SEC
C  Only part of this COMMON block is used here
C
	LOGICAL VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
	COMMON /BINARY_LOGICALS/ VISUAL,SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,
	1SPECTROSCOPIC_SEC_SUB,INTERFEROMETRIC,PAR_PM,PAR_PM_INTERFERO,
	2SOLVE_PAR,INCLINATION_AVAILABLE,RATIO_AVAILABLE,IGNORE_MAIN_PAIR
C
	DEG_RAD=57.29577951308232D0
C
C  Get the elements needed to calculate the errors.  The code in this section
C  is similar to that in NLFUN3.FOR.
	P=ABS(PARAMETRS(1))
	ECCENTR=ABS(PARAMETRS(3))
	IF (ECCENTR.GT.1.0D0) ECCENTR=2.0D0-ECCENTR !  Prevents the impossible 
C  The logicals VISUAL and SPECTROSCOPIC are assumed to have been set by a
C  previous call to BINARY_INDEXES.
	IF (SPECTROSCOPIC.AND..NOT.VISUAL) THEN
	  INCLINA=90.0D0
	ELSE IF (VISUAL) THEN !  Either SPECTROSCOPIC or not SPECTROSCOPIC
	  INCLINA=PARAMETRS(5)
	END IF	
	IF (VISUAL) a=PARAMETRS(INDX(7))
	IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(4:4).EQ.'B') THEN
		!  Visual binary pair or a spectroscopic binary pair
	  RATIO_a=PARAMETRS(INDX(8))
	ELSE !  Single member
C  Ignore ratio of semimajor axes
	END IF
C  The logicals PAR_PM and SOLVE_PAR are assumed to have been set by a
C  previous call to BINARY_INDEXES.
	IF (PAR_PM) THEN
	  PARALLAX=PARAMETRS(INDX(9))
	ELSE
	  IF (SOLVE_PAR) PARALLAX=PARAMETRS(INDX(9))
	END IF
	IF (SPECTROSCOPIC) THEN
	  IF (.NOT.PAR_PM.AND..NOT.SOLVE_PAR) aAU=PARAMETRS(INDX(17))
C  For future expansion, the code for the subcomponent elements would go here
	END IF
C  Now we are ready to calculate the errors.  We assume that 
C  SEMIMAJOR_AXIS_ASSIGNMENT has been set by a previous call to BINARY_INDEXES.
	IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B') THEN
	  IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1) THEN !  a(") of pri.
	    SIGMA_VEL_AMP(1)=ABS(VEL_AMP_PRI)*SQRT(ABS(
	1   (1.0D0/a)**2*COVARIANCE(INDX(7),INDX(7))+
	2   (1.0D0/DTAND(INCLINA))**2*COVARIANCE(5,5)/DEG_RAD**2+
	3   (1.0D0/PARALLAX)**2*COVARIANCE(INDX(9),INDX(9))+
	4   (1.0D0/P)**2*COVARIANCE(1,1)+
	5   (ECCENTR/(1.0D0-ECCENTR**2))**2*COVARIANCE(3,3)+
	6   2.0D0/a/DTAND(INCLINA)*COVARIANCE(INDX(7),5)/DEG_RAD-
	7   2.0D0/a/PARALLAX*COVARIANCE(INDX(7),INDX(9))-
	8   2.0D0/a/P*COVARIANCE(INDX(7),1)+
	9   2.0D0/a*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(INDX(7),3)-
	1   2.0D0/DTAND(INCLINA)/PARALLAX*COVARIANCE(5,INDX(9))/DEG_RAD-
	2   2.0D0/DTAND(INCLINA)/P*COVARIANCE(5,1)/DEG_RAD+
	3   2.0D0/DTAND(INCLINA)*ECCENTR/(1.0D0-ECCENTR**2)*
	3     COVARIANCE(5,3)/DEG_RAD+
	4   2.0D0/PARALLAX/P*COVARIANCE(INDX(9),1)-
	5   2.0D0/PARALLAX*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(INDX(9),3)-
	6   2.0D0/P*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(1,3)))
	  ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.1) THEN !  a(AU) of pri.
	    SIGMA_VEL_AMP(1)=ABS(VEL_AMP_PRI)*SQRT(
	1   (1.0D0/aAU)**2*COVARIANCE(INDX(17),INDX(17))+
	2   (1.0D0/P)**2*COVARIANCE(1,1)+
	3   (1.0D0*ECCENTR/(1.0D0-ECCENTR**2))**2*COVARIANCE(3,3)-
	4   2.0D0/aAU/P*COVARIANCE(INDX(17),1)+
	5   2.0D0/aAU*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(INDX(17),3)-
	6   2.0D0/P**ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(1,3))
	  END IF
	END IF
	IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') THEN !  For secondary
	  IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1) THEN !  a(") of pri.
	    SIGMA_VEL_AMP(2)=ABS(VEL_AMP_SEC)*SQRT(
	1   (1.0D0/(RATIO_a-1.0D0))**2*COVARIANCE(INDX(8),INDX(8))+
	2   (1.0D0/a)**2*COVARIANCE(INDX(7),INDX(7))+
	3   (1.0D0/DTAND(INCLINA))**2*COVARIANCE(5,5)/DEG_RAD**2+
	4   (1.0D0/PARALLAX)**2*COVARIANCE(INDX(9),INDX(9))+
	5   (1.0D0/P)**2*COVARIANCE(1,1)+
	6   (1.0D0*ECCENTR/(1.0D0-ECCENTR**2))**2*COVARIANCE(3,3)+
	7   2.0D0/(RATIO_a-1.0D0)/a*COVARIANCE(INDX(8),INDX(7))+
	8   2.0D0/(RATIO_a-1.0D0)/DTAND(INCLINA)*COVARIANCE(INDX(8),5)/
	9   DEG_RAD-
	9   2.0D0/(RATIO_a-1.0D0)/PARALLAX*COVARIANCE(INDX(8),INDX(9))-
	1   2.0D0/(RATIO_a-1.0D0)/P*COVARIANCE(INDX(8),1)+
	2   2.0D0/(RATIO_a-1.0D0)*ECCENTR/(1.0D0-ECCENTR**2)*
	2     COVARIANCE(INDX(8),3)+
	3   2.0D0/a/DTAND(INCLINA)*COVARIANCE(INDX(7),5)/DEG_RAD-
	4   2.0D0/a/PARALLAX*COVARIANCE(INDX(7),INDX(9))-
	5   2.0D0/a/P*COVARIANCE(INDX(7),1)+
	6   2.0D0/a*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(INDX(7),3)-
	7   2.0D0/DTAND(INCLINA)/PARALLAX*COVARIANCE(5,INDX(9))/DEG_RAD-
	8   2.0D0/DTAND(INCLINA)/P*COVARIANCE(5,1)/DEG_RAD+
	9   2.0D0/DTAND(INCLINA)*ECCENTR/(1.0D0-ECCENTR**2)*
	9     COVARIANCE(5,3)/DEG_RAD+
	1   2.0D0/PARALLAX/P*COVARIANCE(INDX(9),1)-
	2   2.0D0/PARALLAX*ECCENTR/(1.0D0-ECCENTR**2)*
	3   COVARIANCE(INDX(9),3)-
	4   2.0D0/P*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(1,3))
	  END IF
	  IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.2) THEN !  a(") of sec.
	    SIGMA_VEL_AMP(2)=ABS(VEL_AMP_SEC)*SQRT(
	1   (1.0D0/a)**2*COVARIANCE(INDX(7),INDX(7))+
	2   (1.0D0/DTAND(INCLINA))**2*COVARIANCE(5,5)/DEG_RAD**2+
	3   (1.0D0/PARALLAX)**2*COVARIANCE(INDX(9),INDX(9))+
	4   (1.0D0/P)**2*COVARIANCE(1,1)+
	5   (ECCENTR/(1.0D0-ECCENTR**2))**2*COVARIANCE(3,3)+
	6   2.0D0/a/DTAND(INCLINA)*COVARIANCE(INDX(7),5)/DEG_RAD-
	7   2.0D0/a/PARALLAX*COVARIANCE(INDX(7),INDX(9))-
	8   2.0D0/a/P*COVARIANCE(INDX(7),1)+
	9   2.0D0/a*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(INDX(7),3)-
	1   2.0D0/DTAND(INCLINA)/PARALLAX*COVARIANCE(5,INDX(9))/DEG_RAD-
	2   2.0D0/DTAND(INCLINA)/P*COVARIANCE(5,1)/DEG_RAD+
	3   2.0D0/DTAND(INCLINA)*ECCENTR/(1.0D0-ECCENTR**2)*
	3     COVARIANCE(5,3)/DEG_RAD+
	4   2.0D0/PARALLAX/P*COVARIANCE(INDX(9),1)-
	5   2.0D0/PARALLAX*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(INDX(9),3)-
	6   2.0D0/P*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(1,3))
	  ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.1) THEN !  a(AU) of pri.
	    SIGMA_VEL_AMP(2)=ABS(VEL_AMP_SEC)*SQRT(
	1   (1.0D0/(RATIO_a-1.0D0))**2*COVARIANCE(INDX(8),INDX(8))+
	2   (1.0D0/aAU)**2*COVARIANCE(INDX(17),INDX(17))+
	3   (1.0D0/P)**2*COVARIANCE(1,1)+
	4   (ECCENTR/(1.0D0-ECCENTR**2))**2*COVARIANCE(3,3)+
	5   2.0D0/(RATIO_a-1.0D0)/aAU*COVARIANCE(INDX(8),INDX(17))-
	6   2.0D0/(RATIO_a-1.0D0)/P*COVARIANCE(INDX(8),1)+
	7   2.0D0/(RATIO_a-1.0D0)*ECCENTR/(1.0D0-ECCENTR**2)*
	7     COVARIANCE(INDX(8),3)-
	8   2.0D0/aAU/P*COVARIANCE(INDX(17),1)+
	9   2.0D0/aAU*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(INDX(17),3)-
	1   2.0D0/P*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(1,3))
	  ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.2) THEN !  a(AU) of sec.
	    SIGMA_VEL_AMP(2)=ABS(VEL_AMP_SEC)*SQRT(
	1   (1.0D0/aAU)**2*COVARIANCE(INDX(17),INDX(17))+
	2   (1.0D0/P)**2*COVARIANCE(1,1)+
	3   (1.0D0*ECCENTR/(1.0D0-ECCENTR**2))**2*COVARIANCE(3,3)-
	4   2.0D0/aAU/P*COVARIANCE(INDX(17),1)+
	5   2.0D0/aAU*ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(INDX(17),3)-
	6   2.0D0/P**ECCENTR/(1.0D0-ECCENTR**2)*COVARIANCE(1,3))
	  END IF
	END IF
	RETURN
	END
