	SUBROUTINE WRITE_BINARY_HEADER(UNIT,NAME,RIGHTASC,DECLINA,
	1EQUATOR,TYPE,PARALLAX_PROPER_MOTION,SCALE,FILETYPE,ELEMENTS,
	2C_ELEMENTS,N_ELEMENTS)
C
C	Note:  This is the Unix version
C
C	This subroutine writes the header of a binary star file from
C  the information supplied in the argument list.  We assume that the file has
C  been rewound if necessary.
C
C				Donald H. Gudehus, March, 1994
C		Version 2, D. Gudehus, April, 1994
C		Incorporated code for spectroscopic binary
C		Version 2U, D. Gudehus, September, 1997
C		Made INCLUDE files lowercase for UNIX.
C		Version 3, D. Gudehus, May 2, 1998
C		Made adjustment to position of scale units for visual binary
C		Version 4, D. Gudehus, October 8, 1999
C		Changed IF block usage of character dummy variables to
C		a range of indexes so that constants can be used in the
C		calling routine.
C		Version 5, D. Gudehus, December, 2002
C		Added code for 1-D interferometric data
C		Version 6, D. Gudehus, January, 2003
C		Added TYPE='r'
C		Version 7, D. Gudehus, February 5,2004
C		Increased the significance of C_ELEMENTS, and encoded more
C		characters for EQUATOR and SCALE.
C		Version 8, D. Gudehus, March 1, 2008
C		Corrected bug where header lines for "System radial velocity"
C		and "Sine of inclination times semimajor axis in AU" were
C		being duplicated. 
C
C	UNIT			(C)	The logical unit to be used for writing
C	NAME			(C)	The name of the binary star
C	RIGHTASC		(C)	The right ascension of the binary
C	DECLINA			(C)	The declination of the binary
C	EQUATOR			(C)	The epoch of the equator
C	TYPE			(C)	The type of data, absolute (A) or 
C					relative (R).
C	PARALLAX_PROPER_MOTION	(C)	A flag for wheter parallax and
C					proper motion are included (I),
C					or removed (R).
C	SCALE			(C)	The scale, "/unit or km/sec/unit.
C	FILETYPE		(C)	The type of file, X_Y, THETA_RHO,
C					BY_VELOCITY, or JD_VELOCITY.
C	ELEMENTS		(C)	An array containing the elements of
C					the orbit or orbits.  Elements not
C					in use or for which there are no 
C					available values, should be set 
C					equal to -1.0D37.
C	C_ELEMENTS		(C)	A character array containing the 
C					elements of the orbit or orbits.
C					These are written if the 
C					corresponding value of ELEMENT 
C					is > -1.0D37
C	N_ELEMENTS		(C)	The length of the above two arrays.
C	
	CHARACTER*1 CSIGN,CSIGN_D,TYPE,PARALLAX_PROPER_MOTION
	CHARACTER*18 C_ELEMENTS(N_ELEMENTS)
	CHARACTER*11 FILETYPE
	CHARACTER*79 STRING
	CHARACTER*80 NAME
	INTEGER UNIT
	REAL MIN
	REAL*8 SCALE,RIGHTASC,DECLINA,EQUATOR,ELEMENTS(N_ELEMENTS)
	INCLUDE 'binary_parameters.for'
	INCLUDE 'binary_header.for'
C
	STRING(1:5)='Name='
	STRING(6:79)=NAME
	WRITE (UNIT,1000) STRING
1000	FORMAT(' ',A) 
C
	STRING=' '
	STRING(1:16)='Right Ascension='
	ERROR=1.0E-4
	CALL DHHMS(RIGHTASC,ERROR,HR,MIN,SEC,CSIGN)
	ENCODE (2,1002,STRING(17:18)) INT(HR)
1002	FORMAT (I2)
C  Use leading zeroes
	IF (STRING(17:17).EQ.' ') STRING(17:17)='0'
	IF (STRING(18:18).EQ.' ') STRING(18:18)='0'
	ENCODE (2,1002,STRING(20:21)) INT(MIN)
	IF (STRING(20:20).EQ.' ') STRING(20:20)='0'
	IF (STRING(21:21).EQ.' ') STRING(21:21)='0'
	ENCODE (5,2052,STRING(23:27)) SEC
2052	FORMAT (F5.2)
	WRITE (UNIT,1000) STRING
C
	STRING=' '
	STRING(1:12)='Declination='
	CALL DHHMS(DECLINA,ERROR,DEG,MIN,SEC,CSIGN_D)
	STRING(13:13)=CSIGN_D
C  Use a plus sign if necessary
	IF (STRING(13:13).EQ.' ') STRING(13:13)='+'
	ENCODE (2,1002,STRING(14:15)) INT(DEG)
	IF (STRING(14:14).EQ.' ') STRING(14:14)='0'
	IF (STRING(15:15).EQ.' ') STRING(15:15)='0'
	ENCODE (2,1002,STRING(17:18)) INT(MIN)
	IF (STRING(17:17).EQ.' ') STRING(17:17)='0'
	IF (STRING(18:18).EQ.' ') STRING(18:18)='0'
	ENCODE (5,2041,STRING(20:24)) SEC
2041	FORMAT (F4.1)
	WRITE (UNIT,1000) STRING
C
	STRING=' '
	STRING(1:8)='Equator='
	ENCODE (11,2115,STRING(9:19)) EQUATOR
2115	FORMAT (F11.5)
	WRITE (UNIT,1000) STRING
C
	STRING=' '
	STRING(1:5)='Type='
	IF (TYPE(1:1).EQ.'A') STRING(6:19)='Absolute orbit'
	IF (TYPE(1:1).EQ.'R') STRING(6:19)='Relative orbit'
	IF (TYPE(1:1).EQ.'r') STRING(6:40)=
	1'Relative orbit, Equator is constant'
	WRITE (UNIT,1000) STRING
C
	IF (FILETYPE(4:11).NE.'VELOCITY') THEN
	  STRING=' '
	  STRING(1:27)='Parallax and proper motion='
	  IF (PARALLAX_PROPER_MOTION(1:1).EQ.'I') STRING(28:35)='Included'
	  IF (PARALLAX_PROPER_MOTION(1:1).EQ.'R') STRING(28:34)='Removed'
	  WRITE (UNIT,1000) STRING
	END IF
C
	STRING=' '
	STRING(1:6)='Scale='
C  Assume 11 characters for encoding scale
	N=11
	ENCODE (N,2116,STRING(7:6+N)) SCALE
2116	FORMAT (F11.6)
	IF (FILETYPE(4:11).NE.'VELOCITY') THEN
	  STRING(7+N:13+N)=' "/Unit'
	ELSE IF (FILETYPE(4:11).EQ.'VELOCITY') THEN
	  STRING(7+N:18+N)=' km/sec/Unit'
	END IF
	WRITE (UNIT,1000) STRING
C
C  Take care of the elements if they exist
	DO I=1,9
	  IF (ELEMENTS(I).GT.-1.0D37) THEN
	    CALL ALPHA(%REF(HEAD(I)),1,47,IA,NR,NF,1)
	    WRITE(UNIT,5000) HEAD(I),C_ELEMENTS(I)
5000	    FORMAT(' ',A<IA>,A)
	  END IF
	END DO
	IF (ELEMENTS(10).GT.-1.0D37) THEN
	  CALL ALPHA(%REF(HEAD(10)),1,47,IA,NR,NF,1)
	  WRITE(UNIT,5000) HEAD(10),C_ELEMENTS(10)
C  Put X center for 1-D interferometric data in the correct sequence
	ELSE IF (ELEMENTS(89).GT.-1.0D37) THEN
	  CALL ALPHA(%REF(HEAD(89)),1,47,IA,NR,NF,1)
	  WRITE(UNIT,5000) HEAD(89),C_ELEMENTS(89)
	END IF
	DO I=11,12
	  IF (ELEMENTS(I).GT.-1.0D37) THEN
	    CALL ALPHA(%REF(HEAD(I)),1,47,IA,NR,NF,1)
	    WRITE(UNIT,5000) HEAD(I),C_ELEMENTS(I)
	  END IF
	END DO
	IF (ELEMENTS(13).GT.-1.0D37) THEN
	  CALL ALPHA(%REF(HEAD(13)),1,47,IA,NR,NF,1)
	  WRITE(UNIT,5000) HEAD(13),C_ELEMENTS(13)
C  Put Y center for 1-D interferometric data in the correct sequence
	ELSE IF (ELEMENTS(90).GT.-1.0D37) THEN
	  CALL ALPHA(%REF(HEAD(90)),1,47,IA,NR,NF,1)
	  WRITE(UNIT,5000) HEAD(90),C_ELEMENTS(90)
	END IF
	DO I=14,MAXHEAD
	  IF (ELEMENTS(I).GT.-1.0D37) THEN
	    CALL ALPHA(%REF(HEAD(I)),1,47,IA,NR,NF,1)
	    WRITE(UNIT,5000) HEAD(I),C_ELEMENTS(I)
	  END IF
	END DO
C
	IF (FILETYPE(1:3).EQ.'X_Y') THEN
	  WRITE (UNIT,1100)
1100	  FORMAT(' ','   Epoch                 X             Sigma_X',
	1'            Y            Sigma_Y')
	ELSE IF (FILETYPE(1:9).EQ.'THETA_RHO') THEN
	  WRITE (UNIT,1200)
1200	  FORMAT(' ','   Epoch               Theta         Sigma_Theta',
	1'         Rho          Sigma_Rho')
	ELSE IF (FILETYPE(1:11).EQ.'BY_VELOCITY') THEN
	  WRITE (UNIT,1300)
1300	  FORMAT(' ','    Epoch(BY)           v(radial)         Sigma_v')
	ELSE IF (FILETYPE(1:11).EQ.'JD_VELOCITY') THEN
	  WRITE (UNIT,1301)
1301	  FORMAT(' ','    Epoch(JD)           v(radial)         Sigma_v')
	ELSE IF (FILETYPE(1:6).EQ.'BY_RHO') THEN
	  WRITE (UNIT,1400)
1400	  FORMAT(' ','    Epoch(BY)       Theta(interfer.)',
	1 '  Rho(projected)    Sigma_Rho')
	ELSE IF (FILETYPE(1:6).EQ.'JD_RHO') THEN
	  WRITE (UNIT,1401)
1401	  FORMAT(' ','    Epoch(JD)       Theta(interfer.)',
	1 '  Rho(projected)    Sigma_Rho')
	END IF
	RETURN
	END
