	PROGRAM BINARY
C
C  Note:  This is the Unix version
C
C       This program makes use of the nonlinear least squares subroutine,
C  NONLINFIT to solve for the elements of the orbit of a binary star.  One 
C  can solve for a pair of stars or for a single star.  The data may be 
C  astrometric (x and y positions), visual (position angles and separations, 
C  or x and y positions) and/or spectroscopic.  The astrometric, visual 
C  and spectroscopic data are used to solve simultaneously
C  for the orbital elements, including the proper motion, parallax,
C  ratio of semimajor axes, and system radial velocity, when sufficient
C  data is available.  Both the primary and secondary are allowed to have
C  spectroscopic subcomponents.  Any subcomponent amplitude present in the
C  primary or secondary data is a reflex velocity of the actual subcomponent
C  star.  In the notation used in the program, the subcomponent star is 
C  considered to be a secondary of the parent star.  Data files for the 
C  subcomponent can be incorporated into the solution as well.  Up to 5 
C  subcomponents in each can be simultaneously be solved for along with the
C  astrometric and separation data.  Therefore up to 12 stars could be 
C  considered in a restricted manner.  The data files are assumed to be 
C  in the standard binary star format used in the program EDITBINARY.
C  If reflex velocities are present in the spectra of a single star, as
C  in the case of planets orbiting a star, obtaining a solution requires
C  that one assume an orbit of a very long period for an imaginary
C  spectroscopic binary, and then hold those elements constant in the
C  solution for the subcomponent(s).  For example, a period of 10^37 y
C  can be assigned and elements of index 1 (Period), 2 (Time of periastron
C  passage), 3 (Eccentricity), 4 (Angle from node to periastron), and 6
C  (Sine of inclination times semimajor axis) held constant.  The process
C  is made easier by including the key value 9999.99999 for the period in the
C  data file.  This key value automatically sets the period of the imaginary
C  main pair to 1.0X10^37, assigns values to the four other elements, and
C  holds them constant in the solution.
C       For astrometric data, the equator for right ascension and declination
C  is assumed to also be the equator for x and y positions.  For visual
C  and speckle data, the equator is assumed to be the epoch of observation.
C  The data files can optionally include initial estimates of the elements.
C  The angle from the node to periastron is always taken to be that of the 
C  primary, even when only data for the secondary exists.  The interpretation
C  of the semimajor axis depends on particular case.  For primary and 
C  secondary of a particular kind (visual or spectroscopic) solved together, 
C  "a" is that of the primary.  For a single orbit of a particular type, 
C  "a" applies to that particular orbit.
C       Finally, data with a sigma of 9999.99999 are ignored by the program,
C  and the residuals of the visual binary data can be written to a standard
C  file for later analysis.
C
C			Donald H. Gudehus, April, 1986
C			Version 2, Donald H. Gudehus, March, 1994
C			Revised program to read standard binary star
C			file, and handle several possible cases (15).
C			Version 3, Donald H. Gudehus, January, 1995
C			Allowed for astrometrically determined parallax
C			to be corrected by a predetermined correction
C			factor which represents the effect of the finite
C			distance of the standard stars.
C			Version 4, Donald H. Gudehus, May, 1995
C			Various bug fixes for plotting spectroscopic 
C			secondary
C			Version 5, Donald H. Gudehus, June, 1995
C			Added capability to solve for spectroscopic 
C			subcomponents.
C			Version 6, Donald H. Gudehus, October, 1998
C			Added printout of descriptions of each plot.
C			In printout of residuals, changed from ratio of 
C			residual to sigma in file, to ratio of residual to
C			mean error of an observation of average weight of
C			solution.
C			Version 7, Donald H. Gudehus, October, 1999
C			Added capability to write the residuals of the
C			visual binary solution to a standard file for
C			later analysis.
C			Version 7U, Donald H. Gudehus, October 13, 1999
C			Modified program to work under Unix.  Plotting
C			is at this time disabled under Unix.
C			Version 8, Donald H. Gudehus, November 24, 1999
C			Fixed bug in concatenated string in call to
C			MESSAGE.
C			Version 9, Donald H. Gudehus, November 25, 1999
C			Fixed bug where residuals of spectroscopic component
C			files were being displayed when not asked for.  Fixed
C			bug where incorrect index was being displayed for
C			projected velocity amplitude of primary or secondary
C			subcomponent secondary.  Increased number of points
C			to 5000.
C			Version 10, Donald H. Gudehus, December 16, 1999
C			Added ability to exclude various ranges of phase
C			from spectroscopic data so as to test for line
C			blending.  Also added calculation of period in days,
C			and distance in parsecs.
C			Version 11, Donald H. Gudehus, February 21, 2000
C			Added ability to redo solution, excluding points
C			which deviate by more than a given factor times the
C			mean error.
C			Version 11U, Donald H. Gudehus, July 23, 2000
C			Fixed bug where check for SIGMA equal to 9999.99999
C			was not working in the Unix version but was in the
C			OpenVMS version.
C			Version 12, Donald H. Gudehus, April 27, 2001
C			Incorported call to new version of PARALLAXFACTORS.
C			Made WEIGHT_SCALE for X and Y two-dimensional.
C			Version 13, Donald H. Gudehus, May 2, 2001
C			Meaning of proper motion acceleration is now true
C			acceleration rather than coefficient of the
C			quadratic factor.
C			Version 14, Donald H. Gudehus, August 23, 2001
C			Fixed bug where the residual vectors were not
C			being drawn for the primary when both primary and
C			secondary visual binary data were being considered.
C			Fixed bug where the average of initial estimates 
C			of the semimajor axis for a primary and secondary
C			spectroscopic binary was being calculated when
C			it shouldn't have been.
C			Version 15, Donald H. Gudehus, September 24, 2001
C			Added ability to calculate apsidal motion for
C			the subcomponents.  Added INDX to COMMON block
C			so that the values appropriate to the solution,
C			and therefore PARAMETRS, can be used when plotting.
C			Version 16, Donald H. Gudehus, October 16, 2001
C			Added ability for program to be run from a "save"
C			file with alterable parameters.
C			Version 17, D. H. G., November 23, 2002
C			Added plotting of errorbars and increased number of
C			points for data and plotting to 10000.
C			Version 18, D. H. G., Dec. 22, 2002
C			Added calculation of mass function and sum of masses
C			for the spectroscopic cases.  Fixed bug in nel.for
C			where occassional NaN (not a number) situations from
C			user entered data would cause an incorrect count of
C			parameters in usrinp.for.  Fixed the logic in
C			binary_indexes.for for determining when the parallax
C			could be solved for.  This had caused an error in the
C			case of a combined solution with data from a visual
C			secondary with relative coordinates, and a
C			spectroscopic secondary.
C			Version 19, D. H. G., Dec. 31, 2002
C			Fixed bug in optional output of residuals where the
C			ratio of rms average of residual to mean error, for
C			the case of a visual secondary with measurements in
C			units of theta and rho was not being done correctly.
C			Version 20, D. H. G., January 3-14, 2003
C			Added new header line "Relative orbit, Equator is
C			constant" and corresponding TYPE='r' to cover
C			nonstandard case.  Improved code for calculation
C			of mean error in section where  residuals are
C			requested to be viewed and some parameters are
C			optionally held constant in the solution.  Added
C			separate plots of primary and secondary in addition
C			to the combined plot for visual binaries when both
C			files are available.  Allowed scaling of weights
C			directly in theta and rho.  Added a symbol marking
C			the location of the primary in a relative plot.
C			Version 21, D. H. G., July 20, 2003
C			Fixed a typo where MASS_FUNC was misspelled as
C			MASS_FUNCT in the calculation of the mean error
C			of the mass function.  The corrected value is smaller
C			by about a part in 500.  The typo was pointed out by
C			Larry Wasserman of Lowell Observatory.
C			Version 22, D. H. G., February 5, 2004
C			Increased the significance of C_ELEMENTS so that up
C			to 11 characters can be decoded from the initial
C			estimates in the file.
C			Version 23, D. H. G., February 24, 2004
C			Introduced a key value of 9999.99999 for the period
C			to simplify solutions of planetary orbits in the
C			case where reflex velocities are present in only
C			one star.  The key value asigns a period of 10^37 y
C			to an imaginary pair whose elements are held
C			constant in the solution.
C			Version 24, D. H. G., September 10, 2004
C			Fixed plotting problem for case where primary
C			with no secondary data, included parallax information,
C			and the observed-to-true parallax was substantially
C			less than one.
C			Version 25, D. H. G., November 6, 2004
C			Added calculation of a(AU) and a(AU)sin(i) under
C			"Additional Information" for certain combined solution
C			cases.
C			Version 26, D. H. G., April 5, 2005
C			Various improvements
C			Version 27, D. H. G., May 19, 2005
C			Added calculation of expected times of eclipses for
C			the case when there is only a spectroscopic binary.
C			Version 28, D. H. G., March 5, 2008
C			Adjusted number of points for the curves depending on
C			whether the plot was over all time or just one cycle.
C			Version 29, D. H. G., November 10, 2013
C			Added support for 64-bit processors
C			
C
C  Some files that are available are:
C	mucas_l_pri.dat		Lippincott, Parallax and proper
C				motion removed
C	mucas_l_pri_ppm.dat	Parallax and proper motion not 
C				Removed;  scale is about 12.6 smaller
C	mucas_sec.dat		Speckle resolutions of secondary
C	mucas_vel_pri.dat	Spectroscopic observations of primary
C	hr6469_sec.dat		Speckle observations of secondary from
C				Scarfe et al. (1994)
C	hr6469_vel_pri.dat	Spectroscopic observations of primary
C	hr6469_vel_sec.dat	Spectroscopic observations of secondary
C				with one subcomponent
C	47_uma.dat		Spectroscopic observations of two
C				planets about single star from
C				Fischer et al 2002
C
	INCLUDE 'binary_parameters.for'
C
	INTEGER COLOR(MAXNPLOTS),
	1FUNCAT,ITIME(2),NPLPOINTS(MAXNPLOTS),LINVECT(MAXNPLOTS),
	2INC(MAXNPLOTS),LINTYPE(MAXNPLOTS),NSYMBOL(MAXNPLOTS),
	3IPRISUB_S_F(NSUBS_MAX),ISECSUB_S_F(NSUBS_MAX),
	4N_INITIAL_ESTIMATES(MAXNUMPARAM),INDX(MAXNUMPARAM),
	5SEMIMAJOR_AXIS_ASSIGNMENT(2),DIMENSN_DATA,DIMENSN_DATA_USED,
	6DIMENSN_IND,NPOINTS_DATA_VEL_P_SUB_S_OMIT(NSUBS_MAX),
	7NPOINTS_DATA_VEL_S_SUB_S_OMIT(NSUBS_MAX),NCONSTANT(MAXNUMPARAM)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,OPERATE,
	1INVERT(2),INTERACT,CROSSHAIRS,SCALEWEIGHTS,PLOTD,PLOTR,PLOTE,
	2MARK_PRI,CR,SHOW_RESID,WRITE_RESID,MISSING_SIGMA,VISUAL,
	3SPECTROSCOPIC,SPECTROSCOPIC_PRI_SUB,SPECTROSCOPIC_SEC_SUB,
	4INTERFEROMETRIC,PAR_PM,SOLVE_PAR,INCLINATION_AVAILABLE,
	5RATIO_AVAILABLE,LIGHT_TRAVEL_TIME,EXCLUDE_BY_PHASE_PRI,
	6EXCLUDE_BY_PHASE_SEC,EXCLUDE,REDO_SOLUTION,SOLVE_FOR_APSIDAL
	REAL XPLOT(MAXNPOINTS,MAXNPLOTS),DATPLOT(MAXNPOINTS,MAXNPLOTS)
	REAL*8 TIME(MAXNPOINTS,DIMENSN_IND_MAX),TIME_SAVE(MAXNPOINTS),
	1TIME_SAVE_JD(MAXNPOINTS),DATA(MAXNPOINTS,DIMENSN_DATA_MAX),
	2DATA_SAVE(MAXNPOINTS,DIMENSN_DATA_MAX),DATA_TEMP(MAXNPOINTS,2),
	3WEIGHT(MAXNPOINTS,DIMENSN_DATA_MAX),
	3WEIGHT_SAVE(MAXNPOINTS,DIMENSN_DATA_MAX),SIGMA_TEMP(2),
	4RESIDUALS(MAXNPOINTS,DIMENSN_DATA_MAX),
	5PARAMETRS(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	6INITIAL_ESTIMATES(MAXNUMPARAM),
	7INITIAL_ESTIMATES_TEMP(MAXNUMPARAM),
	7INITIAL_ESTIMATES_WRITE(MAXNUMPARAM),INITIAL_ESTIMATES_TEST,
	7MEANERROR(DIMENSN_DATA_MAX),MEANERRORMEANERROR(DIMENSN_DATA_MAX),
	8AVREDCHISQ,REDCHISQ(DIMENSN_DATA_MAX),RDCS1,RDCS2,MEOAW1,MEOAW2,
	9SIGMA(MAXNPOINTS,DIMENSN_DATA_MAX),SIGMA_SAVE(MAXNPOINTS,4),
	1SCALE_P,SCALE_S,SCALE_VEL_P,SCALE_VEL_S,SCALE_INTER_P,SCALE_INTER_S,
	2SCALE_VEL_P_SUB_S(NSUBS_MAX),SCALE_VEL_S_SUB_S(NSUBS_MAX),
	3SUM_P,SUM_S,SUM_VEL_P,SUM_VEL_S,SUM_VEL_P_SUB_S(NSUBS_MAX),
	4SUM_VEL_S_SUB_S(NSUBS_MAX),SUM_INTER_P,SUM_INTER_S,
	5SUM1,SUM2,SUM3,SUM4,RIGHTASC_EPOCH,
	5DECLINA_EPOCH,RIGHTASC_P,DECLINA_P,RIGHTASC_S,DECLINA_S,
	6RIGHTASC_VEL_P,DECLINA_VEL_P,RIGHTASC_VEL_S,DECLINA_VEL_S,
	7RIGHTASC_INTER_P,DECLINA_INTER_P,RIGHTASC_INTER_S,DECLINA_INTER_S,
	7EQUATOR_P,EQUATOR_S,EQUATOR_VIS,EQUATOR_VEL_P,EQUATOR_VEL_S,
	8EQUATOR_INTER_P,EQUATOR_INTER_S,EQUATOR_INTER,TIME_BEGIN,TIME_END,
	9DEG_RAD,TEMP1,TEMP2,TEMP3,TEMP4,TEMP5,WEIGHT_SCALE,
	1WEIGHT_SCALE_XY(2),WEIGHT_SCALE_THETA_RHO(2),DISTANCE,MASS_P,
	2MASS_S,MASS_SUM,MASS_FUNC,
	2PARALLAX,SIGMA_PARALLAX,SIGMA_DISTANCE,SIGMA_SMA_AU,
	3SIGMA_SMA_AU_SIN,OBS_TRUE_PAR,SIGMA_MASS,
	3SIGMA_VEL_AMP(2),SIGMA_VEL_AMP_SUBS(2*NSUBS_MAX),
	4EXCLUDED_PHASES_PRI(MAXNUMPHASES),EXCLUDED_PHASES_SEC(MAXNUMPHASES),
	5THRESHOLD_FACTOR,SMALLEST_PERIOD,ECLIPSE_EPOCHS(2)
C
C  PX and PY are the parallax factors.  They are calculated as part of
C  the fitting process.  EPOCH is used for the proper motion
C  calculation.  Currently, 10000 points can be fit, but this
C  number can be made arbitrarily large.

C  The SCRATCH_BLOCK blocks are used to transfer information to
C  NLFUN3, NLDER3, and BINARY_LTT_COR.  The BINARY COMMON blocks
C  are written to by BINARY_INDEXES.
	CHARACTER*1 TYPE_P,TYPE_S,TYPE_VEL_P,TYPE_VEL_S,TYPE_INTER_P,
	1TYPE_INTER_S
	COMMON /CHAR_SCRATCH_BLOCK/ TYPE_P,TYPE_S,TYPE_VEL_P,TYPE_VEL_S,
	1TYPE_INTER_P,TYPE_INTER_S
C
	INTEGER NPOINTS_DATA_VEL_P_SUB_S(NSUBS_MAX),
	1NPOINTS_DATA_VEL_S_SUB_S(NSUBS_MAX)
	COMMON /INTEGER4_SCRATCH_BLOCK/NPOINTSPXPY,NPOINTS_DATA_P,
	1NPOINTS_DATA_S,NPOINTS_VIS,NPOINTS_DATA_VEL_P,
	2NPOINTS_DATA_VEL_S,NPOINTS_SP,NPOINTS_DATA_VEL_P_SUB_S,
	3NPOINTS_DATA_VEL_S_SUB_S,NPOINTS_VEL_P_S,NPOINTS_VEL_S_S,
	4NPOINTS_DATA_INTER_P,NPOINTS_DATA_INTER_S,NPOINTS_INTER
C
	REAL*8 PX(MAXNPOINTS_COM),PY(MAXNPOINTS_COM),
	1FR_ANGLE(MAXNPOINTS_COM),EPOCH,
	2VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB(NSUBS_MAX),
	3VEL_AMP_SEC_SUB(NSUBS_MAX),VEL_AMP_PRI_SUB_SEC(NSUBS_MAX),
	4VEL_AMP_SEC_SUB_SEC(NSUBS_MAX),INCLINA_WORK,
	5INCLINA_WORK_PRI_SUB(NSUBS_MAX),
	6INCLINA_WORK_SEC_SUB(NSUBS_MAX),ECC_ANOM(MAXNPOINTS_COM),
	7ECC_ANOM_PRI_SUB(MAXNPOINTS_COM,NSUBS_MAX),
	8ECC_ANOM_SEC_SUB(MAXNPOINTS_COM,NSUBS_MAX),LTT_COR(MAXNPOINTS_COM),
	9PHOTOCENTRIC_TRUE_SEMIMAJAX,FUNCTN_TEMP(MAXNPOINTS_COM,4)
	  !  PX and PY are the parallax factors.  VEL_AMP_XXX are returned
	  !  values which are used in BINARY and NLDER3.
	COMMON /REAL8_SCRATCH_BLOCK/ PX,PY,FR_ANGLE,
	1EPOCH,VEL_AMP_PRI,VEL_AMP_SEC,VEL_AMP_PRI_SUB,VEL_AMP_SEC_SUB,
	2VEL_AMP_PRI_SUB_SEC,VEL_AMP_SEC_SUB_SEC,INCLINA_WORK,
	3INCLINA_WORK_PRI_SUB,INCLINA_WORK_SEC_SUB,
	4ECC_ANOM,ECC_ANOM_PRI_SUB,ECC_ANOM_SEC_SUB,LTT_COR,
	5PHOTOCENTRIC_TRUE_SEMIMAJAX,FUNCTN_TEMP
C
	INTEGER SEMIMAJOR_AXIS_ASSIGNMENT_BIN(2),DIMENSN_DATA_BIN,
	1DIMENSN_DATA_USED_BIN,INDX_BIN(MAXNUMPARAM)
	COMMON /BINARY_INTEGER4/ SEMIMAJOR_AXIS_ASSIGNMENT_BIN,
	1DIMENSN_DATA_BIN,DIMENSN_DATA_USED_BIN,INDX_BIN
C
	LOGICAL VISUAL_BIN,SPECTROSCOPIC_BIN,SPECTROSCOPIC_PRI_SUB_BIN,
	1SPECTROSCOPIC_SEC_SUB_BIN,INTERFEROMETRIC_BIN,PAR_PM_BIN,
	2PAR_PM_INTERFERO_BIN,SOLVE_PAR_BIN,INCLINATION_AVAILABLE_BIN,
	3RATIO_AVAILABLE_BIN,IGNORE_MAIN_PAIR_BIN
	COMMON /BINARY_LOGICALS/ VISUAL_BIN,SPECTROSCOPIC_BIN,
	1SPECTROSCOPIC_PRI_SUB_BIN,SPECTROSCOPIC_SEC_SUB_BIN,
	2INTERFEROMETRIC_BIN,PAR_PM_BIN,PAR_PM_INTERFERO_BIN,
	3SOLVE_PAR_BIN,INCLINATION_AVAILABLE_BIN,RATIO_AVAILABLE_BIN,
	4IGNORE_MAIN_PAIR_BIN
C
	CHARACTER*1 ITERATION_SCHEME,
	2PARALLAX_PROPER_MOTION_P,PARALLAX_PROPER_MOTION_S,
	3PARALLAX_PROPER_MOTION_VEL_P,PARALLAX_PROPER_MOTION_VEL_S,
	4PARALLAX_PROPER_MOTION_INTER_P,PARALLAX_PROPER_MOTION_INTER_S,
	5C_NPRISUB,C_NSECSUB,C_I
	CHARACTER*11 FILETYPE_P,FILETYPE_S,FILETYPE_VEL_P,FILETYPE_VEL_S,
	1FILETYPE_VEL_P_SUB_S(NSUBS_MAX),FILETYPE_VEL_S_SUB_S(NSUBS_MAX),
	2FILETYPE_INTER_P,FILETYPE_INTER_S
	CHARACTER*18 C_ELEMENTS(MAXNUMPARAM)
	CHARACTER*32 CHOICE,CHOICE_VEL,CHOICE_INTER,CHOICE_TEMP
	CHARACTER*35 FSTRING
	CHARACTER*47 DESCRIPTION(MAXHEAD)
	CHARACTER*50 XLABEL,YLABEL,TITLE,TITLE_VEL,
	1TITLE_VEL_PRISUB_S(NSUBS_MAX),TITLE_VEL_SECSUB_S(NSUBS_MAX),
	2TITLE_INTER
	CHARACTER*80 NAME_P,NAME_S,NAME_VEL_P,NAME_VEL_S,NAME_INTER_P,
	1NAME_INTER_S,NAME_SAVE_FILE
C
	CHARACTER*80 CNAME
	COMMON /CFILE/ CNAME
C
	BYTE BCNTI
	CHARACTER*1 CNTI
	EQUIVALENCE (BCNTI,CNTI)




	CHARACTER*1 SYSTEM_ENDIAN
	COMMON /SYS_ENDIAN/ SYSTEM_ENDIAN
	INTEGER SYSTEM_NUMBER_BYTES
	COMMON /SYS_NUM_BYTES/ SYSTEM_NUMBER_BYTES


C
	INCLUDE 'binary_header.for'
C
	DATA OMIT/'FF7FFFFF'X/
C
C  SIGMA is the value read from the file, scaled by any user input
C  scaling factor.
C  SIGMA_SAVE is the sigma from the input file, scaled by any user input
C  scaling factor.  It is only needed when the data is from a visual binary
C  in units of rho and theta, and is used in calculating the reduced chi
C  squared.
C  WEIGHT is the weight that is used in NONLINFIT, and calculated from SIGMA.
C  It can also be set to zero for points which deviate beyond a user
C  defined multiple of the mean error, when the solution is rerun.
C  WEIGHT_SAVE is the original WEIGHT before WEIGHT is set to zero.
C  In this case, the residuals for that point are not displayed.
C  Note that DATA, SIGMA, and WEIGHT are two-dimensional arrays where the
C  first index runs from 1 to MAXNPOINTS.  However when the second index
C  increments, the first does not return to 1, but continues to increment by
C  one as the arrays are filled when reading the files.  Therefore the
C  arrays can never be completely filled.
C
C  Get the number of processor bits
	CALL GET_OS_BYTES
C  Make carriage control as in OpenVMS
	OPEN(UNIT=6,STATUS='NEW',CARRIAGECONTROL='FORTRAN')
C  Open the log file
	OPEN(UNIT=7,NAME='binary.log',STATUS='UNKNOWN',
	1CARRIAGECONTROL='FORTRAN')
C
	CALL MESSAGE('*********************************************'//
	1'****************************')
	CALL MESSAGE(' ')
	CALL MESSAGE('      Welcome to BINARY, a program which can '//
	1'solve for the elements')
	CALL MESSAGE('         of a binary or multiple star system,'//
	1'using a combination')
	CALL MESSAGE('             of astrometric, visual, and '//
	1'spectroscopic data.')
	CALL MESSAGE(' ')
	CALL MESSAGE('                    Donald H. Gudehus, '//
	1'November 10, 2013')
	CALL MESSAGE(' ')
	CALL MESSAGE('*********************************************'//
	1'****************************')
	CALL MESSAGE(' ')
C
C  Handle the optional save file and parameters
	N=IARGC()
	IF (N.GT.1) THEN
	  CALL ERROR(-4,'TOO MANY ARGUMENTS')
	END IF
	CALL GETARG(1,NAME_SAVE_FILE)
	CALL ALPHA(%REF(NAME_SAVE_FILE),1,80,NC,NF,NR,1)
	NL=LEN(NAME_SAVE_FILE)
	NA=INDEX(NAME_SAVE_FILE(1:),' ')
	NB=INDEX(NAME_SAVE_FILE(1:),'(')
	IF (NB.EQ.0.OR.NB.GT.NA) THEN !  No parameters
	  N2=NA-1
	  N3=N2
	ELSE !  Parameters present
	  N2=NB-1
	  N3=INDEX(NAME_SAVE_FILE(N2:),')')+N2-1
	  IF (N3.EQ.0) N3=NL !  ) missing
	END IF
C
C  Store any option keywords and parameters in the buffer B in 
C  STRKEY.  If there are none, store parentheses instead.
	CALL STRKEY(NAME_SAVE_FILE(N2:N3))
C  Append the keywords and parameters in the save file to the
C  optional parameters already in the buffer B in strkey.
C  This is done by calling addkey.
	IF (NAME_SAVE_FILE.NE.' ') CALL RESTR(NAME_SAVE_FILE(1:N2))
C  Open a new save file
	CALL SAVE('binary.sve')
C
	BCNTI='09'X
	DEG_RAD=57.29577951308232D0
	DIMENSN_IND=1 !  Time
	FUNCAT=3 !  For NONLINFIT
	DO J=1,DIMENSN_DATA_MAX
	  DO I=1,MAXNPOINTS
	    WEIGHT(I,J)=0.0
	  END DO
	END DO
C  The following value is initially assigned to each element by 
C  GET_BINARY_HEADER and is used here as a test for whether the element was 
C  actually in the header
	INITIAL_ESTIMATES_TEST=-1.0D37
	DO I=1,MAXNUMPARAM
	  INITIAL_ESTIMATES(I)=0.0
	  N_INITIAL_ESTIMATES(I)=0
	  PARAMETRS(I)=0.0
	END DO
C  Initialize the light travel time correction
	DO I=1,MAXNPOINTS_COM
	  LTT_COR(I)=0.0
	END DO
C  Initialize various logicals
	TYPE_P=' '
	TYPE_S=' '
	TYPE_VEL_P=' '
	TYPE_VEL_S=' '
C  Initialize various numbers
	NPRISUB=0
	NSECSUB=0
	NPRISUB_S_F=0
	NSECSUB_S_F=0
C
C  Determine what display and plot devices are available
	CALL GET_DP_AVAILABILITY
C  Obtain the user's choices
	CHOICE=' '
	DO WHILE (CHOICE.NE.'P'.AND.CHOICE.NE.'S'.AND.CHOICE.NE.'B'.AND.
	1CHOICE.NE.'N')
	  CHOICE='S'
	  CALL MESSAGE('Enter a choice for the 2-D positional (XY and/or '//
	1 'Theta-Rho) information:')
	  CALL MESSAGE('            P = Positional data exists for primary')
	  CALL MESSAGE('            S = Positional data exists for'//
	1 ' secondary or for')
	  CALL MESSAGE('                relative orbit (default)')
	  CALL MESSAGE('            B = Positional data exists for both'//
	1 ' stars')
	  CALL MESSAGE('            N = No positional information available')
	  CALL USRINP(%REF(CHOICE),1,'A',10,1,'CHOICE_P=',' ')
	  CALL ALPHA(%REF(CHOICE),1,10,NC,NF,NR,4) 
				!  4 ensures left justification
	END DO
	CHOICE_VEL=' '
	DO WHILE (CHOICE_VEL.NE.'P'.AND.CHOICE_VEL.NE.'S'.AND.
	1CHOICE_VEL.NE.'B'.AND.CHOICE_VEL.NE.'N') 
	  CHOICE_VEL='P'
	  CALL MESSAGE('Enter a choice for the spectroscopic information:')
	  CALL MESSAGE('            P = Velocity data exists for primary'//
	1 ' (default)')
	  CALL MESSAGE('            S = Velocity data exists for secondary')
	  CALL MESSAGE('            B = Velocity data exists for both stars')
	  CALL MESSAGE('            N = No velocity information available')
	  CALL USRINP(%REF(CHOICE_VEL),1,'A',10,1,'CHOICE_S=',' ')
	  CALL ALPHA(%REF(CHOICE_VEL),1,10,NC,NF,NR,4)
	END DO
	IF (CHOICE.EQ.'N'.AND.CHOICE_VEL.EQ.'N') STOP
C  Include the velocity choice in CHOICE 
	CHOICE(4:4)=CHOICE_VEL(1:1)
	CHOICE_INTER=' '
c	DO WHILE (CHOICE_INTER.NE.'P'.AND.CHOICE_INTER.NE.'S'.AND.
c	1CHOICE_INTER.NE.'B'.AND.CHOICE_INTER.NE.'N')
	  CHOICE_INTER='N'
c	  CALL MESSAGE('Enter a choice for the 1-D interferometric'//
c	1 ' (projected separation) information:')
c	  CALL MESSAGE('            P = Positional data exists for primary')
c	  CALL MESSAGE('            S = Positional data exists for'//
c	1 ' secondary or for')
c	  CALL MESSAGE('                relative orbit')
c	  CALL MESSAGE('            B = Positional data exists for both'//
c	1 ' stars')
c	  CALL MESSAGE('            N = No positional information'//
c	1 ' available (default)')
c	  CALL USRINP(%REF(CHOICE_INTER),1,'A',10,1,'CHOICE_I=',' ')
c	  CALL ALPHA(%REF(CHOICE_INTER),1,10,NC,NF,NR,4)
c	END DO
C  Include the interferometric choice in CHOICE
	CHOICE(30:30)=CHOICE_INTER(1:1)
	IF (CHOICE.EQ.'N'.AND.CHOICE_VEL.EQ.'N'.AND.CHOICE_INTER.EQ.'N') STOP
C
	IF (CHOICE(1:1).NE.'N') THEN
	  CALL MESSAGE('Some files of positional data that are available are:')
	  CALL MESSAGE(
	1 '  mucas_l_pri.dat      Lippincott;  Parallax and proper')
	  CALL MESSAGE(
	1 '                               motion removed')
	  CALL MESSAGE(
	1 '  mucas_l_pri_ppm.dat  Lippincott;  Parallax and proper')
	  CALL MESSAGE(
	1 '                               motion not removed')
	  CALL MESSAGE(
	1 '  mucas_sec.dat        Various separations of pair')
	  CALL MESSAGE(
	1 '  hr6469_sec.dat       Scarfe et al;  separations of'//
	2 ' secondary')
	END IF
C  Obtain some preliminary information about the visual data
	IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B') THEN
	  CALL MESSAGE ('Open the primary star''s file for positions')
	  CALL OPENCF(21,3,'FILE_21=',0)
	  REWIND(21)
	  CALL GET_BINARY_HEADER(21,NAME_P,RIGHTASC_P,DECLINA_P,
	1 EQUATOR_P,TYPE_P,PARALLAX_PROPER_MOTION_P,SCALE_P,
	2 FILETYPE_P,INITIAL_ESTIMATES_TEMP,C_ELEMENTS,MAXNUMPARAM)
	  CALL PRINT_BINARY_HEADER(NAME_P,RIGHTASC_P,DECLINA_P,
	1 EQUATOR_P,TYPE_P,PARALLAX_PROPER_MOTION_P,SCALE_P,FILETYPE_P,
	2 INITIAL_ESTIMATES_TEMP,C_ELEMENTS,MAXNUMPARAM)
	  EQUATOR_VIS=EQUATOR_P
	  DO I=1,MAXNUMPARAM
	    IF (INITIAL_ESTIMATES_TEMP(I).GT.INITIAL_ESTIMATES_TEST) THEN
C  We may have initial estimates of a particular element from more than one
C  file, so we need to keep track.
	      N_INITIAL_ESTIMATES(I)=N_INITIAL_ESTIMATES(I)+1
	      INITIAL_ESTIMATES(I)=INITIAL_ESTIMATES(I)+
	1     INITIAL_ESTIMATES_TEMP(I)
	      INITIAL_ESTIMATES_TEMP(I)=INITIAL_ESTIMATES_TEST !  Resets
	    END IF
	  END DO
	  CHOICE(2:2)=PARALLAX_PROPER_MOTION_P !  Possibilities are
		!  I (included) and R (removed)
C  For CHOICE(2:2) equals I or R, we assume all observations have the
C  same equator.
C  Possibilities for TYPE_P are A (absolute) and R (relative) and r
C  (relative with fixed equator).  R and r would be rare for the
C  primary.  Astrometric data would be A.
	  TITLE=NAME_P
	END IF
	IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(1:1).EQ.'S') THEN
	  CALL MESSAGE('Open the secondary star''s file for positions')
	  CALL OPENCF(22,3,'FILE_22=',0)
	  CALL GET_BINARY_HEADER(22,NAME_S,RIGHTASC_S,DECLINA_S,
	1 EQUATOR_S,TYPE_S,PARALLAX_PROPER_MOTION_S,SCALE_S,
	2 FILETYPE_S,INITIAL_ESTIMATES_TEMP,C_ELEMENTS,MAXNUMPARAM)
	  CALL PRINT_BINARY_HEADER(NAME_S,RIGHTASC_S,DECLINA_S,
	1 EQUATOR_S,TYPE_S,PARALLAX_PROPER_MOTION_S,SCALE_S,FILETYPE_S,
	2 INITIAL_ESTIMATES_TEMP,C_ELEMENTS,MAXNUMPARAM)
	  DO I=1,MAXNUMPARAM
C  For case of both primary and secondary files, only use the semimajor 
C  axis of the primary.
	    IF ((INITIAL_ESTIMATES_TEMP(I).GT.INITIAL_ESTIMATES_TEST).AND.
	1   .NOT.(I.EQ.7.AND.CHOICE(1:1).EQ.'B')) THEN
	      N_INITIAL_ESTIMATES(I)=N_INITIAL_ESTIMATES(I)+1
	      INITIAL_ESTIMATES(I)=INITIAL_ESTIMATES(I)+
	1     INITIAL_ESTIMATES_TEMP(I)
	      INITIAL_ESTIMATES_TEMP(I)=INITIAL_ESTIMATES_TEST !  Resets
	    END IF
	  END DO
	  CHOICE(3:3)=PARALLAX_PROPER_MOTION_S !  Possibilities are
		! I (included) and R (removed).
C  If case I, then absolute coordinates are implied
C  If case R, we make an adjustment in CHOICE(3:3).
	  IF (CHOICE(3:3).EQ.'R'.AND.TYPE_S.EQ.'A') CHOICE(3:3)='A'
C  CHOICE=A means parallax and proper motion removed and absolute coordinates.
C  R means parallax and proper motion removed, but relative coordinates.
C  For I or A, we assume all observations have the same equator;
C  For R, we assume all observations have the equator of their epoch of
C  observation, hence their position angles will have to be precessed,
C  unless TYPE='r', in which case the equator is fixed.
	  IF (CHOICE(1:1).EQ.'S') TITLE=NAME_S
	  IF (CHOICE(1:1).EQ.'B') THEN
	    IF (EQUATOR_S.NE.EQUATOR_P)
	1    CALL ERROR(-2,'Equator of primary and secondary do not match')
	  ELSE IF (CHOICE(1:1).EQ.'S') THEN
	    EQUATOR_VIS=EQUATOR_S
	  END IF
	END IF
C  Obtain some preliminary information on the spectroscopic data.
C  We needn't be concerned with the equator for spectroscopic data.
	IF (CHOICE(4:4).NE.'N') THEN
	  CALL MESSAGE('Some files of spectroscopic data that are'//
	1 ' available are:')
	  CALL MESSAGE(
	1 '  mucas_vel_pri.dat    Radial velocities of primary')
	  CALL MESSAGE(
	1 '  hr6469_vel_pri.dat   Scarfe et al;  velocities of'//
	2 ' primary')
	  CALL MESSAGE(
	1 '  hr6469_vel_sec.dat   Scarfe et al;  velocities of'//
	2 ' secondary')
	  CALL MESSAGE(
	1 '                       and one subcomponent')
	  CALL MESSAGE(
	1 '  47_uma.dat           Fischer et al;  reflex velocities'//
	2 ' of planets')
	  CALL MESSAGE(
	1 '                       about a single star')
	END IF
	IF (CHOICE_VEL(1:1).EQ.'P'.OR.CHOICE_VEL(1:1).EQ.'B') THEN
	  CALL MESSAGE ('Open the primary star''s file for velocities')
	  CALL OPENCF(23,3,'FILE_23=',0)
	  REWIND(23)
	  CALL GET_BINARY_HEADER(23,NAME_VEL_P,RIGHTASC_VEL_P,DECLINA_VEL_P,
	1 EQUATOR_VEL_P,TYPE_VEL_P,PARALLAX_PROPER_MOTION_VEL_P,SCALE_VEL_P,
	2 FILETYPE_VEL_P,INITIAL_ESTIMATES_TEMP,C_ELEMENTS,MAXNUMPARAM)
	  CALL PRINT_BINARY_HEADER(NAME_VEL_P,RIGHTASC_VEL_P,DECLINA_VEL_P,
	1 EQUATOR_VEL_P,TYPE_VEL_P,PARALLAX_PROPER_MOTION_VEL_P,
	2 SCALE_VEL_P,FILETYPE_VEL_P,
	3 INITIAL_ESTIMATES_TEMP,C_ELEMENTS,MAXNUMPARAM)
	  DO I=1,MAXNUMPARAM
	    IF (INITIAL_ESTIMATES_TEMP(I).GT.INITIAL_ESTIMATES_TEST) THEN
	      N_INITIAL_ESTIMATES(I)=N_INITIAL_ESTIMATES(I)+1
	      INITIAL_ESTIMATES(I)=INITIAL_ESTIMATES(I)+
	1     INITIAL_ESTIMATES_TEMP(I)
	      INITIAL_ESTIMATES_TEMP(I)=INITIAL_ESTIMATES_TEST !  Resets
	    END IF
	  END DO
	  TITLE_VEL=NAME_VEL_P
C  Decide how many primary subcomponents to solve for
	  NPRISUB_S_F=0
	  NSECSUB_S_F=0
	  NPRISUB=0
	  CALL USRINP(NPRISUB,1,'I',4,1,'NUMBER_OF_SUBCOMPONENTS=',
	1 'Enter the number of primary subcomponents to solve for:'//
	2 '  (default = 0)')
	  ENCODE(1,1001,C_NPRISUB) NPRISUB
1001	  FORMAT(I1)
	  IF (NPRISUB.NE.0) THEN
	    CHOICE(5:5)=C_NPRISUB
	    DO I=1,5
	      J=SETEND(IPRISUB_S_F(I))
	    END DO
	    IPRISUB_S_F(1)=0
	    CALL MESSAGE('Enter the primary subcomponents for which'//
	1   ' independent data files exist;')
	    CALL USRINP(IPRISUB_S_F,5,'I',4,3,
	1   'PRIMARY_SUBCOMPONENT_SECONDARIES=',
	2   'enter 0 for none:  (default = 0)')
	    IF (IPRISUB_S_F(1).EQ.0) THEN
	      NPRISUB_S_F=0
	    ELSE
	      NPRISUB_S_F=INEL(IPRISUB_S_F,5)
	      CALL MESSAGE('Open the primary subcomponent secondary file(s)')
	    END IF
	    DO I=1,NPRISUB_S_F
C  Units 25 to 29 are reserved for the primary subcomponent secondaries
	      ITMP=IPRISUB_S_F(I)
	      ENCODE(1,1001,C_I) ITMP
	      FSTRING='PRIMARY_SUBCOMPONENT_SECONDARY_'//C_I//'='
	      CALL OPENCF(24+ITMP,1,FSTRING(1:33),0)
	      REWIND(24+ITMP)
	      CALL GET_BINARY_HEADER(24+ITMP,NAME_VEL_P,RIGHTASC_VEL_P,
	1     DECLINA_VEL_P,EQUATOR_VEL_P,TYPE_VEL_P,
	2     PARALLAX_PROPER_MOTION_VEL_P,SCALE_VEL_P_SUB_S(ITMP),
	3     FILETYPE_VEL_P_SUB_S(ITMP),INITIAL_ESTIMATES_TEMP,C_ELEMENTS,
	4     MAXNUMPARAM)
	      CALL PRINT_BINARY_HEADER(NAME_VEL_P,RIGHTASC_VEL_P,
	1     DECLINA_VEL_P,EQUATOR_VEL_P,TYPE_VEL_P,
	2     PARALLAX_PROPER_MOTION_VEL_P,SCALE_VEL_P_SUB_S(ITMP),
	3     FILETYPE_VEL_P_SUB_S(ITMP),INITIAL_ESTIMATES_TEMP,
	4     C_ELEMENTS,MAXNUMPARAM)
	      DO II=1,MAXNUMPARAM
	        IF (INITIAL_ESTIMATES_TEMP(II).GT.
	1       INITIAL_ESTIMATES_TEST) THEN
	          N_INITIAL_ESTIMATES(II)=N_INITIAL_ESTIMATES(II)+1
	          INITIAL_ESTIMATES(II)=INITIAL_ESTIMATES(II)+
	1         INITIAL_ESTIMATES_TEMP(II)
	          INITIAL_ESTIMATES_TEMP(II)=INITIAL_ESTIMATES_TEST !  Resets
	        END IF
	      END DO
	      TITLE_VEL_PRISUB_S(ITMP)=NAME_VEL_P
	      CHOICE(8+ITMP:8+ITMP)='T'
	    END DO
	  END IF
	END IF
	IF (CHOICE_VEL(1:1).EQ.'S'.OR.CHOICE_VEL(1:1).EQ.'B') THEN
	  CALL MESSAGE('Open the secondary star''s file for velocities')
	  CALL OPENCF(24,3,'FILE_24=',0)
	  CALL GET_BINARY_HEADER(24,NAME_VEL_S,RIGHTASC_VEL_S,DECLINA_VEL_S,
	1 EQUATOR_VEL_S,TYPE_VEL_S,PARALLAX_PROPER_MOTION_VEL_S,SCALE_VEL_S,
	2 FILETYPE_VEL_S,INITIAL_ESTIMATES_TEMP,C_ELEMENTS,MAXNUMPARAM)
	  CALL PRINT_BINARY_HEADER(NAME_VEL_S,RIGHTASC_VEL_S,DECLINA_VEL_S,
	1 EQUATOR_VEL_S,TYPE_VEL_S,PARALLAX_PROPER_MOTION_VEL_S,
	2 SCALE_VEL_S,FILETYPE_VEL_S,
	2 INITIAL_ESTIMATES_TEMP,C_ELEMENTS,MAXNUMPARAM)
	  DO I=1,MAXNUMPARAM
C  For case of both primary and secondary files, only use the angle 
C  from node to periastron and the semimajor axis of the primary.
	    IF ((INITIAL_ESTIMATES_TEMP(I).GT.INITIAL_ESTIMATES_TEST).AND.
	1   .NOT.(I.EQ.17.AND.CHOICE_VEL(1:1).EQ.'B')) THEN
	      N_INITIAL_ESTIMATES(I)=N_INITIAL_ESTIMATES(I)+1
	      INITIAL_ESTIMATES(I)=INITIAL_ESTIMATES(I)+
	1     INITIAL_ESTIMATES_TEMP(I)
	      INITIAL_ESTIMATES_TEMP(I)=INITIAL_ESTIMATES_TEST !  Resets
	    END IF
	  END DO
	  IF (CHOICE_VEL(1:1).EQ.'S') TITLE_VEL=NAME_VEL_S
C  Decide how many secondary subcomponents to solve for
	  NSECSUB=0
	  CALL USRINP(NSECSUB,1,'I',4,1,'NUMBER_OF_SUBCOMPONENTS=',
	1 'Enter the number of secondary subcomponents to solve for:'//
	2 '  (default = 0)')
	  ENCODE(1,1001,C_NSECSUB) NSECSUB
	  IF (NSECSUB.NE.0) THEN
	    CHOICE(7:7)=C_NSECSUB
	    DO I=1,5
	      J=ISETEND(ISECSUB_S_F(I))
	    END DO
	    ISECSUB_S_F(1)=0
	    CALL MESSAGE('Enter the secondary subcomponents for which'//
	1   ' independent files exist;')
	    CALL USRINP(ISECSUB_S_F,5,'I',4,3,
	1   'SECONDARY_SUBCOMPONENT_SECONDARIES=',
	2   'enter 0 for none:  (default = 0)')
	    IF (ISECSUB_S_F(1).EQ.0) THEN
	      NSECSUB_S_F=0
	    ELSE
	      NSECSUB_S_F=INEL(ISECSUB_S_F,5)
	      CALL MESSAGE('Open the secondary subcomponent secondary file(s)')
	    END IF
	    DO I=1,NSECSUB_S_F
C  Units 30 to 34 are reserved for the secondary subcomponent secondaries
	      ITMP=ISECSUB_S_F(I)
	      ENCODE(1,1001,C_I) ITMP
	      FSTRING='SECONDARY_SUBCOMPONENT_SECONDARY_'//C_I//'='
	      CALL OPENCF(29+ISECSUB_S_F(I),1,FSTRING(1:35),0)
	      REWIND(29+ITMP)
	      CALL GET_BINARY_HEADER(29+ITMP,NAME_VEL_S,RIGHTASC_VEL_S,
	1     DECLINA_VEL_S,EQUATOR_VEL_S,TYPE_VEL_S,
	2     PARALLAX_PROPER_MOTION_VEL_S,SCALE_VEL_S_SUB_S(ITMP),
	3     FILETYPE_VEL_S_SUB_S(ITMP),INITIAL_ESTIMATES_TEMP,C_ELEMENTS,
	4     MAXNUMPARAM)
	      CALL PRINT_BINARY_HEADER(NAME_VEL_S,RIGHTASC_VEL_S,
	1     DECLINA_VEL_S,EQUATOR_VEL_S,TYPE_VEL_S,
	2     PARALLAX_PROPER_MOTION_VEL_S,SCALE_VEL_S_SUB_S(ITMP),
	3     FILETYPE_VEL_S_SUB_S(ITMP),INITIAL_ESTIMATES_TEMP,
	4     C_ELEMENTS,MAXNUMPARAM)
	      DO II=1,MAXNUMPARAM
	        IF (INITIAL_ESTIMATES_TEMP(II).GT.
	1       INITIAL_ESTIMATES_TEST) THEN
	          N_INITIAL_ESTIMATES(II)=N_INITIAL_ESTIMATES(II)+1
	          INITIAL_ESTIMATES(II)=INITIAL_ESTIMATES(II)+
	1         INITIAL_ESTIMATES_TEMP(II)
	          INITIAL_ESTIMATES_TEMP(II)=INITIAL_ESTIMATES_TEST !  Resets
	        END IF
	      END DO
	      TITLE_VEL_SECSUB_S(ITMP)=NAME_VEL_S
	      CHOICE(13+ITMP:13+ITMP)='T'
	    END DO
	  END IF
	END IF
C
C Apsidal motion for main components
C
	IF (N_INITIAL_ESTIMATES(78).NE.0) THEN
	  CHOICE(19:19)='A'
	ELSE
	  SOLVE_FOR_APSIDAL=.FALSE.
	  CALL USRINP(SOLVE_FOR_APSIDAL,1,'L',4,1,
	1 'SOLVE_FOR_APSIDAL_MOTION=',
	2 'Do you wish to solve for apsidal motion for the main '//
	3 'components?  (default = N)')
	  IF (SOLVE_FOR_APSIDAL) THEN
	    CHOICE(19:19)='A'
	    N_INITIAL_ESTIMATES(78)=1
	    INITIAL_ESTIMATES(78)=0.0D0
	    CALL MESSAGE('Enter an initial estimate for the apsidal '//
	1   'motion in degrees per year:  ')
	    CALL USRINP(INITIAL_ESTIMATES(78),1,'F',8,1,'APSIDAL_MOTION=',
	1   '(default = 0.0)')
	  END IF
	END IF
C
C  Apsidal motion for primary subcomponents
C
	DO I=1,NPRISUB
	  IF (N_INITIAL_ESTIMATES(78+I).NE.0) THEN
	    CHOICE(19+I:19+I)='A'
	  ELSE
	    SOLVE_FOR_APSIDAL=.FALSE.
	    WRITE(6,1012) I
1012	    FORMAT(' ','Do you wish to solve for apsidal motion',
	1   ' for primary subcomponent #',I1,'?')
	    CALL USRINP(SOLVE_FOR_APSIDAL,1,'L',4,1,
	1  'SOLVE_FOR_APSIDAL_MOTION_SUB=',
	2  '(default = N)')
	    IF (SOLVE_FOR_APSIDAL) THEN
	      CHOICE(19+I:19+I)='A'
	      N_INITIAL_ESTIMATES(78+I)=1
	      INITIAL_ESTIMATES(78+I)=0.0D0
	      CALL MESSAGE('Enter an initial estimate for the apsidal '//
	1     'motion in degrees per year:  ')
	      CALL USRINP(INITIAL_ESTIMATES(78+I),1,'F',8,1,'APSIDAL_MOTION=',
	1     '(default = 0.0)')
	    END IF
	  END IF
	END DO
C
C  Apsidal motion for secondary subcomponents
C
	DO I=1,NSECSUB
	  IF (N_INITIAL_ESTIMATES(83+I).NE.0) THEN
	    CHOICE(24+I:24+I)='A'
	  ELSE
	    SOLVE_FOR_APSIDAL=.FALSE.
	    WRITE(6,1013) I
1013	    FORMAT(' ','Do you wish to solve for apsidal motion',
	1   ' for secondary subcomponent #',I1,'?')
	    CALL USRINP(SOLVE_FOR_APSIDAL,1,'L',4,1,
	1   'SOLVE_FOR_APSIDAL_MOTION_SUB=',
	2   '(default = N)')
	    IF (SOLVE_FOR_APSIDAL) THEN
	      CHOICE(24+I:24+I)='A'
	      N_INITIAL_ESTIMATES(83+I)=1
	      INITIAL_ESTIMATES(83+I)=0.0D0
	      CALL MESSAGE('Enter an initial estimate for the apsidal '//
	1     'motion in degrees per year:  ')
	      CALL USRINP(INITIAL_ESTIMATES(83+I),1,'F',8,1,'APSIDAL_MOTION=',
	1     '(default = 0.0)')
	    END IF
	  END IF
	END DO
C
C  Calculate the indexes, number of parameters, and logical values
	CALL BINARY_INDEXES(CHOICE,NUMPARAM)
C
C  Set up the parameters which will serve as initial estimates.  These
C  values are optionally contained in the files.  Also save INDX to
C  properly reference PARAMETRS when plotting since NLFUN3 makes use
C  of INDX.
	DO I=1,MAXNUMPARAM
	  INDX(I)=INDX_BIN(I)
	  IF (N_INITIAL_ESTIMATES(I).NE.0) THEN
C  For the case of primary and secondary being solved together, the 
C  semimajor axis of the primary alone is used.  The semimajor axis 
C  estimate of the secondary was excluded earlier, in this case.
	    PARAMETRS(INDX(I))=INITIAL_ESTIMATES(I)/N_INITIAL_ESTIMATES(I)
	  END IF
	END DO
C
C  See if the initial estimate of the period was given as 9999.99999, which
C  indicates that the main pair is to be ignored, and only subcomponents
C  solved for.  This situation is useful for solving for the orbits of
C  planets around stars.  In the calculation, the period is set to 1.0D37,
C  the other elements of the main pair are set to various values,
C  and all the main pair elements are held constant in the solution.
	IF (PARAMETRS(INDX(1)).EQ.9999.99999D0) THEN
	  IGNORE_MAIN_PAIR_BIN=.TRUE.
	  PARAMETRS(INDX(1))=1.0D37 !  P
	  PARAMETRS(INDX(2))=2000.0D0 !  T
	  PARAMETRS(INDX(3))=0.0D0 !  e
	  PARAMETRS(INDX(6))=0.0D0 !  w
	  PARAMETRS(INDX(17))=1.0D0 !  sin(i)*a(AU)
	ELSE
	  IGNORE_MAIN_PAIR_BIN=.FALSE.
	END IF
C
C  We don't have to consider parameters 9 to 15 if parallax and 
C  proper motion have been removed, since in that case there is no need 
C  to solve for these parameters.
C
C  Ask about excluding data points
	IF (CHOICE_VEL(1:1).EQ.'P'.OR.CHOICE_VEL(1:1).EQ.'B') THEN
	  CALL MESSAGE('Do you wish to exclude data points from the '//
	1 'primary spectroscopic')
	  CALL MESSAGE('file based on phase?  This is not '//
	1 'recommended if there are')
	  CALL MESSAGE('subcomponents.  The file''s initial estimates '//
	1 'will be used')
	  EXCLUDE_BY_PHASE_PRI=.FALSE.
	  CALL USRINP(EXCLUDE_BY_PHASE_PRI,1,'L',4,1,
	1 'EXCLUDE_BY_PHASE_PRI=','to calculate phase.  (default = N)')
	  IF (EXCLUDE_BY_PHASE_PRI) THEN
	    CALL MESSAGE('Enter pairs of lower and upper phase limits '//
	1   'which will serve to exclude')
	    CALL MESSAGE('any data points which are so enclosed.  All '//
	1   'limits must be greater')
	    CALL MESSAGE('than zero, but the upper limit of a pair '//
	1   'may for convenience be')
	    CALL USRINP(EXCLUDED_PHASES_PRI,MAXNUMPHASES,'F',8,0,
	1   'EXCLUDED_PHASES_PRI=',
	2   'greater than 1.  A maximum of three pairs can be specified.')
	    NPAIRS_PRI=NELD(EXCLUDED_PHASES_PRI,MAXNUMPHASES)/2
	  END IF
	END IF
	IF (CHOICE_VEL(1:1).EQ.'B'.OR.CHOICE_VEL(1:1).EQ.'S') THEN
	  CALL MESSAGE('Do you wish to exclude data points from the '//
	1 'secondary spectroscopic')
	  CALL MESSAGE('file based on phase?  This is not '//
	1 'recommended if there are')
	  CALL MESSAGE('subcomponents.  The file''s initial estimates '//
	1 'will be used')
	  EXCLUDE_BY_PHASE_SEC=.FALSE.
	  CALL USRINP(EXCLUDE_BY_PHASE_SEC,1,'L',4,1,
	1 'EXCLUDE_BY_PHASE_SEC=','to calculate phase.  (default = N)')
	  IF (EXCLUDE_BY_PHASE_SEC) THEN
	    CALL MESSAGE('Enter pairs of lower and upper phase limits '//
	1   'which will serve to exclude')
	    CALL MESSAGE('any data points which are so enclosed.  All '//
	1   'limits must be greater')
	    CALL MESSAGE('than zero, but the upper limit of a pair '//
	1   'may for convenience be')
	    CALL USRINP(EXCLUDED_PHASES_SEC,MAXNUMPHASES,'F',8,0,
	1   'EXCLUDED_PHASES_SEC=',
	2   'greater than 1.  A maximum of three pairs can be specified.')
	    NPAIRS_SEC=NELD(EXCLUDED_PHASES_SEC,MAXNUMPHASES)/2
	  END IF
	END IF
C  Write some diagnostics to the log file.  Other information is written
C  by binary_indexes.for.
	WRITE (7,1017)
1017	FORMAT(' ','CHOICE')
	DO I=1,32
	  WRITE (7,1018) I,CHOICE(I:I)
1018	  FORMAT(' ',I4,2X,A3)
	END DO
	WRITE (7,1027) DIMENSN_DATA_BIN,DIMENSN_DATA_USED_BIN
1027	FORMAT(' ','DIMENSION_DATA = ',I3,2X,'DIMENSION_DATA_USED = ',I3)
	WRITE (7,1029) SEMIMAJOR_AXIS_ASSIGNMENT_BIN(1),
	1SEMIMAJOR_AXIS_ASSIGNMENT_BIN(2)
1029	FORMAT(' ','SEMIMAJOR_AXIS_ASSIGNMENT(1) = ',I3,2X,
	1'SEMIMAJOR_AXIS_ASSIGNMENT(2) = ',I3)
C
C  Save some logicals for plotting and other purposes.
	VISUAL=VISUAL_BIN
	SPECTROSCOPIC=SPECTROSCOPIC_BIN
	SPECTROSCOPIC_PRI_SUB=SPECTROSCOPIC_PRI_SUB_BIN
	SPECTROSCOPIC_SEC_SUB=SPECTROSCOPIC_SEC_SUB_BIN
	INTERFEROMETRIC=INTERFEROMETRIC_BIN
	PAR_PM=PAR_PM_BIN
	SOLVE_PAR=SOLVE_PAR_BIN
	INCLINATION_AVAILABLE=INCLINATION_AVAILABLE_BIN
	RATIO_AVAILABLE=RATIO_AVAILABLE_BIN
	SEMIMAJOR_AXIS_ASSIGNMENT(1)=SEMIMAJOR_AXIS_ASSIGNMENT_BIN(1)
	SEMIMAJOR_AXIS_ASSIGNMENT(2)=SEMIMAJOR_AXIS_ASSIGNMENT_BIN(2)
	DIMENSN_DATA=DIMENSN_DATA_BIN
	DIMENSN_DATA_USED=DIMENSN_DATA_USED_BIN
	DO I=1,MAXNUMPARAM
	  INDX(I)=INDX_BIN(I)
	END DO
C  Handle some special cases
	IF (PAR_PM) THEN
	  OBS_TRUE_PAR=1.0D0
	  CALL MESSAGE('Enter the ratio of the observed to true '//
	1 'parallax for the astrometric')
	  CALL USRINP(OBS_TRUE_PAR,1,'F',8,1,'OBSERVED_TO_TRUE_PARALLAX=',
	1 'data:  (default = 1.0)')
	END IF
C  If the primary data refers to a photocentric orbit, we will need to
C  adjust the semimajor axis for that data in NLFUN3.
	PHOTOCENTRIC_TRUE_SEMIMAJAX=1.0D0
	IF (TYPE_P.EQ.'A'.OR.TYPE_S.EQ.'A') THEN
	  CALL USRINP(PHOTOCENTRIC_TRUE_SEMIMAJAX,1,'F',8,1,
	1 'RATIO_PHOTOCENTRIC_TO_TRUE_SEMIMAJOR_AXIS=','Enter the ratio '//
	2 'of the photocentric to true semimajor axis:  (default = 1.0)')
	END IF
C
	DO I=1,MAXHEAD
	  DESCRIPTION(INDX(I))=HEAD(I)
	END DO
C  Now obtain the data for the fit
	SUM_P=0.0D0
	SUM_S=0.0D0
	SUM_VEL_P=0.0D0
	SUM_VEL_S=0.0D0
	DO I=1,NSUBS_MAX
	  SUM_VEL_P_SUB_S(I)=0.0D0
	  SUM_VEL_S_SUB_S(I)=0.0D0
	END DO
	SUM_INTER_P=0.0D0
	SUM_INTER_S=0.0D0
	NPOINTS_DATA=0
	NPOINTS_DATA_P=0
	NPOINTS_DATA_S=0
	NPOINTS_VIS=0
	NPOINTS_DATA_VEL_P=0
	NPOINTS_DATA_VEL_S=0
	NPOINTS_SP=0
	DO I=1,NSUBS_MAX
	  NPOINTS_DATA_VEL_P_SUB_S(I)=0
	  NPOINTS_DATA_VEL_S_SUB_S(I)=0
	END DO
	NPOINTS_VEL_P_S=0 !  Primary spectroscopic subcomponent files
	NPOINTS_VEL_S_S=0
	NPOINTS_DATA_INTER_P=0
	NPOINTS_DATA_INTER_S=0
	NPOINTS_INTER=0
C
C  Read the data from the files
C
C  The primary 
	IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B') THEN !  Primary, visual
	  N=NPOINTS_DATA+1
	  DO WHILE (.TRUE.)
	    READ (21,1100,END=991) TIME(N,1),DATA(N,1),SIGMA(N,1),
	1   DATA(N,2),SIGMA(N,2)
1100	    FORMAT(' ',F12.6,4X,F14.5,2X,F14.5,2X,F14.5,2X,F14.5)
	    IF (SIGMA(N,1).NE.9999.99999D0.AND.
	1   SIGMA(N,2).NE.9999.99999D0) THEN
C  To convert from Besselian years to Julian years of emphemeris time,
C  uncomment the following two lines.
C	      CALL BESSYR_JULCENT(TIME(N,1),TIME(N,1))
C	      TIME(N,1)=TIME(N,1)*100+1900.0D0
	      TIME_SAVE(N)=TIME(N,1)
C  Take account of the scale in the primary file.
	      IF (FILETYPE_P.EQ.'X_Y') THEN
	        DATA(N,1)=SCALE_P*DATA(N,1) !  Now in arc seconds
	        DATA(N,2)=SCALE_P*DATA(N,2)
	        SIGMA(N,1)=SCALE_P*SIGMA(N,1) !  Sigma in X
	        SIGMA(N,2)=SCALE_P*SIGMA(N,2) !  Sigma in Y
	      ELSE IF (FILETYPE_P.EQ.'THETA_RHO') THEN
	        TEMP1=DATA(N,1) !  Position angle
	        TEMP2=DATA(N,2) !  Separation
	        DATA(N,1)=SCALE_P*TEMP2*DSIND(TEMP1) !  X in "
	        DATA(N,2)=SCALE_P*TEMP2*DCOSD(TEMP1) !  Y in "
C  Leave the error in angle alone for now, and just scale the error in
C  separation.  We will convert to error in X and Y below.
	        SIGMA(N,2)=SCALE_P*SIGMA(N,2)
C  Save the input sigmas in theta and rho for when we scale weights and
C  compute the mean errors at the end.
	        SIGMA_SAVE(N,1)=SIGMA(N,1) !  Sigma in theta
	        SIGMA_SAVE(N,2)=SIGMA(N,2) !  Sigma in rho
	      END IF
	      DATA_SAVE(N,1)=DATA(N,1)
	      DATA_SAVE(N,2)=DATA(N,2)
	      IF (CHOICE(1:1).EQ.'B') THEN
C  For these N, there is no secondary data
	        WEIGHT(N,3)=0.0D0
	        WEIGHT(N,4)=0.0D0
	        WEIGHT_SAVE(N,3)=WEIGHT(N,3)
	        WEIGHT_SAVE(N,4)=WEIGHT(N,4)
	      END IF
	      N=N+1
	    END IF
	  END DO
 991	  N=N-1
	  CLOSE(UNIT=21)
	  NPOINTS_DATA_P=N
	  NPOINTSPXPY_P=N
	  NPOINTS_DATA=N
	  NPOINTS_VIS=N
C  Check for missing primary sigmas
	  MISSING_SIGMA=.FALSE.
	  DO I=1,NPOINTS_DATA_P
	    IF (SIGMA(I,1).EQ.0.0D0) MISSING_SIGMA=.TRUE.
	  END DO
	  IF (MISSING_SIGMA) THEN
	    IF (FILETYPE_P.EQ.'X_Y') THEN
	      CALL MESSAGE('Some of the primary X positions did not'//
	1 ' have sigmas.')
	      CALL USRINP(SIGMA_TEMP(1),1,'F',8,0,'SIGMA_X=',
	1     'Enter a sigma for X in ":')
	    ELSE IF (FILETYPE_P.EQ.'THETA_RHO') THEN
	      CALL MESSAGE('Some of the primary position angles did not'//
	1 ' have sigmas.')
	      CALL USRINP(SIGMA_TEMP(1),1,'F',8,0,'SIGMA_THETA=',
	1     'Enter a sigma for THETA in degrees:')
	    END IF
	  END IF
	  MISSING_SIGMA=.FALSE.
	  DO I=1,NPOINTS_DATA_P
	    IF (SIGMA(I,2).EQ.0.0D0) MISSING_SIGMA=.TRUE.
	  END DO
	  IF (MISSING_SIGMA) THEN
	    IF (FILETYPE_P.EQ.'X_Y') THEN
	      CALL MESSAGE('Some of the primary Y positions did not'//
	1 ' have sigmas.')
	      CALL USRINP(SIGMA_TEMP(2),1,'F',8,0,'SIGMA_Y=',
	1     'Enter a sigma for Y in ":')
	    ELSE IF (FILETYPE_P.EQ.'THETA_RHO') THEN
	      CALL MESSAGE('Some of the primary separations did not'//
	1 ' have sigmas.')
	      CALL USRINP(SIGMA_TEMP(2),1,'F',8,0,'SIGMA_SEPARATION=',
	1     'Enter a sigma for SEPARATION in ":')
	    END IF
	  END IF
C  Compute the primary weights
	  IF (FILETYPE_P.EQ.'X_Y') THEN
	    DO I=1,NPOINTS_DATA_P
	      DO K=1,2
	        IF (SIGMA(I,K).EQ.0.0D0) THEN
	          SIGMA(I,K)=MAX(1.0D-6,SIGMA_TEMP(K))
C  Save the user sigmas if they were zero in the file
C	          SIGMA_SAVE(I,K)=SIGMA(I,K) !  Not needed for X_Y
	        END IF
	        WEIGHT(I,K)=1.0D0/SIGMA(I,K)**2
	        WEIGHT_SAVE(I,K)=WEIGHT(I,K)
	        SUM_P=SUM_P+WEIGHT(I,K)
	      END DO
	    END DO
	  ELSE IF (FILETYPE_P.EQ.'THETA_RHO') THEN
C  Convert from the errors in angle and separation to errors in X and Y
C  for the primary
	    DO I=1,NPOINTS_DATA_P
	      TEMP1=DATA(I,1) !  X, was converted above
	      TEMP2=DATA(I,2) !  Y
	      DO K=1,2
	        IF (SIGMA(I,K).EQ.0.0D0) THEN
	          SIGMA(I,K)=MAX(1.0D-6,SIGMA_TEMP(K))
	          SIGMA_SAVE(I,K)=SIGMA(I,K)
	        END IF
	      END DO
	      TEMP3=SIGMA(I,1) !  Error in angle from the file
	      TEMP4=SIGMA(I,2) !  Error in separation scaled from the file
	      TEMP5=SQRT(TEMP1**2+TEMP2**2) !  We don't have rho any more
C  We start with x=rho*sin(theta) and y=rho*cos(theta)
	      SIGMA(I,1)=SQRT((TEMP2*TEMP3/DEG_RAD)**2+
	1     (TEMP1/TEMP5*TEMP4)**2)
	      SIGMA(I,2)=SQRT((TEMP1*TEMP3/DEG_RAD)**2+
	1     (TEMP2/TEMP5*TEMP4)**2)
C  Sigmas in theta and rho are in SIGMA_SAVE
	      DO K=1,2
	        WEIGHT(I,K)=1.0D0/SIGMA(I,K)**2
	        WEIGHT_SAVE(I,K)=WEIGHT(I,K)
	        SUM_P=SUM_P+WEIGHT(I,K)
	      END DO
	    END DO
	  END IF
C  Calculate the parallax factors if necessary
	  IF (PARALLAX_PROPER_MOTION_P.EQ.'I') THEN
	    DO I=1,NPOINTSPXPY_P
	      CALL PARALLAXFACTORS(TIME(I,1),RIGHTASC_P,DECLINA_P,
	1     EQUATOR_P,0,PX(I),PY(I))
	      PX(I)=PX(I)*OBS_TRUE_PAR
	      PY(I)=PY(I)*OBS_TRUE_PAR
	    END DO
	  END IF
	END IF
C  The secondary
	IF (CHOICE(1:1).EQ.'B'.OR.CHOICE(1:1).EQ.'S') THEN  
				!  Secondary, visual
	  IF (DIMENSN_DATA.LT.4) CALL ERROR(-3,'Impossible data dimension')
	  N=NPOINTS_DATA+1
	  DO WHILE (.TRUE.)
	    READ (22,1100,END=992) TIME(N,1),DATA(N,3),SIGMA(N,3),
	1   DATA(N,4),SIGMA(N,4)
	    IF (SIGMA(N,3).NE.9999.99999D0.AND.
	1   SIGMA(N,4).NE.9999.99999D0) THEN
C  To convert from Besselian years to Julian years of emphemeris time,
C  uncomment the following two lines.
C	      CALL BESSYR_JULCENT(TIME(N,1),TIME(N,1))
C	      TIME(N,1)=TIME(N,1)*100+1900.0
	      TIME_SAVE(N)=TIME(N,1)
C  Take account of the scale in the file.  We ultimately want X and Y
	      IF (FILETYPE_S.EQ.'X_Y') THEN
	        DATA(N,3)=SCALE_S*DATA(N,3) !  Now in arc seconds
	        DATA(N,4)=SCALE_S*DATA(N,4)
	        SIGMA(N,3)=SCALE_S*SIGMA(N,3)
	        SIGMA(N,4)=SCALE_S*SIGMA(N,4)
	      ELSE IF (FILETYPE_S.EQ.'THETA_RHO') THEN
	        TEMP1=DATA(N,3) !  Position angle
	        TEMP2=DATA(N,4) !  Separation
	        DATA(N,3)=SCALE_S*TEMP2*DSIND(TEMP1) !  X in "
	        DATA(N,4)=SCALE_S*TEMP2*DCOSD(TEMP1) !  Y in "
C  Leave the error in angle alone for now, and just scale the error in
C  separation.  We will convert to error in X and Y below.
	        SIGMA(N,4)=SCALE_S*SIGMA(N,4) !  Still separation
C  Save the input sigmas in theta and rho for when we scale weights and
C  compute the mean errors at the end.
	        SIGMA_SAVE(N,3)=SIGMA(N,3) !  Sigma in theta
	        SIGMA_SAVE(N,4)=SIGMA(N,4) !  Sigma in rho
	      END IF
C  Precess the positions if they have their equator equal to their
C  epoch of observation.
	      IF (CHOICE(3:3).EQ.'R'.AND.TYPE_S.EQ.'R') THEN
C  Precess the given RA and DEC of the star to the epoch of the
C  observation.  In this case, RIGHTASC_S, given in the file, is to be 
C  considered as the right ascension of the primary.
	        CALL PRECESSIO(0,EQUATOR_VIS,TIME(N,1),RIGHTASC_S,
	1       DECLINA_S,RIGHTASC_EPOCH,DECLINA_EPOCH)
C  Get the separation and position angle at the epoch of observation
	        TEMP2=DSQRT(DATA(N,3)**2+DATA(N,4)**2) !  Separation in "
	        TEMP1=DATAN2D(DATA(N,3),DATA(N,4)) !  Position angle in deg
	        IF (TEMP1.LT.0.0D0) TEMP1=TEMP1+360.0D0
C  Get the RA and DEC of the secondary at the epoch of observation
	        TEMP4=DECLINA_EPOCH+TEMP2*DCOSD(1.0D0*TEMP1)/3600.0D0 !  deg
	        TEMP3=RIGHTASC_EPOCH+TEMP2*DSIND(1.0D0*TEMP1)/15.0D0/
	1       3600.0D0/DCOSD((DECLINA_EPOCH+TEMP4)/2.0D0) !  In hours
C  Precess the secondary's position to the equator in the file
	        CALL PRECESSIO(0,TIME(N,1),EQUATOR_VIS,TEMP3,TEMP4,
	1       TEMP3,TEMP4)
C  Get the delta x and y of the secondary relative to the primary
C  at the equator in the file
	        DATA(N,3)=(TEMP3-RIGHTASC_S)*DCOSD((DECLINA_S+
	1       TEMP4)/2.0D0)*15.0D0*3600.0D0 !  "
	        DATA(N,4)=(TEMP4-DECLINA_S)*3600.0D0 !  "
C	        TEMP2=DSQRT(DATA(N,3)**2+DATA(N,4)**2) !  Separation
C	        TEMP1=DATAN2D(DATA(N,3),DATA(N,4)) !  Position angle
	      END IF
	      DATA_SAVE(N,3)=DATA(N,3)
	      DATA_SAVE(N,4)=DATA(N,4)
	      WEIGHT(N,1)=0.0D0
	      WEIGHT(N,2)=0.0D0
	      WEIGHT_SAVE(N,1)=WEIGHT(N,1)
	      WEIGHT_SAVE(N,2)=WEIGHT(N,2)
	      N=N+1
	    END IF
	  END DO
 992	  N=N-1
	  CLOSE(UNIT=22)
	  NPOINTS_DATA=N
	  NPOINTS_DATA_S=NPOINTS_DATA-NPOINTS_DATA_P
	  NPOINTS_VIS=NPOINTS_DATA
	  NPOINTSPXPY_S=NPOINTS_DATA_S
C  Check for missing sigmas for secondary
	  MISSING_SIGMA=.FALSE.
	  DO I=NPOINTS_DATA_P+1,NPOINTS_DATA
	    IF (SIGMA(I,3).EQ.0.0D0) MISSING_SIGMA=.TRUE.
	  END DO
	  IF (MISSING_SIGMA) THEN
	    IF (FILETYPE_S.EQ.'X_Y') THEN
	      CALL MESSAGE('Some of the secondary X positions did not'//
	1 ' have sigmas.')
	      CALL USRINP(SIGMA_TEMP(1),1,'F',8,0,'SIGMA_X=',
	1     'Enter a sigma for X:')
	    ELSE IF (FILETYPE_S.EQ.'THETA_RHO') THEN
	      CALL MESSAGE('Some of the secondary position angles did'//
	1 ' not have sigmas.')
	      CALL USRINP(SIGMA_TEMP(1),1,'F',8,0,'SIGMA_THETA=',
	1     'Enter a sigma for THETA:')
	    END IF
	  END IF
	  MISSING_SIGMA=.FALSE.
	  DO I=NPOINTS_DATA_P+1,NPOINTS_DATA
	    IF (SIGMA(I,4).EQ.0.0) MISSING_SIGMA=.TRUE.
	  END DO
	  IF (MISSING_SIGMA) THEN
	    IF (FILETYPE_S.EQ.'X_Y') THEN
	      CALL MESSAGE('Some of the secondary Y positions did not'//
	1 ' have sigmas.')
	      CALL USRINP(SIGMA_TEMP(2),1,'F',8,0,'SIGMA_Y=',
	1     'Enter a sigma for Y:')
	    ELSE IF (FILETYPE_S.EQ.'THETA_RHO') THEN
	      CALL MESSAGE('Some of the secondary separations did not'//
	1 ' have sigmas.')
	      CALL USRINP(SIGMA_TEMP(2),1,'F',8,0,'SIGMA_SEPARATION=',
	1     'Enter a sigma for SEPARATION:')
	    END IF
	  END IF
C  Compute the weights for the secondary
	  IF (FILETYPE_S.EQ.'X_Y') THEN
	    DO I=NPOINTS_DATA_P+1,NPOINTS_DATA
	      DO K=3,4
	        IF (SIGMA(I,K).EQ.0.0) THEN
	          SIGMA(I,K)=MAX(1.0D-6,SIGMA_TEMP(K-2))
C	          SIGMA_SAVE(I,K)=SIGMA(I,K) !  Not needed for X_Y
	        END IF
	        WEIGHT(I,K)=1.0D0/SIGMA(I,K)**2
	        WEIGHT_SAVE(I,K)=WEIGHT(I,K)
	        SUM_S=SUM_S+WEIGHT(I,K)
	      END DO
	    END DO
	  ELSE IF (FILETYPE_S.EQ.'THETA_RHO') THEN
C  Convert from the errors in angle and separation to errors in X and Y
C  for the secondary
	    DO I=NPOINTS_DATA_P+1,NPOINTS_DATA
	      TEMP1=DATA(I,3) !  X
	      TEMP2=DATA(I,4) !  Y
	      DO K=3,4
	        IF (SIGMA(I,K).EQ.0.0) THEN
	          SIGMA(I,K)=MAX(1.0D-6,SIGMA_TEMP(K-2))
	          SIGMA_SAVE(I,K)=SIGMA(I,K)
	        END IF
	      END DO
	      TEMP3=SIGMA(I,3) !  Error in angle
	      TEMP4=SIGMA(I,4) !  Error in separation
	      TEMP5=SQRT(TEMP1**2+TEMP2**2) !  We don't have rho any more
C  We start with x=rho*sin(theta) and y=rho*cos(theta)
	      SIGMA(I,3)=SQRT((TEMP2*TEMP3/DEG_RAD)**2+
	1     (TEMP1/TEMP5*TEMP4)**2)
	      SIGMA(I,4)=SQRT((TEMP1*TEMP3/DEG_RAD)**2+
	1     (TEMP2/TEMP5*TEMP4)**2)
C  Sigmas in theta and rho are in SIGMA_SAVE
	      DO K=3,4
	        WEIGHT(I,K)=1.0D0/SIGMA(I,K)**2
	        WEIGHT_SAVE(I,K)=WEIGHT(I,K)
	        SUM_S=SUM_S+WEIGHT(I,K)
	      END DO
	    END DO
	  END IF
C  Calculate the parallax factors if necessary
	  IF (PARALLAX_PROPER_MOTION_S.EQ.'I') THEN
	    DO I=NPOINTSPXPY_P+1,NPOINTSPXPY_P+1+NPOINTSPXPY_S
	      CALL PARALLAXFACTORS(TIME(I,1),RIGHTASC_S,DECLINA_S,
	1     EQUATOR_S,0,PX(I),PY(I))
	      PX(I)=PX(I)*OBS_TRUE_PAR
	      PY(I)=PY(I)*OBS_TRUE_PAR
	    END DO
	  END IF
	END IF
C
C  Get the data from the spectroscopic primary file
C
	IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B') THEN  
				!  Primary, spectroscopic
	  N=NPOINTS_DATA+1
	  DO WHILE (.TRUE.)
	    READ (23,3100,END=993) TIME(N,1),DATA(N,5),SIGMA(N,5)
3100	    FORMAT(' ',F15.6,4X,F14.5,2X,F14.5)
	    EXCLUDE=.FALSE.
	    IF (EXCLUDE_BY_PHASE_PRI) THEN
	      CALL BINARY_EXCLUDE_BY_PHASE(PARAMETRS,MAXNUMPARAM,TIME(N,1),
	1     FILETYPE_VEL_P,EXCLUDED_PHASES_PRI,MAXNUMPHASES,NPAIRS_PRI,
	2     EXCLUDE)
	    END IF
	    IF ((SIGMA(N,5).NE.9999.99999D0).AND.(.NOT.EXCLUDE)) THEN
	      IF (FILETYPE_VEL_P.EQ.'BY_VELOCITY') THEN
C  TIME_SAVE_JD is used for plotting
	        CALL BY_JD(TIME(N,1),TIME_SAVE_JD(N))
C  To convert from Besselian years to Julian years of ephemeris time,
C  uncomment the following two lines.
C	        CALL BESSYR_JULCENT(TIME(N,1),TIME(N,1))
C	        TIME(N,1)=TIME(N,1)*100+1900.0
	      ELSE IF (FILETYPE_VEL_P.EQ.'JD_VELOCITY') THEN
C  Convert from Julian Days to Besselian years
	        TIME_SAVE_JD(N)=TIME(N,1)
	        CALL JD_BY(TIME(N,1),TIME(N,1))
C  To convert from Besselian years to Julian years of ephemeris time,
C  uncomment the following line.
C	        TIME(N,1)=(TIME(N,1)-2415020.0)/365.25+1900.0
	      END IF
	      TIME_SAVE(N)=TIME(N,1)
C  Take account of the scale in the file.
	      IF (FILETYPE_VEL_P(4:11).NE.'VELOCITY') 
	1     CALL ERROR(-3,'Non spectroscopic file opened as spectroscopic')
	      DATA(N,5)=SCALE_VEL_P*DATA(N,5) !  Now in km/sec
	      SIGMA(N,5)=SCALE_VEL_P*SIGMA(N,5)
	      DATA_SAVE(N,5)=DATA(N,5)
	      IF (SIGMA(N,5).GT.0.0) THEN
	        WEIGHT(N,5)=1.0D0/SIGMA(N,5)**2
	      ELSE
	        WEIGHT(N,5)=1.0D10 !  Arbitrarily large
	      END IF
	      WEIGHT_SAVE(N,5)=WEIGHT(N,5)
	      SUM_VEL_P=SUM_VEL_P+WEIGHT(N,5)
	      IF (CHOICE(4:4).EQ.'B') THEN
	        WEIGHT(N,6)=0.0
	        WEIGHT_SAVE(N,6)=WEIGHT(N,6)
	      END IF
	      N=N+1
	    END IF
	  END DO
 993	  N=N-1
	  CLOSE(UNIT=23)
	  NPOINTS_DATA=N
	  NPOINTS_DATA_VEL_P=NPOINTS_DATA-NPOINTS_VIS
	  NPOINTS_SP=NPOINTS_DATA_VEL_P
	END IF
C
C  Get the data from the spectroscopic secondary file
C
	IF (CHOICE(4:4).EQ.'B'.OR.CHOICE(4:4).EQ.'S') THEN  
				!  Secondary, spectroscopic
	  IF (DIMENSN_DATA.LT.6) CALL ERROR(-3,'Impossible data dimension')
	  N=NPOINTS_DATA+1
	  DO WHILE (.TRUE.)
	    READ (24,3100,END=994) TIME(N,1),DATA(N,6),SIGMA(N,6)
	    EXCLUDE=.FALSE.
	    IF (EXCLUDE_BY_PHASE_SEC) THEN
	      CALL BINARY_EXCLUDE_BY_PHASE(PARAMETRS,MAXNUMPARAM,TIME(N,1),
	1     FILETYPE_VEL_S,EXCLUDED_PHASES_SEC,MAXNUMPHASES,NPAIRS_SEC,
	2     EXCLUDE)
	    END IF
	    IF ((SIGMA(N,6).NE.9999.99999D0).AND.(.NOT.EXCLUDE)) THEN
	      IF (FILETYPE_VEL_S.EQ.'BY_VELOCITY') THEN
C  TIME_SAVE_JD is used for plotting
	        CALL BY_JD(TIME(N,1),TIME_SAVE_JD(N))
C  To convert from Besselian years to Julian years of ephemeris time,
C  uncomment the following two lines.
C	        CALL BESSYR_JULCENT(TIME(N,1),TIME(N,1))
C	        TIME(N,1)=TIME(N,1)*100+1900.0
	      ELSE IF (FILETYPE_VEL_S.EQ.'JD_VELOCITY') THEN
C  Convert from Julian Days to Besselian years
	        TIME_SAVE_JD(N)=TIME(N,1)
	        CALL JD_BY(TIME(N,1),TIME(N,1))
C  To convert from Besselian years to Julian years of ephemeris time,
C  uncomment the following line.
C	        TIME(N,1)=(TIME(N,1)-2415020.0)/365.25+1900.0
	      END IF
	      TIME_SAVE(N)=TIME(N,1)
C  Take account of the scale in the file.
	      IF (FILETYPE_VEL_S(4:11).NE.'VELOCITY')
	1     CALL ERROR(-3,'Non spectroscopic file opened as spectroscopic')
	      DATA(N,6)=SCALE_VEL_S*DATA(N,6) !  Now in km/sec
	      SIGMA(N,6)=SCALE_VEL_S*SIGMA(N,6)
	      DATA_SAVE(N,6)=DATA(N,6)
	      WEIGHT(N,5)=0.0
	      WEIGHT_SAVE(N,5)=WEIGHT(N,5)
	      IF (SIGMA(N,6).GT.0.0) THEN
	        WEIGHT(N,6)=1.0D0/SIGMA(N,6)**2
	      ELSE 
	        WEIGHT(N,6)=1.0D10
	      END IF
	      WEIGHT_SAVE(N,6)=WEIGHT(N,6)
	      SUM_VEL_S=SUM_VEL_S+WEIGHT(N,6)
	      N=N+1
	    END IF
	  END DO
 994	  N=N-1
	  CLOSE(UNIT=24)
	  NPOINTS_DATA=N
	  NPOINTS_DATA_VEL_S=NPOINTS_DATA-NPOINTS_DATA_VEL_P-NPOINTS_VIS
	  NPOINTS_SP=NPOINTS_DATA_VEL_P+NPOINTS_DATA_VEL_S
	END IF
C
C  Get the data from the spectroscopic primary subcomponent secondary file(s)
C
	DO I=1,NPRISUB_S_F
	  N=NPOINTS_DATA+1
	  DO WHILE (.TRUE.)
	    ITMP=IPRISUB_S_F(I)
	    READ (24+ITMP,3100,END=995) TIME(N,1),
	1   DATA(N,6+ITMP),SIGMA(N,6+ITMP)
	    IF (SIGMA(N,6+ITMP).NE.9999.99999D0) THEN
	      IF (FILETYPE_VEL_P_SUB_S(ITMP).EQ.'BY_VELOCITY') THEN
	        CALL BY_JD(TIME(N,1),TIME_SAVE_JD(N))
	      ELSE IF (FILETYPE_VEL_P_SUB_S(ITMP).EQ.'JD_VELOCITY') THEN
C  Convert from Julian Days to Besselian years
	        TIME_SAVE_JD(N)=TIME(N,1)
	        CALL JD_BY(TIME(N,1),TIME(N,1))
	      END IF
	      TIME_SAVE(N)=TIME(N,1)
C  Take account of the scale in the file.
	      IF (FILETYPE_VEL_P_SUB_S(ITMP)(4:11).NE.'VELOCITY') 
	1     CALL ERROR(-3,'Non spectroscopic file opened as spectroscopic')
	      DATA(N,6+ITMP)=SCALE_VEL_P_SUB_S(ITMP)*DATA(N,6+ITMP)  
					!  Now in km/sec
	      SIGMA(N,6+ITMP)=SCALE_VEL_P_SUB_S(ITMP)*SIGMA(N,6+ITMP)
	      DATA_SAVE(N,6+ITMP)=DATA(N,6+ITMP)
	      IF (SIGMA(N,6+ITMP).GT.0.0) THEN
	        WEIGHT(N,6+ITMP)=1.0D0/SIGMA(N,6+ITMP)**2
	      ELSE
	        WEIGHT(N,6+ITMP)=1.0D10 !  Arbitrarily large
	      END IF
	      SUM_VEL_P_SUB_S(ITMP)=SUM_VEL_P_SUB_S(ITMP)+WEIGHT(N,6+ITMP)
	      IF (CHOICE(4:4).EQ.'B') THEN
	        WEIGHT(N,6+ITMP)=0.0
	      END IF
	      WEIGHT_SAVE(N,6+ITMP)=WEIGHT(N,6+ITMP)
	      N=N+1
	    END IF
	  END DO
 995	  N=N-1
	  CLOSE(UNIT=24+ITMP)
	  NPOINTS_DATA=N
	  NPOINTS_DATA_VEL_P_SUB_S(ITMP)=NPOINTS_DATA-NPOINTS_VIS-
	1 NPOINTS_SP-NPOINTS_VEL_P_S
	  NPOINTS_VEL_P_S=NPOINTS_VEL_P_S+NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	END DO !  Ends loop over primary spectroscopic subcomponent 
	       !  secondary files with index I.
C
C  Get the data from the spectroscopic secondary subcomponent secondary file(s)
C
	DO I=1,NSECSUB_S_F
	  N=NPOINTS_DATA+1
	  DO WHILE (.TRUE.)
	    ITMP=ISECSUB_S_F(I)
	    READ (29+ITMP,3100,END=996) TIME(N,1),
	1   DATA(N,11+ITMP),SIGMA(N,11+ITMP)
	    IF (SIGMA(N,11+ITMP).NE.9999.99999D0) THEN
	      IF (FILETYPE_VEL_S_SUB_S(ITMP).EQ.'BY_VELOCITY') THEN
	        CALL BY_JD(TIME(N,1),TIME_SAVE_JD(N))
	      ELSE IF (FILETYPE_VEL_S_SUB_S(ITMP).EQ.'JD_VELOCITY') THEN
C  Convert from Julian Days to Besselian years
	        TIME_SAVE_JD(N)=TIME(N,1)
	        CALL JD_BY(TIME(N,1),TIME(N,1))
	      END IF
	      TIME_SAVE(N)=TIME(N,1)
C  Take account of the scale in the file.
	      IF (FILETYPE_VEL_S_SUB_S(ITMP)(4:11).NE.'VELOCITY') 
	1     CALL ERROR(-3,'Non spectroscopic file opened as spectroscopic')
	      DATA(N,11+ITMP)=SCALE_VEL_S_SUB_S(ITMP)*DATA(N,11+ITMP)
				!  Now in km/sec
	      SIGMA(N,11+ITMP)=SCALE_VEL_S_SUB_S(ITMP)*SIGMA(N,11+ITMP)
	      DATA_SAVE(N,11+ITMP)=DATA(N,11+ITMP)
	      IF (SIGMA(N,11+ITMP).GT.0.0) THEN
	        WEIGHT(N,11+ITMP)=1.0D0/SIGMA(N,11+ITMP)**2
	      ELSE
	        WEIGHT(N,11+ITMP)=1.0D10 !  Arbitrarily large
	      END IF
	      SUM_VEL_S_SUB_S(ITMP)=SUM_VEL_S_SUB_S(ITMP)+WEIGHT(N,11+ITMP)
	      IF (CHOICE(4:4).EQ.'B') THEN
	        WEIGHT(N,11+ITMP)=0.0
	      END IF
	      WEIGHT_SAVE(N,11+ITMP)=WEIGHT(N,11+ITMP)
	      N=N+1
	    END IF
	  END DO
 996	  N=N-1
	  CLOSE(UNIT=29+ITMP)
	  NPOINTS_DATA=N
	  NPOINTS_DATA_VEL_S_SUB_S(ITMP)=NPOINTS_DATA-NPOINTS_VIS-
	1 NPOINTS_SP-NPOINTS_VEL_P_S-NPOINTS_VEL_S_S
	  NPOINTS_VEL_S_S=NPOINTS_VEL_S_S+NPOINTS_DATA_VEL_S_SUB_S(ITMP)
	END DO !  Ends loop over secondary spectroscopic subcomponent 
	       !  secondary files with index I.
C
C  Ask the user about scaling the weights
C
	IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B') THEN
	  SUM_P=SUM_P/(2.0*NPOINTS_DATA_P)
	  WRITE (6,1201) SUM_P,1.0/SQRT(SUM_P)
1201	  FORMAT(' ','The average XY weight for the visual primary is ',
	1 G14.8,/' and the average sigma is ',G14.8)
	  WRITE (7,1201) SUM_P,1.0/SQRT(SUM_P)
	  SCALEWEIGHTS=.FALSE.
	  IF (FILETYPE_P.EQ.'X_Y') THEN
	    CALL MESSAGE('Do you wish to scale the XY weights of the'//
	1   ' positions of the primary?')
	    CALL USRINP(SCALEWEIGHTS,1,'L',4,1,
	1   'SCALE_WEIGHTS_PRIMARY_POSITION=',
	2   '(default = N)')
	    IF (SCALEWEIGHTS) THEN
	      WEIGHT_SCALE_XY(1)=1.0D0
	      WEIGHT_SCALE_XY(2)=1.0D0
	      CALL MESSAGE('Enter the factors to multiply the weights '//
	1     '(in X and Y) by:')
	      CALL USRINP(WEIGHT_SCALE_XY,2,'F',8,1,
	1    'WEIGHT_SCALE_PRIMARY_POSITION=','(defaults = 1.0 1.0)')
	      DO N=1,NPOINTS_DATA_P
	        WEIGHT(N,1)=WEIGHT_SCALE_XY(1)*WEIGHT(N,1) !  X
	        WEIGHT(N,2)=WEIGHT_SCALE_XY(2)*WEIGHT(N,2) !  Y
	        WEIGHT_SAVE(N,1)=WEIGHT(N,1)
	        WEIGHT_SAVE(N,2)=WEIGHT(N,2)
	        SIGMA(N,1)=SIGMA(N,1)/SQRT(WEIGHT_SCALE_XY(1))
	        SIGMA(N,2)=SIGMA(N,2)/SQRT(WEIGHT_SCALE_XY(2))
	      END DO
	    END IF
	  ELSE IF (FILETYPE_P.EQ.'THETA_RHO') THEN
	    CALL MESSAGE('Do you wish to scale the Theta-Rho weights of'//
	1   ' the positions of the primary?')
	    CALL USRINP(SCALEWEIGHTS,1,'L',4,1,
	1   'SCALE_WEIGHTS_PRIMARY_POSITION=',
	2   '(default = N)')
	    IF (SCALEWEIGHTS) THEN
	      WEIGHT_SCALE_THETA_RHO(1)=1.0D0
	      WEIGHT_SCALE_THETA_RHO(2)=1.0D0
	      CALL MESSAGE('Enter the factors to multiply the weights '//
	1     '(in Theta and Rho) by:')
	      CALL USRINP(WEIGHT_SCALE_THETA_RHO,2,'F',8,1,
	1     'WEIGHT_SCALE_PRIMARY_POSITION=','(defaults = 1.0 1.0)')
	      DO N=1,NPOINTS_DATA_P
C  Sigmas in theta and rho are in SIGMA_SAVE
	        SIGMA_SAVE(N,1)=SIGMA_SAVE(N,1)/
	1       SQRT(WEIGHT_SCALE_THETA_RHO(1))
	        SIGMA_SAVE(N,2)=SIGMA_SAVE(N,2)/
	1       SQRT(WEIGHT_SCALE_THETA_RHO(2))
C  Now recompute the XY sigmas and weights
	        TEMP1=DATA(N,1) !  X, was converted above
	        TEMP2=DATA(N,2) !  Y
	        TEMP3=SIGMA_SAVE(N,1) !  Error in angle from the file
	        TEMP4=SIGMA_SAVE(N,2) !  Error in separation scaled from the file
	        TEMP5=SQRT(TEMP1**2+TEMP2**2) !  We don't have rho any more
C  We start with x=rho*sin(theta) and y=rho*cos(theta)
	        SIGMA(N,1)=SQRT((TEMP2*TEMP3/DEG_RAD)**2+
	1       (TEMP1/TEMP5*TEMP4)**2)
	        SIGMA(N,2)=SQRT((TEMP1*TEMP3/DEG_RAD)**2+
	1       (TEMP2/TEMP5*TEMP4)**2)
	        DO K=1,2
	          WEIGHT(N,K)=1.0D0/SIGMA(N,K)**2
	          WEIGHT_SAVE(N,K)=WEIGHT(N,K)
	        END DO
	      END DO
C  Old way, ignore
C  They wouldn't actually exist if FILETYPE_P were X_Y.
C  We start with theta=tan^-1(y/x)
C  and rho^2=x^2+y^2.  Remember that SIGMA_SAVE is theta and rho
C  and SIGMA is x and y.
C no checking for divide by zero yet
C	      SIGMA_SAVE(N,1)=SQRT(SIGMA(N,1)**2*(DATA(N,2)/
C	1     (1.0+(DATA(N,2)/DATA(N,1))**2)/DATA(N,1)**2)**2+
C	2     SIGMA(N,2)**2*(1.0/(1.0+(DATA(N,2)/DATA(N,1))**2)/
C	3     DATA(N,1))**2)*DEG_RAD
C				!  Theta in degrees
C	      SIGMA_SAVE(N,2)=SQRT(SIGMA(N,1)**2*DATA(N,1)**2/
C	1     (DATA(N,1)**2+DATA(N,2)**2)+
C	2     SIGMA(N,2)**2*DATA(N,2)**2/(DATA(N,1)**2+DATA(N,2)**2))
C				!  Rho in arc "
	    END IF
	  END IF
	END IF
	IF (CHOICE(1:1).EQ.'S'.OR.CHOICE(1:1).EQ.'B') THEN
	  SUM_S=SUM_S/(2.0*NPOINTS_DATA_S)
	  WRITE (6,1202) SUM_S,1.0/SQRT(SUM_S)
1202	  FORMAT(' ','The average XY weight for the visual secondary is ',
	1 G14.8,/' and the average sigma is ',G14.8)
	  WRITE (7,1202) SUM_S,1.0/SQRT(SUM_S)
	  SCALEWEIGHTS=.FALSE.
	  IF (FILETYPE_S.EQ.'X_Y') THEN
	    CALL MESSAGE('Do you wish to scale the XY weights of the'//
	1   ' positions of the secondary?')
	    CALL USRINP(SCALEWEIGHTS,1,'L',4,1,
	1   'SCALE_WEIGHTS_SECONDARY_POSITION=',
	2   '(default = N)')
	    IF (SCALEWEIGHTS) THEN
	      WEIGHT_SCALE_XY(1)=1.0D0
	      WEIGHT_SCALE_XY(2)=1.0D0
	      CALL MESSAGE('Enter the factors to multiply the weights '//
	1     '(in X and Y) by:')
	      CALL USRINP(WEIGHT_SCALE_XY,2,'F',8,1,
	1    'WEIGHT_SCALE_SECONDARY_POSITION=','(defaults = 1.0 1.0)')
	      DO N=NPOINTS_DATA_P+1,NPOINTS_DATA_P+NPOINTS_DATA_S
	        WEIGHT(N,3)=WEIGHT_SCALE_XY(1)*WEIGHT(N,3)
	        WEIGHT(N,4)=WEIGHT_SCALE_XY(2)*WEIGHT(N,4)
	        WEIGHT_SAVE(N,3)=WEIGHT(N,3)
	        WEIGHT_SAVE(N,4)=WEIGHT(N,4)
	        SIGMA(N,3)=SIGMA(N,3)/SQRT(WEIGHT_SCALE_XY(1))
	        SIGMA(N,4)=SIGMA(N,4)/SQRT(WEIGHT_SCALE_XY(2))
	      END DO
	    END IF
	  ELSE IF (FILETYPE_S.EQ.'THETA_RHO') THEN
	    CALL MESSAGE('Do you wish to scale the Theta-Rho weights of'//
	1   ' the positions of the secondary?')
	    CALL USRINP(SCALEWEIGHTS,1,'L',4,1,
	1   'SCALE_WEIGHTS_SECONDARY_POSITION=',
	2   '(default = N)')
	    IF (SCALEWEIGHTS) THEN
	      WEIGHT_SCALE_THETA_RHO(1)=1.0D0
	      WEIGHT_SCALE_THETA_RHO(2)=1.0D0
	      CALL MESSAGE('Enter the factors to multiply the weights '//
	1     '(in Theta and Rho) by:')
	      CALL USRINP(WEIGHT_SCALE_THETA_RHO,2,'F',8,1,
	1     'WEIGHT_SCALE_SECONDARY_POSITION=','(defaults = 1.0 1.0)')
	      DO N=NPOINTS_DATA_P+1,NPOINTS_DATA_P+NPOINTS_DATA_S
C  Sigmas in theta and rho are in SIGMA_SAVE
	        SIGMA_SAVE(N,3)=SIGMA_SAVE(N,3)/
	1       SQRT(WEIGHT_SCALE_THETA_RHO(1))
	        SIGMA_SAVE(N,4)=SIGMA_SAVE(N,4)/
	1       SQRT(WEIGHT_SCALE_THETA_RHO(2))
C  Now recompute the XY sigmas and weights
	        TEMP1=DATA(N,3) !  X, was converted above
	        TEMP2=DATA(N,4) !  Y
	        TEMP3=SIGMA_SAVE(N,3) !  Error in angle from the file
	        TEMP4=SIGMA_SAVE(N,4) !  Error in separation scaled from the file
	        TEMP5=SQRT(TEMP1**2+TEMP2**2) !  We don't have rho any more
C  We start with x=rho*sin(theta) and y=rho*cos(theta)
	        SIGMA(N,3)=SQRT((TEMP2*TEMP3/DEG_RAD)**2+
	1       (TEMP1/TEMP5*TEMP4)**2)
	        SIGMA(N,4)=SQRT((TEMP1*TEMP3/DEG_RAD)**2+
	1       (TEMP2/TEMP5*TEMP4)**2)
	        DO K=3,4
	          WEIGHT(N,K)=1.0D0/SIGMA(N,K)**2
	          WEIGHT_SAVE(N,K)=WEIGHT(N,K)
	        END DO
	      END DO
	    END IF
C  Old way, ignore
C  They wouldn't actually
C  exist if FILETYPE_P were X_Y.  We start with theta=tan^-1(y/x)
C  and rho^2=x^2+y^2.  Remember that SIGMA_SAVE is theta and rho
C  and SIGMA is x and y.
C  No checking for divide by zero yet
C	      SIGMA_SAVE(N,3)=SQRT(SIGMA(N,3)**2*(DATA(N,4)/
C	1     (1.0+(DATA(N,4)/DATA(N,3))**2)/DATA(N,3)**2)**2+
C	2     SIGMA(N,4)**2*(1.0/(1.0+(DATA(N,4)/DATA(N,3))**2)/
C	3     DATA(N,3))**2)*DEG_RAD
C				!  Theta in degrees
C	      SIGMA_SAVE(N,4)=SQRT(SIGMA(N,3)**2*DATA(N,3)**2/
C	1     (DATA(N,3)**2+DATA(N,4)**2)+
C	2     SIGMA(N,4)**2*DATA(N,4)**2/(DATA(N,3)**2+DATA(N,4)**2))
C				!  Rho in arc "
	  END IF
	END IF
	IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B') THEN
	  SUM_VEL_P=SUM_VEL_P/NPOINTS_DATA_VEL_P
	  WRITE (6,1203) SUM_VEL_P,1.0/SQRT(SUM_VEL_P)
1203	  FORMAT(' ','The average weight for the spectroscopic primary',
	1 ' is ',G14.8,/' and the average sigma is ',G14.8)
	  WRITE (7,1203) SUM_VEL_P,1.0/SQRT(SUM_VEL_P)
	  SCALEWEIGHTS=.FALSE.
	  CALL MESSAGE('Do you wish to scale the weights of the'//
	1 ' velocities of the primary?')
	  CALL USRINP(SCALEWEIGHTS,1,'L',4,1,
	1 'SCALE_WEIGHTS_PRIMARY_VELOCITY=','(default = N)')
	  IF (SCALEWEIGHTS) THEN
	    WEIGHT_SCALE=1.0
	    CALL USRINP(WEIGHT_SCALE,1,'F',8,1,
	1   'WEIGHT_SCALE_PRIMARY_VELOCITY=',
	2   'Enter the factor to multiply the weights by:  (default = 1.0)')
	    DO N=NPOINTS_VIS+1,NPOINTS_VIS+NPOINTS_DATA_VEL_P
	      WEIGHT(N,5)=WEIGHT_SCALE*WEIGHT(N,5)
	      WEIGHT_SAVE(N,5)=WEIGHT(N,5)
	      SIGMA(N,5)=SIGMA(N,5)/SQRT(WEIGHT_SCALE)
	    END DO
	  END IF
	END IF
	IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') THEN
	  SUM_VEL_S=SUM_VEL_S/NPOINTS_DATA_VEL_S
	  WRITE (6,1204) SUM_VEL_S,1.0/SQRT(SUM_VEL_S)
1204	  FORMAT(' ','The average weight for the spectroscopic secondary',
	1 ' is ',G14.8,/' and the average sigma is ',G14.8)
	  WRITE (7,1204) SUM_VEL_S,1.0/SQRT(SUM_VEL_S)
	  SCALEWEIGHTS=.FALSE.
	  CALL MESSAGE('Do you wish to scale the weights of the'//
	1 ' velocities of the secondary?')
	  CALL USRINP(SCALEWEIGHTS,1,'L',4,1,
	1 'SCALE_WEIGHTS_SECONDARY_VELOCITY=','(default = N)')
	  IF (SCALEWEIGHTS) THEN
	    WEIGHT_SCALE=1.0
	    CALL USRINP(WEIGHT_SCALE,1,'F',8,1,
	1   'WEIGHT_SCALE_SECONDARY_VELOCITY=',
	1   'Enter the factor to multiply the weights by:  (default = 1.0)')
	    DO N=NPOINTS_VIS+NPOINTS_DATA_VEL_P+1,NPOINTS_VIS+NPOINTS_SP
	      WEIGHT(N,6)=WEIGHT_SCALE*WEIGHT(N,6) 
	      WEIGHT_SAVE(N,6)=WEIGHT(N,6)
	      SIGMA(N,6)=SIGMA(N,6)/SQRT(WEIGHT_SCALE)
	    END DO
	  END IF
	END IF
	DO I=1,NPRISUB_S_F
	  ITMP=IPRISUB_S_F(I)
	  SUM_VEL_P_SUB_S(ITMP)=SUM_VEL_P_SUB_S(ITMP)/
	1 NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	  WRITE (6,1205) ITMP,SUM_VEL_P_SUB_S(ITMP),
	1 1.0/SQRT(SUM_VEL_P_SUB_S(ITMP))
1205	  FORMAT(' ','The average weight for spectroscopic primary ',
	1 'subcomponent secondary ',I1,/
	2 ' is ',G14.8,' and the average sigma is ',G14.8)
	  WRITE (7,1205) ITMP,SUM_VEL_P_SUB_S(ITMP),
	1 1.0/SQRT(SUM_VEL_P_SUB_S(ITMP))
	  SCALEWEIGHTS=.FALSE.
	  CALL MESSAGE('Do you wish to scale the weights of the'//
	1 ' velocities of this primary')
	  CALL USRINP(SCALEWEIGHTS,1,'L',4,1,
	1 'SCALE_WEIGHTS_PRIMARY_VELOCITY=',
	2 'subcomponent secondary?  (default = N)')
	  IF (SCALEWEIGHTS) THEN
	    WEIGHT_SCALE=1.0
	    CALL USRINP(WEIGHT_SCALE,1,'F',8,1,
	1   'WEIGHT_SCALE_PRIMARY_VELOCITY=',
	2   'Enter the factor to multiply the weights by:  (default = 1.0)')
	    IF (I.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+1
	    NHI=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)-1
	    DO N=NLO,NHI
	      WEIGHT(N,6+ITMP)=WEIGHT_SCALE*WEIGHT(N,6+ITMP)
	      WEIGHT_SAVE(N,6+ITMP)=WEIGHT(N,6+ITMP)
	      SIGMA(N,6+ITMP)=SIGMA(N,6+ITMP)/SQRT(WEIGHT_SCALE)
	    END DO
	    NLO=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	  END IF
	END DO
	DO I=1,NSECSUB_S_F
	  ITMP=ISECSUB_S_F(I)
	  SUM_VEL_S_SUB_S(ITMP)=SUM_VEL_S_SUB_S(ITMP)/
	1 NPOINTS_DATA_VEL_S_SUB_S(ITMP)
	  WRITE (6,1206) ITMP,SUM_VEL_S_SUB_S(ITMP),
	1 1.0/SQRT(SUM_VEL_S_SUB_S(ITMP))
1206	  FORMAT(' ','The average weight for spectroscopic secondary ',
	1 'subcomponent secondary ',I1,/
	2 ' is ',G14.8,' and the average sigma is ',G14.8)
	  WRITE (7,1206) ITMP,SUM_VEL_S_SUB_S(ITMP),
	1 1.0/SQRT(SUM_VEL_S_SUB_S(ITMP))
	  SCALEWEIGHTS=.FALSE.
	  CALL MESSAGE('Do you wish to scale the weights of the'//
	1 ' velocities of this secondary')
	  CALL USRINP(SCALEWEIGHTS,1,'L',4,1,
	1 'SCALE_WEIGHTS_SECONDARY_VELOCITY=',
	2 'subcomponent secondary?  (default = N)')
	  IF (SCALEWEIGHTS) THEN
	    WEIGHT_SCALE=1.0
	    CALL USRINP(WEIGHT_SCALE,1,'F',8,1,
	1   'WEIGHT_SCALE_SECONDARY_VELOCITY=',
	2   'Enter the factor to multiply the weights by:  (default = 1.0)')
	    IF (I.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+NPOINTS_VEL_P_S+1
	    NHI=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)-1
	    DO N=NLO,NHI
	      WEIGHT(N,11+ITMP)=WEIGHT_SCALE*WEIGHT(N,11+ITMP)
	      WEIGHT_SAVE(N,11+ITMP)=WEIGHT(N,11+ITMP)
	      SIGMA(N,11+ITMP)=SIGMA(N,11+ITMP)/SQRT(WEIGHT_SCALE)
	    END DO
	    NLO=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)
	  END IF
	END DO
C
	LIGHT_TRAVEL_TIME=.FALSE.
	IF (SPECTROSCOPIC) THEN
	  LIGHT_TRAVEL_TIME=.TRUE.
	  CALL MESSAGE('Do you wish to correct for the light travel time'//
	1 ' across the orbit(s)?')
	  CALL USRINP(LIGHT_TRAVEL_TIME,1,'L',4,1,
	1 'CORRECT_FOR_TRAVEL_TIME=','(default = Y)')
	  IF (LIGHT_TRAVEL_TIME.AND..NOT.VISUAL) THEN
	    CALL MESSAGE('The inclination angle for the main pair cannot'//
	1   ' be solved for.  Enter a')
	    INCLINA_WORK=90.0D0
	    CALL USRINP(INCLINA_WORK,1,'F',8,1,'WORKING_INCLINATION=',
	1   'working value for the light travel time calculation:'//
	2   '  (default = 90 degrees)')
	  END IF
	  IF (LIGHT_TRAVEL_TIME.AND.NPRISUB.NE.0) THEN
	    DO I=1,NSUBS_MAX
	      INCLINA_WORK_PRI_SUB(I)=90.0D0
	    END DO
	    CALL MESSAGE('The inclination angle(s) for the primary'//
	1   ' subcomponent(s) cannot be')
	    CALL USRINP(INCLINA_WORK_PRI_SUB,NSUBS_MAX,'F',8,1,
	1   'WORKING_INCLINATION_PRIMARY_SUBCOMPONENT(S)=',
	2   'solved for.  Enter (a) working value(s):  '//
	3   '(default(s) = 90 degrees)')
	  END IF
	  IF (LIGHT_TRAVEL_TIME.AND.NSECSUB.NE.0) THEN
	    DO I=1,NSUBS_MAX
	      INCLINA_WORK_SEC_SUB(I)=90.0D0
	    END DO
	    CALL MESSAGE('The inclination angle(s) for the secondary'//
	1   ' subcomponent(s) cannot be')
	    CALL USRINP(INCLINA_WORK_SEC_SUB,NSUBS_MAX,'F',8,1,
	1   'WORKING_INCLINATION_SECONDARY_SUBCOMPONENT(S)=',
	2   'solved for.  Enter (a) working value(s):  '//
	3   '(default(s) = 90 degrees)')
	  END IF
	END IF
C
C  Return point for redoing the calculation by excluding points deviating
C  beyond a given factor times the mean error.
	REDO_SOLUTION=.FALSE.
 100	NPOINTS_DATA_P_OMIT=0
	NPOINTS_DATA_S_OMIT=0
	NPOINTS_DATA_VEL_P_OMIT=0
	NPOINTS_DATA_VEL_S_OMIT=0
	NPOINTS_DATA_TOTAL_OMIT=0
	DO I=1,NSUBS_MAX
	  NPOINTS_DATA_VEL_P_SUB_S_OMIT(I)=0
	  NPOINTS_DATA_VEL_S_SUB_S_OMIT(I)=0
	END DO
	NPOINTS_DATA_INTER_P_OMIT=0
	NPOINTS_DATA_INTER_S_OMIT=0
	IF (REDO_SOLUTION) THEN
	  IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B') THEN
	    DO N=1,NPOINTS_DATA_P
	      WEIGHT(N,1)=WEIGHT_SAVE(N,1)
	      WEIGHT(N,2)=WEIGHT_SAVE(N,2)
	      IF (ABS(RESIDUALS(N,1)).GT.THRESHOLD_FACTOR*MEANERROR(1).OR.
	1     ABS(RESIDUALS(N,2)).GT.THRESHOLD_FACTOR*MEANERROR(2)) THEN
	        WEIGHT(N,1)=0.0
	        WEIGHT(N,2)=0.0
	        NPOINTS_DATA_P_OMIT=NPOINTS_DATA_P_OMIT+1
	      END IF
	    END DO
	  END IF
	  IF (CHOICE(1:1).EQ.'S'.OR.CHOICE(1:1).EQ.'B') THEN
	    DO N=NPOINTS_DATA_P+1,NPOINTS_DATA_P+NPOINTS_DATA_S
	      WEIGHT(N,3)=WEIGHT_SAVE(N,3)
	      WEIGHT(N,4)=WEIGHT_SAVE(N,4)
	      IF (ABS(RESIDUALS(N,3)).GT.THRESHOLD_FACTOR*MEANERROR(3).OR.
	1     ABS(RESIDUALS(N,4)).GT.THRESHOLD_FACTOR*MEANERROR(4)) THEN
	        WEIGHT(N,3)=0.0
	        WEIGHT(N,4)=0.0
	        NPOINTS_DATA_S_OMIT=NPOINTS_DATA_S_OMIT+1
	      END IF
	    END DO
	  END IF
	  IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B') THEN
	    DO N=NPOINTS_VIS+1,NPOINTS_VIS+NPOINTS_DATA_VEL_P
	      WEIGHT(N,5)=WEIGHT_SAVE(N,5)
	      IF (ABS(RESIDUALS(N,5)).GT.THRESHOLD_FACTOR*MEANERROR(5)) THEN
	        WEIGHT(N,5)=0.0
	        NPOINTS_DATA_VEL_P_OMIT=NPOINTS_DATA_VEL_P_OMIT+1
	      END IF
	    END DO
	  END IF
	  IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') THEN
	    DO N=NPOINTS_VIS+NPOINTS_DATA_VEL_P+1,NPOINTS_VIS+NPOINTS_SP
	      WEIGHT(N,6)=WEIGHT_SAVE(N,6)
	      IF (ABS(RESIDUALS(N,6)).GT.THRESHOLD_FACTOR*MEANERROR(6)) THEN
	        WEIGHT(N,6)=0.0
	        NPOINTS_DATA_VEL_S_OMIT=NPOINTS_DATA_VEL_S_OMIT+1
	      END IF
	    END DO
	  END IF
	  NPOINTS_DATA_TOTAL_OMIT=NPOINTS_DATA_P_OMIT+NPOINTS_DATA_S_OMIT+
	1 NPOINTS_DATA_VEL_P_OMIT+NPOINTS_DATA_VEL_S_OMIT
	  DO I=1,NPRISUB_S_F
	    ITMP=IPRISUB_S_F(I)
	    IF (I.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+1
	    NHI=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)-1
	    DO N=NLO,NHI
	      WEIGHT(N,6+ITMP)=WEIGHT_SAVE(N,6+ITMP)
	      IF (ABS(RESIDUALS(N,6+ITMP)).GT.THRESHOLD_FACTOR*
	1     MEANERROR(6+ITMP)) THEN
	        WEIGHT(N,6+ITMP)=0.0
	        NPOINTS_DATA_VEL_P_SUB_S_OMIT(ITMP)=
	1       NPOINTS_DATA_VEL_P_SUB_S_OMIT(ITMP)+1
	      END IF
	    END DO
	    NLO=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	    NPOINTS_DATA_TOTAL_OMIT=NPOINTS_DATA_TOTAL_OMIT+
	1   NPOINTS_DATA_VEL_P_SUB_S_OMIT(ITMP)
	  END DO
	  DO I=1,NSECSUB_S_F
	    ITMP=ISECSUB_S_F(I)
	    IF (I.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+NPOINTS_VEL_P_S+1
	    NHI=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)-1
	    DO N=NLO,NHI
	      WEIGHT(N,11+ITMP)=WEIGHT_SAVE(N,11+ITMP)
	      IF (ABS(RESIDUALS(N,11+ITMP)).GT.THRESHOLD_FACTOR*
	1     MEANERROR(11+ITMP)) THEN
	        WEIGHT(N,11+ITMP)=0.0
	        NPOINTS_DATA_VEL_S_SUB_S_OMIT(ITMP)=
	1       NPOINTS_DATA_VEL_S_SUB_S_OMIT(ITMP)+1
	      END IF
	    END DO
	    NLO=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)
	    NPOINTS_DATA_TOTAL_OMIT=NPOINTS_DATA_TOTAL_OMIT+
	1   NPOINTS_DATA_VEL_S_SUB_S_OMIT(ITMP)
	  END DO
	END IF
c will need something here for interferometric
C
	EPOCH=EQUATOR_P
	ITIME(1)=1
	ITIME(2)=NPOINTS_DATA !  Total for all dependent dimensions
	ITERATION_SCHEME=' '
	TOLERANCE=0.0
	INTERMEDIATE=.TRUE.
	OPERATE=LIGHT_TRAVEL_TIME
C  The plotting array can't be loaded until we know what the period of 
C  the orbit is.  Set NPLPOINTS equal to zero.
C  Start the solution by asking for initial estimates
	IF (IGNORE_MAIN_PAIR_BIN) THEN
	  CALL MESSAGE('Initial estimates (including those of an'//
	1 ' imaginary main pair) are:')
	ELSE
	  CALL MESSAGE('Initial estimates are:')
	END IF
	WRITE (6,1310) (DESCRIPTION(I),PARAMETRS(I),I=1,ABS(NUMPARAM))
1310	FORMAT(' ',A47,2X,G16.10)
	WRITE (7,1310) (DESCRIPTION(I),PARAMETRS(I),I=1,ABS(NUMPARAM))
	CALL USRINP(PARAMETRS,ABS(NUMPARAM),'F',8,1,'PARAMETERS=',
	1'Enter the initial estimates of the parameters:  (default = above)')
C  The following section is for the case of P=9999.99999, i.e., planets in
C  orbit about one star
	IF (IGNORE_MAIN_PAIR_BIN) THEN
	  DO I=1,MAXNUMPARAM
	    J=ISETEND(NCONSTANT(I))
	  END DO
	  NCONSTANT(1)=1
	  NCONSTANT(2)=2
	  NCONSTANT(3)=3
	  NCONSTANT(4)=4
	  NCONSTANT(5)=6
	  NCONSTANT(6)=0 !  For taking default
	  CALL MESSAGE('Because the main pair elements are not solved for'//
	1 ' on account of the')
	  CALL MESSAGE('main pair period having been set to the key value'//
	1 ' of 9999.99999, their')
	  CALL MESSAGE('values will be held constant in the solution which'//
	1 ' follows.')
	  CALL MESSAGE('Enter the indexes of the additional parameters'//
	1 ' which are to be held constant')
	  CALL MESSAGE('(starting index = 1):  (default = none except'//
	1 ' those of main pair,')
	  CALL USRINP(NCONSTANT(6),MAXNUMPARAM-5,'I',4,1,'NUMBERS_CONSTANT=',
	1 'i.e., indexes 1, 2, 3, 4, and 6')
	  IF (NCONSTANT(6).EQ.0) THEN
	    NUMCONSTANT=5
	  ELSE
	    NUMCONSTANT=INEL(NCONSTANT,MAXNUMPARAM)
	  END IF
	  DO I=1,NUMPARAM
	    CONSTANTTERMS(I)=.FALSE.
	  END DO
	  DO I=1,NUMCONSTANT
	    CONSTANTTERMS(NCONSTANT(I))=.TRUE.
	  END DO
C  Make NUMPARAM positive so that NONLINFIT doesn't ask the same
C  question about which prameters to hold constant.
	  NUMPARAM=ABS(NUMPARAM)
	END IF
C
	CALL NONLINFIT(TIME,MAXNPOINTS,DIMENSN_IND,DATA,DIMENSN_DATA,WEIGHT,
	1ITIME,FUNCAT,CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	2ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,MEANERROR,
	3MEANERRORMEANERROR,REDCHISQ,AVREDCHISQ,
	4XPLOT,DATPLOT,0,RESIDUALS)
C
C  Write some diagnostics to the log file
	WRITE (7,1315)
1315	FORMAT(' ','INDEX CONSTANTTERMS')
	DO I=1,NUMPARAM
	  WRITE (7,1316) I,CONSTANTTERMS(I)
1316	  FORMAT(' ',I4,2X,L5)
	END DO
	WRITE (7,1027) DIMENSN_DATA,DIMENSN_DATA_USED
	NUMPARAM_SOLV=0
	DO I=1,NUMPARAM
	  IF (.NOT.CONSTANTTERMS(I)) NUMPARAM_SOLV=NUMPARAM_SOLV+1
	END DO
	WRITE (7,1317) NUMPARAM_SOLV
1317	FORMAT(' ','NUMPARAM_SOLV = ',I4)
C
	CALL MESSAGE(' ')
	CALL MESSAGE('     The Elements of the Orbit and Other Parameters')
1320	FORMAT(' The X and Y centers are for the epoch ',F9.4)
	IF (PARALLAX_PROPER_MOTION_P.EQ.'I') THEN
	  WRITE (6,1320) EPOCH
	  WRITE (7,1320) EPOCH
	END IF
1325	FORMAT(' ',A,G16.10,' +-',G13.7)
	WRITE (6,1325) (DESCRIPTION(I),PARAMETRS(I),
	1SQRT(ABS(COVARIANCE(I,I))),I=1,NUMPARAM)
	WRITE (7,1325) (DESCRIPTION(I),PARAMETRS(I),
	1SQRT(ABS(COVARIANCE(I,I))),I=1,NUMPARAM)
C
	CALL MESSAGE(' ')
	CALL MESSAGE('        Additional Information')
	IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1) THEN
	  CALL MESSAGE('The semimajor axis in " is assigned to the'//
	1 ' primary orbit')
	ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.2) THEN
	  CALL MESSAGE('The semimajor axis in " is assigned to the'//
	1 ' secondary orbit')
	ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.3) THEN
	  CALL MESSAGE('The semimajor axis in " is assigned to the'//
	1 ' relative orbit')
	END IF
	IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.1) THEN
	  CALL MESSAGE('The projected semimajor axis in AU is assigned'//
	1 ' to the primary orbit')
	ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.2) THEN
	  CALL MESSAGE('The projected semimajor axis in AU is assigned'//
	1 ' to the secondary orbit')
	END IF
C
	IF (TYPE_P.EQ.'A'.OR.TYPE_S.EQ.'A') THEN
	  WRITE (6,1328) PARAMETRS(INDX(7))*PHOTOCENTRIC_TRUE_SEMIMAJAX,
	1 SQRT(ABS(COVARIANCE(INDX(7),INDX(7))))*PHOTOCENTRIC_TRUE_SEMIMAJAX
1328	  FORMAT(' ','The photocentric semimajor axis in " =',G16.10,
	1 ' +-',G13.7)
	  WRITE (7,1328) PARAMETRS(INDX(7))*PHOTOCENTRIC_TRUE_SEMIMAJAX,
	1 SQRT(ABS(COVARIANCE(INDX(7),INDX(7))))*PHOTOCENTRIC_TRUE_SEMIMAJAX
	END IF
C
	IF ((SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1.OR.SEMIMAJOR_AXIS_ASSIGNMENT(1)
	1.EQ.2.OR.SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.3).AND.SOLVE_PAR) THEN
	  SIGMA_SMA_AU=SQRT(COVARIANCE(INDX(7),INDX(7))/PARAMETRS(INDX(9))**2+
	1 COVARIANCE(INDX(9),INDX(9))*PARAMETRS(INDX(7))**2/
	2 PARAMETRS(INDX(9))**4-2.0D0*COVARIANCE(INDX(7),INDX(9))*
	3 PARAMETRS(INDX(7))/PARAMETRS(INDX(9))**3)
	  WRITE(6,1329) PARAMETRS(INDX(7))/PARAMETRS(INDX(9)),SIGMA_SMA_AU
1329	  FORMAT(' ','The semimajor axis in AU is ',G16.10,' +-',G13.7)
	  WRITE(7,1329) PARAMETRS(INDX(7))/PARAMETRS(INDX(9)),SIGMA_SMA_AU
	END IF
C
	IF ((SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1.OR.SEMIMAJOR_AXIS_ASSIGNMENT(1)
	1.EQ.2.OR.SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.3).AND.SOLVE_PAR.AND.
	2SPECTROSCOPIC) THEN
	  SIGMA_SMA_AU_SIN=SQRT(COVARIANCE(INDX(7),INDX(7))/
	1 PARAMETRS(INDX(9))**2*DSIND(PARAMETRS(INDX(5)))**2+
	2 COVARIANCE(INDX(9),INDX(9))*PARAMETRS(INDX(7))**2/
	3 PARAMETRS(INDX(9))**4*DSIND(PARAMETRS(INDX(5)))**2+
	4 COVARIANCE(INDX(5),INDX(5))*PARAMETRS(INDX(7))**2/
	5 PARAMETRS(INDX(9))**2*DCOSD(PARAMETRS(INDX(5)))**2/DEG_RAD**2-
	6 2.0D0*COVARIANCE(INDX(7),INDX(9))*PARAMETRS(INDX(7))/
	7 PARAMETRS(INDX(9))**3*DSIND(PARAMETRS(INDX(5)))**2+
	8 2.0D0*COVARIANCE(INDX(7),INDX(5))*PARAMETRS(INDX(7))/
	9 PARAMETRS(INDX(9))**2*DSIND(PARAMETRS(INDX(5)))*
	1 DCOSD(PARAMETRS(INDX(5)))/DEG_RAD-
	2 2.0D0*COVARIANCE(INDX(9),INDX(5))*PARAMETRS(INDX(7))**2/
	3 PARAMETRS(INDX(9))**3*DSIND(PARAMETRS(INDX(5)))*
	4 DCOSD(PARAMETRS(INDX(5)))/DEG_RAD)
	  WRITE(6,1330) PARAMETRS(INDX(7))/PARAMETRS(INDX(9))*
	1 DSIND(PARAMETRS(INDX(5))),SIGMA_SMA_AU_SIN
1330	  FORMAT(' ','The semimajor axis in AU times sin(inc.) is ',
	1 G16.10,' +-',G13.7)
	  WRITE(7,1330) PARAMETRS(INDX(7))/PARAMETRS(INDX(9))*
	1 DSIND(PARAMETRS(INDX(5))),SIGMA_SMA_AU_SIN
	END IF
C
	IF (.NOT.IGNORE_MAIN_PAIR_BIN) THEN
	  WRITE(6,1332) PARAMETRS(1)*365.242198781D0,
	1 SQRT(ABS(COVARIANCE(1,1)))*365.242198781D0
		!  Note that the period is always index 1
1332	  FORMAT(' ','The period in days is ',G16.10,' +-',G13.7)
	  WRITE(7,1332) PARAMETRS(1)*365.242198781D0,
	1 SQRT(ABS(COVARIANCE(1,1)))*365.242198781D0
	  CALL BY_JD(PARAMETRS(INDX(2)),TEMP1)
	  WRITE(6,1333) TEMP1,SQRT(ABS(COVARIANCE(2,2)))*365.242198781D0
1333	  FORMAT(' ','The time of periastron passage is ',
	1 F16.5,' +-',F14.5,' JD')
	  WRITE(7,1333) TEMP1,SQRT(ABS(COVARIANCE(2,2)))*365.242198781D0
	END IF
C
	IF (SOLVE_PAR) THEN
	  PARALLAX=PARAMETRS(INDX(9))
	  SIGMA_PARALLAX=SQRT(COVARIANCE(INDX(9),INDX(9)))
	  DISTANCE=1.0/PARALLAX
	  SIGMA_DISTANCE=SIGMA_PARALLAX*DISTANCE**2
	  WRITE(6,1334) DISTANCE,SIGMA_DISTANCE
1334	  FORMAT(' ','The distance in parsecs is ',G16.10,' +-',G13.7)
	END IF
C
	CALL VEL_AMP_ERROR(CHOICE,INDX,PARAMETRS,COVARIANCE,
	1SIGMA_VEL_AMP,SIGMA_VEL_AMP_SUBS)
	IF (.NOT.IGNORE_MAIN_PAIR_BIN) THEN
	  IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'B') THEN
	    WRITE(6,1336) ABS(VEL_AMP_PRI),SIGMA_VEL_AMP(1)
1336	    FORMAT(' ','The projected vel. amp. of the primary',
	1   ' is ',G14.8,' +-',G11.5,' km/sec')
	    WRITE(7,1336) ABS(VEL_AMP_PRI),SIGMA_VEL_AMP(1)
	  END IF
	END IF
	IF (CHOICE(4:4).EQ.'S'.OR.CHOICE(4:4).EQ.'B') THEN
	  WRITE(6,1338) ABS(VEL_AMP_SEC),SIGMA_VEL_AMP(2)
1338	  FORMAT(' ','The projected vel. amp. of the secondary',
	1 ' is ',G14.8,' +-',G11.5,' km/sec')
	  WRITE(7,1338) ABS(VEL_AMP_SEC),SIGMA_VEL_AMP(2)
	END IF
	IF (SPECTROSCOPIC_PRI_SUB) THEN
	  DO I=1,NPRISUB
	    WRITE(6,1340) I,PARAMETRS(INDX(18+5*(I-1)))*365.242198781D0,
	1   SQRT(ABS(COVARIANCE(INDX(18+5*(I-1)),INDX(18+5*(I-1)))))*
	2   365.242198781D0
1340	    FORMAT(' ','The period in days of pri. subcomp. #',I1,
	1   ' is ',G16.10,' +-',G13.7)
	    WRITE(7,1340) I,PARAMETRS(INDX(18+5*(I-1)))*365.242198781D0,
	1   SQRT(ABS(COVARIANCE(INDX(18+5*(I-1)),INDX(18+5*(I-1)))))*
	2   365.242198781D0
C
	    WRITE(6,1343) I,ABS(VEL_AMP_PRI_SUB(I))
1343	    FORMAT(' ','The projected velocity amp. of pri.',
	1   ' subcomp. #',I1,' is ',G14.8,' km/sec')
	    WRITE(7,1343) I,ABS(VEL_AMP_PRI_SUB(I))
	  END DO
	END IF
	DO I=1,NPRISUB_S_F
	  ITMP=IPRISUB_S_F(I)
	  WRITE(6,1344) ITMP,ABS(VEL_AMP_PRI_SUB_SEC(ITMP))
1344	  FORMAT(' ','The projected velocity amp. of pri.',
	1 ' subcomp. sec. #',I1,' is ',G14.8,' km/sec')
	  WRITE(7,1344) ITMP,ABS(VEL_AMP_PRI_SUB_SEC(ITMP))
	END DO
	IF (SPECTROSCOPIC_SEC_SUB) THEN
	  DO I=1,NSECSUB
	    WRITE(6,1345) I,ABS(VEL_AMP_SEC_SUB(I))
1345	    FORMAT(' ','The projected velocity amp. of sec.',
	1   ' subcomp. #',I1,' is ',G14.8,' km/sec')
	    WRITE(7,1345) I,ABS(VEL_AMP_SEC_SUB(I))
	  END DO
	END IF
	DO I=1,NSECSUB_S_F
	  ITMP=ISECSUB_S_F(I)
	  WRITE(6,1346) ITMP,ABS(VEL_AMP_SEC_SUB_SEC(ITMP))
1346	  FORMAT(' ','The projected velocity amp. of sec.',
	1 ' subcomp. sec. #',I1,' is ',G14.8,' km/sec')
	  WRITE(7,1346) ITMP,ABS(VEL_AMP_SEC_SUB_SEC(ITMP))
	END DO
	CALL MESSAGE('The reduced chi squared is')
	N1435=INT(LOG10(1.0*DIMENSN_DATA_MAX))+1
	DO I=1,DIMENSN_DATA
	  WRITE (6,1435) REDCHISQ(I),I
	  WRITE (7,1435) REDCHISQ(I),I
	END DO
1435	FORMAT(' ',G18.8,' for data dimension ',I<N1435>)
	WRITE(6,1438) AVREDCHISQ
1438	FORMAT(' The average reduced chi squared = ',G18.8)
	WRITE(7,1438) AVREDCHISQ
	CALL MESSAGE('The mean error for a point of average weight is')
	N1440=INT(LOG10(1.0*DIMENSN_DATA_MAX))+1
	DO I=1,DIMENSN_DATA
	  WRITE (6,1440) MEANERROR(I),MEANERRORMEANERROR(I),I
	  WRITE (7,1440) MEANERROR(I),MEANERRORMEANERROR(I),I
	END DO
1440	FORMAT(' ',G18.8,' +-',G16.6,' for data dimension ',
	1I<N1440>)
	WRITE(6,1450) NPOINTS_DATA-NPOINTS_DATA_TOTAL_OMIT
1450	FORMAT(' ','The total number of points used in the solution = ',
	1I5)
	WRITE(7,1450) NPOINTS_DATA-NPOINTS_DATA_TOTAL_OMIT
C  For the case of only a spectroscopic binary, calculate the expected times
C  of eclipses on the assumption that the inclination is 90 degrees (no light
C  travel time correction is included).
	IF (SPECTROSCOPIC.AND..NOT.VISUAL) THEN
	  CALL GET_ECLIPSE_EPOCHS(PARAMETRS,ECLIPSE_EPOCHS)
	  CALL BY_JD(ECLIPSE_EPOCHS(1),TEMP1)
	  CALL MESSAGE('Predicted times of eclipses assuming inclination = '//
	1 '90 degrees and no light')
	  CALL MESSAGE('travel time correction')
	  WRITE(6,1460) ECLIPSE_EPOCHS(1),TEMP1
1460	  FORMAT(' ','     Primary in front:  ',F16.6,' BY, ',F17.4,' JD')
	  WRITE(7,1460) ECLIPSE_EPOCHS(1),TEMP1
	  CALL BY_JD(ECLIPSE_EPOCHS(2),TEMP1)
	  WRITE(6,1461) ECLIPSE_EPOCHS(2),TEMP1
1461	  FORMAT(' ','     Primary in back:  ',F16.6,' BY, ',F17.4,' JD')
	  WRITE(7,1461) ECLIPSE_EPOCHS(2),TEMP1
	END IF
	CALL USRINP(CR,1,'L',4,1,'<CR>=','Press the <CR> to see the'//
	1' covariance:')
	DO J=1,NUMPARAM
	  DO I=1,J
	    N1550I=INT(LOG10(REAL(I)))+1
	    N1550J=INT(LOG10(REAL(J)))+1
	    WRITE (6,1500) I,J,COVARIANCE(I,J)
	    WRITE (7,1500) I,J,COVARIANCE(I,J)
1500	    FORMAT(' Covariance(',I<INT(LOG10(REAL(I)))+1>,',',
	1   I<INT(LOG10(REAL(J)))+1>,') = ',G20.8)
	  END DO
	END DO
	CALL MESSAGE(' ')
C  End of Additional Information section
C
C  Ask if the user wants to redo the solution where points beyond
C  some multiple of the MEOAW are rejected.
	CALL MESSAGE('Do you wish to redo the solution, eliminating '//
	1'points which deviate')
	REDO_SOLUTION=.FALSE.
	CALL USRINP(REDO_SOLUTION,1,'L',4,1,'REDO_SOLUTION=',
	1'more than a given factor times the mean error?  (default = N)')
	IF (REDO_SOLUTION) THEN
	  THRESHOLD_FACTOR=3.0D0
	  CALL USRINP(THRESHOLD_FACTOR,1,'F',8,1,'THRESHOLD_FACTOR=',
	1 'Enter the threshold factor:  (default = 3.0)')
	END IF
	IF (REDO_SOLUTION) GO TO 100
C
C  Beginning of plotting section
C
	PLOTD=.TRUE.
	CALL USRINP(PLOTD,1,'L',4,1,'PLOT=',
	1'Do you wish to plot the function and the data?  (default = Y)')
	IF (PLOTD.AND.VISUAL) THEN
	  PLOTR=.TRUE.
	  CALL USRINP(PLOTR,1,'L',4,1,'PLOT_RESIDUALS=',
	1 'Do you wish to plot the residual vectors for the visual'//
	2 ' data?  (default = Y)')
	  IF (TYPE_S.EQ.'R'.OR.TYPE_S.EQ.'r') THEN
	    MARK_PRI=.TRUE.
	    CALL MESSAGE('Do you wish to mark the location of the'//
	1   ' primary with a circle in')
	    CALL USRINP(MARK_PRI,1,'L',4,1,'MARK_PRIMARY=',
	1   'the relative orbit?  (default = Y)')
	  END IF
	END IF
	IF (PLOTD) THEN
	  PLOTE=.TRUE.
	  CALL USRINP(PLOTE,1,'L',4,1,'PLOT_ERRORBARS=',
	1 'Do you wish to plot errorbars for the'//
	2 ' data?  (default = Y)')
	END IF
	IF (PLOTD.AND.VISUAL) THEN
C  Set up the plotting parameters.  First we will plot the visual binary
C  curve and the data, provided that they exist.
C  If the parallax and proper motion are included, that case will be plotted
C  first, then a plot will be made with them removed.
C  Finally, the spectroscopic binary curve and data will be plotted,
C  provided that they exist.
	  INC(1)=1
	  INC(2)=1
	  COLOR(1)=16776960 !  Yellow
	  COLOR(2)=16711935 !  Magenta
	  INVERT(1)=.TRUE.
	  INVERT(2)=.FALSE.
	  INTERACT=.FALSE.
	  CROSSHAIRS=.TRUE.
	  XLABEL=CNTI//'D'//CNTI//' RIGHT ASCENSION (")'
	  YLABEL=CNTI//'D'//CNTI//' DECLINATION (")'
C  Set up TIME for for plotting the first curve of a visual binary.
C  Leave TIME(1,1) as is, and cover slightly more than one period for plotting
C  if parallax and proper motion are not included in a single star, or  not
C  included in both components for a pair.  If they are included, use the 
C  entire available time by finding the maximum and minimum times.
C  For the actual data, TIME_SAVE is available if it is needed.
C  Get the maximum and minimum times
	  TIME_BEGIN=TIME(1,1)
	  TIME_END=TIME(NPOINTS_VIS,1)
	  DO I=1,NPOINTS_VIS
	    IF (TIME(I,1).GT.TIME_END) TIME_END=TIME(I,1)
	    IF (TIME(I,1).LT.TIME_BEGIN) TIME_BEGIN=TIME(I,1)
	  END DO
C  Get the number of points for the drawn curve.  Too few do not resolve things,
C  too many takes time, and a number around 300 produces random gaps.
	  IF (PARALLAX_PROPER_MOTION_P.EQ.'I'.OR.PARALLAX_PROPER_MOTION_S
	1 .EQ.'I') THEN !  Parallax and proper motion not removed from either
			!  nor both.  For spectroscopic binary data only,
			!  this section will be skipped
	    NPOINTS_CURVE=(TIME_END-TIME_BEGIN)/PARAMETRS(1)*200
	    IF (NPOINTS_CURVE.GT.MAXNPOINTS-10) NPOINTS_CURVE=MAXNPOINTS-10
	    IF (NPOINTS_CURVE.LT.3000) NPOINTS_CURVE=3000
	    DO I=1,NPOINTS_CURVE
	      TIME(I,1)=TIME_BEGIN+(I-1)*(TIME_END-TIME_BEGIN)/
	1     (NPOINTS_CURVE-1)
	    END DO
	  ELSE !  Removed from both
	    NPOINTS_CURVE=3000
	    DO I=2,NPOINTS_CURVE
	      TIME(I,1)=TIME(1,1)+(I-1)*PARAMETRS(1)*1.02/(NPOINTS_CURVE-1)
	    END DO
	  END IF
C  Set up the indexes for a call to NLFUN
	  ITIME(1)=1
	  ITIME(2)=NPOINTS_CURVE
C  Set up the X and Y coordinates for each case, for both the curve 
C  and the data
C
C  Case of single star in the visual binary
	  IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'S') THEN
	    NPLOTS=2 !  Solution and data
	    IF (PLOTR) THEN !  Plots the residuals
	      IR=NPLOTS+1
	      NPLOTS=NPLOTS+2 !  Residuals in X and Y
	    END IF
	    IF (PLOTE) THEN !  Plots errorbars
	      IE=NPLOTS+1
	      NPLOTS=NPLOTS+4 !  Errorbars in X and Y
	    END IF
	    IF (MARK_PRI) THEN !  Marks the primary;  is false for 'P'
	      IM=NPLOTS+1
	      NPLOTS=NPLOTS+1
	    END IF
	    NPLPOINTS(1)=NPOINTS_CURVE
	    NPLPOINTS(2)=NPOINTS_VIS
	    LINVECT(1)=256*17 !  Solid and penwidth=17
	    LINVECT(2)=256*23
	    LINTYPE(1)=0 !  Line
	    LINTYPE(2)=-1 !  Symbols only
	    NSYMBOL(1)=0
	    NSYMBOL(2)=1+256*256*5 !  Symbol #1 and symbol height=5/100 inch
            IF (PLOTR) THEN !  Residuals
	      INC(IR)=1
	      INC(IR+1)=1
	      NPLPOINTS(IR)=NPOINTS_VIS !  To plot vectors requires
	      NPLPOINTS(IR+1)=NPOINTS_VIS !  2 plots
	      LINVECT(IR)=2+256*13 !  Dashed and penwidth=13
	      LINVECT(IR+1)=2+256*13
	      LINTYPE(IR)=997 !  Vector
	      LINTYPE(IR+1)=997
	      NSYMBOL(IR)=1 !  No tickmark
	      NSYMBOL(IR+1)=1
	      COLOR(IR)=3050327 !  Sea green, residuals
	      COLOR(IR+1)=3050327
	    END IF
	    IF (PLOTE) THEN !  Errorbars
	      INC(IE)=1
	      INC(IE+1)=1
	      INC(IE+2)=1
	      INC(IE+3)=1
	      NPLPOINTS(IE)=NPOINTS_VIS
	      NPLPOINTS(IE+1)=NPOINTS_VIS
	      NPLPOINTS(IE+2)=NPOINTS_VIS
	      NPLPOINTS(IE+3)=NPOINTS_VIS
	      LINVECT(IE)=256*23 !  Solid and penwidth=23
	      LINVECT(IE+1)=256*23
	      LINVECT(IE+2)=256*23
	      LINVECT(IE+3)=256*23
	      LINTYPE(IE)=997 !  Errorbar
	      LINTYPE(IE+1)=997
	      LINTYPE(IE+2)=997
	      LINTYPE(IE+3)=997
	      NSYMBOL(IE)=100
	      NSYMBOL(IE+1)=100
	      NSYMBOL(IE+2)=100
	      NSYMBOL(IE+3)=100
	      COLOR(IE)=7982541 !  DarkSlateGray3
	      COLOR(IE+1)=7982541
	      COLOR(IE+2)=7982541
	      COLOR(IE+3)=7982541
	    END IF
	    IF (MARK_PRI) THEN !  Marks primary
	      INC(IM)=1
	      NPLPOINTS(IM)=1
	      LINVECT(IM)=256*23
	      LINTYPE(IM)=-1
	      NSYMBOL(IM)=1+256*256*17 !  Circle with height of 17/100 in.
	      COLOR(IM)=16711680 !  Red
	    END IF
C  Load the actual data points
C  If a plot with parallax and proper motion included is made, 
C  we will later make a plot without them.
	    IF (CHOICE(1:1).EQ.'P') THEN
	      DO I=1,NPOINTS_VIS
	        XPLOT(I,2)=DATA(I,1) !  Single star X data points
	        DATPLOT(I,2)=DATA(I,2) !  Single star Y data points
	        IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,2)=OMIT
	      END DO
	      IF (PLOTR) THEN
	        DO I=1,NPOINTS_VIS
	          XPLOT(I,IR)=XPLOT(I,2)
	          DATPLOT(I,IR)=DATPLOT(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR)=OMIT
	          XPLOT(I,IR+1)=XPLOT(I,IR)-RESIDUALS(I,1)
	          DATPLOT(I,IR+1)=DATPLOT(I,IR)-RESIDUALS(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR+1)=OMIT
	        END DO
	      END IF
	      IF (PLOTE) THEN
	        DO I=1,NPOINTS_VIS
	          XPLOT(I,IE)=XPLOT(I,2)
	          DATPLOT(I,IE)=DATPLOT(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	          XPLOT(I,IE+1)=XPLOT(I,2)
	          DATPLOT(I,IE+1)=SIGMA(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	          XPLOT(I,IE+2)=XPLOT(I,2)
	          DATPLOT(I,IE+2)=DATPLOT(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+2)=OMIT
	          XPLOT(I,IE+3)=SIGMA(I,1)
	          DATPLOT(I,IE+3)=DATPLOT(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+3)=OMIT
	        END DO
	      END IF 
	    ELSE IF (CHOICE(1:1).EQ.'S') THEN
	      DO I=1,NPOINTS_VIS
	        XPLOT(I,2)=DATA(I,3) !  Single star X data points
	        DATPLOT(I,2)=DATA(I,4) !  Single star Y data points
	        IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,2)=OMIT
	      END DO
	      IF (PLOTR) THEN
	        DO I=1,NPOINTS_VIS
	          XPLOT(I,IR)=XPLOT(I,2)
	          DATPLOT(I,IR)=DATPLOT(I,2)
	          IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IR)=OMIT
	          XPLOT(I,IR+1)=XPLOT(I,IR)-RESIDUALS(I,3)
	          DATPLOT(I,IR+1)=DATPLOT(I,IR)-RESIDUALS(I,4)
	          IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IR+1)=OMIT
	        END DO
	      END IF
	      IF (PLOTE) THEN
	        DO I=1,NPOINTS_VIS
	          XPLOT(I,IE)=XPLOT(I,2)
	          DATPLOT(I,IE)=DATPLOT(I,2)
	          IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	          XPLOT(I,IE+1)=XPLOT(I,2)
	          DATPLOT(I,IE+1)=SIGMA(I,4)
	          IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	          XPLOT(I,IE+2)=XPLOT(I,2)
	          DATPLOT(I,IE+2)=DATPLOT(I,2)
	          IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IE+2)=OMIT
	          XPLOT(I,IE+3)=SIGMA(I,3)
	          DATPLOT(I,IE+3)=DATPLOT(I,2)
	          IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IE+3)=OMIT
	        END DO
	      END IF
	      IF (MARK_PRI) THEN
	        XPLOT(1,IM)=0.0
	        DATPLOT(1,IM)=0.0
	      END IF
	    END IF
C  Calculate the parallax factors for the curve, if necessary, for 
C  this case of a single star.  
	    IF (PARALLAX_PROPER_MOTION_P.EQ.'I') THEN
	      DO I=1,NPOINTS_CURVE
	        CALL PARALLAXFACTORS(TIME(I,1),RIGHTASC_P,DECLINA_P,
	1       EQUATOR_P,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	        PX(I)=PX(I)*OBS_TRUE_PAR
	        PY(I)=PY(I)*OBS_TRUE_PAR
	      END DO
	    ELSE IF (PARALLAX_PROPER_MOTION_S.EQ.'I') THEN
	      DO I=1,NPOINTS_CURVE
	        CALL PARALLAXFACTORS(TIME(I,1),RIGHTASC_S,DECLINA_S,
	1       EQUATOR_S,0,PX(I),PY(I))
	        PX(I)=PX(I)*OBS_TRUE_PAR
	        PY(I)=PY(I)*OBS_TRUE_PAR
	      END DO
	    END IF
C  Now call NLFUN  to compute X and Y for the curve.  NLFUN will make 
C  use of the parallax factors if CHOICE(2:2)='I' or if CHOICE(3:3)='I'.
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:4)=CHOICE(1:4)
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    IF (CHOICE(1:1).EQ.'P') THEN
	      DO I=1,NPOINTS_CURVE
	        XPLOT(I,1)=DATA(I,1) !  Single star X positions for curve
	        DATPLOT(I,1)=DATA(I,2) !  Single star Y positions for curve
	      END DO
	    ELSE IF (CHOICE(1:1).EQ.'S') THEN
	      DO I=1,NPOINTS_CURVE
	        XPLOT(I,1)=DATA(I,3) !  Single star X positions for curve
	        DATPLOT(I,1)=DATA(I,4) !  Single star Y positions for curve
	      END DO
	    END IF
C  End of case of a single star in a visual binary
C
C  Case of a pair of stars in a visual binary.
C  If either has parallax and proper motion included, we will
C  first make a plot with them included;  then make a second plot
C  with them removed.
	  ELSE IF (CHOICE(1:1).EQ.'B') THEN
	    INC(3)=1
	    INC(4)=1
	    COLOR(3)=3050327 !  Sea green
	    COLOR(4)=16711680 !  Red
            NPLOTS=4 !  Solutions and data for 2 stars
            IF (PLOTR) THEN !  Plots the residuals
              IR=NPLOTS+1
              NPLOTS=NPLOTS+2 !  Residuals in X and Y
            END IF
            IF (PLOTE) THEN !  Plots errorbars
              IE=NPLOTS+1
              NPLOTS=NPLOTS+4 !  Errorbars in X and Y
            END IF
            NPLPOINTS(1)=NPOINTS_CURVE !  Primary orbit
            NPLPOINTS(2)=NPOINTS_DATA_P !  Primary data
	    NPLPOINTS(3)=NPOINTS_CURVE !  Secondary orbit
	    NPLPOINTS(4)=NPOINTS_DATA_S !  Secondary data
            LINVECT(1)=256*17 !  Solid and penwidth=17
            LINVECT(2)=256*23
	    LINVECT(3)=256*17
	    LINVECT(4)=0
            LINTYPE(1)=0 !  Line
            LINTYPE(2)=-1 !  Symbols only
	    LINTYPE(3)=0
	    LINTYPE(4)=-1
            NSYMBOL(1)=0
            NSYMBOL(2)=1+256*256*5 !  Symbol #1 and symbol height=5/100 inch
	    NSYMBOL(3)=0
	    NSYMBOL(4)=2+256*256*8 !  Symbol #2 and symbol height=8/100 inch
            IF (PLOTR) THEN !  Residuals (primary and secondary combined)
              INC(IR)=1
              INC(IR+1)=1
              NPLPOINTS(IR)=NPOINTS_DATA_P+NPOINTS_DATA_S
			!  Pri. and sec. residuals start of vector
              NPLPOINTS(IR+1)=NPOINTS_DATA_P+NPOINTS_DATA_S
			!  Pri. and sec. residuals, end of vector
              LINVECT(IR)=2+256*13 !  Dashed and penwidth=13
              LINVECT(IR+1)=2+256*13
              LINTYPE(IR)=997 !  Vector
              LINTYPE(IR+1)=997
              NSYMBOL(IR)=1 !  No tick mark
              NSYMBOL(IR+1)=1
              COLOR(IR)=16766720 !  Gold, residuals
              COLOR(IR+1)=16766720
            END IF
	    IF (PLOTE) THEN !  Errorbars (primary and seconary combined)
	      INC(IE)=1
	      INC(IE+1)=1
	      INC(IE+2)=1
	      INC(IE+3)=1
	      NPLPOINTS(IE)=NPOINTS_DATA_P+NPOINTS_DATA_S
	      NPLPOINTS(IE+1)=NPOINTS_DATA_P+NPOINTS_DATA_S
	      NPLPOINTS(IE+2)=NPOINTS_DATA_P+NPOINTS_DATA_S
	      NPLPOINTS(IE+3)=NPOINTS_DATA_P+NPOINTS_DATA_S
	      LINVECT(IE)=256*23 !  Solid and penwidth=23
	      LINVECT(IE+1)=256*23
	      LINVECT(IE+2)=256*23
	      LINVECT(IE+3)=256*23
	      LINTYPE(IE)=997 !  Errorbar
	      LINTYPE(IE+1)=997
	      LINTYPE(IE+2)=997
	      LINTYPE(IE+3)=997
	      NSYMBOL(IE)=100
	      NSYMBOL(IE+1)=100
	      NSYMBOL(IE+2)=100
	      NSYMBOL(IE+3)=100
	      COLOR(IE)=7982541 !  DarkSlateGray3
	      COLOR(IE+1)=7982541
	      COLOR(IE+2)=7982541
	      COLOR(IE+3)=7982541
	    END IF
C  Load the actual data points for the primary.  
	    DO I=1,NPOINTS_DATA_P
	      XPLOT(I,2)=DATA(I,1) !  Primary star X data points
	      DATPLOT(I,2)=DATA(I,2) !  Primary star Y data points
C  Add parallax and proper motion to the primary data if necessary
	      IF (PARALLAX_PROPER_MOTION_P.EQ.'R'.AND.
	1     PARALLAX_PROPER_MOTION_S.EQ.'I') THEN
	        CALL PARALLAXFACTORS(TIME_SAVE(I),RIGHTASC_P,DECLINA_P,
	1       EQUATOR_P,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	        PX(I)=PX(I)*OBS_TRUE_PAR
	        PY(I)=PY(I)*OBS_TRUE_PAR
	        DELTATIME=TIME_SAVE(I)-EPOCH
	        DELTATIME2=DELTATIME**2
	        XPLOT(I,2)=XPLOT(I,2)+PARAMETRS(9)*PX(I)+PARAMETRS(10)+
	1       PARAMETRS(11)*DELTATIME+0.5*PARAMETRS(12)*DELTATIME2
	        DATPLOT(I,2)=DATPLOT(I,2)+PARAMETRS(9)*PY(I)+PARAMETRS(13)+
	1       PARAMETRS(14)*DELTATIME+0.5*PARAMETRS(15)*DELTATIME2
	      END IF
	      IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
C  Load the actual data points for the secondary.
	    DO I=1,NPOINTS_DATA_S
	      II=I+NPOINTS_DATA_P
C  Since we are plotting positions relative to the center of mass
C  (absolute positions), if the secondary data is relative to the primary 
C  position, we must adjust the data by (1-1/R)*DATA+RESIDUALS/R.
C  To see this, note that SEC_PLOT=PRIM+DATA and PRIM(1-R)=SEC. Eliminate
C  PRIM, and note that SEC_PLOT=SEC+RESID.  Then eliminate SEC from the
C  last 2 equations.  Note that PRIM is the primary's computed position
C  possibly affected by PHOTOCENTRIC_TRUE_SEMIMAJAX.
	      IF (CHOICE(3:3).EQ.'R') THEN !  Parallax and proper
		!  motion removed;  implies relative
	        XPLOT(I,4)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA(II,3)+
	1       RESIDUALS(II,3)/PARAMETRS(INDX(8)) !  Secondary X
	        DATPLOT(I,4)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA(II,4)+
	1       RESIDUALS(II,4)/PARAMETRS(INDX(8)) !  Secondary Y
	      ELSE !  Case of absolute coordinates.  Leave as is for now
	        XPLOT(I,4)=DATA(II,3) !  Secondary X data points
	        DATPLOT(I,4)=DATA(II,4) !  Secondary Y data points
	      END IF
C  Add parallax and proper motion to the secondary data if necessay.
	      IF (PARALLAX_PROPER_MOTION_P.EQ.'I'.AND.
	1     PARALLAX_PROPER_MOTION_S.EQ.'R') THEN
	        CALL PARALLAXFACTORS(TIME_SAVE(II),RIGHTASC_S,DECLINA_S,
	1       EQUATOR_S,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	        PX(I)=PX(I)*OBS_TRUE_PAR
	        PY(I)=PY(I)*OBS_TRUE_PAR
	        DELTATIME=TIME_SAVE(II)-EPOCH
	        DELTATIME2=DELTATIME**2
	        XPLOT(I,4)=XPLOT(I,4)+PARAMETRS(9)*PX(I)+PARAMETRS(10)+
	1       PARAMETRS(11)*DELTATIME+0.5*PARAMETRS(12)*DELTATIME2
	        DATPLOT(I,4)=DATPLOT(I,4)+PARAMETRS(9)*PY(I)+PARAMETRS(13)+
	1       PARAMETRS(14)*DELTATIME+0.5*PARAMETRS(15)*DELTATIME2
	      END IF
	      IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,4)=OMIT
	    END DO
	    IF (PLOTR) THEN
	      DO I=1,NPOINTS_DATA_P
	        XPLOT(I,IR)=XPLOT(I,2)
	        DATPLOT(I,IR)=DATPLOT(I,2)
	        IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR)=OMIT
	        XPLOT(I,IR+1)=XPLOT(I,IR)-RESIDUALS(I,1)
	        DATPLOT(I,IR+1)=DATPLOT(I,IR)-RESIDUALS(I,2)
	        IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR+1)=OMIT
	      END DO
	      DO I=1,NPOINTS_DATA_S
	        II=I+NPOINTS_DATA_P
	        XPLOT(II,IR)=XPLOT(I,4)
	        DATPLOT(II,IR)=DATPLOT(I,4)
	        IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IR)=OMIT
	        XPLOT(II,IR+1)=XPLOT(II,IR)-RESIDUALS(II,3)
	        DATPLOT(II,IR+1)=DATPLOT(II,IR)-RESIDUALS(II,4)
	        IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IR+1)=OMIT
	      END DO
	    END IF
	    IF (PLOTE) THEN
	      DO I=1,NPOINTS_DATA_P
	        XPLOT(I,IE)=XPLOT(I,2)
	        DATPLOT(I,IE)=DATPLOT(I,2)
	        IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	        XPLOT(I,IE+1)=XPLOT(I,2)
	        DATPLOT(I,IE+1)=SIGMA(I,2)
	        IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	        XPLOT(I,IE+2)=XPLOT(I,2)
	        DATPLOT(I,IE+2)=DATPLOT(I,2)
	        IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+2)=OMIT
	        XPLOT(I,IE+3)=SIGMA(I,1)
	        DATPLOT(I,IE+3)=DATPLOT(I,2)
	        IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+3)=OMIT
	      END DO
	      DO I=1,NPOINTS_DATA_S
	        II=I+NPOINTS_DATA_P
	        XPLOT(II,IE)=XPLOT(I,4)
	        DATPLOT(II,IE)=DATPLOT(I,4)
	        IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IE)=OMIT
	        XPLOT(II,IE+1)=XPLOT(I,4)
	        DATPLOT(II,IE+1)=SIGMA(II,4)
	        IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IE+1)=OMIT
	        XPLOT(II,IE+2)=XPLOT(I,4)
	        DATPLOT(II,IE+2)=DATPLOT(I,4)
	        IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IE+2)=OMIT
	        XPLOT(II,IE+3)=SIGMA(II,3)
	        DATPLOT(II,IE+3)=DATPLOT(I,4)
	        IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IE+3)=OMIT
	      END DO
	    END IF

C  Calculate the parallax factors for the curve if necessary
	    IF (PARALLAX_PROPER_MOTION_P.EQ.'I'.OR.PARALLAX_PROPER_MOTION_S
	1   .EQ.'I') THEN !  Parallax and proper motion not removed for this 
			  !  plot
	      DO I=1,NPOINTS_CURVE
C  TIME was already loaded for the curve
	        CALL PARALLAXFACTORS(TIME(I,1),RIGHTASC_P,DECLINA_P,
	1       EQUATOR_P,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	        PX(I)=PX(I)*OBS_TRUE_PAR
	        PY(I)=PY(I)*OBS_TRUE_PAR
	      END DO
C  Make sure that NLFUN calculates both components correctly
	      CHOICE_TEMP='BII'//CHOICE(4:4)
			!  Will be absolute for both components
	      CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	      DO I=1,MAXNUMPARAM
	        INDX_BIN(I)=INDX(I)
	      END DO
	      CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1     CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    ELSE !  Parallax and proper motion not in either set of data
	      CHOICE_TEMP='BRA'//CHOICE(4:4)
			!  Will be absolute for both components
	      CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	      DO I=1,MAXNUMPARAM
	        INDX_BIN(I)=INDX(I)
	      END DO
	      CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1     CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    END IF
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=DATA(I,1) !  Primary X positions for curve
	      DATPLOT(I,1)=DATA(I,2) !  Primary Y positions for curve
C  We are plotting positions relative to the center of mass
C  (absolute positions).
	      XPLOT(I,3)=DATA(I,3) !  Secondary X positions for curve
	      DATPLOT(I,3)=DATA(I,4) !  Secondary Y positions for curve
	    END DO
	  END IF
C  End of section for pair of stars in visual binary, i.e, CHOICE(1:1)='B'
C
C  Now plot 1 or 2 stars.  Could be with or without parallax and proper motion.
	  CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1 NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2 INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	  CALL RESETTEK
C
C  Now do a second plot without parallax and proper motion if the first plot
C  had parallax and proper motion included.  The plot of a relative orbit
C  was covered above.
	  IF (PARALLAX_PROPER_MOTION_P.EQ.'I'.OR.PARALLAX_PROPER_MOTION_S
	1 .EQ.'I') THEN !  Parallax and proper motion not removed from either
			!  nor both.  For spectroscopic binary data only,
			!  this section will be skipped.
C  We will plot an orbit relative to the center of mass for a single star 
C  and 2 absolute orbits for a pair.  Set up TIME to be slightly more than 
C  one period.
	    CALL MESSAGE('Data without parallax and proper motion')
	    NPOINTS_CURVE=3000
	    DO I=1,NPOINTS_CURVE
	      TIME(I,1)=TIME_SAVE(1)+(I-1)*PARAMETRS(1)*1.02/(NPOINTS_CURVE-1)
	    END DO
C  Case of single star
	    IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'S') THEN
C  Reset because of the change in the number of points for the curve
	      NPLPOINTS(1)=NPOINTS_CURVE
	      IF (CHOICE(1:1).EQ.'P') THEN
C  Load the actual data points for the primary
	        DO I=1,NPOINTS_DATA_P
C  Remove parallax and proper motion from the primary data
	          CALL PARALLAXFACTORS(TIME_SAVE(I),RIGHTASC_P,DECLINA_P,
	1         EQUATOR_P,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	          PX(I)=PX(I)*OBS_TRUE_PAR
	          PY(I)=PY(I)*OBS_TRUE_PAR
	          DELTATIME=TIME_SAVE(I)-EPOCH
	          DELTATIME2=DELTATIME**2
	          XPLOT(I,2)=XPLOT(I,2)-PARAMETRS(INDX(9))*PX(I)-
	1         PARAMETRS(INDX(10))-PARAMETRS(INDX(11))*DELTATIME-
	2         0.5*PARAMETRS(INDX(12))*DELTATIME2
	          DATPLOT(I,2)=DATPLOT(I,2)-PARAMETRS(INDX(9))*PY(I)-
	1         PARAMETRS(INDX(13))-PARAMETRS(INDX(14))*DELTATIME-
	2         0.5*PARAMETRS(INDX(15))*DELTATIME2
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,2)=OMIT
	        END DO
	        IF (PLOTR) THEN
	          DO I=1,NPOINTS_DATA_P
	            XPLOT(I,IR)=XPLOT(I,2)
	            DATPLOT(I,IR)=DATPLOT(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR)=OMIT
	            XPLOT(I,IR+1)=XPLOT(I,IR)-RESIDUALS(I,1)
	            DATPLOT(I,IR+1)=DATPLOT(I,IR)-RESIDUALS(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR+1)=OMIT
	          END DO
	        END IF
	        IF (PLOTE) THEN
	          DO I=1,NPOINTS_DATA_P
	            XPLOT(I,IE)=XPLOT(I,2)
	            DATPLOT(I,IE)=DATPLOT(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	            XPLOT(I,IE+1)=XPLOT(I,2)
	            DATPLOT(I,IE+1)=SIGMA(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	            XPLOT(I,IE+2)=XPLOT(I,2)
	            DATPLOT(I,IE+2)=DATPLOT(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+2)=OMIT
	            XPLOT(I,IE+3)=SIGMA(I,1)
	            DATPLOT(I,IE+3)=DATPLOT(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+3)=OMIT
	          END DO
	        END IF
C  Now get the data for the curve
	        CHOICE_TEMP='PR' !
	        CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	        DO I=1,MAXNUMPARAM
	          INDX_BIN(I)=INDX(I)
	        END DO
	        CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1       CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	        DO I=1,NPOINTS_CURVE
	          XPLOT(I,1)=DATA(I,1) !  Single star X positions for curve
	          DATPLOT(I,1)=DATA(I,2) !  Single star Y positions for curve
	        END DO
	      ELSE IF (CHOICE(1:1).EQ.'S') THEN
C  Load the actual data points for the secondary
	        DO I=1,NPOINTS_DATA_S
C  Remove parallax and proper motion from the secondary data
	          CALL PARALLAXFACTORS(TIME_SAVE(I),RIGHTASC_S,DECLINA_S,
	1         EQUATOR_S,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	          PX(I)=PX(I)*OBS_TRUE_PAR
	          PY(I)=PY(I)*OBS_TRUE_PAR
	          DELTATIME=TIME_SAVE(I)-EPOCH
	          DELTATIME2=DELTATIME**2
	          XPLOT(I,2)=XPLOT(I,2)-PARAMETRS(INDX(9))*PX(I)-
	1         PARAMETRS(INDX(10))-PARAMETRS(INDX(11))*DELTATIME-
	2         0.5*PARAMETRS(INDX(12))*DELTATIME2
	          DATPLOT(I,2)=DATPLOT(I,2)-PARAMETRS(INDX(9))*PY(I)-
	1         PARAMETRS(INDX(13))-PARAMETRS(INDX(14))*DELTATIME-
	2         0.5*PARAMETRS(INDX(15))*DELTATIME2
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,2)=OMIT
	        END DO
	        IF (PLOTR) THEN
	          DO I=1,NPOINTS_DATA_S
	            XPLOT(I,IR)=XPLOT(I,2)
	            DATPLOT(I,IR)=DATPLOT(I,2)
	            IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IR)=OMIT
	            XPLOT(I,IR+1)=XPLOT(I,IR)-RESIDUALS(I,3)
	            DATPLOT(I,IR+1)=DATPLOT(I,IR)-RESIDUALS(I,4)
	            IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IR+1)=OMIT
	          END DO
	        END IF
	        IF (PLOTE) THEN
	          DO I=1,NPOINTS_DATA_S
	            XPLOT(I,IE)=XPLOT(I,2)
	            DATPLOT(I,IE)=DATPLOT(I,2)
	            IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	            XPLOT(I,IE+1)=XPLOT(I,2)
	            DATPLOT(I,IE+1)=SIGMA(I,4)
	            IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	            XPLOT(I,IE+2)=XPLOT(I,2)
	            DATPLOT(I,IE+2)=DATPLOT(I,2)
	            IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IE+2)=OMIT
	            XPLOT(I,IE+3)=SIGMA(I,3)
	            DATPLOT(I,IE+3)=DATPLOT(I,2)
	            IF (WEIGHT(I,4).EQ.0.0D0) DATPLOT(I,IE+3)=OMIT
	          END DO
	        END IF
C  We don't mark the position of the primary in this section because
C  relative orbits are not considered here.
C
C  Now get the data for the curve via NLFUN
	        CHOICE_TEMP='SR'
	        CHOICE_TEMP(4:4)=CHOICE(4:4)
	        CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	        DO I=1,MAXNUMPARAM
	          INDX_BIN(I)=INDX(I)
	        END DO
	        CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1       CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	        DO I=1,NPOINTS_CURVE
	          XPLOT(I,1)=DATA(I,3) !  Single star X positions for curve
	          DATPLOT(I,1)=DATA(I,4) !  Single star Y positions for curve
	        END DO
	      END IF
	    ELSE IF (CHOICE(1:1).EQ.'B') THEN
C  Reset because of the change in the number of points for the curve
	      NPLPOINTS(1)=NPOINTS_CURVE
	      NPLPOINTS(3)=NPOINTS_CURVE
C  Be sure parallax and proper motion are removed from the primary data
	      IF (PARALLAX_PROPER_MOTION_P.EQ.'I') THEN
	        DO I=1,NPOINTS_DATA_P
	          CALL PARALLAXFACTORS(TIME_SAVE(I),RIGHTASC_P,DECLINA_P,
	1         EQUATOR_P,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	          PX(I)=PX(I)*OBS_TRUE_PAR
	          PY(I)=PY(I)*OBS_TRUE_PAR
	          DELTATIME=TIME_SAVE(I)-EPOCH
	          DELTATIME2=DELTATIME**2
	          XPLOT(I,2)=DATA_SAVE(I,1)-PARAMETRS(INDX(9))*PX(I)-
	1         PARAMETRS(INDX(10))-PARAMETRS(INDX(11))*DELTATIME-
	2         0.5*PARAMETRS(INDX(12))*DELTATIME2
	          DATPLOT(I,2)=DATA_SAVE(I,2)-PARAMETRS(INDX(9))*PY(I)-
	1         PARAMETRS(INDX(13))-PARAMETRS(INDX(14))*DELTATIME-
	2         0.5*PARAMETRS(INDX(15))*DELTATIME2
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,2)=OMIT
	        END DO
	      ELSE
	        DO I=1,NPOINTS_DATA_P
	          XPLOT(I,2)=DATA_SAVE(I,1)
	          DATPLOT(I,2)=DATA_SAVE(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,2)=OMIT
	        END DO
	      END IF
C  Do the secondary
	      DO I=1,NPOINTS_DATA_S
	        II=I+NPOINTS_DATA_P
	        IF (CHOICE(3:3).EQ.'R') THEN
	          XPLOT(I,4)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA_SAVE(II,3)+
	1         RESIDUALS(II,3)/PARAMETRS(INDX(8))
	          DATPLOT(I,4)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA_SAVE(II,4)+
	1         RESIDUALS(II,4)/PARAMETRS(INDX(8))
	          IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,4)=OMIT
	        ELSE
	          XPLOT(I,4)=DATA_SAVE(II,3)
	          DATPLOT(I,4)=DATA_SAVE(II,4)
	          IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,4)=OMIT
	        END IF
	      END DO
C  Be sure parallax and proper motion are removed  from the secondary data
	      IF (PARALLAX_PROPER_MOTION_S.EQ.'I') THEN
	        DO I=1,NPOINTS_DATA_S
	          II=I+NPOINTS_DATA_P
	          CALL PARALLAXFACTORS(TIME_SAVE(II),RIGHTASC_S,DECLINA_S,
	1         EQUATOR_S,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	          PX(I)=PX(I)*OBS_TRUE_PAR
	          PY(I)=PY(I)*OBS_TRUE_PAR
	          DELTATIME=TIME_SAVE(II)-EPOCH
	          DELTATIME2=DELTATIME**2
	          XPLOT(I,4)=XPLOT(I,4)-PARAMETRS(9)*PX(I)-PARAMETRS(10)-
	1         PARAMETRS(11)*DELTATIME-0.5*PARAMETRS(12)*DELTATIME2
	          DATPLOT(I,4)=DATPLOT(I,4)-PARAMETRS(9)*PY(I)-
	1         PARAMETRS(13)-PARAMETRS(14)*DELTATIME-0.5*PARAMETRS(15)*
	2         DELTATIME2
	          IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,4)=OMIT
	        END DO
	      END IF
	      IF (PLOTR) THEN
	        DO I=1,NPOINTS_DATA_P
	          XPLOT(I,IR)=XPLOT(I,2)
	          DATPLOT(I,IR)=DATPLOT(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR)=OMIT
	          XPLOT(I,IR+1)=XPLOT(I,IR)-RESIDUALS(I,1)
	          DATPLOT(I,IR+1)=DATPLOT(I,IR)-RESIDUALS(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR+1)=OMIT
	        END DO
	        DO I=1,NPOINTS_DATA_S
	          II=I+NPOINTS_DATA_P
	          XPLOT(II,IR)=XPLOT(I,4)
	          DATPLOT(II,IR)=DATPLOT(I,4)
	          IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IR)=OMIT
	          XPLOT(II,IR+1)=XPLOT(II,5)-RESIDUALS(II,3)
	          DATPLOT(II,IR+1)=DATPLOT(II,IR)-RESIDUALS(II,4)
	          IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IR+1)=OMIT
	        END DO
	      END IF
	      IF (PLOTE) THEN
	        DO I=1,NPOINTS_DATA_P
	          XPLOT(I,IE)=XPLOT(I,2)
	          DATPLOT(I,IE)=DATPLOT(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	          XPLOT(I,IE+1)=XPLOT(I,2)
	          DATPLOT(I,IE+1)=SIGMA(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	          XPLOT(I,IE+2)=XPLOT(I,2)
	          DATPLOT(I,IE+2)=DATPLOT(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+2)=OMIT
	          XPLOT(I,IE+3)=SIGMA(I,1)
	          DATPLOT(I,IE+3)=DATPLOT(I,2)
	          IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+3)=OMIT
	        END DO

	        DO I=1,NPOINTS_DATA_S
	          II=I+NPOINTS_DATA_P
	          XPLOT(II,IE)=XPLOT(I,4)
	          DATPLOT(II,IE)=DATPLOT(I,4)
	          IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IE)=OMIT
	          XPLOT(II,IE+1)=XPLOT(I,4)
	          DATPLOT(II,IE+1)=SIGMA(II,4)
	          IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IE+1)=OMIT
	          XPLOT(II,IE+2)=XPLOT(I,4)
	          DATPLOT(II,IE+2)=DATPLOT(I,4)
	          IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IE+2)=OMIT
	          XPLOT(II,IE+3)=SIGMA(II,3)
	          DATPLOT(II,IE+3)=DATPLOT(I,4)
	          IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(II,IE+3)=OMIT
	        END DO
	      END IF
C  Now get the data for the curve
	      CHOICE_TEMP='BRR'//CHOICE(4:4)
	      CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	      DO I=1,MAXNUMPARAM
	        INDX_BIN(I)=INDX(I)
	      END DO
	      CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1     CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	      DO I=1,NPOINTS_CURVE
	        XPLOT(I,1)=DATA(I,1) !  Primary star X positions for curve
	        DATPLOT(I,1)=DATA(I,2) !  Primary star Y positions for curve
	        IF (CHOICE(3:3).EQ.'R') THEN
	          XPLOT(I,3)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA(I,3)
							!  Secondary X curve
	          DATPLOT(I,3)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA(I,4)
							!  Secondary Y curve
	        ELSE
	          XPLOT(I,3)=DATA(I,3) !  Secondary star X positions for curve
	          DATPLOT(I,3)=DATA(I,4) !  Secondary star Y positions for curve
	        END IF
	      END DO
	    END IF
C  Reset the control locations in the plotting arrays
	    DO J=0,3
	      XPLOT(MAXNPOINTS-J,1)=0.0
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	    CALL RESETTEK
	  END IF
C
C  If both primary and secondary are available, the above section plots both
C  on the same plot.  Here we will plot each by itself, but without any
C  proper motion and parallax that may have been included.
	  IF (CHOICE(1:1).EQ.'B') THEN
	    NPOINTS_CURVE=3000
	    NPLOTS=2 !  Solution and data
	    IF (PLOTR) THEN !  Plots the residuals
	      IR=NPLOTS+1
	      NPLOTS=NPLOTS+2 !  Residuals in X and Y
	    END IF
	    IF (PLOTE) THEN !  Plots errorbars
	      IE=NPLOTS+1
	      NPLOTS=NPLOTS+4 !  Errorbars in X and Y
	    END IF
C  The primary is K=1 and the secondary K=2
	    DO K=1,2
	      IF (K.EQ.1) THEN
	        NPOINTS_TEMP=NPOINTS_DATA_P
	      ELSE IF (K.EQ.2) THEN
	        NPOINTS_TEMP=NPOINTS_DATA_S
	      END IF
	      NPLPOINTS(1)=NPOINTS_CURVE
	      NPLPOINTS(2)=NPOINTS_TEMP
	      LINVECT(1)=256*17 !  Solid and penwidth=17
	      LINVECT(2)=256*23
	      LINTYPE(1)=0 !  Line
	      LINTYPE(2)=-1 !  Symbols only
	      NSYMBOL(1)=0
	      NSYMBOL(2)=1+256*256*5 !  Symbol #1 and symbol height=5/100 inch
	      IF (PLOTR) THEN !  Residuals
	        INC(IR)=1
	        INC(IR+1)=1
	        NPLPOINTS(IR)=NPOINTS_TEMP   !  To plot vectors requires
	        NPLPOINTS(IR+1)=NPOINTS_TEMP !  2 plots
	        LINVECT(IR)=2+256*13 !  Dashed and penwidth=13
	        LINVECT(IR+1)=2+256*13
	        LINTYPE(IR)=997 !  Vector
	        LINTYPE(IR+1)=997
	        NSYMBOL(IR)=1 !  No tickmark
	        NSYMBOL(IR+1)=1
	        COLOR(IR)=3050327 !  Sea green, residuals
	        COLOR(IR+1)=3050327
	      END IF
	      IF (PLOTE) THEN !  Errorbars
	        INC(IE)=1
	        INC(IE+1)=1
	        INC(IE+2)=1
	        INC(IE+3)=1
	        NPLPOINTS(IE)=NPOINTS_TEMP
	        NPLPOINTS(IE+1)=NPOINTS_TEMP
	        NPLPOINTS(IE+2)=NPOINTS_TEMP
	        NPLPOINTS(IE+3)=NPOINTS_TEMP
	        LINVECT(IE)=256*23 !  Solid and penwidth=23
	        LINVECT(IE+1)=256*23
	        LINVECT(IE+2)=256*23
	        LINVECT(IE+3)=256*23
	        LINTYPE(IE)=997 !  Errorbar
	        LINTYPE(IE+1)=997
	        LINTYPE(IE+2)=997
	        LINTYPE(IE+3)=997
	        NSYMBOL(IE)=100
	        NSYMBOL(IE+1)=100
	        NSYMBOL(IE+2)=100
	        NSYMBOL(IE+3)=100
	        COLOR(IE)=7982541 !  DarkSlateGray3
	        COLOR(IE+1)=7982541
	        COLOR(IE+2)=7982541
	        COLOR(IE+3)=7982541
	      END IF
	      IF (MARK_PRI.AND.K.EQ.2) THEN !  Marks the primary
	        IM=NPLOTS+1
	        NPLOTS=NPLOTS+1
	        INC(IM)=1
	        NPLPOINTS(IM)=1
	        LINVECT(IM)=256*23
	        LINTYPE(IM)=-1
	        NSYMBOL(IM)=1+256*256*17 !  Circle with height of 17/100 in.
	        COLOR(IM)=16711680 !  Red
	      END IF
C  Load the actual data points
	      IF (K.EQ.1) THEN
C  Be sure parallax and proper motion are removed from the primary data
	        IF (PARALLAX_PROPER_MOTION_P.EQ.'I') THEN
	          DO I=1,NPOINTS_DATA_P
	            CALL PARALLAXFACTORS(TIME_SAVE(I),RIGHTASC_P,DECLINA_P,
	1           EQUATOR_P,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	            PX(I)=PX(I)*OBS_TRUE_PAR
	            PY(I)=PY(I)*OBS_TRUE_PAR
	            DELTATIME=TIME_SAVE(I)-EPOCH
	            DELTATIME2=DELTATIME**2
C  Better to use INDX as above
	            XPLOT(I,2)=DATA_SAVE(I,1)-PARAMETRS(9)*PX(I)-PARAMETRS(10)-
	1           PARAMETRS(11)*DELTATIME-0.5*PARAMETRS(12)*DELTATIME2
		    DATPLOT(I,2)=DATA_SAVE(I,2)-PARAMETRS(9)*PY(I)-
	1           PARAMETRS(13)-
	1           PARAMETRS(14)*DELTATIME-0.5*PARAMETRS(15)*DELTATIME2
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,2)=OMIT
	          END DO	        
	        ELSE
	          DO I=1,NPOINTS_DATA_P
	            XPLOT(I,2)=DATA_SAVE(I,1)
	            DATPLOT(I,2)=DATA_SAVE(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,2)=OMIT
	          END DO
	        END IF
	        IF (PLOTR) THEN
	          DO I=1,NPOINTS_DATA_P
	            XPLOT(I,IR)=XPLOT(I,2)
	            DATPLOT(I,IR)=DATPLOT(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR)=OMIT
	            XPLOT(I,IR+1)=XPLOT(I,IR)-RESIDUALS(I,1)
	            DATPLOT(I,IR+1)=DATPLOT(I,IR)-RESIDUALS(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IR+1)=OMIT
	          END DO
	        END IF
	        IF (PLOTE) THEN
	          DO I=1,NPOINTS_DATA_P
	            XPLOT(I,IE)=XPLOT(I,2)
	            DATPLOT(I,IE)=DATPLOT(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	            XPLOT(I,IE+1)=XPLOT(I,2)
	            DATPLOT(I,IE+1)=SIGMA(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	            XPLOT(I,IE+2)=XPLOT(I,2)
	            DATPLOT(I,IE+2)=DATPLOT(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+2)=OMIT
	            XPLOT(I,IE+3)=SIGMA(I,1)
	            DATPLOT(I,IE+3)=DATPLOT(I,2)
	            IF (WEIGHT(I,2).EQ.0.0D0) DATPLOT(I,IE+3)=OMIT
	          END DO
	        END IF
	      ELSE IF (K.EQ.2) THEN
	        DO I=1,NPOINTS_DATA_S
	          II=I+NPOINTS_DATA_P
	          IF (CHOICE(3:3).EQ.'R') THEN
	            XPLOT(I,2)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA_SAVE(II,3)+
	1           RESIDUALS(II,3)/PARAMETRS(INDX(8))
	            DATPLOT(I,2)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA_SAVE(II,4)+
	1           RESIDUALS(II,4)/PARAMETRS(INDX(8))
	            IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,2)=OMIT
	          ELSE
	            XPLOT(I,2)=DATA_SAVE(II,3)
	            DATPLOT(I,2)=DATA_SAVE(II,4)
	            IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,2)=OMIT
	          END IF
	        END DO
C  Be sure parallax and proper motion are removed  from the secondary data
	        IF (PARALLAX_PROPER_MOTION_S.EQ.'I') THEN
	          DO I=1,NPOINTS_DATA_S
	            II=I+NPOINTS_DATA_P
	            CALL PARALLAXFACTORS(TIME_SAVE(II),RIGHTASC_S,DECLINA_S,
	1           EQUATOR_S,0,PX(I),PY(I))
C  The parallax factors must be scaled down by the estimated observed-to-
C  true parallax.
	            PX(I)=PX(I)*OBS_TRUE_PAR
	            PY(I)=PY(I)*OBS_TRUE_PAR
	            DELTATIME=TIME_SAVE(II)-EPOCH
	            DELTATIME2=DELTATIME**2
	            XPLOT(I,2)=XPLOT(I,2)-PARAMETRS(9)*PX(I)-PARAMETRS(10)-
	1           PARAMETRS(11)*DELTATIME-0.5*PARAMETRS(12)*DELTATIME2
	            DATPLOT(I,2)=DATPLOT(I,2)-PARAMETRS(9)*PY(I)-
	1           PARAMETRS(13)-PARAMETRS(14)*DELTATIME-0.5*PARAMETRS(15)*
	2           DELTATIME2
	            IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,2)=OMIT
	          END DO
	        END IF
	        IF (PLOTR) THEN
	          DO I=1,NPOINTS_DATA_S
	            II=I+NPOINTS_DATA_P
	            XPLOT(I,IR)=XPLOT(I,2)
	            DATPLOT(I,IR)=DATPLOT(I,2)
	            IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,IR)=OMIT
	            XPLOT(I,IR+1)=XPLOT(I,IR)-RESIDUALS(II,3)
	            DATPLOT(I,IR+1)=DATPLOT(I,IR)-RESIDUALS(II,4)
	            IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,IR+1)=OMIT
	          END DO
	        END IF
	        IF (PLOTE) THEN
	          DO I=1,NPOINTS_DATA_S
	            II=I+NPOINTS_DATA_P
	            XPLOT(I,IE)=XPLOT(I,2)
	            DATPLOT(I,IE)=DATPLOT(I,2)
	            IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	            XPLOT(I,IE+1)=XPLOT(I,2)
	            DATPLOT(I,IE+1)=SIGMA(II,4)
	            IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	            XPLOT(I,IE+2)=XPLOT(I,2)
	            DATPLOT(I,IE+2)=DATPLOT(I,2)
	            IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,IE+2)=OMIT
	            XPLOT(I,IE+3)=SIGMA(II,3)
	            DATPLOT(I,IE+3)=DATPLOT(I,2)
	            IF (WEIGHT(II,4).EQ.0.0D0) DATPLOT(I,IE+3)=OMIT
	          END DO
	        END IF
	        IF (MARK_PRI) THEN
	          XPLOT(1,IM)=0.0
	          DATPLOT(1,IM)=0.0
	        END IF
	      END IF
C  Now get the data for the curve
	      CHOICE_TEMP='BRR'
	      CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	      DO I=1,MAXNUMPARAM
	        INDX_BIN(I)=INDX(I)
	      END DO
	      CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1     CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	      IF (K.EQ.1) THEN
	        DO I=1,NPOINTS_CURVE
	          XPLOT(I,1)=DATA(I,1) !  Primary star X positions for curve
	          DATPLOT(I,1)=DATA(I,2) !  Primary star Y positions for curve
	        END DO
	      ELSE IF (K.EQ.2) THEN
	        DO I=1,NPOINTS_CURVE
	          IF (CHOICE(3:3).EQ.'R') THEN
	            XPLOT(I,1)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA(I,3)
							!  Secondary X curve
	            DATPLOT(I,1)=(1.0-1.0/PARAMETRS(INDX(8)))*DATA(I,4)
							!  Secondary Y curve
	          ELSE
	            XPLOT(I,1)=DATA(I,3) !  Secondary star X positions for curve
	            DATPLOT(I,1)=DATA(I,4) !  Secondary star
					   !  Y positions for curve
	          END IF
	        END DO
	      END IF
	      DO J=0,3
	        XPLOT(MAXNPOINTS-J,1)=0.0
	        DATPLOT(MAXNPOINTS-J,1)=0.0
	      END DO
	      CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1     NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2     INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	      CALL RESETTEK
	    END DO
	  END IF
	END IF
C
C  Spectroscopic Plotting Sections
C  1.  All time, pri. or sec. available
C  2.  All time, both available
C  3.  All time, pri. or sec. available, subcomponent reflex vel. subtracted
C  4.  All time, both available, no subcomponent reflex vel.
C  5.  One cycle, pri. or sec. available, subcomponent reflex vel. subtracted
C  6.  One cycle, both available, subcomponent reflex vel. subtracted
C  7.  One cycle, each pri. subcomponent reflex vel.
C  8.  One cycle, each sec. subcomponent reflex vel.
C  9.  All time, each pri. subcomponent file
C 10.  One cycle, each pri. subcomponent file
C 11.  All time, each sec. subcomponent file
C 12.  One cycle, each sec. subcomponent file
C
	IF (PLOTD.AND.SPECTROSCOPIC) THEN
C  Set up the plotting parameters.  We will plot the spectroscopic binary
C  curve and the data, provided that they exist, for the entire time span of
C  the observations.  If subcomponents exist, they will be plotted also, 
C  but if their periods are too small, they will not be fully sampled and 
C  aliasing will result.
	  INVERT(1)=.FALSE.
	  INVERT(2)=.FALSE.
	  INTERACT=.FALSE.
	  CROSSHAIRS=.TRUE.
	  XLABEL='TIME (years)'
	  YLABEL='VELOCITY (km/s)'
C  Set up indexes for call to NLFUN
C  Find the maximum and minimum times
	  TIME_BEGIN=TIME_SAVE(NPOINTS_VIS+1)
	  TIME_END=TIME_SAVE(NPOINTS_VIS+NPOINTS_SP)
	  DO I=NPOINTS_VIS+1,NPOINTS_VIS+NPOINTS_SP
	    IF (TIME_SAVE(I).GT.TIME_END) TIME_END=TIME_SAVE(I)
	    IF (TIME_SAVE(I).LT.TIME_BEGIN) TIME_BEGIN=TIME_SAVE(I)
	  END DO
C  Find the smallest period of parents and subcomponents
	  SMALLEST_PERIOD=PARAMETRS(1)
	  DO IP=1,NPRISUB
	    IF (PARAMETRS(INDX(18)+(IP-1)*5).LT.SMALLEST_PERIOD)
	1   SMALLEST_PERIOD=PARAMETRS(INDX(18)+(IP-1)*5)
	  END DO
	  DO IP=1,NSECSUB
	    IF (PARAMETRS(INDX(43)+(IP-1)*5).LT.SMALLEST_PERIOD)
	1   SMALLEST_PERIOD=PARAMETRS(INDX(43)+(IP-1)*5)
	  END DO
	  IF (SMALLEST_PERIOD.NE.0.0D0)
	1 NPOINTS_CURVE=(TIME_END-TIME_BEGIN)/SMALLEST_PERIOD*200
	  NPOINTS_CURVE=MIN(NPOINTS_CURVE,MAXNPOINTS-10)
C  We use more points for the curve here because many cycles could be
C  plotted.
	  NPOINTS_CURVE=MAX(NPOINTS_CURVE,8000)
	  ITIME(1)=1
	  ITIME(2)=NPOINTS_CURVE
	  DO I=1,NPOINTS_CURVE
	    TIME(I,1)=TIME_BEGIN+(I-1)*(TIME_END-TIME_BEGIN)/
	1   (NPOINTS_CURVE-1)
	  END DO
C
C  Setup for one star in spectroscopic binary
C
	  IF (CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'S') THEN
	    NPLOTS=2 !  Solution and data
	    NPLPOINTS(1)=NPOINTS_CURVE
	    NPLPOINTS(2)=NPOINTS_SP
	    INC(1)=1
	    INC(2)=1
	    COLOR(1)=16776960 !  Yellow
	    COLOR(2)=16711935 !  Magenta
	    LINVECT(1)=256*17 !  Solid and penwidth=17
	    LINVECT(2)=256*23
	    LINTYPE(1)=0 !  Line
	    LINTYPE(2)=-1 !  Symbols only
	    NSYMBOL(1)=0
	    NSYMBOL(2)=1+256*256*7 !  Symbol #1 and symbol height=7/100 inch
	    IF (PLOTE) THEN !  Errorbars
	      IE=NPLOTS+1
	      NPLOTS=NPLOTS+2 !  Errorbars in velocity
	      INC(IE)=1
	      INC(IE+1)=1
	      NPLPOINTS(IE)=NPOINTS_SP
	      NPLPOINTS(IE+1)=NPOINTS_SP
	      LINVECT(IE)=256*23 !  Solid and penwidth=23
	      LINVECT(IE+1)=256*23
	      LINTYPE(IE)=997 !  Errorbar
	      LINTYPE(IE+1)=997
	      NSYMBOL(IE)=100
	      NSYMBOL(IE+1)=100
	      COLOR(IE)=7982541 !  DarkSlateGray3
	      COLOR(IE+1)=7982541
	    END IF
C  Load the actual data points
	    IF (CHOICE(4:4).EQ.'P') THEN
	      IDD=5
	    ELSE IF (CHOICE(4:4).EQ.'S') THEN
	      IDD=6
	    END IF
	    DO I=1,NPOINTS_SP !  (Primary or secondary)
	      II=I+NPOINTS_VIS
	      XPLOT(I,2)=TIME_SAVE(II)
	      DATPLOT(I,2)=DATA_SAVE(II,IDD)
	      IF (WEIGHT(II,IDD).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
	    IF (PLOTE) THEN !  (Primary or secondary)
	      DO I=1,NPOINTS_SP
	        II=I+NPOINTS_VIS
	        XPLOT(I,IE)=XPLOT(I,2)
	        DATPLOT(I,IE)=DATPLOT(I,2)
	        IF (WEIGHT(II,IDD).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	        XPLOT(I,IE+1)=XPLOT(I,2)
	        DATPLOT(I,IE+1)=SIGMA(II,IDD)
	        IF (WEIGHT(II,IDD).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	      END DO
	    END IF
C  Load the data for the curve
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(19:29)=CHOICE(19:29) !  Apsidal motion
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=TIME(I,1)
	      DATPLOT(I,1)=DATA(I,IDD)
	    END DO
C
C  Setup for pair of spectroscopic binary stars
C
	  ELSE IF (CHOICE(4:4).EQ.'B') THEN
	    NPLOTS=4 !  Data and solutions for 2 stars
	    INC(1)=1
	    INC(2)=1
	    INC(3)=1
	    INC(4)=1
	    COLOR(1)=16776960 !  Yellow
	    COLOR(2)=16711935 !  Magenta
	    COLOR(3)=3050327 !  Sea green
	    COLOR(4)=16711680 !  Red
	    NPLPOINTS(1)=NPOINTS_CURVE
	    NPLPOINTS(2)=NPOINTS_DATA_VEL_P
	    NPLPOINTS(3)=NPOINTS_CURVE
	    NPLPOINTS(4)=NPOINTS_DATA_VEL_S
	    LINVECT(1)=256*17 !  Solid and penwidth=17
	    LINVECT(2)=256*23
	    LINVECT(3)=256*17
	    LINVECT(4)=0
	    LINTYPE(1)=0 !  Line
	    LINTYPE(2)=-1 !  Symbols only
	    LINTYPE(3)=0 !  Line
	    LINTYPE(4)=-1 !  Symbols only
	    NSYMBOL(1)=0
	    NSYMBOL(2)=1+256*256*7 !  Symbol #1 and symbol height=7/100 inch
	    NSYMBOL(3)=0
	    NSYMBOL(4)=2+256*256*7 !  Symbol #2 and symbol height=7/100 inch
	    IF (PLOTE) THEN !  Errorbars
	      IE=NPLOTS+1
	      NPLOTS=NPLOTS+2 !  Errorbars in velocity
	      INC(IE)=1
	      INC(IE+1)=1
	      NPLPOINTS(IE)=NPOINTS_DATA_VEL_P+NPOINTS_DATA_VEL_S
	      NPLPOINTS(IE+1)=NPOINTS_DATA_VEL_P+NPOINTS_DATA_VEL_S
	      LINVECT(IE)=256*23 !  Solid and penwidth=23
	      LINVECT(IE+1)=256*23
	      LINTYPE(IE)=997 !  Errorbar
	      LINTYPE(IE+1)=997
	      NSYMBOL(IE)=100
	      NSYMBOL(IE+1)=100
	      COLOR(IE)=7982541 !  DarkSlateGray3
	      COLOR(IE+1)=7982541
	    ELSE
	      NSYMBOL(4)=3+256*256*7 !  Symbol #3 and symbol height=7/100 inch
	    END IF
C  Load the actual data points
	    DO I=1,NPOINTS_DATA_VEL_P
	      II=I+NPOINTS_VIS
	      XPLOT(I,2)=TIME_SAVE(II)
	      DATPLOT(I,2)=DATA_SAVE(II,5)
	      IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
	    DO I=1,NPOINTS_DATA_VEL_S
	      II=I+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	      XPLOT(I,4)=TIME_SAVE(II)
	      DATPLOT(I,4)=DATA_SAVE(II,6)
	      IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(I,4)=OMIT
	    END DO
	    IF (PLOTE) THEN !  (Primary and secondary errorbars)
	      DO I=1,NPOINTS_DATA_VEL_P
	        II=I+NPOINTS_VIS
	        XPLOT(I,IE)=XPLOT(I,2)
	        DATPLOT(I,IE)=DATPLOT(I,2)
	        IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	        XPLOT(I,IE+1)=XPLOT(I,2)
	        DATPLOT(I,IE+1)=SIGMA(II,5)
	        IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	      END DO
	      DO I=1,NPOINTS_DATA_VEL_S
	        II=I+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	        III=I+NPOINTS_DATA_VEL_P
	        XPLOT(III,IE)=XPLOT(I,4)
	        DATPLOT(III,IE)=DATPLOT(I,4)
	        IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(III,IE)=OMIT
	        XPLOT(III,IE+1)=XPLOT(I,4)
	        DATPLOT(III,IE+1)=SIGMA(II,6)
	        IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(III,IE+1)=OMIT
	      END DO
	    END IF
C  Load the data for the curve
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(19:29)=CHOICE(19:29) !  Apsidal motion
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=TIME(I,1)
	      DATPLOT(I,1)=DATA(I,5)
	    END DO
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,3)=TIME(I,1)
	      DATPLOT(I,3)=DATA(I,6)
	    END DO
	  END IF
C
C  Reset the control locations in the plotting arrays
	  DO J=0,3
	    XPLOT(MAXNPOINTS-J,1)=0.0
	    DATPLOT(MAXNPOINTS-J,1)=0.0
	  END DO
	  CALL MESSAGE('Spectroscopic data over all time')
	  CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1 NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2 INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	  CALL RESETTEK
C
C  If spectroscopic reflex subcomponents exist, subtract them from both the
C  data and the curve and plot again for the entire time span of observations.
C
C  One star in spectroscopic binary with subcomponent(s)
	  IF ((SPECTROSCOPIC_PRI_SUB.AND.CHOICE(4:4).EQ.'P').OR.
	1 (SPECTROSCOPIC_SEC_SUB.AND.CHOICE(4:4).EQ.'S')) THEN
C  The plotting parameters from the previous plot are valid here
C  Evaluate the sum of all subcomponents
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(19:29)=CHOICE(19:29)
C  One of the following statements will be ignored in NLFUN3 since either
C  the primary or the secondary will be used.
	    CHOICE_TEMP(6:6)='a' !  Evaluates the sum of all subcomponents
	    CHOICE_TEMP(8:8)='a'
	    ITIME(1)=1+NPOINTS_VIS
	    ITIME(2)=1+NPOINTS_VIS+NPOINTS_SP
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
C  Load the actual data points minus the calculated subcomponents
	    IF (CHOICE(4:4).EQ.'P') THEN
	      IDD=5
	    ELSE IF (CHOICE(4:4).EQ.'S') THEN
	      IDD=6
	    END IF
	    DO I=1,NPOINTS_SP
	      II=I+NPOINTS_VIS
	      XPLOT(I,2)=TIME_SAVE(II)
	      DATPLOT(I,2)=DATA_SAVE(II,IDD)-DATA(II,IDD)
	      IF (WEIGHT(II,IDD).EQ.0.0D0) DATPLOT(I,2)=OMIT
C  Save the output from NLFUN for use below where we plot one cycle
	      DATA_TEMP(II,IDD-4)=DATA(II,IDD)
	    END DO
	    IF (PLOTE) THEN !  (Primary or secondary minus subcomponents)
	      DO I=1,NPOINTS_SP
	        II=I+NPOINTS_VIS
	        XPLOT(I,IE)=XPLOT(I,2)
	        DATPLOT(I,IE)=DATPLOT(I,2)
	        IF (WEIGHT(II,IDD).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	        XPLOT(I,IE+1)=XPLOT(I,2)
	        DATPLOT(I,IE+1)=SIGMA(II,IDD)
	        IF (WEIGHT(II,IDD).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	      END DO
	    END IF
C  Load the data for the curve
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(6:6)='m'
	    CHOICE_TEMP(8:8)='m'
	    CHOICE_TEMP(19:19)=CHOICE(19:19)
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_CURVE
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=TIME(I,1)
	      DATPLOT(I,1)=DATA(I,IDD)
	    END DO
C  Reset the control locations in the plotting arrays
	    DO J=0,3
	      XPLOT(MAXNPOINTS-J,1)=0.0
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
	    CALL MESSAGE('Spectroscopic data over all time without'//
	1   ' subcomponent reflex velocities')
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	    CALL RESETTEK
C  Pair of spectroscopic binary stars with at least 1 subcomponent
	  ELSE IF ((SPECTROSCOPIC_PRI_SUB.OR.SPECTROSCOPIC_SEC_SUB).AND.
	1 CHOICE(4:4).EQ.'B') THEN
C  Evaluate the sum of the subcomponents alone
C  The plotting parameters from the previous plot are valid here
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(19:29)=CHOICE(19:29)
	    CHOICE_TEMP(6:6)='a' !  Evaluates the sum of all subcomponents
	    CHOICE_TEMP(8:8)='a'
	    ITIME(1)=1+NPOINTS_VIS
	    ITIME(2)=1+NPOINTS_VIS+NPOINTS_DATA_VEL_P+NPOINTS_DATA_VEL_S
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
C  Load the actual data points minus the subcomponents
	    DO I=1,NPOINTS_DATA_VEL_P
	      II=I+NPOINTS_VIS
	      XPLOT(I,2)=TIME_SAVE(II)
	      IF (SPECTROSCOPIC_PRI_SUB) THEN
	        DATPLOT(I,2)=DATA_SAVE(II,5)-DATA(II,5)
	      ELSE
C  If there is no subcomponent, DATA will have a nonzero value and so must
C  not be subtracted
	        DATPLOT(I,2)=DATA_SAVE(II,5)
	      END IF
	      IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,2)=OMIT
C  Save the output from NLFUN for possible use below where we plot one cycle
	      DATA_TEMP(II,1)=DATA(II,5)
	    END DO
	    DO I=1,NPOINTS_DATA_VEL_S
	      II=I+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	      XPLOT(I,4)=TIME_SAVE(II)
	      IF (SPECTROSCOPIC_SEC_SUB) THEN
	        DATPLOT(I,4)=DATA_SAVE(II,6)-DATA(II,6)
	      ELSE
	        DATPLOT(I,4)=DATA_SAVE(II,6)
	      END IF
	      IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(I,4)=OMIT
	      DATA_TEMP(II,2)=DATA(II,6)
	    END DO
	    IF (PLOTE) THEN
	      DO I=1,NPOINTS_DATA_VEL_P
	        II=I+NPOINTS_VIS
	        XPLOT(I,IE)=XPLOT(I,2)
	        DATPLOT(I,IE)=DATPLOT(I,2)
	        IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	        XPLOT(I,IE+1)=XPLOT(I,2)
	        DATPLOT(I,IE+1)=SIGMA(II,5)
	        IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	      END DO
	      DO I=1,NPOINTS_DATA_VEL_S
	        II=I+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	        III=I+NPOINTS_DATA_VEL_P
	        XPLOT(III,IE)=XPLOT(I,4)
	        DATPLOT(III,IE)=DATPLOT(I,4)
	        IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(III,IE)=OMIT
	        XPLOT(III,IE+1)=XPLOT(I,4)
	        DATPLOT(III,IE+1)=SIGMA(II,6)
	        IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(III,IE+1)=OMIT
	      END DO
	    END IF
C  Load the data for the curve
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(6:6)='m'
	    CHOICE_TEMP(8:8)='m'
	    CHOICE_TEMP(19:19)=CHOICE(19:19)
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_CURVE
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=TIME(I,1)
	      XPLOT(I,3)=TIME(I,1)
	      DATPLOT(I,1)=DATA(I,5)
	      DATPLOT(I,3)=DATA(I,6)
	    END DO
C  Reset the control locations in the plotting arrays
	    DO J=0,3
	      XPLOT(MAXNPOINTS-J,1)=0.0
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
	    CALL MESSAGE('Spectroscopic data over all time without'//
	1   ' subcomponent reflex velocities')
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	    CALL RESETTEK
	  END IF
C
C  Now plot one cycle of the spectroscopic data, being sure to subtract
C  any subcomponents.  Make the plot begin at the time of
C  periastron passage, T.  Also, make the abscissa be units of phase.
C
C  We can use a smaller number of points for the curve here because
C  only one cycle is plotted.
	  NPOINTS_CURVE=3000
C  Let XYPLOT and SCALE know about the change in the number of points.
	  NPLPOINTS(1)=NPOINTS_CURVE
	  DO I=1,NPOINTS_CURVE
	    TEMP1=1.0D0*(I-1)/(NPOINTS_CURVE-1) !  Avoids overflow for
			!  very large periods
	    TIME(I,1)=PARAMETRS(2)+TEMP1*PARAMETRS(1)
	  END DO
C  One spectroscopic binary - one cycle
	  IF ((CHOICE(4:4).EQ.'P'.OR.CHOICE(4:4).EQ.'S').AND..NOT.
	1 IGNORE_MAIN_PAIR_BIN) THEN
	    IF (SPECTROSCOPIC_PRI_SUB.OR.SPECTROSCOPIC_SEC_SUB) THEN
	      CALL MESSAGE('Spectroscopic data over one cycle without'//
	1     ' subcomponent reflex velocities')
	    ELSE
	      CALL MESSAGE('Spectroscopic data over one cycle')
	    END IF
C  If apsidal motion exists, we must force w to be constant in the plot
	    IF (CHOICE(19:19).EQ.'A') THEN
	      CALL MESSAGE('Note:  the angle from node to periastron '//
	1     'is assumed to be constant')
	      CHOICE_TEMP=' '
	      CHOICE_TEMP(1:8)=CHOICE(1:8)
	      CHOICE_TEMP(6:6)='m' !  Evaluates only the main component
	      CHOICE_TEMP(8:8)='m'
	      ITIME(1)=1+NPOINTS_VIS
	      ITIME(2)=1+NPOINTS_VIS+NPOINTS_SP
	      CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	      DO I=1,MAXNUMPARAM
	        INDX_BIN(I)=INDX(I)
	      END DO
	      CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1     CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    END IF
C  The plotting parameters from the previous plot are valid here
C  Load the actual data points
	    DO I=1,NPOINTS_SP
	      II=I+NPOINTS_VIS
	      TEMP1=TIME_SAVE(II)-PARAMETRS(2)
	      DO WHILE(TEMP1.LT.0.0D0)
	        TEMP1=TEMP1+PARAMETRS(1)
	      END DO
	      XPLOT(I,2)=MOD(TEMP1,PARAMETRS(1))/PARAMETRS(1)
	      IF (CHOICE(19:19).EQ.'A') THEN !  Apsidal motion for main pair
	        DATPLOT(I,2)=RESIDUALS(II,IDD)+DATA(II,IDD)
	      ELSE
C  If there are subcomponents, subtract them from the main component.  
C  We can use DATA_TEMP from the previous plot.
C  IDD is still valid from above
	        IF (SPECTROSCOPIC_PRI_SUB.OR.SPECTROSCOPIC_SEC_SUB) THEN  
	          DATPLOT(I,2)=DATA_SAVE(II,IDD)-DATA_TEMP(II,IDD-4)
	        ELSE
	          DATPLOT(I,2)=DATA_SAVE(II,IDD)
	        END IF
	      END IF
	      IF (WEIGHT(II,IDD).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
	    IF (PLOTE) THEN !  (Primary or secondary)
	      DO I=1,NPOINTS_SP
	        II=I+NPOINTS_VIS
	        XPLOT(I,IE)=XPLOT(I,2)
	        DATPLOT(I,IE)=DATPLOT(I,2)
	        IF (WEIGHT(II,IDD).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	        XPLOT(I,IE+1)=XPLOT(I,2)
	        DATPLOT(I,IE+1)=SIGMA(II,IDD)
	        IF (WEIGHT(II,IDD).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	      END DO
	    END IF
C  Load the data for the curve
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(6:6)='m'
	    CHOICE_TEMP(8:8)='m'
C  Do not include apsidal motion, since in that situation the orbit
C  does not repeat, and cannot be folded into one cycle.
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_CURVE
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=(TIME(I,1)-PARAMETRS(2))/PARAMETRS(1)
	      DATPLOT(I,1)=DATA(I,IDD)
	    END DO
C  Pair of spectroscopic binary stars - one cycle
	  ELSE IF (CHOICE(4:4).EQ.'B') THEN
C  The plotting parameters from the previous plot are valid here
C  Load the actual data points
	    IF (SPECTROSCOPIC_PRI_SUB.OR.SPECTROSCOPIC_SEC_SUB) THEN
	      CALL MESSAGE('Spectroscopic data over one cycle without'//
	1     ' subcomponent reflex velocities')
	    ELSE
	      CALL MESSAGE('Spectroscopic data over one cycle')
	    END IF
C  If apsidal motion exists, we must force w to be constant in the plot
	    IF (CHOICE(19:19).EQ.'A') THEN
	      CALL MESSAGE('Note:  the angle from node to periastron '//
	1     'is assumed to be constant')
	      CHOICE_TEMP=' '
	      CHOICE_TEMP(1:8)=CHOICE(1:8)
	      CHOICE_TEMP(6:6)='m'
	      CHOICE_TEMP(8:8)='m'
	      ITIME(1)=1+NPOINTS_VIS
	      ITIME(2)=1+NPOINTS_VIS+NPOINTS_SP
	      CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	      DO I=1,MAXNUMPARAM
	        INDX_BIN(I)=INDX(I)
	      END DO
	      CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1     CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    END IF
C  The plotting parameters from the previous plot are valid here
C  Load the data points for the primary
	    DO I=1,NPOINTS_DATA_VEL_P
	      II=I+NPOINTS_VIS
	      TEMP1=TIME_SAVE(II)-PARAMETRS(2)
	      DO WHILE(TEMP1.LT.0.0D0)
	        TEMP1=TEMP1+PARAMETRS(1)
	      END DO
	      XPLOT(I,2)=MOD(TEMP1,PARAMETRS(1))/PARAMETRS(1)
	      IF (CHOICE(19:19).EQ.'A') THEN
	        DATPLOT(I,2)=RESIDUALS(II,5)+DATA(II,5)
	      ELSE
C  If there are subcomponents, subtract them from the main component.  
C  We can use DATA_TEMP from the previous plot.
	        IF (SPECTROSCOPIC_PRI_SUB) THEN  
	          DATPLOT(I,2)=DATA_SAVE(II,5)-DATA_TEMP(II,1)
	        ELSE
	          DATPLOT(I,2)=DATA_SAVE(II,5)
	        END IF
	      END IF
	      IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
C  Load the data points for the secondary
	    DO I=1,NPOINTS_DATA_VEL_S
	      II=I+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	      TEMP1=TIME_SAVE(II)-PARAMETRS(2)
	      DO WHILE(TEMP1.LT.0.0)
	        TEMP1=TEMP1+PARAMETRS(1)
	      END DO
	      XPLOT(I,4)=MOD(TEMP1,PARAMETRS(1))/PARAMETRS(1)
	      IF (CHOICE(19:19).EQ.'A') THEN
	        DATPLOT(I,2)=RESIDUALS(II,6)+DATA(II,6)
	      ELSE
	        IF (SPECTROSCOPIC_SEC_SUB) THEN  
	          DATPLOT(I,4)=DATA_SAVE(II,6)-DATA_TEMP(II,2)
	        ELSE
	          DATPLOT(I,4)=DATA_SAVE(II,6)
	        END IF
	      END IF
	      IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(I,4)=OMIT
	    END DO
	    IF (PLOTE) THEN
	      DO I=1,NPOINTS_DATA_VEL_P
	        II=I+NPOINTS_VIS
	        XPLOT(I,IE)=XPLOT(I,2)
	        DATPLOT(I,IE)=DATPLOT(I,2)
	        IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,IE)=OMIT
	        XPLOT(I,IE+1)=XPLOT(I,2)
	        DATPLOT(I,IE+1)=SIGMA(II,5)
	        IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,IE+1)=OMIT
	      END DO
	      DO I=1,NPOINTS_DATA_VEL_S
	        II=I+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	        III=I+NPOINTS_DATA_VEL_P
	        XPLOT(III,IE)=XPLOT(I,4)
	        DATPLOT(III,IE)=DATPLOT(I,4)
	        IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(III,IE)=OMIT
	        XPLOT(III,IE+1)=XPLOT(I,4)
	        DATPLOT(III,IE+1)=SIGMA(II,6)
	        IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(III,IE+1)=OMIT
	      END DO
	    END IF
C  Load the data for the curve
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(6:6)='m'
	    CHOICE_TEMP(8:8)='m'
C  Do not include apsidal motion, since in that situation the orbit
C  does not repeat, and cannot be folded into one cycle.
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=(TIME(I,1)-PARAMETRS(2))/PARAMETRS(1)
	      DATPLOT(I,1)=DATA(I,5)
	    END DO
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,3)=XPLOT(I,1)
	      DATPLOT(I,3)=DATA(I,6)
	    END DO
	  END IF
	  XLABEL='Phase'
	  IF (.NOT.IGNORE_MAIN_PAIR_BIN) THEN
C  Reset the control locations in the plotting arrays
	    DO J=0,3
	      XPLOT(MAXNPOINTS-J,1)=0.0
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
C  Predetermine the scale for the abscissa
	    XPLOT(MAXNPOINTS-3,1)=6.0
	    XPLOT(MAXNPOINTS-2,1)=10.0
	    XPLOT(MAXNPOINTS-1,1)=0.0
	    XPLOT(MAXNPOINTS,1)=0.1
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	    CALL RESETTEK
	  END IF
	END IF
C
C  Plot one cycle for each primary subcomponent's reflex velocities if they
C  exist.
C
	IF (PLOTD.AND.SPECTROSCOPIC_PRI_SUB) THEN  
	  DO IP=1,NPRISUB
	    NPLOTS=2
C  Calculate the adjusted data points
	    ENCODE(1,1001,C_I) IP
	    CALL MESSAGE('Spectroscopic reflex velocity data over one'//
	1   ' cycle for primary')
	    CALL MESSAGE('subcomponent #'//C_I)
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:4)=CHOICE(1:4)
	    CHOICE_TEMP(5:5)=C_I
	    CHOICE_TEMP(6:6)='o' !  Evaluates only subcomponent IP
C  Do not include apsidal motion, since in that situation the orbit
C  does not repeat, and cannot be folded into one cycle.
	    IF (CHOICE(19+IP:19+IP).EQ.'A') THEN
	      CALL MESSAGE('Note:  the angle from node to periastron '//
	1     'is assumed to be constant')
	    END IF
	    ITIME(1)=1+NPOINTS_VIS
	    ITIME(2)=1+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_DATA_VEL_P
	      II=I+NPOINTS_VIS
	      TEMP1=TIME_SAVE(II)-PARAMETRS(INDX(19)+(IP-1)*5)
	      DO WHILE(TEMP1.LT.0.0)
	        TEMP1=TEMP1+PARAMETRS(INDX(18)+(IP-1)*5)
	      END DO
	      XPLOT(I,2)=MOD(TEMP1,PARAMETRS(INDX(18)+(IP-1)*5))/
	1     PARAMETRS(INDX(18)+(IP-1)*5)
	      DATPLOT(I,2)=RESIDUALS(II,5)+DATA(II,5)
	      IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
	    IF (PLOTE) THEN !  (Primary subcomponent errorbars)
	      NPLOTS=NPLOTS+2
	      NPLPOINTS(3)=NPOINTS_DATA_VEL_P
	      NPLPOINTS(4)=NPOINTS_DATA_VEL_P
	      LINVECT(3)=256*23
	      LINVECT(4)=256*23
	      LINTYPE(3)=997 !  Errorbar
	      LINTYPE(4)=997
	      NSYMBOL(3)=100
	      NSYMBOL(4)=100
	      COLOR(3)=7982541 !  DarkSlateGray3
	      COLOR(4)=7982541 !  DarkSlateGray3
	      DO I=1,NPOINTS_DATA_VEL_P
	        II=I+NPOINTS_VIS
	        XPLOT(I,3)=XPLOT(I,2)
	        DATPLOT(I,3)=DATPLOT(I,2)
	        IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,3)=OMIT
	        XPLOT(I,4)=XPLOT(I,2)
	        DATPLOT(I,4)=SIGMA(II,5)
	        IF (WEIGHT(II,5).EQ.0.0D0) DATPLOT(I,4)=OMIT
	      END DO
	    END IF
C  Now calculate the points for the curve
C  Set up the time appropriate for the subcomponent
	    DO I=1,NPOINTS_CURVE
	      TIME(I,1)=PARAMETRS(INDX(19)+(IP-1)*5)+(I-1)*
	1     PARAMETRS(INDX(18)+(IP-1)*5)/(NPOINTS_CURVE-1)
	    END DO
C  Load the data for the curve
C  Use CHOICE_TEMP from above
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_CURVE
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=(TIME(I,1)-PARAMETRS(INDX(19)+(IP-1)*5))/
	1     PARAMETRS(INDX(18)+(IP-1)*5)
	      DATPLOT(I,1)=DATA(I,5)
	    END DO
C  Reset the control locations in the plotting arrays
	    DO J=0,3
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
C  Predetermine the scale for the abscissa
	    XPLOT(MAXNPOINTS-3,1)=6.0
	    XPLOT(MAXNPOINTS-2,1)=10.0
	    XPLOT(MAXNPOINTS-1,1)=0.0
	    XPLOT(MAXNPOINTS,1)=0.1
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	    CALL RESETTEK
	  END DO
	END IF
C
C  Plot one cycle for each secondary subcomponent's reflex velocities
C  if they exist
C
	IF (PLOTD.AND.SPECTROSCOPIC_SEC_SUB) THEN !  Secondary subcomponent(s)
	  DO IP=1,NSECSUB
	    NPLOTS=2
C  Calculate the adjusted data points
	    ENCODE(1,1001,C_I) IP
	    CALL MESSAGE('Spectroscopic reflex velocity data over one'//
	1   ' cycle for secondary subcomponent #'//C_I)
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:6)=CHOICE(1:6)
	    CHOICE_TEMP(7:7)=C_I
	    CHOICE_TEMP(8:8)='o'
C  Do not include apsidal motion, since in that situation the orbit
C  does not repeat, and cannot be folded into one cycle.
	    IF (CHOICE(24:24).EQ.'A') THEN
	      CALL MESSAGE('Note:  the angle from node to periastron '//
	1     'is assumed to be constant')
	    END IF
	    ITIME(1)=1+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	    ITIME(2)=1+NPOINTS_VIS+NPOINTS_DATA_VEL_P+NPOINTS_DATA_VEL_S
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_DATA_VEL_S
	      II=I+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	      TEMP1=TIME_SAVE(II)-PARAMETRS(INDX(44)+(IP-1)*5)
	      DO WHILE(TEMP1.LT.0.0)
	        TEMP1=TEMP1+PARAMETRS(INDX(43)+(IP-1)*5)
	      END DO
	      XPLOT(I,2)=MOD(TEMP1,PARAMETRS(INDX(43)+(IP-1)*5))/
	1     PARAMETRS(INDX(43)+(IP-1)*5)
	      DATPLOT(I,2)=RESIDUALS(II,6)+DATA(II,6)
	      IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
	    IF (PLOTE) THEN !  (Secondary subcomponent errorbars)
	      NPLOTS=NPLOTS+2
	      NPLPOINTS(3)=NPOINTS_DATA_VEL_S
	      NPLPOINTS(4)=NPOINTS_DATA_VEL_S
	      LINVECT(3)=256*23
	      LINVECT(4)=256*23
	      LINTYPE(3)=997 !  Errorbar
	      LINTYPE(4)=997
	      NSYMBOL(3)=100
	      NSYMBOL(4)=100
	      COLOR(3)=7982541 !  DarkSlateGray3
	      COLOR(4)=7982541 !  DarkSlateGray3
	      DO I=1,NPOINTS_DATA_VEL_S
	        II=I+NPOINTS_VIS+NPOINTS_DATA_VEL_P
	        XPLOT(I,3)=XPLOT(I,2)
	        DATPLOT(I,3)=DATPLOT(I,2)
	        IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(I,3)=OMIT
	        XPLOT(I,4)=XPLOT(I,2)
	        DATPLOT(I,4)=SIGMA(II,6)
	        IF (WEIGHT(II,6).EQ.0.0D0) DATPLOT(I,4)=OMIT
	      END DO
	    END IF
C  Now calculate the points for the curve
C  Set up the time appropriate for the subcomponent
	    DO I=1,NPOINTS_CURVE
	      TIME(I,1)=PARAMETRS(INDX(44)+(IP-1)*5)+(I-1)*
	1     PARAMETRS(INDX(43)+(IP-1)*5)/(NPOINTS_CURVE-1)
	    END DO
C  Use CHOICE_TEMP from above
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_CURVE
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
C  Load the data for the curve
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=(TIME(I,1)-PARAMETRS(INDX(44)+(IP-1)*5))/
	1     PARAMETRS(INDX(43)+(IP-1)*5)
	      DATPLOT(I,1)=DATA(I,6)
	    END DO
C  Reset the Y control locations in the plotting arrays.
	    DO J=0,3
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
C  Predetermine the scale for the abscissa
	    XPLOT(MAXNPOINTS-3,1)=6.0
	    XPLOT(MAXNPOINTS-2,1)=10.0
	    XPLOT(MAXNPOINTS-1,1)=0.0
	    XPLOT(MAXNPOINTS,1)=0.1
	    NPLPOINTS(2)=NPOINTS_DATA_VEL_S
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	    CALL RESETTEK
	  END DO
	END IF
C
C  Now plot any spectroscopic primary subcomponent secondaries if the
C  files exist.  First plot for the entire time span.
C
C  Increase the number of points for the curve.
	NPOINTS_CURVE=8000
	NPLPOINTS(1)=NPOINTS_CURVE
	IF (PLOTD) THEN
	  DO IP=1,NPRISUB_S_F
	    NPLOTS=2
	    ITMP=IPRISUB_S_F(IP)
	    ENCODE(1,1001,C_I) ITMP
	    CALL MESSAGE('Spectroscopic data over all time for primary'//
	1   ' subcomponent #'//C_I)
C  Calculate the adjusted data points
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(8+ITMP:8+ITMP)='o'
	    CHOICE_TEMP(19:29)=CHOICE(19:29) !  Apsidal motion
	    IF (IP.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+1
	    NHI=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)-1
	    ITIME(1)=NLO
	    ITIME(2)=NHI
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	      II=I+NLO-1
	      XPLOT(I,2)=TIME_SAVE(II)
	      DATPLOT(I,2)=RESIDUALS(II,6+ITMP)+DATA(II,6+ITMP)
	      IF (WEIGHT(II,6+ITMP).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
	    IF (PLOTE) THEN !  (Primary subcomponent secondary errorbars)
	      NPLOTS=NPLOTS+2
	      NPLPOINTS(3)=NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	      NPLPOINTS(4)=NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	      LINVECT(3)=256*23
	      LINVECT(4)=256*23
	      LINTYPE(3)=997 !  Errorbar
	      LINTYPE(4)=997
	      NSYMBOL(3)=100
	      NSYMBOL(4)=100
	      COLOR(3)=7982541 !  DarkSlateGray3
	      COLOR(4)=7982541 !  DarkSlateGray3
	      DO I=1,NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	        II=I+NLO-1
	        XPLOT(I,3)=XPLOT(I,2)
	        DATPLOT(I,3)=DATPLOT(I,2)
	        IF (WEIGHT(II,6+ITMP).EQ.0.0D0) DATPLOT(I,3)=OMIT
	        XPLOT(I,4)=XPLOT(I,2)
	        DATPLOT(I,4)=SIGMA(II,6+ITMP)
	        IF (WEIGHT(II,6+ITMP).EQ.0.0D0) DATPLOT(I,4)=OMIT
	      END DO
	    END IF
C  Calculate the points for the curve.  Use CHOICE_TEMP from above.
	    TIME_BEGIN=TIME_SAVE(NLO)
	    TIME_END=TIME_SAVE(NHI)
	    DO I=NLO,NHI
	      IF (TIME_SAVE(I).GT.TIME_END) TIME_END=TIME_SAVE(I)
	      IF (TIME_SAVE(I).LT.TIME_BEGIN) TIME_BEGIN=TIME_SAVE(I)
	    END DO
	    DO I=1,NPOINTS_CURVE
	      TIME(I,1)=TIME_BEGIN+(I-1)*(TIME_END-TIME_BEGIN)/
	1     (NPOINTS_CURVE-1)
	    END DO
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_CURVE
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=TIME(I,1)
	      DATPLOT(I,1)=DATA(I,6+ITMP)
	    END DO
C  Reset the control locations in the plotting arrays
	    DO J=0,3
	      XPLOT(MAXNPOINTS-J,1)=0.0
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	    CALL RESETTEK
	    NLO=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	  END DO
C
C  Now plot any spectroscopic primary subcomponent secondaries for
C  one cycle.
C
C  Decrease the number of points for the curve
	  NPOINTS_CURVE=3000
	  NPLPOINTS(1)=NPOINTS_CURVE
	  DO IP=1,NPRISUB_S_F
	    NPLOTS=2
	    ITMP=IPRISUB_S_F(IP)
	    ENCODE(1,1001,C_I) ITMP
	    CALL MESSAGE('Spectroscopic data for one cycle for primary'//
	1   ' subcomponent #'//C_I)
C  Calculate the adjusted data points
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:8)=CHOICE(1:8)
	    CHOICE_TEMP(8+ITMP:8+ITMP)='o'
C  Do not include apsidal motion, since in that situation the orbit
C  does not repeat, and cannot be folded into one cycle.
	    IF (CHOICE(19+ITMP:19+ITMP).EQ.'A') THEN
	      CALL MESSAGE('Note:  the angle from node to periastron '//
	1     'is assumed to be constant')
	    END IF
	    IF (IP.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+1
	    NHI=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)-1
	    ITIME(1)=NLO
	    ITIME(2)=NHI
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	      II=I+NLO-1
	      TEMP1=TIME_SAVE(II)-PARAMETRS(INDX(19)+(ITMP-1)*5)
	      DO WHILE(TEMP1.LT.0.0)
	        TEMP1=TEMP1+PARAMETRS(INDX(18)+(ITMP-1)*5)
	      END DO
	      XPLOT(I,2)=MOD(TEMP1,PARAMETRS(INDX(18)+(ITMP-1)*5))/
	1     PARAMETRS(INDX(18)+(ITMP-1)*5)
	      DATPLOT(I,2)=RESIDUALS(II,6+ITMP)+DATA(II,6+ITMP)
	      IF (WEIGHT(II,6+ITMP).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
	    IF (PLOTE) THEN !  (Primary subcomponent secondary errorbars)
	      NPLOTS=NPLOTS+2
	      NPLPOINTS(3)=NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	      NPLPOINTS(4)=NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	      LINVECT(3)=256*23
	      LINVECT(4)=256*23
	      LINTYPE(3)=997 !  Errorbar
	      LINTYPE(4)=997
	      NSYMBOL(3)=100
	      NSYMBOL(4)=100
	      COLOR(3)=7982541 !  DarkSlateGray3
	      COLOR(4)=7982541 !  DarkSlateGray3
	      DO I=1,NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	        II=I+NLO-1
	        XPLOT(I,3)=XPLOT(I,2)
	        DATPLOT(I,3)=DATPLOT(I,2)
	        IF (WEIGHT(II,6+ITMP).EQ.0.0D0) DATPLOT(I,3)=OMIT
	        XPLOT(I,4)=XPLOT(I,2)
	        DATPLOT(I,4)=SIGMA(II,6+ITMP)
	        IF (WEIGHT(II,6+ITMP).EQ.0.0D0) DATPLOT(I,4)=OMIT
	      END DO
	    END IF
C  Calculate the points for the curve
C  Set up the time appropriate for the subcomponent
	    DO I=1,NPOINTS_CURVE
	      TIME(I,1)=PARAMETRS(INDX(19)+(ITMP-1)*5)+(I-1)*
	1     PARAMETRS(INDX(18)+(ITMP-1)*5)/(NPOINTS_CURVE-1)
	    END DO
C  Load the data for the curve.  Use CHOICE_TEMP from above
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_CURVE
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=(TIME(I,1)-PARAMETRS(INDX(19)+(ITMP-1)*5))/
	1     PARAMETRS(INDX(18)+(ITMP-1)*5)
	      DATPLOT(I,1)=DATA(I,6+ITMP)
	    END DO
C  Reset the control locations in the plotting arrays
	    DO J=0,3
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
C  Predetermine the scale for the abscissa
	    XPLOT(MAXNPOINTS-3,1)=6.0
	    XPLOT(MAXNPOINTS-2,1)=10.0
	    XPLOT(MAXNPOINTS-1,1)=0.0
	    XPLOT(MAXNPOINTS,1)=0.1
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	    CALL RESETTEK
	    NLO=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	  END DO
C
C  Now plot any spectroscopic secondary subcomponent secondaries.  First
C  plot for the entire time span.
C
C  Increase the number of points for the curve
	  NPOINTS_CURVE=8000
	  NPLPOINTS(1)=NPOINTS_CURVE
	  DO IS=1,NSECSUB_S_F
	    NPLOTS=2
	    ITMP=ISECSUB_S_F(IS)
	    ENCODE(1,1001,C_I) ITMP
	    CALL MESSAGE('Spectroscopic data over all time for secondary'//
	1   ' subcomponent #'//C_I)
C  Calculate the adjusted data points
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:13)=CHOICE(1:13)
	    CHOICE_TEMP(13+ITMP:13+ITMP)='o'
	    CHOICE_TEMP(19:29)=CHOICE(19:29)
	    IF (IS.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+NPOINTS_VEL_P_S+1
	    NHI=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)-1
	    ITIME(1)=NLO
	    ITIME(2)=NHI
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_DATA_VEL_S_SUB_S(ITMP)
	      II=I+NLO-1
	      XPLOT(I,2)=TIME_SAVE(II)
	      DATPLOT(I,2)=RESIDUALS(II,11+ITMP)+DATA(II,11+ITMP)
	      IF (WEIGHT(II,11+ITMP).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
            IF (PLOTE) THEN !  (Secondary subcomponent secondary errorbars)
              NPLOTS=NPLOTS+2
              NPLPOINTS(3)=NPOINTS_DATA_VEL_S_SUB_S(ITMP)
              NPLPOINTS(4)=NPOINTS_DATA_VEL_S_SUB_S(ITMP)
              LINVECT(3)=256*23
              LINVECT(4)=256*23
              LINTYPE(3)=997 !  Errorbar
              LINTYPE(4)=997
              NSYMBOL(3)=100
              NSYMBOL(4)=100
              COLOR(3)=7982541 !  DarkSlateGray3
              COLOR(4)=7982541 !  DarkSlateGray3
              DO I=1,NPOINTS_DATA_VEL_S_SUB_S(ITMP)
                II=I+NLO-1
                XPLOT(I,3)=XPLOT(I,2)
                DATPLOT(I,3)=DATPLOT(I,2)
                IF (WEIGHT(II,11+ITMP).EQ.0.0D0) DATPLOT(I,3)=OMIT
                XPLOT(I,4)=XPLOT(I,2)
                DATPLOT(I,4)=SIGMA(II,11+ITMP)
                IF (WEIGHT(II,11+ITMP).EQ.0.0D0) DATPLOT(I,4)=OMIT
              END DO
            END IF
C  Calculate the points for the curve.  Use CHOICE_TEMP from above.
	    TIME_BEGIN=TIME_SAVE(NLO)
	    TIME_END=TIME_SAVE(NHI)
	    DO I=NLO,NHI
	      IF (TIME_SAVE(I).GT.TIME_END) TIME_END=TIME_SAVE(I)
	      IF (TIME_SAVE(I).LT.TIME_BEGIN) TIME_BEGIN=TIME_SAVE(I)
	    END DO
	    DO I=1,NPOINTS_CURVE
	      TIME(I,1)=TIME_BEGIN+(I-1)*(TIME_END-TIME_BEGIN)/
	1     (NPOINTS_CURVE-1)
	    END DO
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_CURVE
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=TIME(I,1)
	      DATPLOT(I,1)=DATA(I,11+ITMP)
	    END DO
C  Reset the control locations in the plotting arrays
	    DO J=0,3
	      XPLOT(MAXNPOINTS-J,1)=0.0
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	    CALL RESETTEK
	    NLO=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)
	  END DO
C
C  Now plot any spectroscopic secondary subcomponent secondaries for
C  one cycle.
C
C  Reduce the number of points
	  NPOINTS_CURVE=3000
	  NPLPOINTS(1)=NPOINTS_CURVE
	  DO IS=1,NSECSUB_S_F
	    NPLOTS=2
	    ITMP=ISECSUB_S_F(IS)
	    ENCODE(1,1001,C_I) ITMP
	    CALL MESSAGE('Spectroscopic data for one cycle for secondary'//
	1   ' subcomponent #'//C_I)
C  Calculate the adjusted data points
	    CHOICE_TEMP=' '
	    CHOICE_TEMP(1:13)=CHOICE(1:13)
	    CHOICE_TEMP(13+ITMP:13+ITMP)='o'
C  Do not include apsidal motion, since in that situation the orbit
C  does not repeat, and cannot be folded into one cycle.
	    IF (CHOICE(24+ITMP:24+ITMP).EQ.'A') THEN
	      CALL MESSAGE('Note:  the angle from node to periastron '//
	1     'is assumed to be constant')
	    END IF		
	    IF (IS.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+NPOINTS_VEL_P_S+1
	    NHI=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)-1
	    ITIME(1)=NLO
	    ITIME(2)=NHI
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME_SAVE,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_DATA_VEL_S_SUB_S(ITMP)
	      II=I+NLO-1
	      TEMP1=TIME_SAVE(II)-PARAMETRS(INDX(44)+(ITMP-1)*5)
	      DO WHILE(TEMP1.LT.0.0D0)
	        TEMP1=TEMP1+PARAMETRS(INDX(43)+(ITMP-1)*5)
	      END DO
	      XPLOT(I,2)=MOD(TEMP1,PARAMETRS(INDX(43)+(ITMP-1)*5))/
	1     PARAMETRS(INDX(43)+(ITMP-1)*5)
	      DATPLOT(I,2)=RESIDUALS(II,11+ITMP)+DATA(II,11+ITMP)
	      IF (WEIGHT(II,11+ITMP).EQ.0.0D0) DATPLOT(I,2)=OMIT
	    END DO
            IF (PLOTE) THEN !  (Secondary subcomponent secondary errorbars)
              NPLOTS=NPLOTS+2
              NPLPOINTS(3)=NPOINTS_DATA_VEL_S_SUB_S(ITMP)
              NPLPOINTS(4)=NPOINTS_DATA_VEL_S_SUB_S(ITMP)
              LINVECT(3)=256*23
              LINVECT(4)=256*23
              LINTYPE(3)=997 !  Errorbar
              LINTYPE(4)=997
              NSYMBOL(3)=100
              NSYMBOL(4)=100
              COLOR(3)=7982541 !  DarkSlateGray3
              COLOR(4)=7982541 !  DarkSlateGray3
              DO I=1,NPOINTS_DATA_VEL_S_SUB_S(ITMP)
                II=I+NLO-1
                XPLOT(I,3)=XPLOT(I,2)
                DATPLOT(I,3)=DATPLOT(I,2)
                IF (WEIGHT(II,11+ITMP).EQ.0.0D0) DATPLOT(I,3)=OMIT
                XPLOT(I,4)=XPLOT(I,2)
                DATPLOT(I,4)=SIGMA(II,11+ITMP)
                IF (WEIGHT(II,11+ITMP).EQ.0.0D0) DATPLOT(I,4)=OMIT
              END DO
            END IF
C  Calculate the points for the curve
C  Set up the time appropriate for the subcomponent
	    DO I=1,NPOINTS_CURVE
	      TIME(I,1)=PARAMETRS(INDX(44)+(ITMP-1)*5)+(I-1)*
	1     PARAMETRS(INDX(43)+(ITMP-1)*5)/(NPOINTS_CURVE-1)
	    END DO
C  Load the data for the curve.  Use CHOICE_TEMP from above
	    ITIME(1)=1
	    ITIME(2)=NPOINTS_CURVE
	    CALL BINARY_INDEXES(CHOICE_TEMP,NUMPARAM)
	    DO I=1,MAXNUMPARAM
	      INDX_BIN(I)=INDX(I)
	    END DO
	    CALL NLFUN(TIME,MAXNPOINTS,DIMENSN_IND,PARAMETRS,FUNCAT,
	1   CHOICE_TEMP,OPERATE,ITIME,DATA,DIMENSN_DATA)
	    DO I=1,NPOINTS_CURVE
	      XPLOT(I,1)=(TIME(I,1)-PARAMETRS(INDX(44)+(ITMP-1)*5))/
	1     PARAMETRS(INDX(43)+(ITMP-1)*5)
	      DATPLOT(I,1)=DATA(I,11+ITMP)
	    END DO
C  Reset the control locations in the plotting arrays
	    DO J=0,3
	      DATPLOT(MAXNPOINTS-J,1)=0.0
	    END DO
C  Predetermine the scale for the abscissa
	    XPLOT(MAXNPOINTS-3,1)=6.0
	    XPLOT(MAXNPOINTS-2,1)=10.0
	    XPLOT(MAXNPOINTS-1,1)=0.0
	    XPLOT(MAXNPOINTS,1)=0.1
	    CALL XYPLOT(XPLOT,DATPLOT,MAXNPOINTS,
	1   NPLOTS,NPLPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE_VEL,' ',.FALSE.)
	    CALL RESETTEK
	    NLO=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)
	  END DO
	END IF
C
C  Beginning of section for displaying the residuals
C
	SHOW_RESID=.FALSE.
	CALL USRINP(SHOW_RESID,1,'L',4,1,'SHOW_RESIDUALS=',
 	1'Do you wish to see the residuals?  (default = N)')
	IF (SHOW_RESID) THEN
	  IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B') THEN
	    CALL MESSAGE('For the primary:')
	    CALL MESSAGE(' ')
	    IF (FILETYPE_P.EQ.'X_Y') THEN
	      CALL MESSAGE('                            |DELTA_X|   '//
	1'              |DELTA_Y|     DELTA ')
	      CALL MESSAGE('                DELTA_X    ___________  '//
	1'  DELTA_Y    ___________    ______')
	      CALL MESSAGE('   EPOCH          (")     MEAN ERROR(X) '//
	1'    (")     MEAN ERROR(Y)    M.E. ')
	      SUM1=0.0D0
	      SUM2=0.0D0
	      DO I=1,NPOINTS_DATA_P
C  If the weight is zero, the residuals are not displayed
	        IF (WEIGHT(I,1).GT.0.0D0.AND.WEIGHT(I,2).GT.0.0D0) THEN
	          TEMP1=SQRT(RESIDUALS(I,1)**2+RESIDUALS(I,2)**2)
	          TEMP2=SQRT(MEANERROR(1)**2+MEANERROR(2)**2)
	          IF (TEMP2.GT.0.0D0) TEMP2=TEMP1/TEMP2
	          TEMP3=ABS(RESIDUALS(I,1))/MEANERROR(1) !  Relative to solution
	          TEMP4=ABS(RESIDUALS(I,2))/MEANERROR(2)
	          SUM1=SUM1+TEMP3**2
	          SUM2=SUM2+TEMP4**2
	          WRITE(6,4000) TIME_SAVE(I),RESIDUALS(I,1),
	1         TEMP3,RESIDUALS(I,2),TEMP4,TEMP2
4000	          FORMAT (' ',G13.8,1X,G12.6,1X,G12.6,1X,G12.6,1X,G12.6,1X,
	1         G12.6)
	          WRITE(7,4000) TIME_SAVE(I),RESIDUALS(I,1),
	1         TEMP3,RESIDUALS(I,2),TEMP4,TEMP2
	        END IF
	      END DO
C  No error checking here yet
	      WRITE(6,4002) SQRT(SUM1/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT))
4002	      FORMAT(' ','The rms average of (Delta X)/(Mean Error(X)) = ',
	1     G12.6)
	      WRITE(6,4003) SQRT(SUM2/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT))
4003	      FORMAT(' ','The rms average of (Delta Y)/(Mean Error(Y)) = ',
	1     G12.6)
	      WRITE (7,4002) SQRT(SUM1/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT))
	      WRITE (7,4003) SQRT(SUM2/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT))
	    ELSE IF (FILETYPE_P.EQ.'THETA_RHO') THEN
	      CALL MESSAGE('                          |DELTA_THETA| '//
	1'             |DELTA_RHO|    DELTA ')
	      CALL MESSAGE('              DELTA_THETA  ___________  '//
	1' DELTA_RHO   ___________    ______')
	      CALL MESSAGE('   EPOCH          (o)     MEAN ERROR(T) '//
	1'    (")     MEAN ERROR(R)    M.E. ')
C  We will have to convert from residuals in X and Y to those in theta
C  and rho.
	      SUM1=0.0D0
	      SUM2=0.0D0
	      SUM3=0.0D0
	      SUM4=0.0D0
C  First loop through the points to calculate the mean error for an 
C  observation of average weight in theta and rho.  Then begin another 
C  loop where the residuals are compared to those mean errors.
	      DO I=1,NPOINTS_DATA_P
C  Note that DATA_SAVE is in the form:  X and Y
	        IF (WEIGHT(I,1).GT.0.0D0.AND.WEIGHT(I,2).GT.0.0D0) THEN
	          TEMP3=SQRT(DATA_SAVE(I,1)**2+DATA_SAVE(I,2)**2)
				!  Rho observed in "
	          TEMP4=SQRT((DATA_SAVE(I,1)-RESIDUALS(I,1))**2+
	1         (DATA_SAVE(I,2)-RESIDUALS(I,2))**2) !  Rho calculated
	          TEMP4=TEMP3-TEMP4 !  Rho observerd - calculated
	          TEMP1=DATAN2D(DATA_SAVE(I,1),DATA_SAVE(I,2))
C  Change the range from -180 to +180, to 0 to 360.
	          IF (TEMP1.LT.0.0D0) TEMP1=TEMP1+360.0D0 !  Theta observed
	          TEMP5=DATAN2D(DATA_SAVE(I,1)-RESIDUALS(I,1),
	1         DATA_SAVE(I,2)-RESIDUALS(I,2))
	          IF (TEMP5.LT.0.0D0) TEMP5=TEMP5+360.0D0 !  Theta calculated
	          TEMP5=TEMP1-TEMP5
C  Make the magnitude of the difference less than 180.
	          IF (ABS(TEMP5).GT.180.0D0) THEN
	            IF (TEMP5.LT.0.0D0) THEN
	              TEMP5=TEMP5+360.0D0
	            ELSE 
	              TEMP5=TEMP5-360.0D0
	            END IF
	          END IF
C  Note that SIGMA_SAVE is scaled by any user-entered weight scaling,
C  and is here in units of theta and rho.
	          SUM1=SUM1+(TEMP5/SIGMA_SAVE(I,1))**2 !  For delta theta
	          SUM2=SUM2+(TEMP4/SIGMA_SAVE(I,2))**2 !  For delta rho
C  Get the sum of the weights for the mean error of an observation of
C  average weight calculation below.
	          SUM3=SUM3+1.0D0/SIGMA_SAVE(I,1)**2
	          SUM4=SUM4+1.0D0/SIGMA_SAVE(I,2)**2
	        END IF
	      END DO
C	      NADJUST=NUMPARAM_SOLV/DIMENSN_DATA_USED
	      NADJUST=NUMPARAM_SOLV*0.5*(NPOINTS_DATA_P-
	1     NPOINTS_DATA_P_OMIT)/NPOINTS_DATA
	      RDCS1=SUM1/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT-NADJUST)
			!  Reduced xi^2 for theta
	      RDCS2=SUM2/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT-NADJUST)
			!  Reduced xi^2 for rho
	      MEOAW1=SQRT(SUM1/((NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT-
	1     NADJUST)*SUM3/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT)))
					!  MEOAW theta
	      MEOAW2=SQRT(SUM2/((NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT-
	1     NADJUST)*SUM4/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT)))
					!  MEOAW rho
C  Now that we have the MEOAW for theta and rho, we can print out the
C  residuals
	      SUM1=0.0D0
	      SUM2=0.0D0
	      DO I=1,NPOINTS_DATA_P
	        IF (WEIGHT(I,1).GT.0.0D0.AND.WEIGHT(I,2).GT.0.0D0) THEN
	          TEMP1=SQRT(RESIDUALS(I,1)**2+RESIDUALS(I,2)**2)
	          TEMP2=SQRT(MEANERROR(1)**2+MEANERROR(2)**2)
	          IF (TEMP2.GT.0.0D0) TEMP2=TEMP1/TEMP2 !  Same as X_Y case
	          TEMP3=SQRT(DATA_SAVE(I,1)**2+DATA_SAVE(I,2)**2)
				!  Rho observed in "
	          TEMP4=SQRT((DATA_SAVE(I,1)-RESIDUALS(I,1))**2+
	1         (DATA_SAVE(I,2)-RESIDUALS(I,2))**2) !  Rho calculated
	          TEMP4=TEMP3-TEMP4 !  Rho observerd - calculated
	          TEMP1=DATAN2D(DATA_SAVE(I,1),DATA_SAVE(I,2))
C  Change the range from -180 to +180, to 0 to 360.
	          IF (TEMP1.LT.0.0D0) TEMP1=TEMP1+360.0D0 !  Theta observed
	          TEMP5=DATAN2D(DATA_SAVE(I,1)-RESIDUALS(I,1),
	1         DATA_SAVE(I,2)-RESIDUALS(I,2))
	          IF (TEMP5.LT.0.0D0) TEMP5=TEMP5+360.0D0 !  Theta calculated
	          TEMP5=TEMP1-TEMP5
C  Make the magnitude of the difference less than 180.
	          IF (ABS(TEMP5).GT.180.0D0) THEN
	            IF (TEMP5.LT.0.0D0) THEN
	              TEMP5=TEMP5+360.0D0
	            ELSE 
	              TEMP5=TEMP5-360.0D0
	            END IF
	          END IF
C Epoch, theta(obs)-theta(calc), (delta theta)/(m.e. theta), 
C rho(obs)-rho(calc), (delta rho)/(m.e. rho), delta/m.e.
	          WRITE(6,4000) TIME_SAVE(I),TEMP5,
	1         ABS(TEMP5)/MEOAW1,TEMP4,
	2         ABS(TEMP4)/MEOAW2,TEMP2
	          WRITE(7,4000) TIME_SAVE(I),TEMP5,
	1         ABS(TEMP5)/MEOAW1,TEMP4,
	2         ABS(TEMP4)/MEOAW2,TEMP2
	          SUM1=SUM1+(TEMP5/MEOAW1)**2
	          SUM2=SUM2+(TEMP4/MEOAW2)**2
	        END IF
	      END DO
	      WRITE (6,4005) RDCS1
4005	      FORMAT(' ','The reduced chi squared for theta = ',
	1     G12.6)
	      WRITE (7,4005) RDCS1
	      WRITE (6,4006) RDCS2
4006	      FORMAT(' ','The reduced chi squared for rho = ',
	1     G12.6)
	      WRITE (7,4006) RDCS2
	      WRITE (6,4007) MEOAW1
4007	      FORMAT(' ','The mean error for an observation of average',
	1     ' weight for theta = '/10X,G12.6)
	      WRITE (7,4007) MEOAW1
	      WRITE (6,4008) MEOAW2
4008	      FORMAT(' ','The mean error for an observation of average',
	1     ' weight for rho = '/10X,G12.6)
	      WRITE (7,4008) MEOAW2
C
	      WRITE (6,4011) SQRT(SUM1/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT))
4011	      FORMAT(' ','The rms average of (Delta Theta)/',
	1     '(Mean Error(Theta)) = ',G12.6)
	      WRITE (7,4011) SQRT(SUM1/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT))
	      WRITE (6,4012) SQRT(SUM2/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT))
4012	      FORMAT(' ','The rms average of (Delta Rho)/',
	1     '(Mean Error(Rho)) = ',G12.6)
	      WRITE (7,4012) SQRT(SUM2/(NPOINTS_DATA_P-NPOINTS_DATA_P_OMIT))
	    END IF
	  END IF !  Ends section for CHOICE(1:1)='P' OR 'B'
C
	  IF (CHOICE(1:1).EQ.'S'.OR.CHOICE(1:1).EQ.'B') THEN
	    CALL MESSAGE('For the secondary:')
	    CALL MESSAGE(' ')
	    IF (CHOICE(1:1).EQ.'S') THEN
	      NLO=1
	      NHI=NPOINTS_DATA_S
	    ELSE IF (CHOICE(1:1).EQ.'B') THEN
	      NLO=NPOINTS_DATA_P+1
	      NHI=NPOINTS_DATA_P+NPOINTS_DATA_S
	    END IF
	    IF (FILETYPE_S.EQ.'X_Y') THEN
	      CALL MESSAGE('                            |DELTA_X|   '//
	1'              |DELTA_Y|     DELTA ')
	      CALL MESSAGE('                DELTA_X    ___________  '//
	1'  DELTA_Y    ___________    ______')
	      CALL MESSAGE('   EPOCH          (")     MEAN ERROR(X) '//
	1'    (")     MEAN ERROR(Y)    M.E. ')
	      SUM1=0.0D0
	      SUM2=0.0D0
	      DO I=NLO,NHI
	        IF (WEIGHT(I,3).GT.0.0D0.AND.WEIGHT(I,4).GT.0.0D0) THEN
	          TEMP1=SQRT(RESIDUALS(I,3)**2+RESIDUALS(I,4)**2)
	          TEMP2=SQRT(MEANERROR(3)**2+MEANERROR(4)**2)
	          IF (TEMP2.GT.0.0D0) TEMP2=TEMP1/TEMP2
	          TEMP3=ABS(RESIDUALS(I,3))/MEANERROR(3) !  Relative to solution
	          TEMP4=ABS(RESIDUALS(I,4))/MEANERROR(4)
	          SUM1=SUM1+TEMP3**2
	          SUM2=SUM2+TEMP4**2
	          WRITE(6,4000) TIME_SAVE(I),RESIDUALS(I,3),
	1         TEMP3,RESIDUALS(I,4),TEMP4,TEMP2
	          WRITE(7,4000) TIME_SAVE(I),RESIDUALS(I,3),
	1         TEMP3,RESIDUALS(I,4),TEMP4,TEMP2
	        END IF
	      END DO
	      WRITE (6,4002) SQRT(SUM1/(NHI-NLO+1.0D0-NPOINTS_DATA_S_OMIT))
	      WRITE (6,4003) SQRT(SUM2/(NHI-NLO+1.0D0-NPOINTS_DATA_S_OMIT))
	      WRITE (7,4002) SQRT(SUM1/(NHI-NLO+1.0D0-NPOINTS_DATA_S_OMIT))
	      WRITE (7,4003) SQRT(SUM2/(NHI-NLO+1.0D0-NPOINTS_DATA_S_OMIT))
	    ELSE IF (FILETYPE_S.EQ.'THETA_RHO') THEN
	      CALL MESSAGE('                          |DELTA_THETA| '//
	1'             |DELTA_RHO|    DELTA ')
	      CALL MESSAGE('              DELTA_THETA  ___________  '//
	1' DELTA_RHO   ___________    ______')
	      CALL MESSAGE('   EPOCH          (o)     MEAN ERROR(T) '//
	1'    (")     MEAN ERROR(R)    M.E. ')
C  We will have to convert from residuals in X and Y to those in theta
C  and rho.
	      SUM1=0.0D0
	      SUM2=0.0D0
	      SUM3=0.0D0
	      SUM4=0.0D0
C  First loop through the points to calculate the mean error for an 
C  observation of average weight in theta and rho.  Then begin another 
C  loop where the residuals are compared to those mean errors.
	      DO I=NLO,NHI
C  Note that DATA_SAVE is in the form:  X and Y
	        IF (WEIGHT(I,3).GT.0.0D0.AND.WEIGHT(I,4).GT.0.0D0) THEN
	          TEMP3=SQRT(DATA_SAVE(I,3)**2+DATA_SAVE(I,4)**2)
				!  Rho observed in "
	          TEMP4=SQRT((DATA_SAVE(I,3)-RESIDUALS(I,3))**2+
	1         (DATA_SAVE(I,4)-RESIDUALS(I,4))**2) !  Rho calculated
	          TEMP4=TEMP3-TEMP4 !  Rho observerd - calculated
	          TEMP1=DATAN2D(DATA_SAVE(I,3),DATA_SAVE(I,4))
C  Change the range from -180 to +180, to 0 to 360.
	          IF (TEMP1.LT.0.0D0) TEMP1=TEMP1+360.0D0 !  Theta observed
	          TEMP5=DATAN2D(DATA_SAVE(I,3)-RESIDUALS(I,3),
	1         DATA_SAVE(I,4)-RESIDUALS(I,4))
	          IF (TEMP5.LT.0.0D0) TEMP5=TEMP5+360.0D0 !  Theta calculated
	          TEMP5=TEMP1-TEMP5 !  Theta observed - calculated
C  Make the magnitude of the difference less than 180.
	          IF (ABS(TEMP5).GT.180.0D0) THEN
	            IF (TEMP5.LT.0.0D0) THEN
	              TEMP5=TEMP5+360.0D0
	            ELSE 
	              TEMP5=TEMP5-360.0D0
	            END IF
	          END IF
C  Note that SIGMA_SAVE is scaled by any user-entered weight scaling,
C  and is here in units of theta and rho.
	          SUM1=SUM1+(TEMP5/SIGMA_SAVE(I,3))**2 !  For delta theta
	          SUM2=SUM2+(TEMP4/SIGMA_SAVE(I,4))**2 !  For delta rho
C  Get the sum of the weights for the mean error of an observation of
C  average weight calculation below.
	          SUM3=SUM3+1.0D0/SIGMA_SAVE(I,3)**2
	          SUM4=SUM4+1.0D0/SIGMA_SAVE(I,4)**2
	        END IF
	      END DO
C	      NADJUST=NUMPARAM_SOLV/DIMENSN_DATA_USED
	      NADJUST=NUMPARAM_SOLV*0.5*(NPOINTS_DATA_S-
	1     NPOINTS_DATA_S_OMIT)/NPOINTS_DATA
	      RDCS1=SUM1/(NHI-NLO+1-NPOINTS_DATA_S_OMIT-NADJUST)
			!  Reduced xi^2 for theta
	      RDCS2=SUM2/(NHI-NLO+1-NPOINTS_DATA_S_OMIT-NADJUST)
			!  Reduced xi^2 for rho
	      MEOAW1=SQRT(SUM1/((NHI-NLO+1-NPOINTS_DATA_S_OMIT-
	1     NADJUST)*SUM3/(NPOINTS_DATA_S-NPOINTS_DATA_S_OMIT)))
				!  MEOAW theta
	      MEOAW2=SQRT(SUM2/((NHI-NLO+1-NPOINTS_DATA_S_OMIT-
	1     NADJUST)*SUM4/(NPOINTS_DATA_S-NPOINTS_DATA_S_OMIT)))
				!  MEOAW rho
C  Now that we have the MEOAW for theta and rho, we can print out the
C  residuals
	      SUM1=0.0D0
	      SUM2=0.0D0
	      DO I=NLO,NHI
	        IF (WEIGHT(I,3).GT.0.0D0.AND.WEIGHT(I,4).GT.0.0D0) THEN
	          TEMP1=SQRT(RESIDUALS(I,3)**2+RESIDUALS(I,4)**2)
	          TEMP2=SQRT(MEANERROR(3)**2+MEANERROR(4)**2)
	          IF (TEMP2.GT.0.0D0) TEMP2=TEMP1/TEMP2 !  Same as X_Y case
	          TEMP3=SQRT(DATA_SAVE(I,3)**2+DATA_SAVE(I,4)**2)
				!  Rho observed in "
	          TEMP4=SQRT((DATA_SAVE(I,3)-RESIDUALS(I,3))**2+
	1         (DATA_SAVE(I,4)-RESIDUALS(I,4))**2) !  Rho calculated
	          TEMP4=TEMP3-TEMP4 !  Rho observerd - calculated
	          TEMP1=DATAN2D(DATA_SAVE(I,3),DATA_SAVE(I,4))
	          IF (TEMP1.LT.0.0D0) TEMP1=TEMP1+360.0D0 !  Theta observed
	          TEMP5=DATAN2D(DATA_SAVE(I,3)-RESIDUALS(I,3),
	1         DATA_SAVE(I,4)-RESIDUALS(I,4))
	          IF (TEMP5.LT.0.0D0) TEMP5=TEMP5+360.0D0 !  Theta calculated
	          TEMP5=TEMP1-TEMP5 !  Theta observed - calculated
C  Make the magnitude of the difference less than 180.
	          IF (ABS(TEMP5).GT.180.0D0) THEN
	            IF (TEMP5.LT.0.0) THEN
	              TEMP5=TEMP5+360.0D0
	            ELSE 
	              TEMP5=TEMP5-360.0D0
	            END IF
	          END IF
	          IF (MEOAW1.GT.0.0D0.AND.MEOAW2.GT.0.0D0) THEN
	            WRITE(6,4000) TIME_SAVE(I),TEMP5,
	1           ABS(TEMP5)/MEOAW1,TEMP4,
	2           ABS(TEMP4)/MEOAW2,TEMP2
	            WRITE(7,4000) TIME_SAVE(I),TEMP5,
	1           ABS(TEMP5)/MEOAW1,TEMP4,
	2           ABS(TEMP4)/MEOAW2,TEMP2
	          END IF
	          SUM1=SUM1+(TEMP5/MEOAW1)**2
	          SUM2=SUM2+(TEMP4/MEOAW2)**2
	        END IF
	      END DO
C
	      WRITE (6,4005) RDCS1 !  Reduced chi squared, Theta
	      WRITE (7,4005) RDCS1
	      WRITE (6,4006) RDCS2
	      WRITE (7,4006) RDCS2
	      WRITE (6,4007) MEOAW1
	      WRITE (7,4007) MEOAW1
	      WRITE (6,4008) MEOAW2
	      WRITE (7,4008) MEOAW2
C
	      WRITE (6,4011) SQRT(SUM1/(NHI-NLO-NPOINTS_DATA_S_OMIT+1.0D0))
				!  RMS average Delta_Th/ME_Th
	      WRITE (6,4012) SQRT(SUM2/(NHI-NLO-NPOINTS_DATA_S_OMIT+1.0D0))
	      WRITE (7,4011) SQRT(SUM1/(NHI-NLO-NPOINTS_DATA_S_OMIT+1.0D0))
	      WRITE (7,4012) SQRT(SUM2/(NHI-NLO-NPOINTS_DATA_S_OMIT+1.0D0))
	    END IF
	  END IF
C  Now display the spectroscopic residuals
	  IF (CHOICE(4:4).EQ.'P') THEN
	    CALL MESSAGE('For the spectroscopic primary:')
	    CALL MESSAGE('    EPOCH(BY)     EPOCH(JD)     RESIDUAL(km/s)'//
	1  ' RESIDUAL/MEAN ERROR  PHASE')
	    SUM1=0.0D0
	    NLO=NPOINTS_VIS+1
	    NHI=NPOINTS_VIS+NPOINTS_DATA_VEL_P
	    DO I=NLO,NHI
	      IF (WEIGHT(I,5).GT.0.0D0) THEN
	        IF (FILETYPE_VEL_P.NE.'JD_VELOCITY')  
	1       CALL BY_JD(TIME_SAVE(I),TIME_SAVE_JD(I))
	        TEMP1=RESIDUALS(I,5)
	        IF (MEANERROR(5).GT.0.0D0) THEN
	          TEMP2=TEMP1/MEANERROR(5)
	        ELSE
	          TEMP2=0.0D0
	        END IF
	        SUM1=SUM1+TEMP2**2
	        TEMP3=TIME_SAVE(I)-PARAMETRS(2)
	        DO WHILE(TEMP3.LT.0.0D0)
	          TEMP3=TEMP3+PARAMETRS(1)
	        END DO
	        TEMP3=MOD(TEMP3,PARAMETRS(1))/PARAMETRS(1)
	        WRITE(6,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
4010	        FORMAT(' ',F14.6,2X,F14.4,2X,G14.8,4X,G14.8,2X,F9.6)
	        WRITE(7,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	      END IF
	    END DO
	    WRITE (6,4015) SQRT(SUM1/(NHI-NLO+1.0D0-NPOINTS_DATA_VEL_P_OMIT))
4015	    FORMAT(' ','The rms average of (Delta V)/(Mean Error(V)) = ',
	1     G12.6)
	    WRITE (7,4015) SQRT(SUM1/(NHI-NLO+1.0D0-NPOINTS_DATA_VEL_P_OMIT))
	  ELSE IF (CHOICE(4:4).EQ.'S') THEN
	    CALL MESSAGE('For the spectroscopic secondary:')
	    CALL MESSAGE('    EPOCH(BY)     EPOCH(JD)     RESIDUAL(km/s)'//
	1  ' RESIDUAL/MEAN ERROR  PHASE')
	    SUM1=0.0D0
	    NLO=1+NPOINTS_VIS
	    NHI=NPOINTS_VIS+NPOINTS_DATA_VEL_S
	    DO I=NLO,NHI
	      IF (WEIGHT(I,6).GT.0.0D0) THEN
	        IF (FILETYPE_VEL_S.NE.'JD_VELOCITY')  
	1       CALL BY_JD(TIME_SAVE(I),TIME_SAVE_JD(I))
	        TEMP1=RESIDUALS(I,6)
	        IF (MEANERROR(6).GT.0.0D0) THEN
	          TEMP2=TEMP1/MEANERROR(6)
	        ELSE
	          TEMP2=0.0D0
	        END IF
	        SUM1=SUM1+TEMP2**2
	        TEMP3=TIME_SAVE(I)-PARAMETRS(2)
	        DO WHILE(TEMP3.LT.0.0D0)
	          TEMP3=TEMP3+PARAMETRS(1)
	        END DO
	        TEMP3=MOD(TEMP3,PARAMETRS(1))/PARAMETRS(1)
	        WRITE(6,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	        WRITE(7,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	      END IF
	    END DO
	    WRITE (6,4015) SQRT(SUM1/(NHI-NLO+1.0-NPOINTS_DATA_VEL_S_OMIT))
	    WRITE (7,4015) SQRT(SUM1/(NHI-NLO+1.0-NPOINTS_DATA_VEL_S_OMIT))
	  ELSE IF (CHOICE(4:4).EQ.'B') THEN
	    CALL MESSAGE('For the spectroscopic primary:')
	    CALL MESSAGE('    EPOCH(BY)     EPOCH(JD)     RESIDUAL(km/s)'//
	1  ' RESIDUAL/MEAN ERROR  PHASE')
	    SUM1=0.0
	    NLO=NPOINTS_VIS+1
	    NHI=NPOINTS_VIS+NPOINTS_DATA_VEL_P
	    DO I=NLO,NHI
	      IF (WEIGHT(I,5).GT.0.0D0) THEN
	        IF (FILETYPE_VEL_P.NE.'JD_VELOCITY')  
	1       CALL BY_JD(TIME_SAVE(I),TIME_SAVE_JD(I))
	        TEMP1=RESIDUALS(I,5)
	        IF (MEANERROR(5).GT.0.0D0) THEN
	          TEMP2=TEMP1/MEANERROR(5)
	        ELSE
	          TEMP2=0.0D0
	        END IF
	        SUM1=SUM1+TEMP2**2
	        TEMP3=TIME_SAVE(I)-PARAMETRS(2)
	        DO WHILE(TEMP3.LT.0.0D0)
	          TEMP3=TEMP3+PARAMETRS(1)
	        END DO
	        TEMP3=MOD(TEMP3,PARAMETRS(1))/PARAMETRS(1)
	        WRITE(6,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	        WRITE(7,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	      END IF
	    END DO
	    WRITE (6,4015) SQRT(SUM1/(NHI-NLO+1.0-NPOINTS_DATA_VEL_P_OMIT))
	    WRITE (7,4015) SQRT(SUM1/(NHI-NLO+1.0-NPOINTS_DATA_VEL_P_OMIT))
	    CALL MESSAGE('For the spectroscopic secondary:')
	    CALL MESSAGE('    EPOCH(BY)     EPOCH(JD)     RESIDUAL(km/s)'//
	1  ' RESIDUAL/MEAN ERROR  PHASE')
	    SUM1=0.0D0
	    NLO=NPOINTS_VIS+NPOINTS_DATA_VEL_P+1
	    NHI=NPOINTS_VIS+NPOINTS_DATA_VEL_P+NPOINTS_DATA_VEL_S
	    DO I=NLO,NHI
	      IF (WEIGHT(I,6).GT.0.0D0) THEN
	        IF (FILETYPE_VEL_S.NE.'JD_VELOCITY')  
	1       CALL BY_JD(TIME_SAVE(I),TIME_SAVE_JD(I))
	        TEMP1=RESIDUALS(I,6)
	        IF (MEANERROR(6).GT.0.0D0) THEN
	          TEMP2=TEMP1/MEANERROR(6)
	        ELSE
	          TEMP2=0.0D0
	        END IF
	        SUM1=SUM1+TEMP2**2
	        TEMP3=TIME_SAVE(I)-PARAMETRS(2)
	        DO WHILE(TEMP3.LT.0.0D0)
	          TEMP3=TEMP3+PARAMETRS(1)
	        END DO
	        TEMP3=MOD(TEMP3,PARAMETRS(1))/PARAMETRS(1)
	        WRITE(6,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	        WRITE(7,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	      END IF
	    END DO
	    WRITE (6,4015) SQRT(SUM1/(NHI-NLO+1.0-NPOINTS_DATA_VEL_S_OMIT))
	    WRITE (7,4015) SQRT(SUM1/(NHI-NLO+1.0-NPOINTS_DATA_VEL_S_OMIT))
	  END IF


C  Now display the spectroscopic primary subcomponent secondary residuals
	  DO IS=1,NPRISUB_S_F
	    ITMP=IPRISUB_S_F(IS)
	    ENCODE(1,1001,C_I) ITMP
	    CALL MESSAGE('For the spectroscopic primary subcomponent'//
	1   ' secondary #'//C_I//':')
	    CALL MESSAGE('    EPOCH(BY)     EPOCH(JD)     RESIDUAL(km/s)'//
	1   ' RESIDUAL/MEAN ERROR  PHASE')
	    SUM1=0.0D0
	    IF (IS.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+1
	    NHI=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)-1
	    DO I=NLO,NHI
	      IF (FILETYPE_VEL_P_SUB_S(ITMP).NE.'JD_VELOCITY')
	1     CALL BY_JD(TIME_SAVE(I),TIME_SAVE_JD(I))
	      TEMP1=RESIDUALS(I,6+ITMP)
	      IF (MEANERROR(6+ITMP).GT.0.0D0) THEN
	        TEMP2=TEMP1/MEANERROR(6+ITMP)
	      ELSE
	        TEMP2=0.0D0
	      END IF
	      SUM1=SUM1+TEMP2**2
	      TEMP3=TIME_SAVE(I)-PARAMETRS(INDX(19)+(IS-1)*5)
	      DO WHILE(TEMP3.LT.0.0D0)
	        TEMP3=TEMP3+PARAMETRS(INDX(18)+(IS-1)*5)
	      END DO
	      TEMP3=MOD(TEMP3,PARAMETRS(INDX(18)+(IS-1)*5))/
	1     PARAMETRS(INDX(18)+(IS-1)*5)
	      WRITE(6,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	      WRITE(7,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	    END DO
	    WRITE (6,4015) SQRT(SUM1/(NHI-NLO+1.0D0))
	    WRITE (7,4015) SQRT(SUM1/(NHI-NLO+1.0D0))
	    NLO=NLO+NPOINTS_DATA_VEL_P_SUB_S(ITMP)
	  END DO
C  Now display the spectroscopic secondary subcomponent secondary residuals
	  DO IS=1,NSECSUB_S_F
	    ITMP=ISECSUB_S_F(IS)
	    ENCODE(1,1001,C_I) ITMP
	    CALL MESSAGE('For the spectroscopic secondary subcomponent'//
	1   ' secondary #'//C_I//':')
	    CALL MESSAGE('    EPOCH(BY)     EPOCH(JD)     RESIDUAL(km/s)'//
	1   ' RESIDUAL/MEAN ERROR  PHASE')
	    SUM1=0.0D0
	    IF (IS.EQ.1) NLO=NPOINTS_VIS+NPOINTS_SP+NPOINTS_VEL_P_S+1
	    NHI=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)-1
	    DO I=NLO,NHI
	      IF (FILETYPE_VEL_S_SUB_S(ITMP).NE.'JD_VELOCITY')
	1     CALL BY_JD(TIME_SAVE(I),TIME_SAVE_JD(I))
	      TEMP1=RESIDUALS(I,11+ITMP)
	      IF (MEANERROR(11+ITMP).GT.0.0D0) THEN
	        TEMP2=TEMP1/MEANERROR(11+ITMP)
	      ELSE
	        TEMP2=0.0D0
	      END IF
	      SUM1=SUM1+TEMP2**2
	      TEMP3=TIME_SAVE(I)-PARAMETRS(INDX(44)+(IS-1)*5)
	      DO WHILE(TEMP3.LT.0.0D0)
	        TEMP3=TEMP3+PARAMETRS(INDX(43)+(IS-1)*5)
	      END DO
	      TEMP3=MOD(TEMP3,PARAMETRS(INDX(43)+(IS-1)*5))/
	1     PARAMETRS(INDX(43)+(IS-1)*5)
	      WRITE(6,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	      WRITE(7,4010) TIME_SAVE(I),TIME_SAVE_JD(I),TEMP1,TEMP2,TEMP3
	    END DO
	    WRITE (6,4015) SQRT(SUM1/(NHI-NLO+1.0D0))
	    WRITE (7,4015) SQRT(SUM1/(NHI-NLO+1.0D0))
	    NLO=NLO+NPOINTS_DATA_VEL_S_SUB_S(ITMP)
	  END DO
	END IF
C
C  Beginning of section for writing the residuals of position to a 
C  standard binary star file.  The residuals are written as X and Y
C  and as a relative orbit.
C
	WRITE_RESID=.FALSE.
	IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B'.OR.
	1CHOICE(1:1).EQ.'S') THEN
	  CALL USRINP(WRITE_RESID,1,'L',4,1,'WRITE_RESIDUALS=',
 	1 'Do you wish to write the visual binary xy residuals to a '//
	2 'file?  (default = N)')
	END IF
	IF (WRITE_RESID) THEN
C  Initialize INITIAL_ESTIMATES_WRITE so that the element values will
C  not be written to the file by WRITE_BINARY_HEADER
	  DO I=1,MAXNUMPARAM
	    INITIAL_ESTIMATES_WRITE(I)=-1.0D37
	  END DO
C  Do the primary;  unit 50 is reserved for the primary residuals
	  IF (CHOICE(1:1).EQ.'P'.OR.CHOICE(1:1).EQ.'B') THEN
	    CALL OPENCF(50,2,'FILE_50=',0)
C  Make the header indicate that the TYPE is relative, PARALLAX_PROPER_MOTION
C  is removed, and FILETYPE is X_Y.  In addition, since there is no
C  knowledge of the elements, we don't write their values.
	    CALL WRITE_BINARY_HEADER(50,NAME_P,RIGHTASC_P,DECLINA_P,
	1   EQUATOR_P,'R','R',SCALE_P,'X_Y',INITIAL_ESTIMATES_WRITE,
	2   C_ELEMENTS_WRITE,MAXNUMPARAM)
	    DO I=1,NPOINTS_DATA_P
	      WRITE(50,3000) TIME_SAVE(I),RESIDUALS(I,1)/SCALE_P,
	1     SIGMA(I,1)/SCALE_P,
	2     RESIDUALS(I,2)/SCALE_P,SIGMA(I,2)/SCALE_P
3000          FORMAT(' ',F12.6,4X,F14.5,2X,F14.5,2X,F14.5,2X,F14.5)
	    END DO
	    CLOSE (UNIT=50)
	  END IF
C  Do the secondary;  unit 51 is reserved for the secondary residuals
	  IF (CHOICE(1:1).EQ.'S'.OR.CHOICE(1:1).EQ.'B') THEN
	    CALL OPENCF(51,2,'FILE_51=',0)
	    IF (CHOICE(1:1).EQ.'S') THEN
	      NLO=1
	      NHI=NPOINTS_DATA_S
	    ELSE IF (CHOICE(1:1).EQ.'B') THEN
	      NLO=NPOINTS_DATA_P+1
	      NHI=NPOINTS_DATA_P+NPOINTS_DATA_S
	    END IF
C  Make the header indicate that the TYPE is relative, PARALLAX_PROPER_MOTION
C  is removed, and FILETYPE is X_Y.  In addition, since there is no
C  knowledge of the elements, we don't write their values.
	    CALL WRITE_BINARY_HEADER(51,NAME_S,RIGHTASC_S,DECLINA_S,
	1   EQUATOR_S,'R','R',SCALE_S,'X_Y',INITIAL_ESTIMATES_WRITE,
	2   C_ELEMENTS_WRITE,MAXNUMPARAM)
	    DO I=NLO,NHI
	      WRITE(51,3000) TIME_SAVE(I),RESIDUALS(I,3)/SCALE_S,
	1     SIGMA(I,3)/SCALE_S,
	2     RESIDUALS(I,4)/SCALE_S,SIGMA(I,4)/SCALE_S
	    END DO
	    CLOSE (UNIT=51)
	  END IF
	END IF
C
C  For future development, write the spectroscopic residuals to a file
C
C  Calculate the masses
C
	IF (VISUAL) THEN
C  If the data is from a relative orbit, calculate the
C  sum of the masses from Kepler's 3rd law.
	  IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.3) THEN
	    IF (.NOT.SOLVE_PAR) THEN
	      CALL USRINP(DISTANCE,1,'F',8,0,'DISTANCE=',
	1     'Enter the distance in parsecs:')
	      CALL MESSAGE('The error in distance is assumed to be zero.')
	      PARALLAX=1.0D0/DISTANCE
	      SIGMA_PARALLAX=0.0D0
C  The following can be safely overwritten at this point.  There is no
C  conflict with the spectroscopic cases.
	      PARAMETRS(INDX(9))=PARALLAX
	      COVARIANCE(1,INDX(9))=0.0D0
	      COVARIANCE(INDX(7),INDX(9))=0.0D0
	      COVARIANCE(INDX(9),INDX(9))=0.0D0
C  Else use the solved-for parallax
	    ELSE
	      PARALLAX=PARAMETRS(INDX(9))
	      CALL USRINP(PARALLAX,1,'F',8,1,'PARALLAX=',
	1     'Enter the parallax:  (default = value from solution)')
	      CALL MESSAGE('The errors in parallax are assumed to be those'//
	1     ' of the solution.')
	      PARAMETRS(INDX(9))=PARALLAX
	      SIGMA_PARALLAX=SQRT(COVARIANCE(INDX(9),INDX(9)))
			!  For convenience
            END IF
	    MASS_SUM=PARAMETRS(INDX(7))**3/PARALLAX**3/PARAMETRS(1)**2
	    SIGMA_MASS=SQRT((3.0D0*PARAMETRS(INDX(7))**2/(PARAMETRS(1)**2*
	1   PARALLAX**3))**2*COVARIANCE(INDX(7),INDX(7))+
	2   (2.0D0*PARAMETRS(INDX(7))**3/(PARAMETRS(1)**3*
	3   PARALLAX**3))**2*COVARIANCE(1,1)+
	4   (3.0D0*PARAMETRS(INDX(7))**3/(PARAMETRS(1)**2*PARALLAX*4))**2*
	5   SIGMA_PARALLAX**2-12.0D0*PARAMETRS(INDX(7))**5/(PARAMETRS(1)**5*
	6   PARALLAX**6)*COVARIANCE(INDX(7),1)-18.0D0*PARAMETRS(INDX(7))**5/
	7   (PARAMETRS(1)**4*PARALLAX**7)*COVARIANCE(INDX(7),INDX(9))+
	8   12.0D0*PARAMETRS(INDX(7))**6/(PARAMETRS(1)**5*PARALLAX**7)*
	9   COVARIANCE(1,INDX(9)))
	    WRITE(6,5000) MASS_SUM,SIGMA_MASS
5000	    FORMAT(' ','The sum of the masses in solar units is ',G14.8,
	1   ' +-',G13.7)
	    WRITE(7,5000) MASS_SUM,SIGMA_MASS
C
C  If we only know "a" of the primary or that of the secondary and not the
C  ratio, no mass can be derived.
	  ELSE IF ((SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1.OR.
	1 SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.2).AND..NOT.RATIO_AVAILABLE) THEN
C
C  If we know a of the primary and the ratio of semimajor axes, we can
C  solve for the individual masses.  Note that the program also assigns
C  the semimajor axis to the primary when the ratio is known.
	  ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(1).EQ.1.AND.
	1 RATIO_AVAILABLE) THEN
	    IF (PARALLAX_PROPER_MOTION_P.NE.'I'.AND.
	1   PARALLAX_PROPER_MOTION_S.NE.'I'.AND..NOT.SOLVE_PAR) THEN
C  This case is never spectroscopic
	      CALL USRINP(DISTANCE,1,'F',8,0,'DISTANCE=',
	1     'Enter the distance in parsecs:')
	      CALL MESSAGE('The error in distance is assumed to be zero.')
	      PARALLAX=1.0D0/DISTANCE
	      SIGMA_PARALLAX=0.0D0
C  The following can be safely overwritten at this point
	      COVARIANCE(INDX(7),INDX(9))=0.0D0
	      COVARIANCE(INDX(8),INDX(9))=0.0D0
	      COVARIANCE(1,INDX(9))=0.0D0
	    ELSE
C  This case could also be spectroscopic
C  Use the solved-for parallax
	      PARALLAX=PARAMETRS(INDX(9))
	      CALL USRINP(PARALLAX,1,'F',8,1,'PARALLAX=',
	1     'Enter the parallax:  (default = value from solution)')
	      CALL MESSAGE('The errors in parallax are assumed to be those'//
	1     ' of the solution.')
	      SIGMA_PARALLAX=SQRT(COVARIANCE(INDX(9),INDX(9)))
			!  For convenience
	    END IF
	    MASS_P=(PARAMETRS(INDX(8))-1.0D0)*PARAMETRS(INDX(8))**2*
	1   PARAMETRS(INDX(7))**3/PARALLAX**3/PARAMETRS(1)**2
	    TEMP1=(3.0D0*PARAMETRS(INDX(8))**2-2.0D0*PARAMETRS(INDX(8)))/
	1   (PARAMETRS(INDX(8))**3-PARAMETRS(INDX(8))**2)
	    SIGMA_MASS=
	1   SQRT(ABS((3.0D0*MASS_P/PARAMETRS(INDX(7)))**2*
	2   COVARIANCE(INDX(7),INDX(7))+
	3   (2.0D0*MASS_P/PARAMETRS(1))**2*COVARIANCE(1,1)+
	4   (3.0D0*MASS_P/PARALLAX)**2*SIGMA_PARALLAX**2+
	5   (TEMP1*MASS_P)**2*COVARIANCE(INDX(8),INDX(8))-12.0D0*MASS_P**2/
	6   (PARAMETRS(INDX(7))*PARAMETRS(1))*COVARIANCE(INDX(7),1)-
	7   18.0D0*MASS_P**2/(PARAMETRS(INDX(7))*PARALLAX)*
	8   COVARIANCE(INDX(7),INDX(9))+12.0D0*MASS_P**2/(PARAMETRS(1)*
	9   PARALLAX)*
	1   COVARIANCE(1,INDX(9))+6.0D0*MASS_P**2/PARAMETRS(INDX(7))*TEMP1*
	2   COVARIANCE(INDX(7),INDX(8))-4.0D0*MASS_P**2/PARAMETRS(1)*TEMP1*
	3   COVARIANCE(1,INDX(8))-6.0D0*MASS_P**2/PARALLAX*TEMP1*
	4   COVARIANCE(INDX(8),INDX(9))))
	    WRITE(6,5001) MASS_P,SIGMA_MASS
5001	    FORMAT(' ','The mass of the primary in solar units is ',
	1   G14.8,'+- ',G13.7)
	    WRITE(7,5001) MASS_P,SIGMA_MASS
	    MASS_S=PARAMETRS(INDX(8))**2*PARAMETRS(INDX(7))**3/
	1   PARALLAX**3/PARAMETRS(1)**2
C  It is, of course, easier to use MASS_S=MASS_P/(R-1)
	    TEMP1=2.0D0/PARAMETRS(INDX(8))
	    SIGMA_MASS=
	1   SQRT(ABS((3.0D0*MASS_S/PARAMETRS(INDX(7)))**2*
	2   COVARIANCE(INDX(7),INDX(7))+(2.0D0*MASS_S/PARAMETRS(1))**2*
	3   COVARIANCE(1,1)+(3.0D0*MASS_S/PARALLAX)**2*SIGMA_PARALLAX**2+
	4   (TEMP1*MASS_S)**2*COVARIANCE(INDX(8),INDX(8))-12.0D0*MASS_S**2/
	5   (PARAMETRS(INDX(7))*PARAMETRS(1))*COVARIANCE(INDX(7),1)-
	6   18.0D0*MASS_S**2/(PARAMETRS(INDX(7))*PARALLAX)*
	7   COVARIANCE(INDX(7),INDX(9))+12.0D0*MASS_S**2/(PARAMETRS(1)*
	8   PARALLAX)*
	9   COVARIANCE(1,INDX(9))+6.0D0*MASS_S**2/PARAMETRS(INDX(7))*TEMP1*
	1   COVARIANCE(INDX(7),INDX(8))-4.0D0*MASS_S**2/PARAMETRS(1)*TEMP1*
	2   COVARIANCE(1,INDX(8))-6.0D0*MASS_S**2/PARALLAX*TEMP1*
	3   COVARIANCE(INDX(8),INDX(9))))
	    WRITE(6,5002) MASS_S,SIGMA_MASS
5002	    FORMAT(' ','The mass of the secondary in solar units is ',
	1   G14.8,'+- ',G13.7)
	    WRITE(7,5002) MASS_S,SIGMA_MASS
C  We still need to make use of the ratios from the subcomponents
	  END IF
	ELSE IF (SPECTROSCOPIC.AND..NOT.VISUAL
	1.AND..NOT.IGNORE_MAIN_PAIR_BIN) THEN
	  IF (.NOT.RATIO_AVAILABLE) THEN
	    MASS_FUNC=PARAMETRS(INDX(17))**3/PARAMETRS(1)**2
	    SIGMA_MASS=SQRT((3.0D0*MASS_FUNC/PARAMETRS(INDX(17)))**2*
	1   COVARIANCE(INDX(17),INDX(17))+(2.0D0*MASS_FUNC/PARAMETRS(1))**2*
	2   COVARIANCE(1,1)-12.0D0*MASS_FUNC**2/PARAMETRS(INDX(17))/
	3   PARAMETRS(1)*COVARIANCE(INDX(17),1))
	    IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.1) THEN
	      WRITE(6,5050) MASS_FUNC,SIGMA_MASS
5050	      FORMAT(' ','The mass function, M_b^3sin^3(i)/(M_a+M_b)^2 in ',
	1     'solar units is',/' ',G14.8,' +- ',G14.8)
	      WRITE(7,5050) MASS_FUNC,SIGMA_MASS
	    ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.2) THEN
	      WRITE(6,5052) MASS_FUNC,SIGMA_MASS
5052	      FORMAT(' ','The mass function, M_a^3sin^3(i)/(M_a+M_b)^2 in ',
	1     'solar units is',/' ',G14.8,' +- ',G14.8)
	      WRITE(7,5052) MASS_FUNC,SIGMA_MASS
	    END IF
	  ELSE !  Ratio available
C  In this case, the projected semimajor axis is assigned to the primary
	    MASS_SUM=PARAMETRS(INDX(17))**3*PARAMETRS(INDX(8))**3/
	1   PARAMETRS(1)**2
	    SIGMA_MASS=SQRT((3.0D0*MASS_SUM/PARAMETRS(INDX(17)))**2*
	1   COVARIANCE(INDX(17),INDX(17))+(2.0D0*MASS_SUM/
	2   PARAMETRS(1))**2*
	3   COVARIANCE(1,1)+(3.0D0*MASS_SUM/PARAMETRS(INDX(8)))**2*
	4   COVARIANCE(INDX(8),INDX(8))-12.0D0*MASS_SUM**2/
	5   PARAMETRS(INDX(17))/
	6   PARAMETRS(1)*COVARIANCE(INDX(17),1)+18.0D0*MASS_SUM**2/
	7   PARAMETRS(INDX(17))/PARAMETRS(INDX(8))*
	8   COVARIANCE(INDX(17),INDX(8))-12.0D0*MASS_SUM**2/PARAMETRS(1)/
	9   PARAMETRS(INDX(8))*COVARIANCE(1,INDX(8)))
	    WRITE(6,5054) MASS_SUM,SIGMA_MASS
5054	    FORMAT(' ','The sum of the masses times sin^3(i) in solar ',
	1   'units is ',/' ',G14.8,' +- ',G14.8)
	    WRITE(7,5054) MASS_SUM,SIGMA_MASS
	    SIGMA_MASS=SQRT(COVARIANCE(INDX(8),INDX(8)))
	    WRITE(6,5056) PARAMETRS(INDX(8))-1,SIGMA_MASS
5056	    FORMAT(' ','The ratio of the masses (a to b) is ',
	1   G14.8,' +- ',G14.8)
	    WRITE(7,5056) PARAMETRS(INDX(8))-1,SIGMA_MASS
	  END IF
	END IF
	IF (SPECTROSCOPIC.AND.INCLINATION_AVAILABLE
	1.AND..NOT.IGNORE_MAIN_PAIR_BIN) THEN
C  The inclination comes from the visual data.
	  IF (.NOT.RATIO_AVAILABLE) THEN
	    MASS_FUNC=PARAMETRS(INDX(17))**3/PARAMETRS(1)**2/
	1   DSIND(PARAMETRS(INDX(5)))**3
	    SIGMA_MASS=SQRT((3.0D0*MASS_FUNC/PARAMETRS(INDX(17)))**2*
	1   COVARIANCE(INDX(17),INDX(17))+(2.0D0*MASS_FUNC/PARAMETRS(1))**2*
	2   COVARIANCE(1,1)+(3.0D0*MASS_FUNC/DSIND(PARAMETRS(INDX(5)))*
	3   DCOSD(PARAMETRS(INDX(5))))**2*COVARIANCE(INDX(5),INDX(5))-
	4   12.0D0*MASS_FUNC**2/PARAMETRS(INDX(17))/PARAMETRS(1)*
	5   COVARIANCE(INDX(17),1)-18.0D0*MASS_FUNC**2/PARAMETRS(INDX(17))/
	6   DSIND(PARAMETRS(INDX(5)))*DCOSD(PARAMETRS(INDX(5)))*
	7   COVARIANCE(INDX(17),INDX(5))+12.0D0*MASS_FUNC**2/PARAMETRS(1)/
	8   DSIND(PARAMETRS(INDX(5)))*DCOSD(PARAMETRS(INDX(5)))*
	9   COVARIANCE(1,INDX(5)))
	    IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.1) THEN
	      WRITE(6,5060) MASS_FUNC,SIGMA_MASS
5060	      FORMAT(' ','The mass function, M_b^3/(M_a+M_b)^2 in ',
	1     'solar units is',/' ',G14.8,' +- ',G14.8)
	      WRITE(7,5060) MASS_FUNC,SIGMA_MASS
	    ELSE IF (SEMIMAJOR_AXIS_ASSIGNMENT(2).EQ.2) THEN
	      WRITE(6,5062) MASS_FUNC,SIGMA_MASS
5062	      FORMAT(' ','The mass function, M_a^3/(M_a+M_b)^2 in ',
	1     'solar units is',/' ',G14.8,' +- ',G14.8)
	      WRITE(7,5062) MASS_FUNC,SIGMA_MASS
	    END IF
	  ELSE
C  In this case, since the inclination and ratio are available, it
C  follows that the semimajor axis is assigned to the primary and is
C  in units of arc seconds.  It also follows that the parallax was
C  solved for
	    MASS_SUM=PARAMETRS(INDX(7))**3*PARAMETRS(INDX(8))**3/
	1   PARALLAX**3/PARAMETRS(1)**2
	    SIGMA_MASS=SQRT((3.0D0*MASS_SUM/PARAMETRS(INDX(7)))**2*
	1   COVARIANCE(INDX(7),INDX(7))+(2.0D0*MASS_SUM/PARAMETRS(1))**2*
	2   COVARIANCE(1,1)+(3.0D0*MASS_SUM/PARAMETRS(INDX(8)))**2*
	3   COVARIANCE(INDX(8),INDX(8))+(3.0D0*MASS_SUM/
	4   PARAMETRS(INDX(9)))**2*
	5   COVARIANCE(INDX(9),INDX(9))-12.0D0*MASS_SUM**2/
	6   PARAMETRS(INDX(7))/PARAMETRS(1)*
	7   COVARIANCE(INDX(7),1)+18.0D0*MASS_SUM**2/
	8   PARAMETRS(INDX(7))/PARAMETRS(INDX(8))*
	9   COVARIANCE(INDX(7),INDX(8))-18.0D0*MASS_SUM**2/
	1   PARAMETRS(INDX(7))/PARAMETRS(INDX(9))*
	2   COVARIANCE(INDX(7),INDX(9))-12.0D0*
	3   MASS_SUM**2/PARAMETRS(1)/PARAMETRS(INDX(8))*
	4   COVARIANCE(INDX(1),INDX(8))+12.0D0*MASS_SUM**2/
	5   PARAMETRS(1)/PARAMETRS(INDX(9))*
	6   COVARIANCE(INDX(1),INDX(9))-18.0D0*
	7   MASS_SUM**2/PARAMETRS(INDX(8))/PARAMETRS(INDX(9))*
	8   COVARIANCE(INDX(8),INDX(9)))
	    WRITE(6,5064) MASS_SUM,SIGMA_MASS
5064	    FORMAT(' ','The sum of the masses in solar ',
	1   'units is ',G14.8,' +- ',G14.8)
	    WRITE(7,5064) MASS_SUM,SIGMA_MASS
	    SIGMA_MASS=SQRT(COVARIANCE(INDX(8),INDX(8)))
	    WRITE(6,5056) PARAMETRS(INDX(8))-1,SIGMA_MASS
	    WRITE(7,5056) PARAMETRS(INDX(8))-1,SIGMA_MASS
	  END IF
	END IF
C
	CALL SAVEC('SAVE') !  Closes SAVE file
	END
